	//
	// EstDat.Hpp
	//
	// Definicin de las distintas clases para manejo de
	// estructuras de datos.
	//
	// Francisco Charte - Jan, 30 Julio
	//

#ifndef __cplusplus
#error No se puede usar EstDat en lenguaje C   // Controlar que se utiliza el compilador C++
#endif

#ifndef _ESTDAT_HPP_    // No incluir el archivo ms de una vez

#define _ESTDAT_HPP_

#include <iostream.h> // Referencia a clases istream y ostream
#include <dos.h>
#include <stdlib.h>  // Para poder usar NULL

#include <mem.h> // Copiar y comparar bloques de memoria

	// Si el modelo de memoria seleccionado permite varios segmentos de datos
	// habr que utilizar las versiones far de algunas funciones

#if defined(__LARGE__) || defined(__HUGE__) || defined(__COMPACT__)

	#define istrcpy( orig, dest )	_fstrcpy( (orig), (dest) )
	#define istrcmp( orig, dest )	_fstrcmp( (orig), (dest) )
	#define istrcat( orig, dest )	_fstrcat( (orig), (dest) )
	#define istrchr( cade, cara )	_fstrchr( (cade), (cara) )
	#define istrlen( cade )			_fstrlen( (cade) )
	#define imemcmp( orig, dest, long ) _fmemcmp( (orig), (dest), (long) )

#else

	#define istrcpy( orig, dest )	strcpy( (orig), (dest) )
	#define istrcmp( orig, dest )	strcmp( (orig), (dest) )
	#define istrcat( orig, dest )	strcat( (orig), (dest) )
	#define istrchr( cade, cara )	strchr( (cade), (cara) )
	#define istrlen( cade )			strlen( (cade) )
	#define imemcmp( orig, dest, long ) memcmp( (orig), (dest), (long) )

#endif

enum Bool { Falso, Verdad } ; // Enumeracin bi-estado

	//
	// Todas las definiciones siguientes son plantillas, que toman
	// siempre un tipo de dato, Tipo, sobre el que basan todas sus
	// operaciones. Tipo puede ser cualquier tipo de dato, incluso
	// los creados por nosotros mismos, siempre que para l estn
	// definidos:
	//
	//	Constructor de copia, y operador de asignacin
	//
	//	Operador relacional ==
	//
	//	Operador << para inserccin en un stream
	//
	//	Operador >> para extraccin de un stream
	//
	//	Conversin de int a la clase del objeto, para crear elemento nulo
	//


template <class Tipo>
////////////////////////////////////////////
// Clase : Elemento
//
// Deriva de : -
//
// Base de : -
//
// Finalidad : Servir como unidad bsica o
//	       elemento de una lista
//
// Fecha de creacin : 30 Julio 1992
//
// Fecha de revisin : -
////////////////////////////////////////////
class Elemento
 {
	Tipo	Contenido ;  // Contenido del objeto

	Elemento 	* Anterior, * Siguiente ;  // Punteros para enlazar con otros objetos

	friend class Lista<Tipo> ; // Se da acceso a los miembros a la clase Lista

 } ;

template <class Tipo>
////////////////////////////////////////////
// Clase : Lista
//
// Deriva de : -
//
// Base de : Pila, Cola
//
// Finalidad : Mantener listas doblemente
//	       enlazadas, y servir como
//	       base para otras estructuras
//
// Fecha de creacin : 30 Julio 1992
//
// Fecha de revisin : -
////////////////////////////////////////////
class Lista
 {
	public :

		Lista() ; 	// Constructor por defecto

		Lista(  Lista & ) ; // Constructor de copia y asignacin

		Lista( int ) ; // Para crear un objeto nulo

		~Lista() ;	// Destructor

		void operator= (  Lista & ) ; // Asignar una lista a otra

		int operator== (  Lista & ) ; // Relacin de igualdad entre dos objetos

	    // Operadores para insertar y extraer elementos de la lista

		Lista<Tipo> & operator << (  Tipo ) ;

		Lista<Tipo> & operator >> ( Tipo & ) ;

	    // Mtodos para establecer el orden de extraccin e inserccin

		Lista<Tipo> & ExtrInicio() ;
		Lista<Tipo> & ExtrFinal() ;
		Lista<Tipo> & Extr(  Tipo & ) ;

		Lista<Tipo> & InseInicio() ;
		Lista<Tipo> & InseFinal() ;
		Lista<Tipo> & Inse(  Tipo & ) ;

	    // Mtodos informativos

		unsigned Elementos() ; // Devuelve el nmero de elementos en la lista

		Bool Esta(  Tipo & ) ; // Indica si est o no en la lista un determinado elemento

		Tipo Anterior(  Tipo & ) ; // Devuelve el elemento anterior
		Tipo * pAnterior(  Tipo & )  ; // Devuelve un puntero al elemento anterior

		Tipo Siguiente(  Tipo & ) ; // Devuelve el elemento siguiente
		Tipo * pSiguiente(  Tipo & )  ; // Devuelve un puntero al elemento siguiente

		void Primero(  Tipo & ) ; // Pasa el elemento indicado al principio de la lista

		void Ultimo(  Tipo & ) ; // Pasa el elemento indicado al final de la lista

		Tipo Primero() ; // Devuelve el primer elemento
		Tipo * pPrimero()  ; // Devuelve un puntero al primer elemento

		Tipo Ultimo() ; // Devuelve el ltimo elemento
		Tipo * pUltimo()  ; // Devuelve un puntero al ltimo elemento

		// Este mtodo llama a la funcin pasada como parmetro por
		// cada elemento de la lista, parando o siguiendo segn el
		// valor devuelto, que tambin devuelve.

		int Procesa( int (*) ( Tipo & ) ) ;

		// Este operador servir para comprobar el estado de la lista

		int operator ! () ;

		// Desactivar el indicador de error

		void Vale() ;

		// Operadores para poder almacenar o recuperar una lista

		friend ostream & operator << ( ostream &, Lista & ) ;

		friend istream & operator >> ( istream &, Lista & ) ;

	private :

		Elemento<Tipo> * EsElemento(  Tipo & ) ; // Devolver el elemento en que est un dato

		Elemento<Tipo> * Inicio, * Fin ; // Inicio y fin de la lista

		Elemento<Tipo> * Entrada, * Salida ; // Elementos en los que se producir la prxima inserccin/extraccin

		Bool Estado ;

		static unsigned Cont ;

		static int Cuenta( Tipo & ) ; // Mtodo privado para contar elementos
 } ;

template <class Tipo>
unsigned Lista<Tipo>::Cont ; // Hay que declarar el miembro esttico

template <class Tipo>
////////////////////////////////////////////
// Clase : Pila
//
// Deriva de : Lista
//
// Base de : -
//
// Finalidad : Mantener pilas de datos
//
// Fecha de creacin : 31 Julio
//
// Fecha de revisin : -
////////////////////////////////////////////
class Pila : public Lista<Tipo>
 {
	public :

		Pila() : Lista<Tipo>() {}

		Pila( Pila & P ) : Lista<Tipo>( P ) {}

		// Operadores especficos para inserccin/extraccin de elementos

		Pila & operator << (  Tipo ) ;

		Pila & operator >> ( Tipo & ) ;
 } ;

template <class Tipo>
////////////////////////////////////////////
// Clase : Cola
//
// Deriva de : Lista
//
// Base de : -
//
// Finalidad : Mantener colas de datos
//
// Fecha de creacin : 31 Julio
//
// Fecha de revisin : -
////////////////////////////////////////////
class Cola : public Lista<Tipo>
 {
	public :

		Cola() : Lista<Tipo>() {}

		Cola(  Cola & C ) : Lista<Tipo>( C ) {}

		// Operadores especficos para inserccin/extraccin de elementos

		Cola & operator << (  Tipo ) ;

		Cola & operator >> ( Tipo & ) ;
 } ;

template <class Tipo>
////////////////////////////////////////////
// Clase : Matriz
//
// Deriva de : -
//
// Base de : -
//
// Finalidad : Mantener matrices que se
//	       vayan autodimensionando
//
// Fecha de creacin : 31 Julio
//
// Fecha de revisin : -
////////////////////////////////////////////
class Matriz
 {
	public :

		Matriz() ; // Constructor por defecto

		Matriz(  Matriz & ) ; // Constructor de copia

		Matriz( int ) ; // Para construir un objeto nulo

		~Matriz() ; // Destructor

		void operator= (  Matriz & ) ; // Operador de asignacin

		int operator== (  Matriz & ) ; // Operador de relacin de igualdad

		// Operador para acceder a un elemento de la matriz

		Tipo & operator [] (  unsigned Elemento ) ;

		unsigned Elementos() { return Ultimo ; } // Devuelve el nmero de elementos en la matriz

		// Llamar a una determinada funcin para todos los elementos

		int Procesa( int (*) ( Tipo & ) ) ;


		friend ostream & operator << ( ostream &, Matriz ) ;

		friend istream & operator >> ( istream &, Matriz & ) ;

	private :

		const Factor ; // Factor de incremento de la matriz

		Tipo * Datos ; // Puntero a la matriz

		unsigned nElementos, Ultimo ; // Nmero de elementos en la matriz y ltimo elemento utilizado
 } ;

	//
	// Mtodo : Lista
	//
	// Constructor por defecto de la clase. Simplemente inicializa
	// con NULL el puntero al primer elemento, y desactiva el indicador
	// de error.
	//

template <class Tipo>
inline Lista<Tipo>::Lista()

	{ Inicio = Salida = Entrada = Fin = NULL ; Estado = Verdad ; }

	//
	// Mtodo : Esta
	//
	// Devuelve Verdad o Falso dependiendo de que un cierto
	// dato se encuentre actualmente en la lista o no, para
	// lo que se sirve del mtodo EsElemento, que puede -
	// devolver NULL o un puntero a un elemento.
	//

template <class Tipo>
inline Bool Lista<Tipo>::Esta(  Tipo & Dato )

	{ return EsElemento( Dato ) != NULL ? Verdad : Falso ; }

	//
	// Mtodo : Anterior
	//
	// Devuelve el dato que est en la lista delante del que se pasa
	// como parmetro, o NULL si no hay uno delante.
	//

template <class Tipo>
Tipo Lista<Tipo>::Anterior(  Tipo & Dato )
 {
	Tipo NULO = NULL ; // Se crea un objeto nulo

	return EsElemento( Dato ) == NULL ||
		 EsElemento( Dato ) == Inicio ? Estado = Falso, NULO : ( EsElemento( Dato ) -> Anterior -> Contenido ) ;
 }

	//
	// Mtodo : pAnterior
	//
	// Devuelve la direccin del dato que est en la lista
	// delante del que se pasa como parmetro, o NULL si
	// no hay uno delante.
	//

template <class Tipo>
Tipo * Lista<Tipo>::pAnterior(  Tipo & Dato )
 {
	return EsElemento( Dato ) == NULL ||    // Si elemento pasado es nulo
	       EsElemento( Dato ) == Inicio ? // o no hay anterior
	       Estado = Falso, (Tipo *) 0 : // devolver NULL
	       & ( EsElemento( Dato ) -> Anterior -> Contenido ) ; // sino la direccin
 }

	//
	// Mtodo : Siguiente
	//
	// Devuelve el dato que est en la lista detrs del que se pasa
	// como parmetro, o NULL si no hay uno detrs
	//

template <class Tipo>
Tipo Lista<Tipo>::Siguiente(  Tipo & Dato )
 {
	Tipo NULO = NULL ;

	return EsElemento( Dato ) == NULL ||
		 EsElemento( Dato ) == Fin ?
		 Estado = Falso, NULO :
		 EsElemento( Dato ) -> Siguiente -> Contenido ;
 }

	//
	// Mtodo : pSiguiente
	//
	// Devuelve la direccin del dato que est en la lista
	// detrs del que se pasa como parmetro, o NULL si
	// no hay uno detrs
	//

template <class Tipo>
Tipo * Lista<Tipo>::pSiguiente(  Tipo & Dato )
 {
	return EsElemento( Dato ) == NULL ||
	       EsElemento( Dato ) == Fin ?
		   Estado = Falso, (Tipo *) 0 :
	       & ( EsElemento( Dato ) -> Siguiente -> Contenido ) ;
 }

	//
	// Mtodo : Primero
	//
	// Mueve el elemento indicado al principio de la lista
	//

template <class Tipo>
void Lista<Tipo>::Primero(  Tipo & Dato )
 {
	Elemento<Tipo> * Temporal = EsElemento( Dato ) ;

	if( Temporal != Inicio ) // Si no es ya el primer elemento
	 {
		Temporal -> Anterior -> Siguiente = Temporal -> Siguiente ;

		if( Temporal != Fin ) // Se rompe el ES

			Temporal -> Siguiente -> Anterior = Temporal -> Anterior ;

		else
				      // Si era el ltimo
			Fin = Temporal -> Anterior ; // El ltimo es ahora el anterior

		Inicio -> Anterior = Temporal ;

		Temporal -> Siguiente = Inicio ;
		Temporal -> Anterior = NULL ;

		Inicio = Temporal ;
	 }
 }

	//
	// Mtodo : Ultimo
	//
	// Mueve el elemento indicado al final de la lista
	//

template <class Tipo>
void Lista<Tipo>::Ultimo(  Tipo & Dato )
 {
	Elemento<Tipo> * Temporal = EsElemento( Dato ) ;

	if( Temporal != Fin ) // Si no es ya el ltimo
	 {
		if( Temporal != Inicio ) // Se rompe el EA

			Temporal -> Anterior -> Siguiente = Temporal -> Siguiente ;

		Temporal -> Siguiente -> Anterior = Temporal -> Anterior ;

		if( Temporal == Inicio ) // Si era el primero

			Inicio = Temporal -> Siguiente ; // El primero es ahora el siguiente

		Fin -> Siguiente = Temporal ;

		Temporal -> Anterior = Fin ;
		Temporal -> Siguiente = NULL ;

		Fin = Temporal ;
	 }
 }

	//
	// Mtodo : Primero
	//
	// Devuelve el primer dato que hay en la lista, o NULL si
	// la lista est vaca.
	//

template <class Tipo>
Tipo Lista<Tipo>::Primero()
 {
	Tipo NULO = NULL ;

	return Inicio == NULL ? Estado = Falso, NULO : Inicio -> Contenido ;
 }

	//
	// Mtodo : pPrimero
	//
	// Devuelve la direccin del primer dato que hay en la lista,
	// o NULL si la lista est vaca.
	//

template <class Tipo>
Tipo * Lista<Tipo>::pPrimero()
 {
	return Inicio == NULL ?
               Estado = Falso, (Tipo *) 0 :
               & ( Inicio -> Contenido ) ;
 }

	//
	// Mtodo : Ultimo
	//
	// Devuelve el ltimo dato que hay en la lista, o NULL si
	// la lista est vaca.
	//

template <class Tipo>
Tipo Lista<Tipo>::Ultimo()
 {
	Tipo NULO = NULL ;

	return Inicio == NULL ? Estado = Falso, NULO : Fin -> Contenido ;
 }

	//
	// Mtodo : pUltimo
	//
	// Devuelve la direccin del ltimo dato que hay en la lista,
        // o NULL si la lista est vaca.
	//

template <class Tipo>
Tipo * Lista<Tipo>::pUltimo()
 {
	return Inicio == NULL ?
               Estado = Falso, (Tipo *) 0 :
					& ( Fin -> Contenido ) ;
 }

	//
	// Mtodo : operador !
	//
	// Permite comprobar si en la lista se ha producido un error,
	// en la forma "if ( ! Lista ) ..."
	//

template <class Tipo>
inline int Lista<Tipo>::operator ! ()

	{ return Estado == Falso ; }

	//
	// Mtodo : Vale
	//
	// Desactiva el indicador de error, que se activa automticamente
	// al intentar acceder a un elemento inexistente.
	//

template <class Tipo>
inline void Lista<Tipo>::Vale()

	{ Estado = Verdad ; }

	//
	// Mtodo : Lista
	//
	// Constructor de copia y asignacin. Toma como parmetro
	// otra lista, y copia todos sus elementos en la que se crea.
	//

template <class Tipo>
Lista<Tipo>::Lista(  Lista<Tipo> & Otra ) // Constructor de copia
 {
	Inicio = Salida = Fin = Entrada = NULL ;    // En principio la lista est vaca

	Estado = Verdad ; // Y no hay error

		// Temporal tiene la direccin del primer elemento de la lista

	Tipo * Temporal = Otra . pPrimero() ;

	while( Temporal != NULL ) // Recorrer toda la lista
	 {
		*this << *Temporal ; // Insertar el dato en la nueva lista

		Temporal = Otra . pSiguiente( *Temporal ) ; // Siguiente elemento de la lista
	 }
 }

	//
	// Mtodo =
	//
	// Operador que permite asignar una lista a otra.
	//

template <class Tipo>
void Lista<Tipo>::operator= (  Lista<Tipo> & L )
 {
	Tipo X ; // Objeto auxiliar

	ExtrInicio() ; // Comenzar a extraer desde el primer elemento

	while(1)  // Extraer todos los elementos de la lista
	 {
		*this >> X ; // Extraer elemento

		if( ! *this ) // Si la lista ya est vaca

			break ; // fin
	 }

	Vale() ; // Borrar indicador de error

		// Temporal tiene la direccin del primer elemento de la lista

	Tipo * Temporal = L . pPrimero() ;

	while( Temporal != NULL ) // Recorrer toda la lista
	 {
		*this << *Temporal ; // Insertar el dato en la nueva lista

		Temporal = L . pSiguiente( * Temporal ) ; // Siguiente elemento de la lista
	 }
 }

	//
	// Mtodo : Lista
	//
	// Crea un objeto nulo
	//


template <class Tipo>
inline Lista<Tipo>::Lista( int )
 {
	Inicio = Fin = Entrada = Salida = NULL ; // La lista es nula, est vaca
 }
	//
	// Mtodo : ~Lista
	//
	// Destructor de la clase. Libera la memoria asignada para
	// mantener los elementos de la lista.
	//

template <class Tipo>
Lista<Tipo>::~Lista() // Destructor
 {
	if( Inicio != NULL ) // Si hay elementos
	 do
	  {
		Entrada = Inicio ; // Tomar la direccin de uno

		Inicio = Entrada -> Siguiente ; // Inicio apunta al siguiente

		delete Entrada ; // Borra el actual

	  } while( Entrada != Fin ) ; // Mientras queden
 }

	//
	// Mtodo : operador ==
	//
	// Permite establecer una relacin de igualdad entre dos objetos
	//

template <class Tipo>
int Lista<Tipo>::operator== (  Lista<Tipo> & L )
 {
	Elemento<Tipo> * El1 = Inicio ; // Primer elemento del objeto implcito

	Elemento<Tipo> * El2 = L . Inicio ; // Primer elemento de la lista L

	while( El1 != Fin && El2 != Fin &&          // Mientras no se haya llegado al final de la lista
		   El1 -> Contenido == El2 -> Contenido ) // y los elementos sean iguales
	 {
		El1 = El1 -> Siguiente ; // Pasar a los siguientes elementos
		El2 = El2 -> Siguiente ;
	 }

	return ( El1 == Fin && El2 == Fin ) ; // Si se ha llegado al final de las dos listas es que eran iguales
 }

	//
	// Mtodo : ExtrInicio
	//
	// Asigna al miembro Salida el valor adecuado para extraer
	// valores desde el principio de la lista.
	//

template <class Tipo>
Lista<Tipo> & Lista<Tipo>::ExtrInicio() // Extraer del principio de la lista
 {
	Salida = Inicio ;

	return *this ;
 }

	//
	// Mtodo : ExtrFinal
	//
	// Asigna al miembro Salida el valor adecuado para extraer
	// valores desde el final de la lista.
	//

template <class Tipo>
Lista<Tipo> & Lista<Tipo>::ExtrFinal() // Extraer del final de la lista
 {
	Salida = Fin ;

	return *this ;
 }

	//
	// Mtodo : Extr
	//
	// Asigna al miembro Salida el valor recibido como parmetro,
	// para extraer elementos a partir de ese.
	//

template <class Tipo>
Lista<Tipo> & Lista<Tipo>::Extr(  Tipo & Dato ) // Extraer a partir de un elemento dado
 {
	Salida = EsElemento( Dato ) ;

	return *this ;
 }

	//
	// Mtodo : InseInicio
	//
	// Asigna al miembro Entrada el valor adecuado para que los
	// nuevos elementos se inserten al principio de la lista.
	//

template <class Tipo>
Lista<Tipo> & Lista<Tipo>::InseInicio() // Insertar al inicio de la lista
 {
	Entrada = Inicio ;

	return *this ;
 }

	//
	// Mtodo : InseFinal
	//
	// Asigna al miembro Entrada el valor adecuado para que los
	// nuevos elementos se inserten al final de la lista.
	//

template <class Tipo>
Lista<Tipo> & Lista<Tipo>::InseFinal() // Insertar al final de la lista
 {
	Entrada = NULL ;

	return *this ;
 }

	//
	// Mtodo : Inse
	//
	// Asigna al miembro Entrada el valor recibido como parmetro,
	// para que los nuevos elementos se inserten a partir de l.
	//

template <class Tipo>
Lista<Tipo> & Lista<Tipo>::Inse(  Tipo & Dato ) // Insertar a partir de un cierto elemento
 {
	Entrada = EsElemento( Dato ) ;

	return *this ;
 }

	//
	// Mtodo : Procesa
	//
	// Recibe como parmetro un puntero a una funcin, a la que
	// llamar por cada elemento que exista en la lista. Si la
	// funcin devuelve en cualquier momento un valor distinto
	// de 0 el proceso se cancela, devolviendo el mismo valor
	// que haya devuelto la funcin.
	//

template <class Tipo>
int Lista<Tipo>::Procesa( int ( *F ) ( Tipo & ) )  // Llamar a F por cada elemento de la lista
 {
	Elemento<Tipo> * Temporal = Inicio ; // Temporal apunta al primer elemento

	int Retorno ;

	while( Temporal != Fin ) // Mientras no lleguemos al final de la lista
	 {
		Retorno = F( Temporal -> Contenido ) ; // Llamar a la funcin pasando el dato del elemento

		if( Retorno ) // Si el valor de retorno no fue 0

			return Retorno ; // Terminar devolviendo ese mismo valor

		Temporal = Temporal -> Siguiente ; // Continuar con el siguiente elemento
	 }

	return 0 ; // Devolver 0 si se procesaron todos los elementos
 }

	//
	// Mtodo : Elementos
	//
	// Devuelve el nmero de elementos existentes actualmente en
	// la lista.
	//

template <class Tipo>
unsigned Lista<Tipo>::Elementos() // Devuelve el nmero de elementos en la lista
 {
	Cont = 0 ; // Poner a 0 el contador

	Procesa( Cuenta ) ; // Llamar al mtodo Cuenta por cada elemento

	return Cont ; // Devolver al contador
 }

	//
	// Mtodo : Cuenta
	//
	// Este mtodo es llamado automticamente por el mtodo Procesa.
	// Su finalidad es incrementar el contador por cada elemento
	// existente en la lista.
	//

template <class Tipo>
int Lista<Tipo>::Cuenta( Tipo & ) // El parmetro no se va a utilizar
 {
	Cont++ ; // Se incrementa el contador

	return 0 ; // Devolver 0 para que el proceso no se detenga
 }

	//
	// Mtodo : operador <<
	//
	// Permite insertar un nuevo elemento en la lista.
	//

template <class Tipo>
Lista<Tipo> & Lista<Tipo>::operator << (  Tipo Dato )
 {
	Elemento<Tipo> * Nuevo = new Elemento<Tipo> ; // Se crea un nuevo elemento

	Nuevo -> Contenido = Dato ; // Se guarda en l el dato pasado como parmetro

	if( Inicio == NULL ) // Si es el primer elemento que se aade a la lista
	 {
		Inicio = Fin = Nuevo ; // Tanto el inicio como el final de la lista es el nuevo elemento

		Salida = Nuevo ; // La extraccin de elementos ser a partir del nuevo elemento

		Entrada = NULL ; // La entrada de elementos ser a partir del final de la lista

		Nuevo -> Anterior = Nuevo -> Siguiente = NULL ; // Ahora mismo no hay ni anterior ni siguiente
	 }
	else // Si la lista ya tena elementos

	 if( Entrada == NULL )	// Si hay que aadir al final de la lista
	 {
		Nuevo -> Anterior = Fin ; // El anterior al nuevo es el que hasta ahora era el ltimo

		Nuevo -> Siguiente = NULL ; // El nuevo no tiene siguiente, puesto que va a ser el ltimo

		Fin -> Siguiente = Nuevo ; // El que antes era el ltimo ya s tiene siguiente

		Fin = Nuevo ; // El nuevo elemento es ahora el ltimo
	 }
	else // Si hay que aadir delante del elemento indicado por Entrada
	 {
		Nuevo -> Siguiente = Entrada ; // El siguiente al nuevo ser Entrada

		Nuevo -> Anterior = Entrada -> Anterior ; // El anterior al nuevo es el anterior a Entrada

		if( Entrada != Inicio ) // Si entrada tiene un anterior

			Entrada -> Anterior -> Siguiente = Nuevo ; // ese anterior tendr como siguiente al nuevo

		Entrada -> Anterior = Nuevo ; // El anterior a Entrada es el nuevo

		if( Entrada == Inicio )  // Si se est aadiendo al inicio de la lista

			Inicio = Nuevo ; // el nuevo elemento se ha convertido en el primero
	 }

	return *this ; // Devolver una referencia a la lista en la que se est insertando
 }

	//
	// Mtodo : operador >>
	//
	// Facilita un elemento de la lista, extrayndolo.
	//


template <class Tipo>
Lista<Tipo> & Lista<Tipo>::operator >> ( Tipo & Dato )
 {
	if( Salida == NULL ) // Si la lista est vaca

		Estado = Falso ; // Activar indicador de error en la ista

	else // Si hay datos
	 {
		Dato = Salida -> Contenido ; // Coger el dato correspondiente al elemento Salida

		// Eliminar el elemento Salida de la lista

		Elemento<Tipo> * Temporal = Salida -> Siguiente ; // Temporal apunta al elemento siguiente al que se ha extraido

		if( Salida != Inicio ) // Si hay un elemento anterior al que se ha extraido

			Salida -> Anterior -> Siguiente = Salida -> Siguiente ; // Ese elemento tendr como siguiente al siguiente del que se ha extraido

		if( Salida != Fin ) // Si haba un siguiente

			Salida -> Siguiente -> Anterior = Salida -> Anterior ; // El anterior es ahora el anterior al que se ha extraido

		if( Salida == Fin ) // Si se ha extraido el ltimo

			Fin = Salida -> Anterior ; // El ltimo es ahora el anterior al que se ha extraido

		if( Salida == Inicio ) // Si se ha extraido el primero

			Inicio = Salida -> Siguiente ; // El primero es ahora el siguiente al que se ha extraido

		delete Salida ; // Eliminar el elemento que se ha extraido

		Salida = Temporal ; // La prxima salida ser el siguiente al que se ha extraido
	 }

	return *this ; // Devolver referencia a la lista de la que se est extrayendo
 }

	//
	// Mtodo : EsElemento
	//
	// Busca un determinado dato entre los elementos que hay en la
	// lista, devolviendo un puntero al elemento que lo contiene o
	// NULL si no lo encuentra.
	//

template <class Tipo>
Elemento<Tipo> * Lista<Tipo>::EsElemento(  Tipo & Dato )
 {
        if( Inicio != NULL && Inicio -> Contenido == Dato ) // Si es el primer elemento

            return Inicio ;

	Elemento<Tipo> * Temporal = Inicio ; // Temporal apunta al primer elemento

	if( Temporal != NULL ) // Si la lista no est vaca
        do
	 {
		Temporal = Temporal -> Siguiente ; // Si no continuar buscando en el siguiente

		if( Temporal == NULL ) // Si el puntero es nulo

			break ;

		if( Temporal -> Contenido == Dato ) // Si hemos encontrado el dato que buscamos

			return Temporal ; // Devolver el puntero al elemento

	 } while( Temporal != Fin ) ;

	return NULL ; // Si no se ha encontrado devolver NULL
 }

	//
	// Mtodo : operador <<
	//
	// Permite escribir una lista en un stream.
	//

template <class Tipo>
ostream & operator<< ( ostream & Stream, Lista<Tipo> & L )
 {
	L . Vale() ; // Borrar indicador de error

	Tipo Dato = L . Primero() ; // Dato es el primer dato que hay en la lista

	do
	 {
		if( ! L ) // Si la lista est en estado de error, porque no haya ms elementos

			break ; // interrumpir el proceso

		Stream << Dato ; // Escribir el dato extraido en el stream

		Dato = L . Siguiente( Dato ) ; // Pasar al siguiente dato

	 } while( 1 ) ;

	 L . Vale() ; // Borrar el indicador de error

	 return Stream ; // Devolver el mismo stream sobre el que se trabaja
 }

	//
	// Mtodo : operador >>
	//
	// Permite leer una lista desde un stream.
	//

template <class Tipo>
istream & operator >> ( istream & Stream, Lista<Tipo> & L )
 {
	Tipo Dato ;

	do
	 {
		Stream >> Dato ; // Leer un dato

		if( ! Stream ) // Si no haba ms

			break ; // interrumpir el proceso

		L << Dato ; // Aadir el dato a la lista

	 } while( 1 ) ;

	return Stream ; // Devolver el stream sobre el que se trabaja
 }

	//
	// Mtodos de la clase Pila
	//

template <class Tipo>
Pila<Tipo> & Pila<Tipo>::operator << (  Tipo Dato )
 {
	InseInicio() ;

	Lista<Tipo>::operator<< ( Dato ) ;

	return *this ;
 }

template <class Tipo>
Pila<Tipo> & Pila<Tipo>::operator >> ( Tipo & Dato )
 {
	ExtrInicio() ;

	Lista<Tipo>::operator>> ( Dato ) ;

	return *this ;
 }

	//
	// Mtodos de la clase Cola
	//

template <class Tipo>
Cola<Tipo> & Cola<Tipo>::operator << (  Tipo Dato )
 {
	InseFinal() ;

	Lista<Tipo>::operator<< ( Dato ) ;

	return *this ;
 }

template <class Tipo>
Cola<Tipo> & Cola<Tipo>::operator >> ( Tipo & Dato )
 {
	ExtrInicio() ;

	Lista<Tipo>::operator>> ( Dato ) ;

	return *this ;
 }

	//
	// Mtodos de la clase Matriz
	//

	//
	// Mtodo : Matriz
	//
	// Constructor por defecto de la clase. Crea la matriz
	// con un tamao inicial de Factor elementos
	//

template <class Tipo>
Matriz<Tipo>::Matriz() : Factor( 16 ), nElementos( Factor ), Ultimo( 0 )
 {
	// Factor es el nmero de elementos inicialmente en la matriz

	// Ultimo es el ltimo elemento utilizado hasta ahora

	Datos = new Tipo[ nElementos ] ; // Creacin de la matriz
 }

	//
	// Mtodo : Matriz
	//
	// Constructor de copia, permite crear un objeto a partir
	// de otro ya existente.
	//

template <class Tipo>
Matriz<Tipo>::Matriz(  Matriz<Tipo> & M ) : Factor( M . Factor ), nElementos( M . nElementos ), Ultimo( M . Ultimo )
 {
	Datos = new Tipo[ nElementos ] ; // Asignar la memoria necesaria

	  // Copiar el contenido de la matriz M a la matriz implcita

	for( int I = 0 ; I < Ultimo ; I++ )

		Datos[ I ] = M . Datos[ I ] ;
 }

	//
	// Mtodo : Matriz
	//
	// Genera un objeto nulo
	//

template <class Tipo>
Matriz<Tipo>::Matriz( int ) : Factor( 16 ), nElementos( Factor ), Ultimo( 0 )
 {
	Datos = new Tipo[ nElementos ] ; // Creacin de la matriz
 }

	//
	// Mtodo : operador =
	//
	// Permite asignar un objeto a otro
	//

template <class Tipo>
void Matriz<Tipo>::operator= (  Matriz<Tipo> & M )
 {
	if( Datos != 0 )

		delete [] Datos ; // Se libera la memoria ya asignada

	nElementos = M . nElementos ; // Se toma el nmero de elementos
	Ultimo = M . Ultimo ; // de la matriz y el ltimo usado

	Datos = new Tipo[ nElementos ] ; // Se asigna memoria para la nueva matriz

		// Se copia el contenido de la matriz M

	for( int I = 0 ; I < Ultimo ; I++ )

		Datos[ I ] = M . Datos[ I ] ;
 }

	//
	// Mtodo : operador ==
	//
	// Permite establecer una relacin de igualdad entre dos
	// objetos Matriz.
	//

template <class Tipo>
int Matriz<Tipo>::operator== (  Matriz<Tipo> & M )
 {
	if( nElementos != M . nElementos || // Si el nmero de elementos
		Ultimo != M . Ultimo ) // no coincide

		return 0 ; // las matrices no son iguales

		// Comparar sus contenidos

	if( imemcmp( Datos, M . Datos, sizeof( Tipo ) * Ultimo ) )

		return 0 ; // Si son diferentes

	return 1 ; // Si son iguales
 }

	//
	// Mtodo : ~Matriz
	//
	// El destructor lo nico que tiene que hacer es liberar la
	// memoria asignada para contener la matriz.
	//

template <class Tipo>
Matriz<Tipo>::~Matriz()
 {
	if( Datos != 0 )

		delete [] Datos ; // Se libera la matriz
 }

	//
	// Mtodo : operador []
	//
	// Devuelve una referencia al elemento indicado, permitiendo
	// as tanto la asignacin de valores como el uso de ellos.
	//

template <class Tipo>
Tipo & Matriz<Tipo>::operator [] (  unsigned N )
 {
	if( N >= nElementos ) // Si se intenta acceder a un elemento ms all de la dimensin actual
	 {
		// Se crea una nueva matriz con Factor elementos ms que antes, o
		// con N+1 elementos, depende del elemento al que se intente acceder

		unsigned Cuantos = nElementos + Factor > N ? nElementos + Factor : N + 1 ;

		Tipo * Temporal = new Tipo[ Cuantos ] ; // Se crea la nueva matriz

		// Se pasan todos los elementos de la matriz anterior a la nueva

		if( Datos != 0 )
		 {
			movedata( FP_SEG( Datos ), FP_OFF( Datos ),
					  FP_SEG( Temporal ), FP_OFF( Temporal ),
					  sizeof( Tipo ) * nElementos ) ;

			delete [] Datos ; // Se borra la antigua matriz
		 }

		Datos = Temporal ; // Tomar la direccin de la nueva matriz

		nElementos = Cuantos ; // La matriz ahora tiene Cuantos elementos
	 }

	if( N+1 > Ultimo ) // Si se ha accedido a un elemento mayor que el ltimo hasta ahora

		Ultimo = N+1 ; // este es el ltimo elemento al que se ha accedido

	return Datos[ N ] ; // Devolver la referencia al dato
 }

	//
	// Mtodo : Procesa
	//
	// Llama a la funcin pasada como parmetro para todos
	// los elementos del matriz que se han utilizado
	// hasta ahora.
	//

template <class Tipo>
int Matriz<Tipo>::Procesa( int (* F ) ( Tipo & ) )
 {
	int Retorno ;

		// Desde el primer elemento hasta el ltimo utilizado

	for( int Cont = 0 ; Cont < Ultimo ; Cont++ )
	 {
		Retorno = F( Datos[ Cont ] ) ; // Llamar a la funcin pasando el elemento

		if( Retorno ) // Si el valor de retorno no es 0

			return Retorno ; // terminar el proceso devolviendo ese mismo valor
	 }

	return 0 ; // Devolver 0, el proceso se termin totalmente
 }

	//
	// Mtodo : operador <<
	//
	// Permite escribir una Matriz en un stream
	//

template <class Tipo>
ostream & operator << ( ostream & Stream, Matriz<Tipo> M )
 {
		// Escribir todos los elementos usados hasta ahora

	for( int Cont = 0 ; Cont < M.Ultimo ; Cont++ )

		Stream << M[ Cont ] << endl ; // El Tipo de dato debe tener definido el operador de insercin

	return Stream ; // Devolver una referencia al stream
 }

	//
	// Mtodo : operador >>
	//
	// Permite leer una Matriz desde un stream
	//

template <class Tipo>
istream & operator >> ( istream & Stream, Matriz<Tipo> & M )
 {
	do
	 {
		Tipo Dato ;

		Stream >> Dato ; // Se lee un dato. Tipo debe tener definido el operador de extraccin

		if( ! Stream ) // Si se produce un error en el stream

			break ; // detener la extraccin

		M[ M.Ultimo ] = Dato ; // Guardar el dato leido en la ltima posicin

	 } while( 1 ) ;

	return Stream ; // Devolver la referencia al stream
 }

#endif // _ESTDAT_HPP_
