	//
	// GDiscos.Cpp
	//
	// Implementacin de los mtodos correspondientes a
	// las clases definidas en GDiscos.Hpp
	//
	// Septiembre 1992
	//

#include <fstream.h>
#include <string.h>
#include <io.h>
#include <dos.h>
#include <ctype.h>

#include "GDiscos.Hpp"

	//
	// Mtodos de la clase GArchivo
	//

	//
	// Mtodo : Constructor
	//
	// Recibe un nombre de archivo, que
	// utiliza para leer la informacin
	// correspondiente del disco.
	//

GArchivo::GArchivo( char * N )
 {
	istrcpy( Nombre, N ) ; // Copia el nombre del archivo

	LeeInformacion() ; // Y lee la informacin correspondiente a l
 }

	//
	// Mtodo : Operador de asignacin
	//
	// Asigna un objeto de la clase a otro
	//

void GArchivo::operator = (  GArchivo & O )
 {
		// Copia los distintos miembros

	istrcpy( Nombre, O . Nombre ) ;

	AtributoActual = O . AtributoActual ;
	Hora = O . Hora ;
	Fecha = O . Fecha ;
	Bytes = O . Bytes ;
 }

	//
	// Mtodo : Operador relacional de igualdad
	//
	// Permite saber si dos objetos de la clase son iguales
	//

int GArchivo::operator == (  GArchivo & O )
 {
		// Los objetos son iguales

	return ! istrcmp( Nombre, O . Nombre )  // si los nombres son iguales
		&& AtributoActual == O . AtributoActual // y coinciden el atributo
		&& Hora == O . Hora && Fecha == O . Fecha // la fecha, la hora
		&& Bytes == O . Bytes ;  // y los bytes
 }

	//
	// Mtodo : Operador de insercin en stream
	//
	// Permite escribir un objeto en un stream
	//

ostream & operator << ( ostream & Stream, GArchivo & O )
 {
	Stream . write( O . Nombre, 13 ) ; // Se escribe el nombre

			// Y el resto de los datos
	Stream . write( (char * ) & O . AtributoActual, sizeof( O . AtributoActual ) ) ;
	Stream . write( (char * ) & O . Hora, sizeof( O . Hora ) ) ;
	Stream . write( (char * ) & O . Fecha, sizeof( O . Fecha ) ) ;
	Stream . write( (char * ) & O . Bytes, sizeof( O . Bytes ) ) ;

	return Stream ;
 }

	//
	// Mtodo : Operador de extraccin de stream
	//
	// Permite leer un objeto de un stream
	//

istream & operator >> ( istream & Stream, GArchivo & O )
 {
	Stream . read( O . Nombre, 13 ) ; // Se lee el nombre

			// Y el resto de los datos
	Stream . read( (char * ) & O . AtributoActual, sizeof( O . AtributoActual ) ) ;
	Stream . read( (char * ) & O . Hora, sizeof( O . Hora ) ) ;
	Stream . read( (char * ) & O . Fecha, sizeof( O . Fecha ) ) ;
	Stream . read( (char * ) & O . Bytes, sizeof( O . Bytes ) ) ;

	return Stream ;
 }

	//
	// Mtodo : Borra
	//
	// Elimina el archivo del disco, e inutiliza el objeto
	//

void GArchivo::Borra()
 {
	if( ! remove( Nombre ) ) // Si no hay problema al borrar el archivo

		istrcpy( Nombre, "NULO" ) ; // ya no existe
 }

	//
	// Mtodo : Renombra
	//
	// Permite cambiar el nombre a un archivo
	//

void GArchivo::Renombra(  char * NuevoNombre )
 {
	char Unidad[ MAXDRIVE ], Directorio[ MAXDIR ],
		 Archivo[ MAXFILE + MAXEXT ], Extension[ MAXEXT ] ;

			// Obtener los distintos componentes que se pueden
			// haber facilitado en la cadena

	int X = fnsplit( NuevoNombre, Unidad, Directorio, Archivo, Extension ) ;

	if( ! X & FILENAME ) // Si no se ha especificado nombre de archivo

		return ; // volver sin hacer nada

	if( X & EXTENSION ) // Si se ha especificado extensin

		istrcat( Archivo, Extension ) ; // aadirla al nombre

	if( ! rename( Nombre, Archivo ) ) // Si se renombra sin problemas

		istrcpy( Nombre, Archivo ) ; // este es el nuevo nombre del archivo
 }

	//
	// Mtodo : Copia
	//
	// Copia el archivo con otro nombre o en otro lugar
	//

void GArchivo::Copia(  char * Destino )
 {
		// El archivo de entrada es el objeto implcito

	ifstream Entrada( Nombre, ios::binary ) ;

		// Se abre el archivo de salida

	ofstream Salida( Destino, ios::binary ) ;

	if( ! Entrada || ! Salida ) // Si alguna de las aperturas ha fallado

		return ; // no se puede continuar

	char * Buffer = new char[ 1024 ] ; // Asignar 1 Kbyte para transferencia

	int iSalida = 0 ; // Indicador de salida

	while( ! iSalida ) // Mientras no se active el indicador de salida
	 {
		Entrada . read( Buffer, 1024 ) ; // Se lee un kbyte del archivo

		if( ! Entrada ) // Si se llego al final del archivo

			iSalida = 1 ; // activar el indicador de salida

			// En cualquier caso almacenar en el destino los bytes leidos

		Salida . write( Buffer, Entrada . gcount() ) ;
	 }

	delete [] Buffer ; // Se libera la memoria asignada
 }

	//
	// Mtodo : Mueve
	//
	// Mueve el archivo a otro directorio
	//

void GArchivo::Mueve(  char * Camino )
 {
	char Unidad[ MAXDRIVE ], Directorio[ MAXDIR ],
		 Archivo[ MAXFILE + MAXEXT ], Extension[ MAXEXT ] ;

			// Obtener los distintos componentes que se pueden
			// haber facilitado en la cadena

	int X = fnsplit( Camino, Unidad, Directorio, Archivo, Extension ) ;

	if( ! X & DIRECTORY ) // Si no se ha especificado nuevo directorio

		return ; // volver sin hacer nada

	if( ! rename( Nombre, Directorio ) ) // Si se mueve sin problemas

		istrcpy( Nombre, "NULO" ) ; // el archivo ya no est en el directorio actual
 }

	//
	// Mtodo : EstaTexto
	//
	// Busca un texto en el archivo
	//

int GArchivo::EstaTexto(  char * Texto )
 {
	ifstream Entrada( Nombre, ios::binary ) ; // Abrir el archivo para lectura

	char Buffer[ 1024 ] ; // Asumir una lnea como mximo de 1K

	int Salida = 0 ; // Indicador de salida

	while( ! Salida ) // Mientras no se active el indicador de salida
	 {
		Entrada . getline( Buffer, 1024 ) ; // Leer una lnea

		if( strstr( Buffer, Texto ) ) // Si el texto est en esta lnea

			return 1 ; // informar de ello

		if( ! Entrada ) // Si hemos llegado al fin de archivo

			Salida = 1 ; // activar el indicador de salida
	 }

	return 0 ;  // El texto no se encontr
 }

	//
	// Mtodo : Atributo
	//
	// Permite modificar el estado de un atributo
	//

void GArchivo::Atributo( iAtributo Cual, int Como )
 {
		// Establecer o borrar el atributo, segn lo indicado en Cual

	_chmod( Nombre, 1, Como ? AtributoActual | Cual :
							  AtributoActual & Cual ) ;

		// Recuperar los atributos actuales

	AtributoActual = (iAtributo ) _chmod( Nombre, 0, 0 ) ;
 }

	//
	// Mtodo : LeeInformacion
	//
	// Lee del disco la informacin concerniente a un archivo
	//

void GArchivo::LeeInformacion()
 {
	struct ffblk I ; // Estructura para recoger la informacin

	if( findfirst( Nombre, & I, ( GArchivo::SoloLectura |
								  GArchivo::Oculto |
								  GArchivo::Sistema |
								  GArchivo::Directorio |
								  GArchivo::Archivo ) ) ) // Buscar el archivo, si no se encuentra

		istrcpy( Nombre, "NULO" ) ; // el archivo no es vlido

	else // en caso contrario

		*this = GArchivo( I ) ; // Se asigna la informacin al objeto
 }


	//
	// Mtodos de la clase GDirectorio
	//

	//
	// Mtodo : Constructor
	//
	// Recibe el camino de un directorio y construye con l
	// el objeto
	//

GDirectorio::GDirectorio( char * C )
 {
	char Actual[ MAXPATH ] ;

	getcwd( Actual, MAXPATH ) ; // Se preserva el directorio actual

	if( ! chdir( C ) ) // Si se cambia el directorio sin problemas
	 {
		Selecciona( "*.*" ) ; // Se seleccionan todos los archivos
		istrcpy( Camino, C ) ;
		chdir( Actual ) ; // Se vuelve al directorio establecido
	 }
	else
		istrcpy( Camino, "NULO" ) ; // El directorio no es vlido
 }

	//
	// Mtodo : Constructor de copia
	//
	// Crea un objeto tomando como origen la informacin de otro
	//

GDirectorio::GDirectorio(  GDirectorio & O )

		: Archivos( O . Archivos )  // Copia la lista de archivos
 {
	istrcpy( Camino, O . Camino ) ; // Copia el camino y la referencia
	istrcpy( Referencia, O . Referencia ) ;
 }

	//
	// Mtodo : Operador de asignacin
	//
	// Permite asignar un objeto de la clase a otro
	//

void GDirectorio::operator = (  GDirectorio & O )
 {
	Archivos = O . Archivos ; // Asigna la lista de archivos
	istrcpy( Camino, O . Camino ) ; // El camino y la referencia
	istrcpy( Referencia, O . Referencia ) ;
 }

	//
	// Mtodo : Operador relacional de igualdad
	//
	// Permite comparar dos objetos de la clase para saber si son iguales
	//

int GDirectorio::operator == (  GDirectorio & O )
 {
	return ! istrcmp( Camino, O . Camino ) &&  // Si los caminos son iguales
			 istrcmp( Referencia, O . Referencia ) && // y la referencia
			 Archivos == O . Archivos ; // y la lista de archivos

			 // los objetos son iguales
 }

	//
	// Mtodo : Operador []
	//
	// Permite obtener una referencia a un archivo
	//

GArchivo & GDirectorio::operator[] ( int N )
 {
	GArchivo NULO ;

	return N > NumeroArchivos() ? NULO : Archivos[ N ] ;
 }

	//
	// Mtodo : Operador de insercin en stream
	//
	// Facilita la escritura de objetos en un stream
	//

ostream & operator << ( ostream & Stream, GDirectorio & O )
 {
	Stream . write( O . Camino, MAXPATH ) ; // Se escribe el camino
	Stream . write( O . Referencia, 13 ) ; // la referencia y

	return Stream << O . Archivos ; // La lista de archivos
 }

	//
	// Mtodo : Operador de extraccin de stream
	//
	// Facilita la lectura de objetos de un stream
	//

istream & operator >> ( istream & Stream, GDirectorio & O )
 {
	Stream . read( O . Camino, MAXPATH ) ; // Se lee el camino
	Stream . read( O . Referencia, 13 ) ; // la referencia

	return Stream >> O . Archivos ; // y la lista de archivos
 }

	//
	// Mtodo : Borra
	//
	// Borra el directorio
	//

void GDirectorio::Borra()
 {
	if( ! rmdir( Camino ) )	// Si se borr sin problemas

		istrcpy( Camino, "NULO" ) ;
 }

	//
	// Mtodo : Selecciona
	//
	// Selecciona archivos del directorio actual a partir
	// de un referencia
	//

void GDirectorio::Selecciona( char * Ref )
 {
	struct ffblk InfArchivo ;

	Archivos = Matriz<GArchivo>() ; // Eliminar informacin anterior

	if( findfirst( Ref, & InfArchivo, GArchivo::SoloLectura |
									  GArchivo::Oculto |
									  GArchivo::Sistema |
									  GArchivo::Directorio |
									  GArchivo::Archivo	 ) ) // Si no se encuentra ninguno

		return ;

	do {
			// Preservar informacin del objeto creado

		Archivos[ Archivos . Elementos() ] = GArchivo( InfArchivo ) ;

    } while( ! findnext( & InfArchivo ) ) ; // Mientras existan archivos

 }

	//
	// Mtodo : BorraArchivos
	//
	// Borra los archivos actualmente seleccionados
	//

void GDirectorio::BorraArchivos()
 {
		// Recorrer todos los archivos

	for( int I = 0 ; I < Archivos . Elementos() ; I++ )

				// Si el archivo no es un directorio ni una etiqueta

		if( ! Archivos[ I ] . Atributo( GArchivo::Directorio ) &&
			! Archivos[ I ] . Atributo( GArchivo::Etiqueta ) )

				Archivos[ I ] . Borra() ; // se borra el archivo
 }

	//
	// Mtodo : CopiaArchivos
	//
	// Copia los archivos actualmente seleccionados
	//

void GDirectorio::CopiaArchivos( char * Destino )
 {
		// Recorrer todos los archivos

	for( int I = 0 ; I < Archivos . Elementos() ; I++ )

				// Si el archivo no es un directorio ni una etiqueta

		if( ! Archivos[ I ] . Atributo( GArchivo::Directorio ) &&
			! Archivos[ I ] . Atributo( GArchivo::Etiqueta ) )

				Archivos[ I ] . Copia( Destino ) ; // se borra el archivo
 }

	//
	// Mtodo : MueveArchivos
	//
	// Mueve los archivos actualmente seleccionados
	//

void GDirectorio::MueveArchivos( char * Camino )
 {
		// Recorrer todos los archivos

	for( int I = 0 ; I < Archivos . Elementos() ; I++ )

				// Si el archivo no es un directorio ni una etiqueta

		if( ! Archivos[ I ] . Atributo( GArchivo::Directorio ) &&
			! Archivos[ I ] . Atributo( GArchivo::Etiqueta ) )

				Archivos[ I ] . Mueve( Camino ) ; // se borra el archivo
 }

	//
	// Mtodos de la clase GUnidad
	//

	//
	// Mtodo : Constructor
	//
	// Recibe una letra de unidad y ruye el objeto
	// con la informacin correspondiente a ella
	//

GUnidad::GUnidad( char U )
 {
	char Actual = getdisk() ; // Se preserva la unidad actual

	U = toupper( U ) - 'A' ; // Se convierte en unidad

	setdisk( U ) ; // Intentar cambiar a la unidad indicada

	if( getdisk() != U ) // Si no se puede
	 {
		UnidadActual = 0 ; // La unidad no es vlida
		return ;
	 }

	UnidadActual = U ; // La unidad es vlida

	char Temporal[ MAXPATH ] ;

	getcwd( Temporal, MAXPATH ) ; // Obtener el directorio actual

	DirectorioActual = GDirectorio( Temporal ) ; // Se crea el objeto

	struct dfree Informacion ;

	getdfree( 0, & Informacion ) ; // Se obtiene informacin de ocupacin

		// Se obtiene el nmero de bytes totales de la unidad

	Capacidad = (long ) Informacion . df_sclus *
				Informacion . df_bsec *
				Informacion . df_total ;

		// Se obtiene el nmero de bytes libres

	Libre = (long ) Informacion . df_sclus *
			Informacion . df_bsec *
			Informacion . df_avail ;

	setdisk( Actual ) ; // Se restituye la unidad
 }

	//
	// Mtodo : Operador de asignacin
	//
	// Permite la asignacin entre objetos pertenecientes a la clase
	//

void GUnidad::operator = (  GUnidad & O )
 {
		// Se asignan los distintos miembros

	UnidadActual = O . UnidadActual  ;
	Capacidad = O . Capacidad ;
	Libre = O . Libre ;
	DirectorioActual = O . DirectorioActual ;
 }

	//
	// Mtodo : Operador relacional de igualdad
	//
	// Facilita la comparacin entre dos objetos de la clase
	//

int GUnidad::operator == (  GUnidad & O )
 {
		// Los objetos son iguales si son iguales todos sus miembros

	return UnidadActual == O . UnidadActual &&
		   Capacidad == O . Capacidad &&
		   Libre == O . Libre &&
		   DirectorioActual == O . DirectorioActual ;
 }

	//
	// Mtodo : Operador de insercin en stream
	//
	// Permite escribir un objeto en un stream
	//

ostream & operator << ( ostream & Stream, GUnidad & O )
 {
		// Escribe los miembros del objeto

	Stream . write( (char *) & O . UnidadActual, sizeof( char ) ) ;
	Stream . write( (char *) & O . Capacidad, sizeof( long ) ) ;
	Stream . write( (char *) & O . Libre, sizeof( long ) ) ;

		// Incluida la informacin del directorio actual

	return Stream << O . DirectorioActual ;
 }

	//
	// Mtodo : Operador de extraccin de stream
	//
	// Permite leer un objeto de un stream
	//

istream & operator >> ( istream & Stream, GUnidad & O )
 {
		// Lee los miembros del objeto

	Stream . read( (char *) & O . UnidadActual, sizeof( char ) ) ;
	Stream . read( (char *) & O . Capacidad, sizeof( long ) ) ;
	Stream . read( (char *) & O . Libre, sizeof( long ) ) ;

		// Incluida la informacin del directorio actual

	return Stream >> O . DirectorioActual ;
 }

	//
	// Mtodos de la clase GDisco
	//

	//
	// Mtodo : Constructor por defecto
	//
	// Obtiene la informacin acerca de las unidades del equipo
	// y la que est fijada actualmente
	//

GDisco::GDisco() : NumeroUnidades( setdisk( getdisk() ) ),
				   UnidadActual( (char )( getdisk() + 'A' ) )
 {
 }

	//
	// Mtodo : Operador de asignacin
	//
	// Permite asignar un objeto a otro
	//

void GDisco::operator = (  GDisco & O )
 {
	NumeroUnidades = O . NumeroUnidades ;

	UnidadActual = O . UnidadActual ;
 }

	//
	// Mtodo : Operador relacional de igualdad
	//
	// Facilita la comparacin entre dos objetos de la clase
	//

int GDisco::operator == ( GDisco & O )
 {
	return NumeroUnidades == O . NumeroUnidades &&
		   UnidadActual == O . UnidadActual ;
 }

	//
	// Mtodo : Operador de insercin en stream
	//
	// Permite escribir un objeto de la clase en un stream
	//

ostream & operator << ( ostream & Stream, GDisco & O )
 {
	Stream . write( (char *) & O . NumeroUnidades, sizeof( int ) ) ;

	return Stream << O . UnidadActual ;
 }

	//
	// Mtodo : Operador de extraccin de stream
	//
	// Permite leer un objeto de la clase de un stream
	//

istream & operator >> ( istream & Stream, GDisco & O )
 {
	Stream . read( (char *) & O . NumeroUnidades, sizeof( int ) ) ;

	return Stream >> O . UnidadActual ;
 }

