	//
	// GDiscos.Hpp
	//
	// Clases para el control de archivos, directorios y unidades
	//
	// Septiembre 1992
	//

#ifndef _GDISCOS_HPP_
#define _GDISCOS_HPP

#ifndef __cplusplus
#define No se puede usar GDiscos en C
#endif

#include <dir.h>
#include <dos.h>
#include <iostream.h>
#include <stdio.h>

	// Si el modelo de memoria seleccionado permite varios segmentos de datos
	// habr que utilizar las versiones far de algunas funciones

#if defined(__LARGE__) || defined(__HUGE__) || defined(__COMPACT__)

	#define istrcpy( orig, dest )	           _fstrcpy( (orig), (dest) )
	#define istrcmp( orig, dest )	           _fstrcmp( (orig), (dest) )
	#define istrcat( orig, dest )	           _fstrcat( (orig), (dest) )

#else

	#define istrcpy( orig, dest )	   strcpy( (orig), (dest) )
	#define istrcmp( orig, dest )	   strcmp( (orig), (dest) )
	#define istrcat( orig, dest )	   strcat( (orig), (dest) )

#endif

#include "EstDat.Hpp"

	//
	// Clase : GArchivo
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Mantener la informacin de
	//			   un archivo
	//
	// Fecha de creacin : 1 Septiembre 1992
	//
	// Fecha de revisin : -
	//

class GArchivo
 {
	public :

				// Enumeracin con los posibles atributos de un archivo

			enum iAtributo { SoloLectura = FA_RDONLY,
							 Oculto = FA_HIDDEN,
							 Sistema = FA_SYSTEM,
							 Etiqueta = FA_LABEL,
							 Directorio = FA_DIREC,
							 Archivo = FA_ARCH } ;

		struct  GHora   // Hora de creacin o ltima modificacin
		 {
			unsigned    Segundos	: 5,
					      Minutos	: 6,
					      Horas		: 5 ;

			GHora() {}

			GHora(  unsigned T ) // Constructor
			 {
				Segundos = T & 0x001F ;
				Minutos  = ( T >> 5 ) & 0x003F ;
				Horas    = ( T >> 11 ) & 0x001F ;
			 }

			int operator == (  GHora & O ) // Relacin de igualdad
			 {
				return Segundos == O . Segundos &&
					   Minutos  == O . Minutos &&
					   Horas    == O . Horas ;
			 }

		 } ;

		struct GFecha	// Fecha de creacin o ltima modificacin
		 {
			unsigned    Dia	: 5,
					      Mes	: 4,
					      Ano	: 7 ;

			GFecha() {}

			GFecha(  unsigned F ) // Constructor
			 {
				Dia = F & 0x001F ;
				Mes = ( F >> 5 ) & 0x000F ;
				Ano = ( F >> 9 ) & 0x007F ;
			 }

			int operator == (  GFecha & O ) // Relacin de igualdad
			 {
				return Dia == O . Dia &&
					   Mes == O . Mes &&
					   Ano == O . Ano ;
			 }

		 } ;

				// Constructores

			GArchivo( char * ) ; // Asocia el objeto a un archivo indicado

			GArchivo( struct ffblk & ) ; // Crea el objeto con una informacin

			GArchivo() ; // Constructor por defecto

			GArchivo( int ) ; // Constructor de objeto nulo

			GArchivo(  GArchivo & ) ; // Constructor de copia

			// Operador de asignacin

		void operator = (  GArchivo & ) ;

			// Operador de relacin de igualdad

		int operator == (  GArchivo & ) ;

			// Operador de insercin en un stream

		friend ostream & operator << ( ostream &, GArchivo & ) ;

			// Operador de extraccin de un stream

		friend istream & operator >> ( istream &, GArchivo & ) ;

		void Borra() ; // Borra el archivo

		void Renombra(  char * ) ; // Le asigna un nuevo nombre

		void Copia(  char * ) ; // Lo copia

		void Mueve(  char * ) ; // Mueve el archivo a otro lugar

		int EstaTexto(  char * ) ; // Busca un texto en el archivo

		int Atributo( iAtributo ) ; // Devuelve el atributo indicado

		void Atributo( iAtributo, int ) ; // Fija el estado de un atributo

		operator char * () ; // Devuelve el nombre del archivo

		long Tamano() ; // Devuelve el nmero de bytes del archivo

		GHora HoraUM() ; // Devuelve la hora de ltima modificacin

		GFecha FechaUM() ; // Devuelve la fecha de ltima modificacin

	private :

		char Nombre[ 13 ] ; // Nombre del archivo

		iAtributo AtributoActual ; // Atributos

		GHora Hora ; // Hora de ltima modificacin

		GFecha Fecha ; // Fecha de ltima modificacin

		long Bytes ; // Tamao actual del archivo

		void LeeInformacion() ; // Lee la informacin del archivo
 } ;

	//
	// Mtodos inline de la clase
	//

	//
	// Mtodo : Constructor
	//
	// Recibe una estructura ffblk, conteniendo
	// toda la informacin necesaria sobre el
	// objeto a crear.
	//

inline GArchivo::GArchivo( struct ffblk & I )

			 // Copia los datos

		: AtributoActual( iAtributo( I . ff_attrib ) ),
		  Hora( I . ff_ftime ), Fecha( I . ff_fdate ),
		  Bytes( I . ff_fsize )

	{ istrcpy( Nombre, I . ff_name ) ; } // y el nombre

	//
	// Mtodo : Constructor
	//
	// Constructor por defecto, crea un objeto vaco
	//

inline GArchivo::GArchivo()

	{ istrcpy( Nombre, "NULO" ) ; } // El archivo es nulo

	//
	// Mtodo : Constructor
	//
	// Constructor para crear un objeto nulo
	//

inline GArchivo::GArchivo( int )

	{ istrcpy( Nombre, "NULO" ) ; }

	//
	// Mtodo : Constructor de copia
	//
	// Crea un objeto tomando la informacin de otro como origen
	//

inline GArchivo::GArchivo( GArchivo & O )

			// Copia los datos

		: AtributoActual( O . AtributoActual ), Hora( O . Hora ),
		  Fecha( O . Fecha ), Bytes( O . Bytes )

	{ istrcpy( Nombre, O . Nombre ) ; } // y el nombre

	//
	// Mtodo : Atributo
	//
	// Devuelve el estado del atributo que se indique
	//

inline int GArchivo::Atributo( iAtributo Cual )

	{ return AtributoActual & Cual ; } // Ver si est activado

	//
	// Mtodo : Operador ( char * )
	//
	// Devuelve el nombre del archivo
	//

inline GArchivo::operator char * ()

	{ return Nombre ; }

	//
	// Mtodo : Tamano
	//
	// Devuelve la ocupacin del archivo en disco
	//

inline long GArchivo::Tamano()

	{ return Bytes ; }

	//
	// Mtodo : HoraUM
	//
	// Devuelve la hora en que se cre o modific el archivo
	// por ltima vez
	//

inline GArchivo::GHora GArchivo::HoraUM()

	{ return Hora ; }

	//
	// Mtodo : FechaUM
	//
	// Devuelve la fecha en que se cre o modific el archivo
	// por ltima vez
	//

inline GArchivo::GFecha GArchivo::FechaUM()

	{ return Fecha ; }

	//
	// Clase : GDirectorio
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Mantener la informacin de
	//			   un directorio completo
	//
	// Fecha de creacin : 1 Septiembre 1992
	//
	// Fecha de revisin : -
	//

class GDirectorio
 {
	public :

				// Constructores

			GDirectorio( char * ) ; // Recibe un camino

			GDirectorio() ; // Constructor por defecto

			GDirectorio( int ) ; // Para crear objetos nulos

			GDirectorio(  GDirectorio & ) ; // Constructor de copia

			// Operador de asignacin

		void operator = (  GDirectorio & ) ;

			// Operador relacional de igualdad

		int operator == (  GDirectorio & ) ;

			// Operador de insercin en un stream

		friend ostream & operator << ( ostream &, GDirectorio & ) ;

			// Operador de extraccin de un stream

		friend istream & operator >> ( istream &, GDirectorio & ) ;

		operator char * () ; // Devuelve el nombre del directorio

		void Borra() ; // Borra el directorio

		void Selecciona( char * ) ; // Selecciona archivos a partir de una referencia

		void BorraArchivos() ; // Borra los archivos seleccionados

		void CopiaArchivos( char * ) ; // Copia los archivos seleccionados

		void MueveArchivos( char * ) ; // Mueve los archivos seleccionados

		GArchivo & operator[] ( int ) ; // Devuelve referencia a un archivo

		int NumeroArchivos() ; // Devuelve el nmero de archivos

	private :

		char Camino[ MAXPATH ] ; // Camino del directorio

		Matriz<GArchivo> Archivos ; // Archivos que hay en el directorio

		char Referencia[ 13 ] ; // Referencia de seleccin actual
 } ;

	//
	// Mtodos inline de la clase
	//

	//
	// Mtodo : Constructor por defecto
	//
	// Crea un objeto vaco
	//

inline GDirectorio::GDirectorio()

	{ istrcpy( Camino, "NULO" ) ; } // Camino nulo

	//
	// Mtodo : Constructor
	//
	// Crea un objeto nulo
	//

inline GDirectorio::GDirectorio( int )

	{ istrcpy( Camino, "NULO" ) ; } // Camino nulo

	//
	// Mtodo : Operador (char *)
	//
	// Devuelve el nombre del directorio
	//

inline GDirectorio::operator char * ()

	{ return Camino ; }

	//
	// Mtodo : NumeroArchivos
	//
	// Devuelve el nmero de archivos contenidos en el directorio
	//

inline int GDirectorio::NumeroArchivos()

	{ return Archivos . Elementos() ; }

	//
	// Clase : GUnidad
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Mantener la informacin de
	//			   una unidad de disco
	//
	// Fecha de creacin : 1 Septiembre 1992
	//
	// Fecha de revisin : -
	//

class GUnidad
 {
	public :

				// Constructores

			GUnidad( char ) ; // Recibe una unidad como parmetro

			GUnidad() ; // Constructor por defecto

			GUnidad( int ) ; // Creacin de objeto nulo

			GUnidad(  GUnidad & ) ; // Constructor de copia

			// Operador de asignacin

		void operator = (  GUnidad & ) ;

			// Operador de relacin de igualdad

		int operator == (  GUnidad & ) ;

			// Operador de insercin en un stream

		friend ostream & operator << ( ostream &, GUnidad & ) ;

			// Operador de extraccin de un stream

		friend istream & operator >> ( istream &, GUnidad & ) ;

		operator char () ; // Obtener la letra de la unidad

		GDirectorio & Directorio() ; // Devuelve referencia del directorio actual

		void Directorio( char * ) ; // Fija un nuevo directorio

		long bytesCapacidad() ; // Devuelve la capacidad de la unidad

		long bytesLibres() ; // Devuelve el espacio libre

	private :

		char UnidadActual ; // Letra con la unidad

		long Capacidad, Libre ; // Bytes de capacidad y libres

		GDirectorio DirectorioActual ; // Informacin del directorio actual

 } ;

	//
	// Mtodos inline de la clase
	//

	//
	// Mtodo : Constructor por defecto
	//
	// Crea un objeto vaco
	//

inline GUnidad::GUnidad() : UnidadActual( 0 )  // El objeto est vaco

	{ }

	//
	// Mtodo : Constructor
	//
	// Crea un objeto nulo
	//

inline GUnidad::GUnidad( int ) : UnidadActual( 0 ) // El objeto es nulo

	{ }

	//
	// Mtodo : Constructor de copia
	//
	// Crea un objeto con la misma informacin de uno dado
	//

inline GUnidad::GUnidad(  GUnidad & O )

			// Se copian los miembros

		: UnidadActual( O . UnidadActual ), Capacidad( O . Capacidad ),
		  Libre( O . Libre ), DirectorioActual( O . DirectorioActual )

	{ }

	//
	// Mtodo : Operador (char )
	//
	// Devuelve la letra de la unidad
	//

inline GUnidad::operator char ()

	{ return UnidadActual + 'A' ; }

	//
	// Mtodo : Directorio
	//
	// Devuelve el nombre del directorio actual
	//

inline GDirectorio & GUnidad::Directorio()

	{ return DirectorioActual ; }

	//
	// Mtodo : Directorio
	//
	// Establece el directorio por defecto
	//

inline void GUnidad::Directorio( char * Camino )

	{ DirectorioActual = GDirectorio( Camino ) ; }

	//
	// Mtodo : bytesCapacidad()
	//
	// Devuelve la capacidad de la unidad
	//

inline long GUnidad::bytesCapacidad()

	{ return Capacidad ; }

	//
	// Mtodo : bytesLibres()
	//
	// Devuelve el espacio libre en la unidad
	//

inline long GUnidad::bytesLibres()

	{ return Libre ; }

	//
	// Clase : GDisco
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Mantener informacin general
	//			   sobre las unidades del sistema
	//
	// Fecha de creacin : 1 Septiembre 1992
	//
	// Fecha de revisin : -
	//

class GDisco
 {
	public :

			GDisco() ; // Constructor por defecto

			GDisco( int ) ; // Creacin de objeto nulo

			GDisco(  GDisco & ) ; // Constructor de copia

			// Operador de asignacin

		void operator = (  GDisco & ) ;

			// Operador de relacin de igualdad

		int operator == (  GDisco & ) ;

			// Operador de insercin en un stream

		friend ostream & operator << ( ostream &, GDisco & ) ;

			// Operador de extraccin de un stream

		friend istream & operator >> ( istream &, GDisco & ) ;

		int Unidades() ; // Devuelve el nmero de unidades existentes

		GUnidad & Actual() ; // Devuelve la unidad actual

		void Actual( char ) ; // Fija la unidad actual

	private :

		int NumeroUnidades ; // Nmero de unidades que tiene el equipo

		GUnidad UnidadActual ; // Unidad activa actualmente
 } ;

	//
	// Mtodos inline de la clase
	//

	//
	// Mtodo : Constructor
	//
	// Crea un objeto nulo
	//

inline GDisco::GDisco( int ) : NumeroUnidades( 0 )

	{ }

	//
	// Mtodo : Constructor de copia
	//
	// Crea un objeto tomando como origen otro de la misma clase
	//

inline GDisco::GDisco(  GDisco & O )

		: NumeroUnidades( O . NumeroUnidades ),
		  UnidadActual( O . UnidadActual )

	{ }

	//
	// Mtodo : Unidades
	//
	// Devuelve el nmero de unidades que hay en el equipo
	//

inline int GDisco::Unidades()

	{ return NumeroUnidades ; }

	//
	// Mtodo : Actual
	//
	// Devuelve la unidad actual
	//

inline GUnidad & GDisco::Actual()

	{ return UnidadActual ; }

	//
	// Mtodo : Actual
	//
	// Fija la unidad de disco actual
	//

inline void GDisco::Actual( char U )

	{ UnidadActual = GUnidad( U ) ; }

#endif // _GDISCOS_HPP_
