	//
	// GEntrada.Cpp
	//
	// Implementacin de los mtodos correspondientes a las clases
	// definidas en GEntrada.Hpp
	//
	// 31 Agosto 1992
	//

#include <ctype.h>  // Funciones de clasificacin de caracteres

#include <stdlib.h> // Funciones de conversin

	// Si el modelo de memoria seleccionado permite varios segmentos de datos
	// habr que utilizar las versiones far de algunas funciones

#if defined(__LARGE__) || defined(__HUGE__) || defined(__COMPACT__)

	#define istrcpy( orig, dest )              _fstrcpy( (orig), (dest) )
	#define istrcmp( orig, dest )              _fstrcmp( (orig), (dest) )
	#define istrcat( orig, dest )              _fstrcat( (orig), (dest) )
	#define istrchr( cade, cara )              _fstrchr( (cade), (cara) )
	#define istrlen( cade )                            _fstrlen( (cade) )
	#define imemcmp( orig, dest, long )      _fmemcmp( (orig), (dest), (long) )

#else

	#define istrcpy( orig, dest )              strcpy( (orig), (dest) )
	#define istrcmp( orig, dest )              strcmp( (orig), (dest) )
	#define istrcat( orig, dest )              strcat( (orig), (dest) )
	#define istrchr( cade, cara )              strchr( (cade), (cara) )
	#define istrlen( cade )                            strlen( (cade) )
	#define imemcmp( orig, dest, long )      memcmp( (orig), (dest), (long) )

#endif

#include "GEntrada.Hpp"

	//
	// Mtodos de la clase GCampo
	//

	//
	// Mtodo : Constructor
	//
	// Construye un objeto almacenando los parmetros
	// recibidos en los miembros que se han definido
	// al efecto.
	//

GCampo::GCampo( char * Nombre, int C, int L, TipoCampo * T )

				// Se construyen los miembros

				: Columna( C ), Linea( L ), Tipo( T )

 {
	// Se asigna memoria para preservar el nombre del campo

	ID = new char[ istrlen( Nombre ) + 1 ] ;

	// Se copia el nombre

	istrcpy( ID, Nombre ) ;
 }

	//
	// Mtodo : Constructor de copia
	//
	// Crea un objeto tomando como origen los miembros de otro
	//

GCampo::GCampo(  GCampo & O )

		// Se inicializan los miembros

		: Columna( O . Columna ), Linea( O . Linea ), Tipo( O . Tipo )

 {
	// Se asigna memoria para el identificador o nombre

	ID = new char[ istrlen( O . ID ) + 1 ] ;

	istrcpy( ID, O . ID ) ; // Copia el nombre
 }

	//
	// Mtodo : Destructor
	//
	// Libera la memoria que se hubiese asignado
	//

GCampo::~GCampo()
 {
	if( ID ) // Si existe un nombre

		delete [] ID ; // liberar la memoria que ocupa
 }

	//
	// Mtodo : Operador de asignacin
	//
	// Permite asignar un objeto a otro
	//

void GCampo::operator = (  GCampo & O )
 {
		// Se copian los distintos miembros

	Columna = O . Columna ;
	Linea = O . Linea ;

	Tipo = O . Tipo ;

	if( ID ) // Si ya existe un nombre

		delete [] ID ; // Se libera

	ID = new char[ istrlen( O . ID ) + 1 ] ; // Se asigna memoria

	istrcpy( ID, O . ID ) ; // y se copia el nuevo ttulo
 }

	//
	// Mtodo : Operador relacional de igualdad
	//
	// Permite establecer una relacin de igualdad entre dos objetos
	//

int GCampo::operator == (  GCampo & O )
 {
		// El campo es igual si coinciden el ttulo y el
		// resto de los miembros

	return ! istrcmp( ID, O . ID ) &&
			 Columna == O . Columna && Linea == O . Linea ;
 }

	//
	// Mtodo : Operador de insercin en stream
	//
	// Permite almacenar un objeto en un stream
	//

ostream & operator << ( ostream & Stream, GCampo & O )
 {
      // Almacena el buffer donde est el campo

   Stream . write( O . Imagen(), O . Tipo -> Cols() * O . Tipo -> Lins() ) ;

	return Stream ; // Se devuelve la referencia al stream
 }

	//
	// Mtodo : Operador de extraccin de un stream
	//
	// Permite recuperar un objeto en un stream
	//

istream & operator >> ( istream & Stream, GCampo & O )
 {
      // Recupera el contenido del campo

   Stream . read( O . Imagen(), O . Tipo -> Cols() * O . Tipo -> Lins() ) ;

	return Stream ; // Devolver la referencia al stream
 }

	//
	// Mtodo : Esta
	//
	// Devuelve 1 si la posicin indicada pertenece al campo
	//

int GCampo::Esta( int Col, int Lin )
 {
	return ( Col >= Columna && Col <= Columna + Tipo -> Cols() - 1 &&
			 Lin >= Linea && Lin <= Linea + Tipo -> Lins() - 1 ) ;
 }

	//
	// Mtodos de la clase TipoCampo
	//

	//
	// Mtodo : Constructor
	//
	// Construye el objeto asignando la memoria necesaria para contenerlo
	//

TipoCampo::TipoCampo( int Cs, int Ls )

	// Inicializar miembros

		: Columnas( Cs ), Lineas( Ls )
 {
	// Se asigna memoria

		Buffer = new char[ Columnas * Lineas + 1 ] ;
 }

	//
	// Mtodo : Constructor de copia
	//
	// Construye el objeto tomando como origen los datos de otro
	//

TipoCampo::TipoCampo(  TipoCampo & O )

	// Inicializar miembros

		: Columnas( O . Columnas ), Lineas( O . Lineas ),
		  ColCursor( O . ColCursor ), LinCursor( O . LinCursor )
 {
	// Se asigna la memoria necesaria

		Buffer = new char[ Columnas * Lineas + 1 ] ;

	// Copiar la informacin de un objeto a otro

		movedata( FP_SEG( O . Buffer ), FP_OFF( O . Buffer ),
				  FP_SEG( Buffer ), FP_OFF( Buffer ),
				  Columnas * Lineas ) ;
 }

	//
	// Mtodo : Procesa
	//
	// Este mtodo es el que por defecto procesar los eventos que
	// se produzcan en un campo, controlando movimiento, borrados,
	// insercin, etc. Mientras este mtodo no sea redefinido en
	// la clase derivada, ser el proceso utilizado por defecto.
	//

int TipoCampo::Procesa( Evento & Suceso )
 {
	switch( Suceso . Quien ) // Quin ha producido el evento?
	 {
		case Evento::Teclado : // El teclado

			switch( Suceso . Tecla ) // Veamos qu tecla ha sido pulsada
			 {
				case GTeclado::ARRIBA : // Lnea anterior

					if( LinCursor > 1 ) // Si no estamos en la primera lnea

						LinCursor-- ; // Subir a la anterior

					break ;

				case GTeclado::ABAJO : // Lnea siguiente

					if( LinCursor < Lineas ) // Si no estamos en la ltima

						LinCursor++ ; // Ir a la siguiente

					break ;

				case GTeclado::IZQUIERDA : // Carcter anterior

					if( ColCursor > 1 ) // Si no estamos en la primera columna

						ColCursor-- ; // Ir a la anterior

					break ;

				case GTeclado::DERECHA : // Carcter siguiente

					if( ColCursor < Columnas ) // Si no estamos en la ltima

						ColCursor++ ; // Ir a la siguiente

					break ;

				case GTeclado::INICIO : // Principio del campo

					ColCursor = LinCursor = 1 ; // Primera columna y lnea

					break ;

				case GTeclado::FIN : // Fin del campo

					ColCursor = Columnas ; // Ultima columna
					LinCursor = Lineas ; // y ltima lnea

					break ;

				case GTeclado::CTRL_IZQUIERDA : // Palabra anterior

						// Hasta encontrar un espacio

					while( *(Buffer+(LinCursor-1)*Columnas+ColCursor-1) != ' ' )
					 {
						if( ColCursor > 1 ) // Si no estamos al principio

							ColCursor-- ; // Retroceder una columna

						else if( LinCursor > 1 ) // Si no estamos en la primera lnea
						 {
							LinCursor-- ;          // Ir al fin de la lnea anterior
							ColCursor = Columnas ;
						 }
						else
							break ;
					 }

						// Hasta saltar esos espacios

					while( *(Buffer+(LinCursor-1)*Columnas+ColCursor-1) == ' ' )
					 {
						if( ColCursor > 1 ) // Si no estamos al principio

							ColCursor-- ; // Retroceder una columna

						else if( LinCursor > 1 ) // Si no estamos en la primera lnea
						 {
							LinCursor-- ;          // Ir al fin de la lnea anterior
							ColCursor = Columnas ;
						 }
						else
							break ;
					 }

					break ;

				case GTeclado::CTRL_DERECHA : // Palabra siguiente

						// Hasta encontrar un espacio

					while( *(Buffer+(LinCursor-1)*Columnas+ColCursor-1) != ' ' )
					 {
						if( ColCursor < Columnas ) // Si no estamos al final

							ColCursor++ ; // Avanzar una columna

						else if( LinCursor < Lineas ) // Si no estamos en la ltima lnea
						 {
							LinCursor++ ;  // Ir al inicio de la lnea siguiente
							ColCursor = 1 ;
						 }
						else
							break ;
					 }

						// Hasta saltar los espacios

					while( *(Buffer+(LinCursor-1)*Columnas+ColCursor-1) == ' ' )
					 {
						if( ColCursor < Columnas ) // Si no estamos al final

							ColCursor++ ; // Avanzar una columna

						else if( LinCursor < Lineas ) // Si no estamos en la ltima lnea
						 {
							LinCursor++ ;  // Ir al inicio de la lnea siguiente
							ColCursor = 1 ;
						 }
						else
							break ;
					 }

					break ;

				case GTeclado::BS : // Retroceder y borrar carcter

					if( ColCursor > 1 ) // Si no estamos al principio de lnea

						ColCursor-- ; // retroceder a la posicin anterior

					else if( LinCursor > 1 ) // Si hay ms lneas
					 {
						ColCursor = Columnas ; // Ir al final de la
						LinCursor-- ;   // lnea anterior
					 }
					else
						break ;

				case GTeclado::SUPR : // Borrar carcter

						// Mover todo el contenido del campo una
						// posicin hacia atrs

					char * Posicion =

						Buffer + (LinCursor-1) * Columnas + ColCursor - 1 ;

					movedata( FP_SEG( Posicion+1 ), FP_OFF( Posicion+1 ),
							  FP_SEG( Posicion ), FP_OFF( Posicion ),
							  size_t( Columnas*Lineas - ( Posicion-Buffer+1 ) ) ) ;

						// El ltimo carcter ahora estar en blanco

					*( Buffer + Lineas * Columnas - 1 ) = ' ' ;

					break ;

				case GTeclado::INSERT :

					break ; // Ignorar la pulsacin de esta tecla

				case GTeclado::CTRL_BS : // Borrar hasta fin palabra

						// Mientras no encontremos un espacio

					while( * ( Buffer + (LinCursor-1) * Columnas + ColCursor - 1 ) != ' ' )
					 {
								// Mover todo el contenido del campo una
								// posicin hacia atrs

							char * Posicion =

								Buffer + (LinCursor-1) * Columnas + ColCursor - 1 ;

							movedata( FP_SEG( Posicion+1 ), FP_OFF( Posicion+1 ),
									  FP_SEG( Posicion ), FP_OFF( Posicion ),
									  size_t( Columnas*Lineas - ( Posicion - Buffer + 1 ) ) ) ;

								// El ltimo carcter ahora estar en blanco

						*( Buffer + Lineas * Columnas - 1 ) = ' ' ;
					 }

					break ;

				case GTeclado::CTRL_FIN : // Borrar hasta fin de lnea

						// Desde la posicin actual hasta el final de la lnea

					for( int ColTemp = ColCursor ; ColTemp <= Columnas ; ColTemp++ )

							// Asignar un espacio a cada posicin

						*( Buffer + (LinCursor-1) * Columnas + ColTemp -  1 ) = ' ' ;

					break ;

				case GTeclado::INTRO : // Inicio siguiente lnea

					ColCursor = 1 ; // Ir al principio de la lnea

					if( LinCursor < Lineas ) // Y si no estamos en la ltima lnea

						LinCursor++ ; // avanzar a la siguiente

					break ;

				default : // Cualquier otro carcter

					char Caracter = Suceso . Tecla ;

					if( Valida( Caracter ) ) // Si el carcter es vlido

						Inserta( Caracter ) ; // insertarlo en el campo

					break ;

			 } // switch Tecla

			break ;

		case Evento::Raton : // El ratn, ha sido pulsado el botn izquierdo

			ColCursor = Suceso . Posicion . X ; // Fijar el cursor en esa posicin
			LinCursor = Suceso . Posicion . Y ;

			break ;

	 } // switch Quien

	return 0 ;
 }

	//
	// Mtodo : Inserta
	//
	// Inserta el carcter en la posicin actual del cursor dentro
	// del campo, desplazando a los caracteres existentes o sustituyendo
	// el carcter existente, segn el estado de la tecla Insert
	//

void TipoCampo::Inserta( char C )
 {
	GTeclado Teclado ;

	if( Teclado . InsertActivo() ) // Si Insert est activa
	 {
		char * Posicion = Buffer + (LinCursor-1) * Columnas + ColCursor - 1 ;

			// Abrir un espacio desplazando el contenido del campo

		char * Temporal = new char[ size_t( Columnas*Lineas-(Posicion-Buffer+1) ) ] ;

		movedata( FP_SEG( Posicion ), FP_OFF( Posicion ),
				  FP_SEG( Temporal ), FP_OFF( Temporal ),
				  size_t( Columnas*Lineas - ( Posicion - Buffer + 1 ) ) ) ;

		movedata( FP_SEG( Temporal ), FP_OFF( Temporal ),
				  FP_SEG( Posicion + 1 ), FP_OFF( Posicion + 1 ),
				  size_t( Columnas*Lineas - ( Posicion - Buffer + 1 ) ) ) ;

		delete [] Temporal ;
	 }

	// Tanto si se est insertando como no, hay que almacenar el carcter

	*( Buffer + (LinCursor-1) * Columnas + ColCursor - 1 ) = C ;

	if( ColCursor < Columnas ) // Si no estamos en la ltima posicin

		ColCursor++ ; // Avanzamos a la siguiente

	else // en caso contrario

		if( LinCursor < Lineas ) // si hay ms lneas
		 {
			LinCursor++ ; // Avanzar al principio de la siguiente lnea
			ColCursor = 1 ;
		 }
 }

	//
	// Mtodos de la clase Entero
	//

	//
	// Mtodo : Constructor
	//
	// Construye el objeto pasando los parmetros apropiados
	// al constructor base
	//

Entero::Entero( int Ancho, int Inicial )

	// Llama al constructor base pasando los parmetros apropiados

		: TipoCampo( Ancho, 1 )

 {
		ColCursor = LinCursor = 1 ; // Inicializa posicin del cursor

	// Almacena el valor inicial en el buffer

		itoa( Inicial, Buffer, 10 ) ;

		while( istrlen( Buffer ) < Ancho ) // Mientras no se haya completado el campo

			istrcat( Buffer, " " ) ;
 }

	//
	// Mtodos de la clase Decimal
	//

	//
	// Mtodo : Constructor
	//
	// Construye el objeto haciendo uso del constructor base
	//

Decimal::Decimal( int Ancho, float Inicial )

	// Llama al constructor base

		: TipoCampo( Ancho, 1 )
 {
		ColCursor = LinCursor = 1 ; // Inicializa posicin del cursor

	// Almacena el valor inicial en el buffer

		gcvt( Inicial, Ancho, Buffer ) ;

		while( istrlen( Buffer ) < Ancho ) // Mientras no se haya completado el campo

			istrcat( Buffer, " " ) ;
 }

	//
	// Mtodo : Valida
	//
	// Valida la entrada de caracteres en el campo
	//

int Decimal::Valida( char & C )
 {
		// La entrada es vlida si el carcter

	return isdigit( C ) || // es un dgito numrico

			 // o el signo - y estamos al principio del campo

		   C == '-' && ColCursor == 1 ||

			 // o el punto y no hay otro punto en el campo

		   C == '.' && istrchr( Buffer, '.' ) == NULL ;
 }

	//
	// Mtodos de la clase Cadena
	//

	//
	// Mtodo : Constructor
	//
	// Construye el objeto sirvindose del constructor de la clase base
	//

Cadena::Cadena( int Ancho, int Alto, char * Inicial )

	// Llama al constructor base

		: TipoCampo( Ancho, Alto )
 {
		ColCursor = LinCursor = 1 ; // Inicializa posicin del cursor

	// Copiar el valor inicial en el campo

		if( Inicial ) // Si no es nulo

			istrcpy( Buffer, Inicial ) ;

		else

			istrcpy( Buffer, "" ) ; // No hay valor inicial

		while( istrlen( Buffer ) < Ancho * Alto ) // Mientras no se haya completado el campo

			istrcat( Buffer, " " ) ;
 }

	//
	// Mtodos de la clase GFecha
	//

	//
	// Mtodo : Constructor
	//
	// Permite construir un objeto GFecha a partir de una cadena
	//

GFecha::GFecha( char * S )
 {
	char * Temp = S ;

	Dia = atoi( Temp ) ; // Obtiene el da

	Temp = istrchr( Temp, '/' ) + 1 ; // Temp apunta detrs de la primera barra

	Mes = atoi( Temp ) ; // Obtiene el mes

	Temp = istrchr( Temp, '/' ) + 1 ; // Temp apunta detrs de la segunda barra

	Ano = atoi( Temp ) ; // Obtiene el ao
 }

	//
	// Mtodos de la clase Fecha
	//

	//
	// Mtodo : Constructor
	//
	// Construye un objeto asignndole un valor inicial
	//

Fecha::Fecha( GFecha F )

	// Llama al constructor base para crear el objeto

		: TipoCampo( 8, 1 )  // 8 columnas por 1 lnea

 {
	ColCursor = LinCursor = 1 ; // Inicializa posicin del cursor

	char Dia[ 3 ], Mes[ 3 ], Ano[ 3 ] ; // Objetos para convertir

	itoa( F . Dia, Dia, 10 ) ; // Convertir el da
	itoa( F . Mes, Mes, 10 ) ; // Convertir el mes
	itoa( F . Ano, Ano, 10 ) ; // Convertir el ao

	istrcpy( Buffer, "" ) ; // El buffer est vaco

	if( istrlen( Dia ) == 1 ) // Si el da tiene un slo dgito

		istrcat( Buffer, "0" ) ; // Aadir un 0 de relleno

	istrcat( Buffer, Dia ) ; // Insertar el da
	istrcat( Buffer, "/" ) ; // y una barra de separacin

		// Repetir el proceso para el mes y el ao

	if( istrlen( Mes ) == 1 )

		istrcat( Buffer, "0" ) ;

	istrcat( Buffer, Mes ) ;
	istrcat( Buffer, "/" ) ;

   if( istrlen( Ano ) == 1 )

       istrcat( Buffer, "0" ) ;

	istrcat( Buffer, Ano ) ;
 }

	//
	// Mtodo : Procesa
	//
	// Procesa los eventos de una forma particular, permitiendo
	// pasar automticamente entre los distintos componentes de
	// una fecha
	//

int Fecha::Procesa( Evento & Suceso )
 {
	if( Suceso . Quien == Evento::Teclado ) // Si es una pulsacin

		switch( Suceso . Tecla ) // Veamos que tecla
		 {
			case GTeclado::INTRO : // Siguiente apartado

					// Situar el cursor en el siguiente componente de la fecha

				ColCursor = ColCursor < 3 ? 4 : 7 ;

				break ;

			case GTeclado::IZQUIERDA : // Carcter anterior

				if( ColCursor > 1 ) // Si no estamos al principio del campo

					ColCursor = ColCursor == 7 ? 5 :
								ColCursor == 4 ? 2 : ColCursor - 1 ;

				break ;

			case GTeclado::DERECHA : // Carcter siguiente

				if( ColCursor < 8 ) // Si no estamos al final del campo

					ColCursor = ColCursor == 2 ? 4 :
								ColCursor == 5 ? 7 : ColCursor + 1 ;

				break ;

	 case GTeclado::BS : // Retrocede y borra un carcter

				if( ColCursor > 1 ) // Si no estamos al principio del campo

			  // Retrocede un carcter

					ColCursor = ColCursor == 7 ? 5 :
								ColCursor == 4 ? 2 : ColCursor - 1 ;

	    else

		break  ; // En caso contrario no se puede borrar carcter anterior

	 case GTeclado::SUPR : // Borra el carcter sobre el que est el cursor

			     // Si estamos en el primer dgito de un apartado

	      if( ColCursor == 1 || ColCursor == 4  || ColCursor == 7 )
	       {
		  *( Buffer + ColCursor - 1 ) = *( Buffer + ColCursor ) ;
		  *( Buffer + ColCursor ) = ' ' ;
	       }
	      else // Es el segundo carcter del apartado

		   *( Buffer + ColCursor - 1 ) = ' ' ;

	      break ;

			default : // Si no es ninguna de las anteriores

					// Aplicar el proceso por defecto

				TipoCampo::Procesa( Suceso ) ;
		 }

	else // Se ha pulsado el botn izquierdo del ratn
	 {
		ColCursor = Suceso . Posicion . X ; // Tomar la posicin

			// Si el cursor est en una de las barras

		if( ColCursor == 3 || ColCursor == 6 )

			ColCursor++ ; // Avanzar al siguiente apartado
	 }

	return 0 ;
 }

	//
	// Mtodo : Inserta
	//
	// Inserta el carcter en la posicin actual del cursor dentro
	// del campo, desplazando a los caracteres existentes o sustituyendo
	// el carcter existente, segn el estado de la tecla Insert
	//

void Fecha::Inserta( char C )
 {
	GTeclado Teclado ;

	if( Teclado . InsertActivo() ) // Si estamos insertando

			// Si estamos en el primer dgito de cualquiera de los apartados

		if( ColCursor == 1 || ColCursor == 4 || ColCursor == 7 )

				// Desplazar ese dgito a la siguiente posicin

			*( Buffer + ColCursor ) = *( Buffer + ColCursor - 1 ) ;

	// En cualquier caso almacenar el carcter recibido

	*( Buffer + ColCursor - 1 ) = C ;

	if( ColCursor < 8 ) // Si no estamos en la ltima posicin

		ColCursor++ ; // Pasar a la siguiente

	if( ColCursor == 3 || ColCursor == 6 ) // Si estamos en una de las barras

		ColCursor++ ; // Pasar al siguiente apartado
 }

	//
	// Mtodos de la clase Boton
	//

	//
	// Mtodo : Constructor
	//
	// Crea el objeto sirvindose del constructor base
	//

Boton::Boton( char * S, unsigned C )

	// Llama al constructor base

		: TipoCampo( istrlen( S ) + 2, 3 ), Codigo( C )

 {
   ColCursor = LinCursor = 2 ; // Inicializa posicin del cursor

   // Crear en el buffer la imagen del botn

   char * Temp = Buffer ;

   *Temp++ = 218 ;

   for( int I = 0 ; I < istrlen( S ) ; I++ )

	*Temp++ = 196 ;

   *Temp++ = 183 ;
   *Temp++ = 179 ;

   for( I = 0 ; I < istrlen( S ) ; I++ )

	*Temp++ = S[ I ] ;

   *Temp++ = 186 ;
   *Temp++ = 212 ;

   for( I = 0 ; I < strlen( S ) ; I++ )

	*Temp++ = 205 ;

   *Temp = 188 ;
 }


	//
	// Mtodo : Procesa
	//
	// Procesa los eventos que se pueden producir estando activo
	// un campo de este tipo
	//

int Boton::Procesa( Evento & Suceso )
 {
		// Si se ha pulsado la tecla Intro o la barra o el ratn

	if( Suceso . Quien == Evento::Teclado &&
		( Suceso . Tecla == GTeclado::INTRO || Suceso . Tecla == ' ' ) ||
		Suceso . Quien == Evento::Raton )

			return Codigo ; // Devolver el cdigo asociado con el botn

	return 0 ; // En cualquier otro caso ignorar el evento
 }

	//
	// Mtodos de la clase GSMultiple
	//

	//
	// Mtodo : Constructor
	//
	// Construye un objeto a partir de su representacin
	//

GSMultiple::GSMultiple( char * S )
 {
		// Poner a 0 todos los indicadores

	for( int I = 0 ; I < 32 ; I++, Opcion[ I ] . Estado = 0 ) ;

	char * Temp = istrchr( S, '[' ) ; // Posicin del primer indicador

	for( I = 0 ; I < 32 && Temp ; I++ ) // 32 posibles indicadores
	 {
		Opcion[ I ] . Estado = *(Temp+1) == 'X' ; // Estado del indicador I

		Temp = istrchr( Temp+1, '[' ) ; // Posicin del siguiente indicador
	 }
 }

	//
	// Mtodos de la clase SMultiple
	//

	//
	// Mtodo : Constructor
	//
	// Permite construir un objeto sirvindose del constructor
	// de la clase base
	//

SMultiple::SMultiple( int Ancho, int Alto, GSMultiple Inicial )

	// Llama al constructor base

		: TipoCampo( Ancho * 3, Alto )

 {
	ColCursor = 2 ; LinCursor = 1 ; // Inicializa posicin del cursor

	// Formar en buffer la imagen del campo, segn los valores de Inicial

	char * Temp = Buffer ;

	for( int L = 0 ; L < Alto ; L++ ) // Bucle de lneas

		for( int C = 0 ; C < Ancho ; C++ )  // Bucle de columnas
		 {
			*Temp++ = '[' ; // Delimitador izquierdo

				// X o ' ', dependiendo del estado de esa posicin

			*Temp++ = ( Inicial . Opcion[ L * Ancho + C ] . Estado ?
							'X' :
							' ' ) ;

			*Temp++ = ']' ; // Delimitador derecho
		 }
 }

	//
	// Mtodo : Procesa
	//
	// Procesa los eventos que se produzcan mientras est activo
	// este campo
	//

int SMultiple::Procesa( Evento & Suceso )
 {
	if( Suceso . Quien == Evento::Teclado ) // Si es una tecla

		switch( Suceso . Tecla ) // Veamos cual
		 {
			case GTeclado::IZQUIERDA : // Opcin de la izquierda

				if( ColCursor != 2 ) // Si no estamos en la primera

					ColCursor -= 3 ; // retroceder a la anterior

				break ;

			case GTeclado::DERECHA : // Opcin de la derecha

				if( ColCursor < Columnas - 1 ) // Si no estamos en la ltima

					ColCursor += 3 ; // ir a la siguiente

				break ;

			case GTeclado::ARRIBA : // Opcin superior

				if( LinCursor > 1 ) // Si no estamos en la primera lnea

					LinCursor-- ; // podemos subir a la anterior

				break ;

			case GTeclado::ABAJO : // Opcin inferior

				if( LinCursor < Lineas ) // Si no estamos en la ltima lnea

					LinCursor++ ; // podemos bajar a la siguiente

				break ;

			case GTeclado::INTRO :  // Invertir el estado
			case ' ' :

				*( Buffer + (LinCursor-1) * Columnas + ColCursor - 1 ) =

				   *( Buffer + (LinCursor-1) * Columnas + ColCursor - 1 ) == ' ' ? 'X' : ' ' ;

				break ;

		 } // switch Tecla

	else // Se ha pulsado el botn izquierdo del ratn
	 {
			// Tomar la posicin del ratn

		ColCursor = Suceso . Posicion . X ;
		LinCursor = Suceso . Posicion . Y ;

				// Si estamos en la apertura

		if( *( Buffer + (LinCursor-1) * Columnas + ColCursor - 1 ) == '[' )

			ColCursor++ ; // Entrar en el campo

				// Si estamos en el cierre

		if( *( Buffer + (LinCursor-1) * Columnas + ColCursor - 1 ) == ']' )

			ColCursor-- ; // Entrar en el campo

			// Invertir su estado

		*( Buffer + (LinCursor-1) * Columnas + ColCursor - 1 ) =

		   *( Buffer + (LinCursor-1) * Columnas + ColCursor - 1 ) == ' ' ? 'X' : ' ' ;
	 }

	return 0 ;
 }

	//
	// Mtodos de la clase GSExclusiva
	//

	//
	// Mtodo : Constructor
	//
	// Construye un objeto a partir de su representacin
	//

GSExclusiva::GSExclusiva( char * S )
 {
	char * Temp = istrchr( S, '[' ) ; // Posicin del primer indicador

	if( ! Temp ) // Si la cadena es nula

		Opcion = 1 ; // se asume como activa la primera opcin

	else
	 {

		Opcion = 0 ; // Nmero del indicador activo

		int C = 1 ; // Contador

		while( ! Opcion ) // Hasta que se encuentre la opcin activa

			if( *(Temp+1) == 'X' ) // Si se encuentra la marca

				Opcion = C ; // esta es la opcin activa

			else // si no
			 {
				C++ ;    // Avanzar a la siguiente opcin

				Temp = istrchr( Temp+1, '[' ) ; // hasta la siguiente posicin
			 }
	 }
 }

	//
	// Mtodos de la clase SExclusiva
	//

	//
	// Mtodo : Constructor
	//
	// Construye el objeto sirvindose del constructor base, y
	// le asigna un valor inicial
	//

SExclusiva::SExclusiva( int Ancho, int Alto, GSExclusiva Inicial )

	// Llama al constructor base

			: TipoCampo( Ancho * 3, Alto )

 {
	 ColCursor = 2 ; LinCursor = 1 ; // Inicializa posicin cursor

	// Formar en buffer la imagen del campo, segn los valores de Inicial

	char * Temp = Buffer ;

	for( int L = 0 ; L < Alto ; L++ ) // Bucle de lneas

		for( int C = 0 ; C < Ancho ; C++ )  // Bucle de columnas
		 {
			*Temp++ = '[' ; // Delimitador izquierdo

				// X o ' ', dependiendo del estado de esa posicin

			*Temp++ = ( Inicial . Opcion == L * Ancho + C + 1 ?
							'X' :
							' ' ) ;

			*Temp++ = ']' ; // Delimitador derecho
		 }
 }

	//
	// Mtodo : Procesa
	//
	// Procesa los eventos que se produzcan mientras est activo
	// este campo
	//

int SExclusiva::Procesa( Evento & Suceso )
 {
		// Quitar la seleccin del apartado actual

	*( Buffer + (LinCursor-1) * Columnas + ColCursor - 1 ) = ' ' ;

	if( Suceso . Quien == Evento::Teclado ) // Si es una tecla

		switch( Suceso . Tecla ) // Veamos cual
		 {
			case GTeclado::IZQUIERDA : // Opcin de la izquierda

				if( ColCursor != 2 ) // Si no estamos en la primera

					ColCursor -= 3 ; // retroceder a la anterior

				break ;

			case GTeclado::DERECHA : // Opcin de la derecha

				if( ColCursor < Columnas - 1 ) // Si no estamos en la ltima

					ColCursor += 3 ; // ir a la siguiente

				break ;

			case GTeclado::ARRIBA : // Opcin superior

				if( LinCursor > 1 ) // Si no estamos en la primera lnea

					LinCursor-- ; // podemos subir a la anterior

				break ;

			case GTeclado::ABAJO : // Opcin inferior

				if( LinCursor < Lineas ) // Si no estamos en la ltima lnea

					LinCursor++ ; // podemos bajar a la siguiente

				break ;

		 } // switch Tecla

	else // Se ha pulsado el botn izquierdo del ratn
	 {
			// Tomar la posicin del ratn

		ColCursor = Suceso . Posicion . X ;
		LinCursor = Suceso . Posicion . Y ;

				// Si estamos en la apertura

		if( *( Buffer + (LinCursor-1) * Columnas + ColCursor - 1 ) == '[' )

			ColCursor++ ; // Entrar en el campo

				// Si estamos en el cierre

		if( *( Buffer + (LinCursor-1) * Columnas + ColCursor - 1 ) == ']' )

			ColCursor-- ; // Entrar en el campo
	 }

		 // Activar la nueva opcin seleccionada

	*( Buffer + (LinCursor-1) * Columnas + ColCursor - 1 ) = 'X' ;

	return 0 ;
 }

	//
	// Mtodos de la clase GPlantilla
	//

	//
	// Mtodo : Constructor
	//
	// Construye una plantilla de peticin de campos a partir de una
	// lista de campos, la asocia con una ventana y le asigna un nombre
	//

GPlantilla::GPlantilla( char * Nombre, GVentana * V, Cola<GCampo> & C,
						GVenBuf::Color TN, GVenBuf::Color FN,
						GVenBuf::Color TA, GVenBuf::Color FA )


	// Inicializa los distintos miembros

			: Ventana( V ), Campos( C ),
			  TintaNormal( TN ), FondoNormal( FN ),
			  TintaActivo( TA ), FondoActivo( FA )

 {
	ID = new char[ istrlen( Nombre ) + 1 ] ; // Asigna memoria

	istrcpy( ID, Nombre ) ; // y copia el nombre de la plantilla

   Inicializa() ; // Inicializa el aspecto en la ventana
 }

	//
	// Mtodo : Constructor
	//
	// Construye una plantilla de peticin de campos a partir de un
	// slo campo, la asocia con una ventana y le asigna un nombre
	//

GPlantilla::GPlantilla( char * Nombre, GVentana * V, GCampo & C,
						GVenBuf::Color TN, GVenBuf::Color FN,
						GVenBuf::Color TA, GVenBuf::Color FA )


	// Inicializa los distintos miembros

			: Ventana( V ),
			  TintaNormal( TN ), FondoNormal( FN ),
			  TintaActivo( TA ), FondoActivo( FA )

 {
	ID = new char[ istrlen( Nombre ) + 1 ] ; // Asigna memoria

	istrcpy( ID, Nombre ) ; // y copia el nombre de la plantilla

    Campos << C ; // Aade el campo como nico elemento de la cola

    Inicializa() ;
 }

	//
	// Mtodo : Constructor de copia
	//
	// Permite crear un objeto a partir de otro
	//

GPlantilla::GPlantilla(  GPlantilla & O )

		//  Se construyen los miembros

			: Ventana( O . Ventana ), Campos( O . Campos ),
			  TintaNormal( O . TintaNormal ), FondoNormal( O . FondoNormal ),
			  TintaActivo( O . TintaActivo ), FondoActivo( O . FondoActivo )

 {
	ID = new char[ istrlen( O . ID ) + 1 ] ; // Memoria para el nuevo ttulo

	istrcpy( ID, O . ID ) ; // Copiar el nombre

	Inicializa() ;
 }

	//
	// Mtodo : Destructor
	//
	// Si se asign memoria para el nombre la libera
	//

GPlantilla::~GPlantilla()
 {
	if( ID ) // Si se asign memoria

		delete [] ID ; // la libera
 }

	//
	// Mtodo : Operador de asignacin
	//
	// Facilita la asignacin de un objeto a otro
	//

void GPlantilla::operator = (  GPlantilla & O )
 {
		// Se asignan los miembros

	Ventana = O . Ventana ;

	Campos = O . Campos ;

	TintaNormal = O . TintaNormal ;
	FondoNormal = O . FondoNormal ;
	TintaActivo = O . TintaActivo ;
	FondoActivo = O . FondoActivo ;

	if( ID ) // Si hay un ttulo

		delete [] ID ; // eliminarlo

	ID = new char[ istrlen( O . ID ) + 1 ] ; // Asignar memoria

	istrcpy( ID, O . ID ) ; // y copiar el nuevo ttulo
 }

	//
	// Mtodo : Operador relacional de igualdad
	//
	// Permite saber si dos objetos son iguales
	//

int GPlantilla::operator == (  GPlantilla & O )
 {
		// Dos objetos son iguales si

	return ! istrcmp( ID, O . ID ) && // coincide el nombre
			Ventana == O . Ventana && // la ventana en que estn
			Campos == O . Campos ; // y tienen los mismos campos
 }

	//
	// Mtodo : Operador de insercin en stream
	//
	// Facilita el almacenamiento de un objeto de la clase
	// en el stream
	//

ostream & operator << ( ostream & Stream, GPlantilla & O )
 {
	return Stream << O . Campos ; // Se escriben los campos
 }

	//
	// Mtodo : Operador de extraccin de stream
	//
	// Permite la recuperacin de un objeto desde un stream
	//

istream & operator >> ( istream & Stream, GPlantilla & O )
 {
	return Stream >> O . Campos ; // Se leen los campos
 }

	//
	// Mtodo : Operador de concatenacin
	//
	// Construye una cola a partir de dos objetos GCampo
	//

Cola<GCampo> operator + ( GCampo & O1, GCampo & O2 )
 {
	Cola<GCampo> Temporal ; // Se crea la cola

	Temporal << O1 << O2 ; // Se aaden en ella los dos elementos

	return Temporal ; // y se devuelve
 }

	//
	// Mtodo : Operador de concatenacin
	//
	// Facilita el aadir un nuevo campo a una cola existente
	//

Cola<GCampo> operator+ ( Cola<GCampo> & Cola, GCampo & O )
 {
	Cola << O ; // Se aade el elemento

	return Cola ; // y se devuelve la cola
 }

	//
	// Mtodo : Operador () sobrecargado
	//
	// Devuelve la imagen del campo cuyo nombre se indique
	//

char * GPlantilla::operator () ( char * Nombre )
 {
	GCampo * Campo = Campos . pPrimero() ; // Se obtiene el primer campo

	while( Campo != NULL ) // Mientras no lleguemos al ltimo
	 {
		if( ! istrcmp( Campo -> ID, Nombre ) ) // Si coincide el nombre

			return Campo -> Imagen() ; // Devolver la imagen

		Campo = Campos . pSiguiente( *Campo ) ; // Si no ir al siguiente
	 }

	return NULL ; // No se encontr
 }

	//
	// Mtodo : Inicializa
	//
	// Imprime en la ventana la informacin de la plantilla, y
	// fija como actual el primero, destacndolo del resto.
	//

void GPlantilla::Inicializa()
 {
	Actual = Campos . pPrimero() ; // Comencemos por el primer campo

	while( Actual != NULL )  // Mientras no hayamos llegado al ltimo
	 {
		MuestraNormal() ; // Imprimir el campo con atributo normal

		Actual = Campos . pSiguiente( * Actual ) ; // Ir al siguiente
	 }

	Actual = Campos . pPrimero() ; // El campo actual ser el primero
 }

	//
	// Mtodo : Procesa
	//
	// Procesa los eventos que recibe de GEntrada, traducindolos
	// en movimientos entre los campos o envindoselos al campo
	// actual, segn proceda.
	//

int GPlantilla::Procesa( Evento & Suceso )
 {
	switch( Suceso . Quien ) // Inspeccionemos el suceso
	 {
		case Evento::Teclado : // Es una pulsacin de una tecla

			switch( Suceso . Tecla ) // Veamos que tecla se ha pulsado
			 {
				case GTeclado::TAB : // Ir al siguiente campo

						// Se toma el puntero al siguiente campo

					Actual = Campos . pSiguiente( * Actual ) ;

					if( Actual == NULL ) // Si era el ltimo

						Actual = Campos . pPrimero() ; // Pasar al primero

					break ;

				case GTeclado::SHIFT_TAB : // Ir al campo anterior

						// Se toma el puntero al campo anterior

					Actual = Campos . pAnterior( * Actual ) ;

					if( Actual == NULL ) // Si era el primero

						Actual = Campos . pUltimo() ; // pasar al ltimo

					break ;

				default : // Proceso por defecto

						// El evento lo habr de procesar el propio campo

					return Actual -> Tipo -> Procesa( Suceso ) ;

			 } // switch Tecla

			break ;

		case Evento::Raton : // Es la pulsacin del botn izquierdo del ratn

				// Se halla la posicin donde se puls el botn en
				// coordenadas relativas a la ventana

			int C = Suceso . Posicion . X / 8 + 1 - Ventana -> rdbuf() -> Col() ;
			int L = Suceso . Posicion . Y / 8 + 1 - Ventana -> rdbuf() -> Lin() ;

			GCampo * T = Campos . pPrimero() ; // Puntero al primer campo

				// Vamos a intentar averiguar en qu campo se
				// encuentra esa posicin

			while( T != NULL ) // Mientras queden campos por inspeccionar

				if( T -> Esta( C, L ) ) // Si esa posicin cae en ese campo

					break ; // Detener el proceso de bsqueda

				else

					T = Campos . pSiguiente( * T ) ; // Si no seguir buscando

			if( T != NULL ) // Si el cursor estaba en un campo

				if( T == Actual ) // Si es el campo actual
				 {
					C -= Actual -> Col() ;  // Convertir en coordenada de campo
					L -= Actual -> Lin() ;

					Suceso . Posicion . X = C + 1 ;  // Preservar la posicin
					Suceso . Posicion . Y = L + 1 ;

					return Actual -> Tipo -> Procesa( Suceso ) ; // que lo procese l
				 }
				else // si no
				 {
					MuestraNormal() ; // Quita el atributo destacado

					Actual = T ; // El campo actual ser el sealado

					MuestraDestacado() ; // Destaca ese campo

	       GEvento Eventos ;

		       // Se simula la pulsacin del botn en ese campo

	       Eventos . Inserta( 1, 0,
		       ( C + Ventana -> rdbuf() -> Col() - 1 ) * 8,
		       ( L + Ventana -> rdbuf() -> Lin() - 1 ) * 8 ) ;
				 }

			break ;

	 } // switch Quien

	return 0 ;
 }

	//
	// Mtodo : MuestraNormal
	//
	// Imprime en la ventana el campo actual, utilizando el atributo normal
	//

void GPlantilla::MuestraNormal()
 {
		// Fija el atributo normal

	*Ventana << TintaA( TintaNormal ) << FondoA( FondoNormal ) ;

	Muestra() ; // Imprime el campo
 }

	//
	// Mtodo : MuestraDestacado
	//
	// Imprime en la ventana el campo actual utilizando el atributo
	// destacado, para as distinguirlo de los dems
	//

void GPlantilla::MuestraDestacado()
 {
		// Fija el atributo destacado

	*Ventana << TintaA( TintaActivo ) << FondoA( FondoActivo ) ;

	Muestra() ; // Imprime el campo
 }

	//
	// Mtodo : Muestra
	//
	// Imprime el contenido del campo actual
    //

void GPlantilla::Muestra()
 {
		// Espacio para ir creando lneas de texto

	char * LinTexto = new char[ Actual -> Tipo -> Cols() + 1 ] ;

	char * Temp = Actual -> Imagen() ; // Para ir tomando datos del buffer

	for( int L = 0 ; L < Actual -> Tipo -> Lins() ; L++ )
	 {
		int P = 0 ; // Posicin en la LinTexto

		for( int C = 0 ; C < Actual -> Tipo -> Cols() ; C++ )

			LinTexto[ P++ ] = *Temp++ ; // Ir tomando caracteres

		LinTexto[ P ] = 0 ; // Terminador de cadena

			// Se inserta la cadena en la posicin adecuada

		*Ventana << PosCursor( Actual -> Col(), Actual -> Lin() + L )
				 << LinTexto ;
	 }

   delete [] LinTexto ;
 }

	//
	// Mtodos de la clase GEntrada
	//

	//
	// Mtodo : Operador de concatenacin
	//
	// Permite construir una cola de plantillas a partir de
	// dos objetos individuales
	//

Cola<GPlantilla> operator + ( GPlantilla & O1, GPlantilla & O2 )
 {
	Cola<GPlantilla> Cola ; // Se crea la cola

	Cola << O1 << O2 ; // Se aaden los dos objetos

	return Cola ; // Se devuelve la cola
 }

	//
	// Mtodo : Operador de concatenacin
	//
	// Facilita el aadir una plantilla a una lista existente
	//

Cola<GPlantilla> operator + ( Cola<GPlantilla> & Cola, GPlantilla & O )
 {
	Cola << O ; // Se aade el objeto

	return Cola ; // Y se devuelve
 }

	//
	// Mtodo : Ejecuta
	//
	// Este mtodo es el que realmente gestiona toda el proceso,
	// sirvindose de los datos almacenados en los objetos y de
	// los mtodos Valida() y Procesa(), definidos para cada tipo
	// de campo.
	//

int GEntrada::Ejecuta()
 {
	int Cod = 0 ; // Cdigo de salida

	Actual = Paginas . pPrimero() ; // La pgina actual es la primera

	if( Actual == NULL ) // Si no hay pginas por pedir

		return 0 ; // fin del proceso

	int Salida = 0 ; // Indicador de salida

	while( ! Salida ) // Mientras no se active el indicador de salida
	 {
		Actual -> Inicializa() ; // Inicializa el contenido de la plantilla

		Pantalla -> Encima( Actual -> Ventana ) ; // Pone la ventana sobre las dems

		if( ! Pantalla -> EsVisible( Actual -> Ventana ) )

			Pantalla -> Muestra( Actual -> Ventana ) ; // Hay que mostrar la ventana

		int SalidaInt = 0 ; // Indicador de salida del bucle interior

		while( ! SalidaInt ) // Mientras no se active el indicador
		 {
			GEvento Control ;
			Evento Suceso ;

			Actual -> MuestraDestacado() ; // Mostrar resaltado el campo actual

			*Actual -> Ventana << PosCursor(

			   Actual -> Actual -> Col() + Actual -> Actual -> Tipo -> Col() - 1,
			   Actual -> Actual -> Lin() + Actual -> Actual -> Tipo -> Lin() - 1 ) ;

			GTeclado Teclado ;

			Teclado . InsertActivo() ? // Si Insert est activo

					// Poner el cursor en forma de bloque

				Actual -> Ventana -> rdbuf() -> AspectoCursor( GVenBuf::CursorBloque ) :

					// Si no poner el cursor en forma de lnea

				Actual -> Ventana -> rdbuf() -> AspectoCursor( GVenBuf::CursorNormal ) ;

			Pantalla -> Refresca() ; // Actualizar el aspecto de la pantalla

			do {
				Control >> Suceso ;

			} while( Suceso . Quien == Evento::Vacio ) ;

			Actual -> MuestraNormal() ; // Quitar el atributo destacado

			switch( Suceso . Quien ) // Veamos quien ha producido el evento
			 {
				case Evento::Teclado : // El teclado

					switch( Suceso . Tecla ) // Veamos que tecla
					 {
						case GTeclado::AVPAG : // Siguiente pgina

								// Pasamos a la pgina siguiente

							Actual = Paginas . pSiguiente( * Actual ) ;

							if( ! Actual ) // Si no hay siguiente

								Salida = 1 ; // es el final de la peticin

							SalidaInt = 1 ; // Salir al bucle exterior

							break ;

						case GTeclado::REPAG : // Anterior pgina

								// Pasamos a la pgina anterior

							Actual = Paginas . pAnterior( * Actual ) ;

							if( ! Actual ) // Si no hay anterior

									// Continuamos en la primera pgina

								Actual = Paginas . pPrimero() ;

							SalidaInt = 1 ; // Salir al bucle exterior

							break ;

						case GTeclado::ESC : // Salir

							SalidaInt = Salida = 1 ; // Provocar la salida

							break ;

						default: // Proceso por defecto

								// Pasar el evento para que sea
								// procesado por la plantilla

							Cod = Actual -> Procesa( Suceso ) ;

							if( Cod ) // Si hay valor de retorno

								SalidaInt = Salida = 1 ; // Salir

							break ;

					 } // switch Tecla

					break ;

				case Evento::Raton : // El raton

					switch( Suceso . Boton ) // Veamos que botn
					 {
						case Evento::ClickI : // Botn izquierdo

								// Obtener columna y lnea del cursor

							int C = Suceso . Posicion . X / 8 + 1 ;
							int L = Suceso . Posicion . Y / 8 + 1 ;

							   // Veamos sobre qu ventana se ha pulsado

							GVentana * T =

								(*Pantalla)( Pantalla -> Ventana( C, L ) ) ;

							if( T == Actual -> Ventana ) // Si es la ventana actual
		      {
								  // Que sea la plantilla la que procese

								Cod = Actual -> Procesa( Suceso ) ;

									// Si hay valor de retorno != 0

								if( Cod )

										// Provocar la salida

									SalidaInt = Salida = 1 ;
		      }
							else // En caso contrario veamos si se
								 // ha pulsado sobre otra de las ventanas
							 {
									// Se toma la primera plantilla

								GPlantilla * X = Paginas . pPrimero() ;

								while( X != NULL ) // Mientras no fin

										// Si esa es la ventana en que se puls

									if( X -> Ventana == T )

										break ; // Detener la bsqueda

									else // en caso contrario

											// continuar buscando

										X = Paginas . pSiguiente( *X ) ;

								if( X != NULL ) // Si se encontr
								 {
									Actual = X ; // Esa plantilla es ahora la actual

									SalidaInt = 1 ; // Salir al bucle exterior
								 }
							 }
								break ;

						case Evento::ClickD : // Botn derecho

							SalidaInt = Salida = 1 ; // Provocar la salida

							break ;

					 } // switch Boton

					break ;

			 } // switch Quien

		 } // while SalidaInt

	 } // while Salida

	 return Cod ; // Devuelve cdigo de salida

 }  // GEntrada::Ejecuta

	//
	// Mtodo : Ejecuta2
	//
	// Este mtodo es el que realmente gestiona toda el proceso,
	// sirvindose de los datos almacenados en los objetos y de
	// los mtodos Valida() y Procesa(), definidos para cada tipo
	// de campo.
	//

int GEntrada::Ejecuta2()
 {
	int Cod = 0 ; // Cdigo de salida

	Actual = Paginas . pPrimero() ; // La pgina actual es la primera

	if( Actual == NULL ) // Si no hay pginas por pedir

		return 0 ; // fin del proceso

	int Salida = 0 ; // Indicador de salida

	while( ! Salida ) // Mientras no se active el indicador de salida
	 {
		Actual -> Inicializa() ; // Inicializa el contenido de la plantilla

		Pantalla -> Encima( Actual -> Ventana ) ; // Pone la ventana sobre las dems

		if( ! Pantalla -> EsVisible( Actual -> Ventana ) )

			Pantalla -> Muestra( Actual -> Ventana ) ; // Hay que mostrar la ventana

		int SalidaInt = 0 ; // Indicador de salida del bucle interior

		while( ! SalidaInt ) // Mientras no se active el indicador
		 {
			GEvento Control ;
			Evento Suceso ;

			Actual -> MuestraDestacado() ; // Mostrar resaltado el campo actual

			*Actual -> Ventana << PosCursor(

			   Actual -> Actual -> Col() + Actual -> Actual -> Tipo -> Col() - 1,
			   Actual -> Actual -> Lin() + Actual -> Actual -> Tipo -> Lin() - 1 ) ;

			GTeclado Teclado ;

			Teclado . InsertActivo() ? // Si Insert est activo

					// Poner el cursor en forma de bloque

				Actual -> Ventana -> rdbuf() -> AspectoCursor( GVenBuf::CursorBloque ) :

					// Si no poner el cursor en forma de lnea

				Actual -> Ventana -> rdbuf() -> AspectoCursor( GVenBuf::CursorNormal ) ;

			Pantalla -> Refresca() ; // Actualizar el aspecto de la pantalla

			do {
				Control >> Suceso ;

			} while( Suceso . Quien == Evento::Vacio ) ;

			Actual -> MuestraNormal() ; // Quitar el atributo destacado

			switch( Suceso . Quien ) // Veamos quien ha producido el evento
			 {
				case Evento::Teclado : // El teclado

					switch( Suceso . Tecla ) // Veamos que tecla
					 {
						case GTeclado::AVPAG : // Siguiente pgina

								// Pasamos a la pgina siguiente

							Actual = Paginas . pSiguiente( * Actual ) ;

							if( ! Actual ) // Si no hay siguiente

								Salida = 1 ; // es el final de la peticin

							SalidaInt = 1 ; // Salir al bucle exterior

							break ;

						case GTeclado::REPAG : // Anterior pgina

								// Pasamos a la pgina anterior

							Actual = Paginas . pAnterior( * Actual ) ;

							if( ! Actual ) // Si no hay anterior

									// Continuamos en la primera pgina

								Actual = Paginas . pPrimero() ;

							SalidaInt = 1 ; // Salir al bucle exterior

							break ;

						case GTeclado::ESC : // Salir

							SalidaInt = Salida = 1 ; // Provocar la salida

							break ;

						default: // Proceso por defecto

								// Pasar el evento para que sea
								// procesado por la plantilla

							Cod = Actual -> Procesa( Suceso ) ;

							if( Cod ) // Si hay valor de retorno

								SalidaInt = Salida = 1 ; // Salir

							break ;

					 } // switch Tecla

					break ;
				case Evento::Raton : // El raton

					switch( Suceso . Boton ) // Veamos que botn
					 {
						case Evento::ClickI : // Botn izquierdo

								// Obtener columna y lnea del cursor

							int C = Suceso . Posicion . X / 8 + 1 ;
							int L = Suceso . Posicion . Y / 8 + 1 ;

							   // Veamos sobre qu ventana se ha pulsado

							GVentana * T =

								(*Pantalla)( Pantalla -> Ventana( C, L ) ) ;

							if( T == Actual -> Ventana ) // Si es la ventana actual
		      {
								  // Que sea la plantilla la que procese

								Cod = Actual -> Procesa( Suceso ) ;

									// Si hay valor de retorno != 0

								if( Cod )

										// Provocar la salida

									SalidaInt = Salida = 1 ;
		      }
							else // En caso contrario veamos si se
								 // ha pulsado sobre otra de las ventanas
							 {
									// Se toma la primera plantilla

								GPlantilla * X = Paginas . pPrimero() ;

								while( X != NULL ) // Mientras no fin

										// Si esa es la ventana en que se puls

									if( X -> Ventana == T )

										break ; // Detener la bsqueda

									else // en caso contrario

											// continuar buscando

										X = Paginas . pSiguiente( *X ) ;

								if( X != NULL ) // Si se encontr
								 {
									Actual = X ; // Esa plantilla es ahora la actual

									SalidaInt = 1 ; // Salir al bucle exterior
								 }
								else
								{
									Salida = SalidaInt = 1 ; // Salir
								}
							 }
								break ;

						case Evento::ClickD : // Botn derecho

							SalidaInt = Salida = 1 ; // Provocar la salida

							break ;

					 } // switch Boton

					break ;

			 } // switch Quien

		 } // while SalidaInt

	 } // while Salida

	 return Cod ; // Devuelve cdigo de salida

 }  // GEntrada::Ejecuta

	//
	// Mtodo : Operador () sobrecargado
	//
	// Permite obtener un puntero a una plantilla, cuyo nombre
	// se ha de facilitar
	//

GPlantilla * GEntrada::operator() ( char * Nombre )
 {
	GPlantilla * Plantilla = Paginas . pPrimero() ;

	while( Plantilla != NULL ) // Mientras no lleguemos al ltimo
	 {
		if( ! istrcmp( Plantilla -> ID, Nombre ) ) // Si coincide el nombre

			return Plantilla ; // Devolver el puntero

		Plantilla = Paginas . pSiguiente( *Plantilla ) ; // Si no ir al siguiente
	 }

	return NULL ; // No se encontr
 }

   //
   // Mtodo : Operador () sobrecargado
   //
   // Devuelve la imagen de un campo contenido en una plantilla
   //

char * GEntrada::operator () ( char * Plantilla, char * Campo )
 {
      // El operador () devuelve un puntero a la plantilla, que se
      // utiliza para acceder al operador () de GPlantilla, que
      // devuelve el contenido del campo

   return operator()(Plantilla) -> operator()(Campo) ;
 }
