	//
	// GEntrada.Hpp
	//
	// Definicin de las clases para creacin de mscaras de entrada de datos
	//
	// 27 Agosto 1992

#ifndef _GENTRADA_HPP_

#define _GENTRADA_HPP_

#ifndef __cplusplus
#error No se puede usar GEntrada en C
#endif

#include <ctype.h>

#include "EstDat.Hpp" // Para utilizar colas de objetos

#include "GVentana.Hpp" // Hay referencias a GVentana y GES

#include "GEvento.Hpp" // La peticin se realiza en base a eventos

class TipoCampo ; // Referencia avanzada

	//
	// Clase : Campo
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Mantener la informacin necesaria
	//			   para procesar un campo
	//
	// Fecha de creacin : 27 Agosto 1992
	//
	// Fecha de revisin : -
	//

class GCampo
 {
	public :

		friend class GEntrada ; // La clase GEntrada tiene acceso a esta
		friend class GPlantilla ;

			  // Constructor estndar, que define las caractersticas
			  // del campo a crear

			GCampo( char *, int, int, TipoCampo * ) ;

			GCampo() ; // Constructor por defecto

			GCampo(  GCampo & ) ; // Constructor de copia

			GCampo( int ) ; // Constructor de objeto nulo

			~GCampo() ; // Destructor

			// Operador de asignacin

		void operator = (  GCampo & ) ;

			// Operador de relacin de igualdad

		int operator == (  GCampo & ) ;

			// Operador de insercin en stream

		friend ostream & operator << ( ostream &, GCampo & ) ;

			// Operador de extraccin de stream

		friend istream & operator >> ( istream &, GCampo & ) ;

		char * Imagen() ; // Devuelve la imagen del campo

		int Col() ; // Informa sobre la columna en que est el campo

		int Lin() ; // Sobre la lnea

		  // Informa si una determinada posicin cae en este campo

		int Esta( int, int ) ;

	private :

		int Columna, Linea ; // Posicin del campo en la ventana

			// Tipo del objeto

		TipoCampo * Tipo ;

		char * ID ; // Nombre del campo

 } ; // GCampo

	//
	// Clase : TipoCampo
	//
	// Deriva de : -
	//
	// Base de : Entero, Fecha, Cadena, etc.
	//
	// Finalidad : Servir como base virtual de
	//			   la que derivarn los distintos
	//			   tipos de campo
	//
	// Fecha de creacin : 27 Agosto 1992
	//
	// Fecha de revisin : -
	//

class TipoCampo
 {
	public :

		TipoCampo( int, int ) ; // Constructor

		TipoCampo(  TipoCampo & ) ; // Constructor de copia

		~TipoCampo() ; // Destructor

			// Toma un evento y lo procesa

		virtual int Procesa( Evento & ) ;

			// Valida un carcter

		virtual int Valida( char & ) = 0 ;

			// Insertar un carcter en el campo

		virtual void Inserta( char ) ;

		int Cols() ; // Informa sobre el nmero de columnas del campo

		int Lins() ; // El nmero de lneas

		int Col() ; // Devuelve la columna en la que est actualmente el cursor

		int Lin() ; // Devuelve la lnea en la que est el cursor

		operator char * () ; // Devuelve el contenido en forma de cadena

		operator int () ; // Devuelve el contenido convertido a entero

		operator long () ; // Convertido a entero largo

		operator float () ; // Convertido en nmero de punto flotante

	protected :

		char * Buffer ; // Espacio sobre el que hay que actuar

		int Columnas, Lineas ; // Dimensiones del campo

			// Posicin del cursor dentro del campo

		int ColCursor, LinCursor ;

 } ; // TipoCampo

	//
	// Clase : Entero
	//
	// Deriva de : TipoCampo
	//
	// Base de : -
	//
	// Finalidad : Gestionar la peticin de nmeros enteros
	//
	// Fecha de creacin : 27 Agosto 1992
	//
	// Fecha de revisin : -
	//

class Entero : public TipoCampo
 {
	public :

		Entero( int, int = 0 ) ; // Constructor

	protected :

		int Valida( char & ) ; // Valida entradas de caracteres
 } ;

	//
	// Clase : Decimal
	//
	// Deriva de : TipoCampo
	//
	// Base de : -
	//
	// Finalidad : Gestionar la peticin de nmeros decimales
	//
	// Fecha de creacin : 27 Agosto 1992
	//
	// Fecha de revisin : -
	//

class Decimal : public TipoCampo
 {
	public :

		// El constructor recibe el nmero de columnas del campo, un
		// posible valor inicial, el tipo de ajuste y el carcter de relleno

		Decimal( int, float = 0 ) ;

	protected :

		int Valida( char & ) ; // Valida la entrada de caracteres
 } ;

	//
	// Clase : Cadena
	//
	// Deriva de : TipoCampo
	//
	// Base de : -
	//
	// Finalidad : Gestionar la peticin de cadenas
	//
	// Fecha de creacin : 27 Agosto 1992
	//
	// Fecha de revisin : -
	//

class Cadena : public TipoCampo
 {
	public :

		  // El constructor recibe el nmero de columnas y lneas del campo,
		  // una cadena inicial, por defecto nula, y el carcter de relleno
        // a utilizar.

		Cadena( int, int, char * = NULL ) ;

	protected :

		int Valida( char & ) ; // Valida la entrada de caracteres
 } ;

	//
	// Clase : GFecha
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Almacenar un fecha
	//
	// Fecha de creacin : 27 Agosto 1992
	//
	// Fecha de revisin : -
	//

class GFecha
 {
	public :

		GFecha( char * )  ;  // Construir una fecha a partir de una cadena

		unsigned 	Dia : 5,
					Mes : 4,
					Ano : 7 ;

 } ; // GFecha

	//
	// Clase : Fecha
	//
	// Deriva de : TipoCampo
	//
	// Base de : -
	//
	// Finalidad : Gestionar la peticin de fechas
	//
	// Fecha de creacin : 27 Agosto 1992
	//
	// Fecha de revisin : -
	//

class Fecha : public TipoCampo
 {
	public :

		  // El constructor puede recibir una fecha inicial, que
		  // por defecto es nula

		Fecha( GFecha = GFecha( "" ) ) ;

	protected :

		int Valida( char & ) ; // Valida la entrada de caracteres

		int Procesa( Evento & ) ; // Procesa los eventos

		void Inserta( char ) ; // Insertar un carcter en el campo
 } ;

	//
	// Clase : Boton
	//
	// Deriva de : TipoCampo
	//
	// Base de : -
	//
	// Finalidad : Gestionar el tratamiento de
	//			   botones durante la peticin
	//
	// Fecha de creacin : 27 Agosto 1992
	//
	// Fecha de revisin : -
	//

class Boton : public TipoCampo
 {
	public :

		  // El constructor recibe el ttulo del botn y el valor
		  // que se ha de volver en caso de pulsarse

		Boton( char *, unsigned ) ;

	protected :

		int Valida( char & ) ; // Valida la entrada de caracteres

		int Procesa( Evento & ) ; // Procesa los eventos

		unsigned Codigo ; // Cdigo asociado con el botn
 } ;

	//
	// Clase : GSMultiple
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Mantener la informacin de un
	//			   campo de seleccin mltiple
	//
	// Fecha de creacin : 27 Agosto 1992
	//
	// Fecha de revisin : -
	//

class GSMultiple
 {
	public :

		GSMultiple( char * ) ; // Construir el objeto a partir de su imagen

			// Hasta 32 indicadores de 1 bit cada uno

		struct { unsigned Estado : 1 ; } Opcion[ 32 ] ;

 } ; // GSMultiple

	//
	// Clase : SMultiple
	//
	// Deriva de : TipoCampo
	//
	// Base de : -
	//
	// Finalidad : Gestionar la peticin de
	//			   campos de seleccin mltiple
	//
	// Fecha de creacin : 27 Agosto 1992
	//
	// Fecha de revisin : -
	//

class SMultiple : public TipoCampo
 {
	public :

		  // Nmero de columnas y lneas del campo, que puede
		  // tener un valor inicial, por defecto nulo

		SMultiple( int, int, GSMultiple = GSMultiple( "" ) ) ;

	protected :

		int Valida( char & ) ; // Valida las entradas de caracteres

		int Procesa( Evento & ) ; // Procesa los eventos
 } ;

	//
	// Clase : GSExclusiva
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Mantener la informacin de un
	//			   campo de seleccin exclusiva
	//
	// Fecha de creacin : 27 Agosto 1992
	//
	// Fecha de revisin : -
	//

class GSExclusiva
 {
	public :

		GSExclusiva( char * ) ; // Construir el objeto a partir de su imagen

		long Opcion ;

 } ; // GSExclusiva

	//
	// Clase : SExclusiva
	//
	// Deriva de : TipoCampo
	//
	// Base de : -
	//
	// Finalidad : Gestionar la peticin de
	//			   campos de seleccin exclusiva
	//
	// Fecha de creacin : 27 Agosto 1992
	//
	// Fecha de revisin : -
	//

class SExclusiva : public TipoCampo
 {
	public :

			// El constructor recibe el nmero de columnas y lneas
			// del campo, y de forma opcional un valor inicial

		SExclusiva( int, int, GSExclusiva = GSExclusiva( "" ) ) ;

	protected :

		int Valida( char & ) ; // Valida la entrada de caracteres

		int Procesa( Evento & ) ; // Procesa los eventos
 } ;

	//
	// Clase : GPlantilla
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Implementar la gestin de
	//			   conjuntos de objetos Campo,
	//			   formando mscaras de entrada
	//
	// Fecha de creacin : 27 Agosto 1992
	//
	// Fecha de revisin : -
	//

class GPlantilla
 {
	public :

		friend class GEntrada ;

			   // Constructor estndar

			GPlantilla( char *, GVentana *, Cola<GCampo> &,
						GVenBuf::Color = GVenBuf::Blanco,
						GVenBuf::Color = GVenBuf::Negro,
						GVenBuf::Color = GVenBuf::Negro,
						GVenBuf::Color = GVenBuf::GrisClaro ) ;

			GPlantilla( char *, GVentana *, GCampo &,
						GVenBuf::Color = GVenBuf::Blanco,
						GVenBuf::Color = GVenBuf::Negro,
						GVenBuf::Color = GVenBuf::Negro,
						GVenBuf::Color = GVenBuf::GrisClaro ) ;

			GPlantilla() ; // Constructor por defecto

			GPlantilla(  GPlantilla & ) ; // Constructor de copia

			GPlantilla( int ) ; // Constructor de objeto nulo

			~GPlantilla() ; // Destructor

			// Operador de asignacin

		void operator = (  GPlantilla & ) ;

			// Operador de relacin de igualdad

		int operator == (  GPlantilla & ) ;

			// Operador de insercin en stream

		friend ostream & operator << ( ostream &, GPlantilla & ) ;

			// Operador de extraccin de stream

		friend istream & operator >> ( istream &, GPlantilla & ) ;

			// Permite crear una cola de campos a partir
			// de dos campos individuales

		friend Cola<GCampo> operator + ( GCampo &, GCampo & ) ;

			// Facilita el aadir un campo a una lista

		friend Cola<GCampo> operator + ( Cola<GCampo> &, GCampo & ) ;

			// Devuelve la imagen del campo cuyo nombre se indica

		char * operator() ( char * ) ;

			// Muestra el estado actual de la plantilla en ventana,
			// y fija como campo actual el primero

		void Inicializa() ;

			// Procesa el evento recibido

		int Procesa( Evento & ) ;

	private :

		GVentana * Ventana ; // Ventana en la que se mostrar la mscara

		Cola<GCampo> Campos ; // Cola de los campos que componen la mscara

		char * ID ; // Nombre asignado a la plantilla

			// Colores con los que se representar los campos

		GVenBuf::Color TintaNormal, FondoNormal,
					   TintaActivo, FondoActivo ;

      GCampo * Actual ; // Campo que se est procesando actualmente

		void MuestraNormal() ; // Cambia los atributos del campo
		void MuestraDestacado() ; // actual para distinguirlo del resto

		void Muestra() ; // Imprime el campo actual

 } ; // GPlantilla

	//
	// Clase : GEntrada
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Gestionar el proceso de la
	//			   peticin, permitiendo crear
	//			   varias mscaras de peticin
	//			   asociadas, en forma de pginas,
	//			   recogiendo y procesando los eventos.
	//
	// Fecha de creacin : 27 Agosto 1992
	//
	// Fecha de revisin : -
	//

class GEntrada
 {
	public :
			   // Constructor estndar

			GEntrada( GES *, Cola<GPlantilla> & ) ;

			GEntrada( GES *, GPlantilla & ) ;

			GEntrada() ; // Constructor por defecto

			GEntrada( GEntrada & ) ;

		void operator = ( GEntrada & ) ;

			// Permite crear una lista de mscaras a
			// partir de dos objetos individuales

		friend Cola<GPlantilla> operator + ( GPlantilla &, GPlantilla & ) ;

			// Facilita el aadir mscaras a una lista

		friend Cola<GPlantilla> operator + ( Cola<GPlantilla> &, GPlantilla & ) ;

		int Ejecuta() ; // Procesa la peticin

		int Ejecuta2() ; // Procesa la peticin

			// Devuelve un puntero a la plantilla cuyo nombre se indica

		GPlantilla * operator() ( char * ) ;

        // Devuelve la imagen de un campo

      char * operator () ( char *, char * ) ;

	private :

		GES * Pantalla ; // Objeto GES sobre el que se realizar la salida

		Cola<GPlantilla> Paginas ; // Lista de mscaras

		GPlantilla * Actual ; // Plantilla que se est procesando actualmente

 } ; // GEntrada

	//
	// Implementacin mtodos inline
	//

	//
	// Mtodo : Constructor por defecto
	//
	// Crea un objeto vaco
	//

inline GCampo::GCampo() : Tipo( NULL ), ID( NULL ) // Se asigna valor nulo

	{}

	//
	// Mtodo : Constructor de objeto nulo
	//
	// Construye un objeto nulo
	//

inline GCampo::GCampo( int ) : Tipo( NULL ), ID( NULL ) // Se asigna valor nulo

	{}

	//
	// Mtodo : Imagen
	//
	// Devuelve un puntero a la imagen del campo
	//

inline char * GCampo::Imagen()

		// Devuelve la direccin del buffer, contenido en Tipo

	{ return (char *) *Tipo ; }

	//
	// Mtodo : Col
	//
	// Devuelve la columna en la que est situado el campo en la ventana
	//

inline int GCampo::Col()

	{ return Columna ; }

	//
	// Mtodo : Lin
	//
	// Devuelve la lnea en la que est situado el campo
	//

inline int GCampo::Lin()

	{ return Linea ; }

	//
	// Mtodo : Destructor
	//
	// Libera la memoria asignada por el constructor
	//

inline TipoCampo::~TipoCampo()

	{ if( Buffer != NULL ) delete [] Buffer ; }

	//
	// Mtodo : Cols
	//
	// Devuelve el nmero de columnas del campo
	//

inline int TipoCampo::Cols()

	{ return Columnas ; }

	//
	// Mtodo : Lins
	//
	// Devuelve el nmero de lneas del campo
	//

inline int TipoCampo::Lins()

	{ return Lineas ; }

	//
	// Mtodo : Col
	//
	// Devuelve la posicin horizontal del cursor
	//

inline int TipoCampo::Col()

	{ return ColCursor ; }

	//
	// Mtodo : Lin
	//
	// Devuelve la posicin vertical del cursor
	//

inline int TipoCampo::Lin()

	{ return LinCursor ; }

	//
	// Mtodo : Operador de conversin a puntero char
	//
	// Devuelve la direccin del buffer donde est contenida la informacin
	//

inline TipoCampo::operator char * ()

	{ return Buffer ; }

	//
	// Mtodo : Operador de conversin a entero
	//
	// Convierte el contenido de la cadena a entero, y lo devuelve
	//

inline TipoCampo::operator int ()

	{ return atoi( Buffer ) ; }

	//
	// Mtodo : Operador de conversin a entero largo
	//
	// Convierte el contenido de la cadena a entero largo
	//

inline TipoCampo::operator long ()

	{ return atol( Buffer ) ; }

	//
	// Mtodo : Operador de conversin a nmero de punto flotante
	//
	// Convierte el contenido de la cadena en nmero de punto flotante
	//

inline TipoCampo::operator float ()

	{ return atof( Buffer ) ; }

	//
	// Mtodo : Valida
	//
	// Valida la entrada de caracteres, indicando si son o no
	// admisibles en este tipo de campo
	//

inline int Entero::Valida( char & C )

		// Es vlido si es un dgito o el signo menos al principio del campo

	{ return isdigit( C ) || C == '-' && ColCursor == 1 ; }

	//
	// Mtodo : Valida
	//
	// Valida la entrada de caracteres en el campo
	//

inline int Cadena::Valida( char & C )

	{ return isprint( C ) || strchr( "", C ) ; }

	//
	// Mtodo : Valida
	//
	// Valida la entrada de caracteres en el campo
	//

inline int Fecha::Valida( char & C )

	{ return isdigit( C ) ; }

	//
	// Mtodo : Valida
	//
	// Valida la entrada de caracteres en el campo
	//

inline int Boton::Valida( char & )

	{ return 0 ; } // En este campo no se pueden entrar caracteres

	//
	// Mtodo : Valida
	//
	// Valida la entrada de caracteres en el campo. Los campos
	// de esta clase siempre rechazan la entrada de cualquier carcter
	//

inline int SMultiple::Valida( char & )

	{ return 0 ; }

	//
	// Mtodo : Valida
	//
	// Valida la entrada de caracteres en los campos de esta clase
	//

inline int SExclusiva::Valida( char & )

	{ return 0 ; }

	//
	// Mtodo : Constructor por defecto
	//
	// Crea un objeto vaco
	//

inline GPlantilla::GPlantilla() : ID( NULL ), Ventana( NULL ) // Valores nulos

	{}

	//
	// Mtodo : Constructor nulo
	//
	// Permite crear un objeto nulo
	//

inline GPlantilla::GPlantilla( int ) : Ventana( NULL ), ID( NULL )

	{}

	//
	// Mtodo : Constructor
	//
	// Crea un objeto a partir de una cola de plantillas, asocindolo
	// con un objeto GES
	//

inline GEntrada::GEntrada( GES * P, Cola<GPlantilla> & O )

	// Inicializar los miembros

		: Pantalla( P ), Paginas( O )

		{}

	//
	// Mtodo : Constructor
	//
	// Crea un objeto a partir de una sola plantilla, asocindolo
	// con un objeto GES
	//

inline GEntrada::GEntrada( GES * P, GPlantilla & O )

	// Inicializar los miembros

		: Pantalla( P )

		{ Paginas << O ; }

	//
	// Mtodo : Constructor por defecto
	//
	// Crea un objeto vaco
	//

inline GEntrada::GEntrada() :

		Pantalla( 0 ) // El objeto es nulo

		{}

	//
	// Mtodo : Constructor de copia
	//
	// Permite crear un objeto a partir de otro
	//

inline GEntrada::GEntrada( GEntrada & O ) :

	Pantalla( O . Pantalla ), Paginas( O . Paginas )

	{}

	//
	// Mtodo : Operador de asignacin
	//
	// Facilita la asignacin de un objeto a otro

inline void GEntrada::operator = ( GEntrada & O )

	{ Pantalla = O . Pantalla ; Paginas = O . Paginas ; Actual = O . Actual ; }

#endif // _GENTRADA_HPP_
