	//ͻ
	// GEvento.Cpp                            
	//                                        
	// Implementacin de los mtodos de las   
	// clases Evento y GEvento.               
	//                                        
	// 5 Agosto 1992                          
	//ͼ

#include <dos.h> // Para utilizar biostime()

#include <fstream.h> // Para utilizar streams en disco

#include "GEvento.Hpp" // Definicin de las clases

	// Se declaran las variables estticas

GTeclado GEvento::Teclado ;
GRaton GEvento::Raton ;
Cola<Evento> GEvento::Sucesos ;
Cola<Evento> * GEvento::pMacro ;
int GEvento::Registro = 0 ;

	//
	// Mtodos de la clase Evento
	//

	//
	// Mtodo : operador ==
	//
	// Su finalidad es permitir establecer una relacin
	// de igualdad entre dos eventos, que se consideran
	// igual si coinciden todos sus datos. Es til bsicamente
	// para estructurar los eventos en una cola.
	//

int Evento::operator == ( const Evento & X ) const
 {
	  // Devuelve 1 si todo coincide, 0 en caso contrario

	return Quien == X . Quien &&
			 Boton == X . Boton &&
			 Posicion . X == X . Posicion . X &&
			 Posicion . Y == X . Posicion . Y &&
			 Tecla == X . Tecla &&
			 AltI == X . AltI && AltD == X . AltD &&
			 CtrlI == X . CtrlI && CtrlD == X . CtrlD &&
			 ShiftI == X . ShiftI && ShiftD == X . ShiftD &&
			 Cuando == X . Cuando ;
 }

	//
	// Este constructor acta como conversor de int a Evento,
	// creando un evento vaco. Esto slo es necesario para
	// las estructuras de datos, que devuelven NULL en algunas
	// ocasiones, valor numrico que se convierte a un evento nulo.
	//

Evento::Evento( const int )
 {
	Quien = Vacio ;
	Boton = NClick ;
	Posicion . X = Posicion . Y = NULL ;
	Tecla = NULL ;
	AltI = AltD = CtrlI = CtrlD = ShiftI = ShiftD = NULL ;
	Cuando = NULL ;
 }

	//
	// Mtodos de la clase GEvento
	//

	//
	// Mtodo : Inserta
	//
	// Esta versin de Inserta permite aadir un evento producido
	// por el teclado, facilitando slo el cdigo de la tecla.
	//

void GEvento::Inserta( const unsigned Tecla )
 {
	Evento Nuevo ; // Se va a crear un nuevo evento

	Nuevo . Quien = Evento::Teclado ; // Lo ha producido el teclado
	Nuevo . Tecla = Tecla ; // El cdigo es Tecla

	Nuevo . Cuando = biostime( 0, 0L ) ; // Momento en que se produjo

	Nuevo . AltI = Teclado . AltIPulsado() ; // Estado de las teclas muertas
	Nuevo . AltD = Teclado . AltDPulsado() ;
	Nuevo . CtrlI = Teclado . CtrlIPulsado() ;
	Nuevo . CtrlD = Teclado . CtrlDPulsado() ;
	Nuevo . ShiftI = Teclado . ShiftIPulsado() ;
	Nuevo . ShiftD = Teclado . ShiftDPulsado() ;

	Sucesos << Nuevo ; // Se aade el evento a la cola

	if( Registro ) // Si se est registrando una macro

		*pMacro << Nuevo ; // insertar el evento en ella.
 }

	//
	// Mtodo : Inserta
	//
	// Esta versin del mtodo permite aadir un evento
	// producido por el ratn, facilitando el estado de
	// sus botones izquierdo y derecho y la posicin en
	// la que se encuentra el cursor.
	//

void GEvento::Inserta( const int BI, const int BD, const int X, const int Y )
 {
	Evento  Nuevo ; // Se crea un nuevo evento

	Nuevo . Quien = Evento::Raton ; // Lo ha producido el ratn

		  // Ver si se ha pulsado un botn o los dos

	Nuevo . Boton = BI && BD ? Evento::ClickID : BI ? Evento::ClickI : Evento::ClickD ;

	Nuevo . Posicion . X = X ; // Posicin del cursor en el
	Nuevo . Posicion . Y = Y ; // momento

	Nuevo . Cuando = biostime( 0, 0L ) ; // Momento en que se registra el evento

	Sucesos << Nuevo ; // Se aade el evento a la lista

	if( Registro ) // Si se est registrando una macro

		*pMacro << Nuevo ; // insertar el evento en ella.

 }

	//
	// Mtodo : operador >>
	//
	// Este operador permite solicitar un evento al gestor,
	// que explorar teclado, ratn y finalmente extraer
	// un evento de la cola y lo devolver.
	//                                    zz

GEvento & GEvento::operator >> ( Evento & X )
 {
	int Tecla ;

		// Si hay alguna tecla pulsada

	if( ( Tecla = Teclado . Explora() ) != GTeclado::NADA )

		Inserta( Tecla ) ; // Registra el evento

		// Si est pulsado algn botn del ratn

	int BI = Raton . EstadoBoton( GRaton::Izquierdo ),
		 BD = Raton . EstadoBoton( GRaton::Derecho ) ;

	if( BI || BD )

		Inserta( BI, BD, Raton . PosicionHorizontal(), Raton . PosicionVertical() ) ;

	Sucesos >> X ; // Extrae un evento de la cola y lo deja en X

	if( ! Sucesos ) // Si no se pudo extraer porque no haba
	 {
		Sucesos . Vale() ; // Borrar el indicador de error

		X = (Evento ) NULL ; // El evento es nulo
	 }

	return * this ; // Devolver una referencia al gestor
 }

	//
	// Mtodo : FinMacro
	//
	// Finalida el registro de la macro y la almacena en un archivo
	// cuyo nombre se pasa como parmetro. Devuelve 0 si se falla
	// en la grabacin o 1 en caso contrario.
	//

int GEvento::FinMacro( char * Archivo )
 {
	if( ! Registro ) // Si no se est registrando una macro

		return 0 ; // no se puede grabar

	ofstream Salida( Archivo, ios::binary ) ; // Abrir el archivo

	if( ! Salida ) // Si no se puede

		return 0 ; // devolver 0

	Salida << *pMacro ; // Grabar la macro en el archivo

	delete pMacro ; // Borrar la cola

	Registro = 0 ; // Desactivar el registro de la macro

	return 1 ; // No hubo problemas
 }

	//
	// Mtodo : Macro
	//
	// Reproduce los eventos registrados en una macro. Hay
	// que pasar el nombre del archivo en que se registro la
	// macro.
	//

int GEvento::Macro( char * Archivo )
 {
	ifstream Entrada( Archivo, ios::binary ) ; // Abrir el archivo para lectura

	if( ! Entrada ) // Si no se puede

		return 0 ; // devolver 0

	Entrada >> Sucesos ; // Aadir la macro a la lista de sucesos

	return 1 ; // No hubo problemas
 }

	//
	// Mtodo : operador <<
	//
	// Facilita la insercin de un evento en un stream
	//

ostream & operator << ( ostream & Stream, Evento & X )
 {
	Stream . write( (char *) & X, sizeof( X ) ) ; // Inserta los datos que definen el evento

	return Stream ; // Devolver una referencia al stream
 }

	//
	// Mtodo : operador >>
	//
	// Facilita la extraccin de un evento desde un stream
	//

istream & operator >> ( istream & Stream, Evento & X )
 {
	Stream . read( (char *) & X, sizeof( X ) ) ; // Recupera los datos

	return Stream ; // y devuelve una referencia a un stream
 }