	//ͻ
	//   GEvento.Hpp                               
	//                                             
	// Implementacin de las clases necesarias     
	// para gestionar eventos.                     
	//                                             
	// 5 Agosto 1992                               
	//ͼ

#ifndef _GEVENTO_HPP_

#define _GEVENTO_HPP_

#ifndef __cplusplus
#error No se puede utilizar GEvento en C
#endif

#include <iostream.h>

#include "EstDat.Hpp" // Estructuras de datos

#include "GRaton.Hpp" // Los eventos pueden venir producidos por el ratn
#include "GTeclado.Hpp" // o por el teclado.

	//
	// Clase : Evento
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Contener la informacin concerniente
	//	       a un evento, y facilitar los mtodos
	//	       necesarios para poder disponerlos en
	//	       una cola.
	//
	// Fecha de creacin : 5 Agosto 1992
	//
	// Fecha de revisin : -
	//

class Evento
 {
	public :

		  // Los eventos pueden venir producidos por el ratn
		  // el teclado o estar vacos.

		enum { Vacio, Raton, Teclado } Quien ;

		  // El ratn puede haber registrado una pulsacin en
		  // su botn izquierdo, derecho o en ambos.

		enum { NClick, ClickI, ClickD, ClickID } Boton ;

		  // Para almacenar la posicin del cursor del ratn.

		struct { int X, Y ; } Posicion ;

		unsigned Tecla ; // Tecla que se ha pulsado

		  // Indicadores de estado teclas muertas

		unsigned short	AltI, AltD,
				CtrlI, CtrlD,
				ShiftI, ShiftD;

		long Cuando ; // Momento en que se produjo el evento

		Evento( const Evento & ) ; // Constructor de copia y asignacin

		Evento() ; // Construir un evento vaco

		Evento( const int ) ; // Para construir un elemento NULL

	  int operator== ( const Evento & ) const ; // Relacin de igualdad

		// Operadores para poder escribir y leer un evento en un stream

	  friend ostream & operator << ( ostream &, Evento & ) ;
	  friend istream & operator >> ( istream &, Evento & ) ;

 } ; // Evento

	//
	// Clase : GEvento
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Proveer los mtodos necesarios
	//	       para gestionar los eventos.
	//
	// Fecha de creacin : 5 Agosto 1992
	//
	// Fecha de revisin : -
	//

class GEvento
 {
	public :

	  void Inserta( const unsigned Tecla ) ; // Inserta un evento de teclado

		// Inserta un evento de ratn
	  void Inserta( const int BI, const int BD, const int X, const int Y ) ;

	  GEvento & operator >> ( Evento & ) ; // Extraer un evento

	  void InicioMacro() ; // Inicia el registro de una macro

	  int FinMacro( char * ) ; // Finaliza el registro de la macro y lo almacena

	  int Macro( char * ) ; // Reproduce una macro

	private :

		static GTeclado Teclado ; // Para acceder al teclado

		static GRaton Raton ; // Para acceder al ratn

		static Cola<Evento> Sucesos ; // Cola de eventos producidos

		static Cola<Evento> * pMacro ; // Cola adicional para registrar macros

		static int Registro ;

 } ;  // GEvento

	//
	// Mtodos inline de la clase Evento
	//

	//
	// Este constructor permite crear un objeto a partir de otro,
	// simplemente copiando cada uno de los elementos.
	//

inline Evento::Evento( const Evento & X ) :
					 Quien( X . Quien ), Boton( X . Boton ),
					 Posicion( X . Posicion ), Tecla( X . Tecla ),
					 AltI( X . AltI ), AltD( X . AltD ),
					 CtrlI( X . CtrlI ), CtrlD( X . CtrlD ),
					 ShiftI( X . ShiftI ), ShiftD( X . ShiftD ),
					 Cuando( X . Cuando )
	{}

	//
	// El constructor por defecto crea un evento vaco
	//

inline Evento::Evento() : Quien( Vacio )

	{}

	//
	// Simplemente activa el indicador de registro de macro y
	// asigna memoria para la cola de eventos.
	//

inline void GEvento::InicioMacro()

	{ if( ! Registro ) { Registro = 1 ; pMacro = new Cola<Evento> ; } }

#endif // _GEVENTO_HPP_