	//
	// GMenu.Cpp
	//
	// Implementacin de los mtodos de las clases GMenu
	//
	// 26 Agosto 1992
	//

#include <ctype.h> // Para utilizar toupper()

	// Si el modelo de memoria seleccionado permite varios segmentos de datos
	// habr que utilizar las versiones far de algunas funciones

#if defined(__LARGE__) || defined(__HUGE__) || defined(__COMPACT__)

	#define istrcpy( orig, dest )   _fstrcpy( (orig), (dest) )
	#define istrcmp( orig, dest )   _fstrcmp( (orig), (dest) )
	#define istrcat( orig, dest )   _fstrcat( (orig), (dest) )
	#define istrchr( cade, cara )   _fstrchr( (cade), (cara) )
	#define istrlen( cade )                 _fstrlen( (cade) )
	#define imemcmp( orig, dest, long ) _fmemcmp( (orig), (dest), (long) )

#else

	#define istrcpy( orig, dest )   strcpy( (orig), (dest) )
	#define istrcmp( orig, dest )   strcmp( (orig), (dest) )
	#define istrcat( orig, dest )   strcat( (orig), (dest) )
	#define istrchr( cade, cara )   strchr( (cade), (cara) )
	#define istrlen( cade )                 strlen( (cade) )
	#define imemcmp( orig, dest, long ) memcmp( (orig), (dest), (long) )

#endif

#include "GMenu.Hpp"
#include "GEvento.Hpp"

	//
	// Mtodos de la clase GOpcion
	//

GRaton GMenu::Raton ; // Instancia de miembro esttico

	//
	// Mtodo : Constructor de copia
	//
	// Permite crear un objeto tomando otro como origen
	//

GOpcion::GOpcion(  GOpcion & O ) :

				  // Inicializa los miembros tomando como fuente el objeto

				  Nodo( O . Nodo ), Codigo( O . Codigo ),
				  Linea( O . Linea )
 {
	  // Se asigna espacio para el ttulo

	Titulo = new char[ istrlen( O . Titulo ) + 1 ] ;

	istrcpy( Titulo, O . Titulo ) ; // Y se copia
 }

	//
	// Mtodo : Constructor estndar 1
	//
	// Crea una opcin asocindola con un submen
	//

GOpcion::GOpcion( char * T, GNodo & N ) :

				  Nodo( & N ), Codigo( 0 ) // No hay funcin
 {
	Titulo = new char[ istrlen( T ) + 1 ] ; // Se asigna memoria para el ttulo

	istrcpy( Titulo, T ) ; // Se copia
 }

	//
	// Mtodo : Constructor estndar 2
	//
	// Crea una opcin asocindola con un mensaje
	//

GOpcion::GOpcion( char * T, unsigned C ) :

	   Nodo( 0 ), Codigo( C ) // Se guarda la direccin
 {
	Titulo = new char[ istrlen( T ) + 1 ] ;

	istrcpy( Titulo, T ) ;
 }

	//
	// Mtodo : Destructor
	//
	// Libera la asignacin de memoria realizada en el ructor
	//

GOpcion::~GOpcion()
 {
	if( Titulo != 0 )  // Si se realiz asignacin de memoria

		delete [] Titulo ; // liberarla
 }

	//
	// Mtodo : Operador de asignacin
	//
	// Permite asignar unas opciones a otras
	//

void GOpcion::operator = (  GOpcion & O )
 {
	if( Titulo != 0 ) // Si el objeto implcito ya tena un ttulo

		delete [] Titulo ; // borrarlo

	Titulo = new char[ istrlen( O . Titulo ) + 1 ] ; // Asignar memoria

	istrcpy( Titulo, O . Titulo ) ; // Copiar el ttulo

	Nodo = O . Nodo ;   // Y los dems contenidos
	Linea = O . Linea ;
	Codigo = O . Codigo ;
 }

	//
	// Mtodo : Operador de relacin de igualdad
	//
	// Permite saber si dos opciones son iguales
	//

int GOpcion::operator == (  GOpcion & O )
 {
	return ! istrcmp( Titulo, O . Titulo ) && Linea == O . Linea &&
		   ( Codigo == 0 ? *Nodo == *O . Nodo : Codigo == O . Codigo ) ;
 }

	//
	// Mtodo : Operador de insercin
	//
	// Permite almacenar una opcin en un stream
	//

ostream & operator << ( ostream & Stream, GOpcion & O )
 {
	int N = istrlen( O . Titulo ) + 1 ; // Obtener la longitud del ttulo

	Stream . write( (char *) & N, sizeof( int ) ) ;
	Stream . write( O . Titulo, N ) ;
	Stream . write( (char *) & O . Linea, sizeof( int ) ) ;
	Stream . write( (char *) & O . Codigo, sizeof( unsigned ) ) ;

		  // Si hay un nodo asociado insertarlo tambin

	return O . Codigo == 0 ? Stream << *(O . Nodo) : Stream ;
 }

	//
	// Mtodo : Operador de extraccin
	//
	// Permite extraer una opcin de un stream
	//

istream & operator >> ( istream & Stream, GOpcion & O )
 {
	int N ;

	Stream . read( (char *) & N, sizeof( int ) ) ;
	Stream . read( O . Titulo, N ) ;
	Stream . read( (char *) & O . Linea, sizeof( int ) ) ;
	Stream . read( (char *) & O . Codigo, sizeof( unsigned ) ) ;

	return O . Codigo == 0 ? Stream >> *(O . Nodo) : Stream ;
 }

	//
	// Mtodo : Operador de concatenacin
	//
	// Permite crear una cola de opciones tomando
	// como partida dos objetos
	//

Cola<GOpcion> operator + ( GOpcion & O1, GOpcion & O2 )
 {
	Cola<GOpcion> Temporal ; // Nueva cola de opciones

	Temporal << O1 << O2 ; // Se insertan los dos objetos

	return Temporal ;
 }

	//
	// Mtodo : Operador de concatenacin
	//
	// Permite aadir una opcin a una lista de opciones
	//

Cola<GOpcion> operator + ( Cola<GOpcion> & Cola, GOpcion & O )
 {
	Cola << O ;

	return Cola ;
 }

	//
	// Mtodos de la clase GNodo
	//

	//
	// Mtodo : Constructor de copia
	//
	// Permite crear un objeto tomando otro como origen
	//

GNodo::GNodo(  GNodo & O ) :

					 // Se copian los miembros

							Opciones( O . Opciones ),
							Actual( O . Actual ), Columna( O . Columna ),
							Codigo( O . Codigo )
 {
	Titulo = new char[ istrlen( O . Titulo ) + 1 ] ;

	istrcpy( Titulo, O . Titulo ) ;
 }

	//
	// Mtodo : Constructor estndar 1
	//
	// Crea un submen, asignndole un nombre, y lo asocia
	// a una lista de opciones
	//

GNodo::GNodo( char * T, Cola<GOpcion> & Cola ) :

			  Opciones( Cola ), Codigo( 0 )
 {
	Titulo = new char[ istrlen( T ) + 1 ] ;

	istrcpy( Titulo, T ) ;
 }

	//
	// Mtodo : Constructor estndar 2
	//
	// Crea un submen, asignndole un nombre, y lo asocia
	// con un mensaje
	//

GNodo::GNodo( char * T, unsigned C ) :

			  Codigo( C )
 {
	Titulo = new char[ istrlen( T ) + 1 ] ;

	istrcpy( Titulo, T ) ;
 }

	//
	// Mtodo : Destructor
	//
	// Libera la memoria asignada en el ructor
	//

GNodo::~GNodo()
 {
	if( Titulo != 0 )

		delete [] Titulo ;
 }

	//
	// Mtodo : Operador de asignacin
	//
	// Permite asociar un submen a otro
	//

void GNodo::operator = (  GNodo & O )
 {
	if( Titulo != 0 ) // Si ya existe un ttulo

		delete [] Titulo ; // liberar la memoria que tiene asignada

	  // Asignaer memoria

	Titulo = new char[ istrlen( O . Titulo ) + 1 ] ;

	// y copiar el nuevo ttulo

	istrcpy( Titulo, O . Titulo ) ;

	Opciones = O . Opciones ; // Copiar los dems miembros
	Actual = O . Actual ;
	Columna = O . Columna ;
	Codigo = O . Codigo ;
 }

	//
	// Mtodo : Operador de relacin de igualdad
	//
	// Permite saber si dos submens son iguales
	//

int GNodo::operator == (  GNodo & O )
 {
	return ! istrcmp( Titulo, O . Titulo ) && Columna == O . Columna &&
			( Codigo == 0 ? Opciones == O . Opciones : Codigo == O . Codigo ) ;
 }

	//
	// Mtodo : Operador de insercin
	//
	// Facilita el escribir un submen en un stream
	//

ostream & operator << ( ostream & Stream, GNodo & O )
 {
	int N = istrlen( O . Titulo ) + 1 ;

	Stream . write( (char *) & N, sizeof( int ) ) ;
	Stream . write( O . Titulo, N ) ;
	Stream . write( (char *) & O . Columna, sizeof( int ) ) ;
	Stream . write( (char *) & O . Codigo, sizeof( unsigned ) ) ;

	return O . Codigo == 0 ? Stream << O . Opciones : Stream ;
 }

	//
	// Mtodo : Operador de extraccin
	//
	// Facilita el leer un submen desde un stream
	//

istream & operator >> ( istream & Stream, GNodo & O )
 {
	int N ;

	Stream . read( (char * ) & N, sizeof( int ) ) ;
	Stream . read( O . Titulo, N ) ;
	Stream . read( (char *) & O . Columna, sizeof( int ) ) ;
	Stream . read( (char *) & O . Codigo, sizeof( unsigned ) ) ;

	return O . Codigo == 0 ? Stream >> O . Opciones : Stream ;
 }

	//
	// Mtodo : Operador de concatenacin
	//
	// Crea una cola de submens tomando como partida
	// dos submens
	//

Cola<GNodo> operator + ( GNodo & O1, GNodo & O2 )
 {
	Cola<GNodo> Temporal ;

	Temporal << O1 << O2 ;

	return Temporal ;
 }

	//
	// Mtodo : Operador de concatenacin
	//
	// Aade un submen a una lista de submens
	//

Cola<GNodo> operator + ( Cola<GNodo> & Cola, GNodo & O )
 {
	Cola << O ;

	return Cola ;
 }

	//
	// Mtodo : Constructor
	//
	// Crea el men
	//

GMenu::GMenu(   GES * Destino,
		Cola<GNodo> & Cola,
		GVenBuf::Color TN, GVenBuf::Color FN,
		GVenBuf::Color TS, GVenBuf::Color FS,
		GVenBuf::Color TIN, GVenBuf::Color FIN,
		GVenBuf::Color TIS, GVenBuf::Color FIS,
		 RaizMenu * P ) :
Pantalla( Destino ), Nodos( Cola ),
TintaNormal( TN ), FondoNormal( FN ),
TintaSeleccionado( TS ), FondoSeleccionado( FS ),
TintaInicialNormal( TIN ), FondoInicialNormal( FIN ),
TintaInicialSeleccionada( TIS ), FondoInicialSeleccionada( FIS ),
Procesos( P )
{

	CreaBarraMenu() ; // Se crea la barra de men

	// Ahora hay que crear las ventanas de cada submen
	GNodo * Nodo = Nodos . pPrimero() ; // Se obtiene el primer nodo

	while( Nodo != NULL ) // Mientras no hayamos llegado al ltimo nodo
	{
		// Crear el submen correspondiente, Nodo, en la columna
		// establecida al crear la barra de men, y en la lnea 3
		CreaSubMenus( Nodo, Nodo->Columna, 3 ) ;

		Nodo = Nodos . pSiguiente( * Nodo ) ; // y pasar al siguiente
	}

	// Hecho esto habr que mostrar la barra de men, y obtener eventos
	// para actuar en consecuencia, mostrando y ocultando ventanas, etc.
	Pantalla -> Encima( "Barra menu" ) ;
	Pantalla -> Muestra( "Barra menu" ) ;

	GNodo * Actualp = Nodos . pPrimero() ; // Se obtiene el primer nodo del men

	// Se obtiene un puntero a la ventana correspondiente
	// a la barra del men principal
	GVentana * Ventanap = (*Pantalla)( "Barra menu" ), * VentanaS ;

	// Nos situamos en la opcin actual, la primera
	*Ventanap << PosCursor( Actualp -> Columna, 1 ) ;

	// Y la destacamos del resto
	// Imprime( *Ventanap, Actualp -> Titulo, Destacado ) ;

}

	//
	// Mtodo : Destructor
	//
	// Libera los recursos de la clase GMenu
	//

GMenu::~GMenu()
{

	// Hay que eliminar todas las ventanas que se han creado
	Pantalla -> Elimina( "Barra menu" ) ; // La barra de men

	// Y las ventanas de los submens
	GNodo* Nodo = Nodos . pPrimero() ; // Se obtiene el primer nodo

	while( Nodo != NULL ) // Mientras no hayamos llegado al ltimo nodo
	{
		LiberaSubMenus( Nodo ) ; // Borrar ese nodo y los contenidos

		Nodo = Nodos . pSiguiente( * Nodo ) ; // y pasar al siguiente
	}
	Pantalla -> Refresca() ; // Refrescar para que desaparezca el men
}

	//
	// Mtodos de la clase GMenu
	//

	//
	// Mtodo : Ejecuta
	//
	// Procesa el men
	//

int GMenu::Ejecuta()
{
	int Salida = 0 ; // Indicador de salida
	int Nivel = 0 ; // Nivel actual en el men

	GNodo * Actualp = Nodos . pPrimero() ; // Se obtiene el primer nodo del men

	// Se obtiene un puntero a la ventana correspondiente
	// a la barra del men principal
	GVentana * Ventanap = (*Pantalla)( "Barra menu" ), * VentanaS ;

	int Cod = 0 ; // Valor de retorno, que a la salida indicar
	// si se puls ESC o fue una funcin la que provoc
	// la parada

	while( ! Salida ) // Mientras no se active el indicador de salida
	{
		GNodo * Anteriores[16] ; // Matriz de nodos recorridos

		Pantalla -> Refresca() ; // Refresca el aspecto de la pantalla

		GEvento Control ; // Para obtener eventos
		Evento Suceso ; // Para contener un evento

		do {
			Control >> Suceso ; // Leer eventos

		} while( Suceso . Quien == Evento::Vacio ) ; // hasta encontrar uno no vaco

		switch( Suceso . Quien ) // Veamos quien ha producido el evento
		{
			case Evento::Teclado : // Ha sido el teclado

				switch( Suceso . Tecla ) // Veamos que tecla se ha pulsado
				{
					case GTeclado::INTRO : // Intro o barra espaciadora
					case ' ' :

						if( ! Nivel ) // Si estamos en la barra de men
						{
							if( Actualp -> Codigo == 0 ) // Si no hay mensaje
							{
								Nivel ++ ; // Ir al siguiente nivel

								// Mostrar la ventana correspondiente
								Pantalla -> Encima( Actualp -> Titulo ) ;
								Pantalla -> Muestra( Actualp -> Titulo ) ;

								// Se obtiene la ventana del submen
								VentanaS = (*Pantalla)( Actualp-> Titulo ) ;

								// Destacar la opcin actual en el submen
								*VentanaS << PosCursor( 1, Actualp -> Actual -> Linea ) ;
								Imprime( *VentanaS, Actualp -> Actual -> Titulo, Destacado ) ;
							}
							else // Si hay un mensaje
							{
								// Llamar preservando el valor de retorno
								Cod = Procesos -> Procesa( Actualp -> Codigo ) ;

								if( Cod )
								{
									Salida = 1 ; // Salir
								}
								else
								{
									for( int i = Nivel+1; i > 0; i-- )
									{
										Control.Inserta( GTeclado::ESC );
									}
								}
							}
						}
						else // Si estamos en un submen
						{
							// Si la opcin actual tiene asociada un mensaje
							if( Actualp -> Actual -> Codigo != 0 )
							{
								Cod = Procesos -> Procesa( Actualp -> Actual -> Codigo ) ;
	
								if( Cod )
								{
									Salida = 1 ;
								}
								else
								{
									for( int i = Nivel+1; i > 0; i-- )
									{
										Control.Inserta( GTeclado::ESC );
									}
								}
							}
							else // En caso contrario es que tiene asociado otro submen
							{
								Nivel++ ; // Se baja un nivel ms
	
								// Se preserva en la matriz el nodo anterior
								Anteriores[Nivel-2] = Actualp ;

								Actualp = Actualp -> Actual -> Nodo ;

								// Mostrar la ventana correspondiente
								Pantalla -> Encima( Actualp -> Titulo ) ;
								Pantalla -> Muestra( Actualp -> Titulo ) ;

								// Se obtiene la ventana del submen
								VentanaS = (*Pantalla)( Actualp-> Titulo ) ;

								// Destacar la opcin actual en el submen
								*VentanaS << PosCursor( 1, Actualp -> Actual -> Linea ) ;
								Imprime( *VentanaS, Actualp -> Actual -> Titulo, Destacado ) ;
							}
						}

					break ;

					case GTeclado::IZQUIERDA :

						// Slo es posible desplazarse a la izquierda
						// si el nivel es 0  1

						if( Nivel < 2 )
						{
							// Se elimina el atributo destacado de la opcin
							*Ventanap << PosCursor( Actualp -> Columna, 1 ) ;
							Imprime( *Ventanap, Actualp -> Titulo, Normal ) ;

							if( Nivel == 1 ) // Si hay un submen abierto
							{
								// se oculta
								Pantalla -> Oculta( VentanaS ) ;
							}
							// Ir al submen anterior
							Actualp = Nodos . pAnterior( * Actualp ) ;

							if( Actualp == NULL ) // Si no haba anterior
							{
								Actualp = Nodos . pUltimo() ; // Cogemos el ltimo
							}
							// Destacar la nueva opcion seleccionada
							*Ventanap << PosCursor( Actualp -> Columna, 1 ) ;
							Imprime( *Ventanap, Actualp -> Titulo, Destacado ) ;

							if( Nivel == 1 ) // Si hay un submen abierto
							{
								// Si hay una funcin asociada
								if( Actualp -> Codigo != 0 ) {
								Nivel = 0 ; // Se vuelve al nivel de la barra
							}
							else
							{
								// Se obtiene la ventana correspondiente
								VentanaS = (*Pantalla)( Actualp-> Titulo ) ;
	
								// se muestra
								Pantalla -> Encima( VentanaS ) ;
								Pantalla -> Muestra( VentanaS ) ;

								// y se destaca la opcin seleccionada en l
								*VentanaS << PosCursor( 1, Actualp -> Actual -> Linea ) ;
								Imprime( *VentanaS, Actualp -> Actual -> Titulo, Destacado ) ;
							}
						}
					}

				break ;

				case GTeclado::DERECHA :

					// Slo es posible desplazarse a la derecha
					// si el nivel es 0  1

					if( Nivel < 2 )
					{
						// Se elimina el atributo destacado de la opcin
						*Ventanap << PosCursor( Actualp -> Columna, 1 ) ;
						Imprime( *Ventanap, Actualp -> Titulo, Normal ) ;

						if( Nivel == 1 ) // Si hay un submen abierto
						{
							// se oculta
							Pantalla -> Oculta( VentanaS ) ;
						}
						// Ir al submen siguiente
						Actualp = Nodos . pSiguiente( * Actualp ) ;

						if( Actualp == NULL ) // Si no haba siguiente
						{
							Actualp = Nodos . pPrimero() ; // Cogemos el primero
						}
						// Destacar la nueva opcin seleccionada
						*Ventanap << PosCursor( Actualp -> Columna, 1 ) ;
						Imprime( *Ventanap, Actualp -> Titulo, Destacado ) ;

						if( Nivel == 1 ) // Si hay un submen abierto
						{
							// Si hay una funcin asociada
							if( Actualp -> Codigo != 0 )
							{
								Nivel = 0 ; // Volver al nivel de la barra
							}
							else
							{
								// Se obtiene la ventana correspondiente
								VentanaS = (*Pantalla)( Actualp-> Titulo ) ;

								// se muestra
								Pantalla -> Encima( VentanaS ) ;
								Pantalla -> Muestra( VentanaS ) ;

								// y se destaca la opcin seleccionada en l
								*VentanaS << PosCursor( 1, Actualp -> Actual -> Linea ) ;
								Imprime( *VentanaS, Actualp -> Actual -> Titulo, Destacado ) ;
							}
						}

					}

				break ;

				case GTeclado::ARRIBA :

					if( Nivel ) // Slo es posible si hay un submen abierto
					{
						// Quitar el resaltado de la opcin actual
						*VentanaS << PosCursor( 1, Actualp -> Actual -> Linea ) ;
						Imprime( *VentanaS, Actualp -> Actual -> Titulo, Normal ) ;

						// Ahora la opcin actual ser la anterior
						Actualp -> Actual = Actualp -> Opciones .
							pAnterior( * Actualp -> Actual ) ;

						if( Actualp -> Actual == NULL ) // Si no haba anterior
						{
							// La opcin actual ser la ltima
							Actualp -> Actual = Actualp -> Opciones . pUltimo();
						}
						// Resaltar la opcin actual
						*VentanaS << PosCursor( 1, Actualp -> Actual -> Linea ) ;
						Imprime( *VentanaS, Actualp -> Actual -> Titulo, Destacado ) ;
					}

				break ;

				case GTeclado::ABAJO :

					if( Nivel )
					{
						// Quitar el resaltado de la opcin actual
						*VentanaS << PosCursor( 1, Actualp -> Actual -> Linea ) ;
						Imprime( *VentanaS, Actualp -> Actual -> Titulo, Normal ) ;

						// Ahora la opcin actual ser la siguiente
						Actualp -> Actual = Actualp -> Opciones .
						pSiguiente( * Actualp -> Actual ) ;

						if( Actualp -> Actual == NULL ) // Si no haba siguiente
						{
							// La opcin actual ser la primera
							Actualp -> Actual = Actualp -> Opciones . pPrimero() ;
						}
						// Resaltar la opcin actual
						*VentanaS << PosCursor( 1, Actualp -> Actual -> Linea ) ;
						Imprime( *VentanaS, Actualp -> Actual -> Titulo, Destacado ) ;
					}

				break ;

				case GTeclado::ESC :

					if( ! Nivel ) // Si estamos en la barra
					{
						Salida = 1 ; // Activar indicador de salida
					}
					else
					{
						// Se oculta la ventana del submen
						Pantalla -> Oculta( VentanaS ) ;

						if( Nivel > 1 ) // Si estbamos en un submen
						{
							Actualp = Anteriores[Nivel-2] ; // Recuperar el anterior
						}
						// Se obtiene la ventana del nuevo submen activo
						VentanaS = (*Pantalla)( Actualp-> Titulo ) ;

						Nivel-- ; // Ir al nivel superior
					}

				break ;

				default : // Si es cualquier otra tecla

					if( ! Nivel ) // Si estamos en la barra de men
					{
						// Hay que buscar si existe en los nodos
						// una inicial que coincida con la tecla
						// pulsada

						// Se toma el primer nodo
						GNodo * Temp = Nodos . pPrimero() ;

						while( Temp != NULL ) // Mientras haya nodos
						{
							// Si la inicial de esta opcin
							// coincide con la tecla pulsada

							if( toupper( * (istrchr( Temp -> Titulo, '*' ) + 1) )
								== toupper( Suceso . Tecla ) )
							{
								break ; // Salir del bucle
							}
							// Si no continuar buscando
							Temp = Nodos . pSiguiente( * Temp ) ;
						}
						if( Temp != NULL ) // Si se encontr un nodo
						{
							// Quitar el resalte de la opcin anterior
							*Ventanap << PosCursor( Actualp -> Columna, 1 ) ;
							Imprime( *Ventanap, Actualp -> Titulo, Normal ) ;

							// La nueva opcin es Temp
							Actualp = Temp ;

							// Resaltar la nueva opcin seleccionada
							*Ventanap << PosCursor( Actualp -> Columna, 1 ) ;
							Imprime( *Ventanap, Actualp -> Titulo, Destacado ) ;

							// Simula la pulsacin de la tecla Intro
							Control . Inserta( GTeclado::INTRO ) ;
						}

					}
					else
					{
						// Hay que buscar si existe en las opciones
						// del submen actual una inicial que
						// coincida con la tecla pulsada

						// Se toma la primera de las opciones
						GOpcion * Temp = Actualp -> Opciones . pPrimero() ;

						while( Temp != NULL ) // Mientras haya opciones
						{
							// Si la inicial de esta opcin coincide
							if( toupper( * (istrchr( Temp -> Titulo, '*' ) + 1) )
								== toupper( Suceso . Tecla ) )
							{
								break ; // Salir del bucle
							}
							// Si no continuar buscando
							Temp = Actualp -> Opciones . pSiguiente( * Temp ) ;
						}

						if( Temp != NULL ) // Si se encontr un nodo
						{
							// Quitar el resalte de la opcin anterior
							*VentanaS << PosCursor( 1, Actualp -> Actual -> Linea ) ;
							Imprime( *VentanaS, Actualp -> Actual -> Titulo, Normal ) ;

							// La nueva opcin es Temp
							Actualp -> Actual = Temp ;

							// Resaltar la nueva opcin seleccionada
							*VentanaS << PosCursor( 1, Temp -> Linea ) ;
							Imprime( *VentanaS, Temp -> Titulo, Destacado ) ;

							// Simula la pulsacin de la tecla Intro
							Control . Inserta( GTeclado::INTRO ) ;
						}
					}

				break ;

		} // switch Tecla

		break ;

			case Evento::Raton : // Ha sido el ratn

				switch( Suceso . Boton ) // Veamos que botn se ha pulsado
				{
					case Evento::ClickI : // Botn izquierdo, seleccionar
					case Evento::ClickD : // Botn derecho, cerrar
					
						// Hay que averiguar que ventana haba
						// bajo el cursor del ratn

						// Convertir coordenadas de cursor a coordenadas
						// de pantalla
						int Col = Suceso . Posicion . X / 8 + 1 ;
						int Lin = Suceso . Posicion . Y / 8 + 1 ;

						// Se obtiene un puntero a la ventana en la
						// que se encuentre esa posicin
						GVentana * Temp = (*Pantalla)
						( Pantalla -> Ventana( Col, Lin ) ) ;

						if( Temp != NULL ) // Haba una ventana debajo
						{
							if( Temp == Ventanap ) // Si es la barra del men
							{
								// Hay que ver qu opcin existe en la posicin
								// en que se puls el botn
	
								// Comenzar por el primer nodo
								GNodo * Temp = Nodos . pPrimero() ;

								while( Temp != NULL ) // Mientras queden nodos
								{
									 // Si se puls en esta opcin
									if( Col >= Temp -> Columna + 1 &&
										Col <= Temp -> Columna + Longitud( Temp -> Titulo ) + 2 )
									{
										break ; // interrumpir el bucle
									}
									Temp = Nodos . pSiguiente( * Temp ) ;
								}

								if( Temp != NULL ) // Si se encontr el nodo
								{
									// Cerrar posibles submens abiertos

									while( Nivel ) // Mientras haya submens abiertos
									{
										// Oculta la ventana
										Pantalla -> Oculta( VentanaS ) ;

										if( Nivel > 1 ) // Si estbamos en un submen
										{
											Actualp = Anteriores[Nivel-2] ; // Recuperar el anterior
										}
										// Se obtiene la ventana del nuevo submen activo
										VentanaS = (*Pantalla)( Actualp-> Titulo ) ;

										Nivel-- ; // Ir al nivel superior
									}
									// Quitar el resalte de la opcin anterior
									*Ventanap << PosCursor( Actualp -> Columna, 1 ) ;
									Imprime( *Ventanap, Actualp -> Titulo, Normal ) ;

									// La nueva opcin es Temp
									Actualp = Temp ;

									// Resaltar la nueva opcin seleccionada
									*Ventanap << PosCursor( Actualp -> Columna, 1 ) ;
									Imprime( *Ventanap, Actualp -> Titulo, Destacado ) ;

									// Simula la pulsacin de la tecla Intro
									Control . Inserta( GTeclado::INTRO ) ;
								}
							}
							else if( Temp == VentanaS ) // Si es el submen
							{
								// Hay que averiguar qu opcin hay en
								// la posicin donde se puls el botn

								// Se toma la primera de las opciones
								GOpcion * Temp = Actualp -> Opciones . pPrimero() ;

								while( Temp != NULL ) // Mientras haya opciones
								{
									// Si fue en esta opcin
									if( Lin == Temp -> Linea + VentanaS -> rdbuf() -> Lin() )
									{
										break ; // Salir del bucle
									}
									// Si no continuar buscando
									Temp = Actualp -> Opciones . pSiguiente( * Temp ) ;
								}
								if( Temp != NULL ) // Si se encontr un nodo
								{
									// Quitar el resalte de la opcin anterior
									*VentanaS << PosCursor( 1, Actualp -> Actual -> Linea ) ;
									Imprime( *VentanaS, Actualp -> Actual -> Titulo, Normal ) ;

									// La nueva opcin es Temp
									Actualp -> Actual = Temp ;

									// Resaltar la nueva opcin seleccionada
									*VentanaS << PosCursor( 1, Temp -> Linea ) ;
									Imprime( *VentanaS, Temp -> Titulo, Destacado ) ;

									// Simula la pulsacin de la tecla Intro
									Control . Inserta( GTeclado::INTRO ) ;
								}
							} // Si no es que era otra ventana cualquiera
							else {
									for( int i = Nivel+1; i > 0; i-- )
									{
										Control.Inserta( GTeclado::ESC );
									}
							}
						}

					break ;
/*
					case Evento::ClickD : // Botn derecho, cerrar

						// Simula la pulsacin de la tecla ESC
						Control . Inserta( GTeclado::ESC ) ;

					break ;
*/
				} // switch Boton

			break ;

		 } // switch Quien

	 } // while

	// Quitar el resalte de la opcin anterior
	*Ventanap << PosCursor( Actualp -> Columna, 1 ) ;
	Imprime( *Ventanap, Actualp -> Titulo, Normal ) ;

	Pantalla -> Refresca() ; // Refresca el aspecto de la pantalla

	return Cod ; // Devolver el valor de retorno

}  // Ejecuta

	//
	// Mtodo : EstaEnBarraMenu
	//
	// Indica si una tecla esta en la barra del menu.
	//

int GMenu::EstaEnBarraMenu( unsigned int Tecla )
{
	// Hay que buscar si existe en los nodos
	// una inicial que coincida con la tecla
	// pulsada

	if( isascii( Tecla ) == 0 ) {
		return 0;
	}
	// Se toma el primer nodo
	GNodo * Temp = Nodos . pPrimero() ;

	while( Temp != NULL ) // Mientras haya nodos
	{
		// Si la inicial de esta opcin
		// coincide con la tecla pulsada

		if( toupper( * (istrchr( Temp -> Titulo, '*' ) + 1) )
			== toupper( Tecla ) )
		{
			return 1 ; // Salir del bucle
		}
		// Si no continuar buscando
		Temp = Nodos . pSiguiente( * Temp ) ;
	}
	return 0 ; 
}

	//
	// Mtodo : ActivaBarraMenu
	//
	// Activa la barra del menu
	//

void GMenu::ActivaBarraMenu( int Refrescar )
{
	GVentana		*Ventanap;



	// Se obtiene un puntero a la ventana correspondiente
	// a la barra del men principal
	Ventanap = (*Pantalla)( "Barra menu" );

	Ventanap->rdbuf()->BordeVentana( GVenBuf::Doble );

	if( Refrescar )
	{
		Pantalla -> Refresca() ; // Refresca el aspecto de la pantalla
	}

}

	//
	// Mtodo : DesactivaBarraMenu
	//
	// Desactiva la barra del menu
	//

void GMenu::DesactivaBarraMenu( int Refrescar )
{
	GVentana		*Ventanap;



	// Se obtiene un puntero a la ventana correspondiente
	// a la barra del men principal
	Ventanap = (*Pantalla)( "Barra menu" );

	Ventanap->rdbuf()->BordeVentana( GVenBuf::Simple );

	if( Refrescar )
	{
		Pantalla -> Refresca() ; // Refresca el aspecto de la pantalla
	}
}

    //
    // Mtodo : CreaBarraMenu
    //
    // Crea la ventana en la que aparecer la barra horizontal
    // de opciones, mostrando en ella los ttulos.
    //

void GMenu::CreaBarraMenu()
 {
	  // Se crea la barra de men

	GVentana Ventana( "Barra menu", 78, 1,
					   GVenBuf::Grueso, TintaNormal, FondoNormal ) ;

	Ventana << AspCursor( GVenBuf::SinCursor ) ; // No hay cursor

	int Col = 1 ; // Se comenzar a escribir a partir de la primera columna

	GNodo * Nodo = Nodos . pPrimero() ; // Vamos a recorrer los nodos

	while( Nodo != NULL ) // Mientras no lleguemos al ltimo
	 {
		Nodo -> Columna = Col ; // Se guarda la columna donde se va a mostrar

		gotoxy( Col, 1 ) ; // Posicionar el cursor

		   // Obtener la longitud del ttulo ms los espacios separadores

		int Lon = Longitud( Nodo -> Titulo ) + 2 ;

		Col += Lon ; // Sumar la longitud para obtener la siguiente columna

		if( Col > 79 ) // Si nos hemos salida de la ventana

			break ; // detener el proceso

			 // Si todo va bien aadir el ttulo delimitado por dos espacios

		Imprime( Ventana, Nodo -> Titulo, Normal ) ;

			// y procesar el siguiente nodo

		Nodo = Nodos . pSiguiente( *Nodo ) ;

	 } // while

	 Pantalla -> Anade( Ventana, 1, 1 ) ; // Aadir la ventana a la pantalla
 }

   //
   // Mtodo : CreaSubMenus
   //
   // Crea la ventana correspondiente al nodo que recibe como
   // parmetro, situndola en las coordenadas especificadas.
   // Imprime en ella los ttulos de las opciones, y en caso
   // de que alguna opcin tenga asociado un submen se llama
   // a s misma recursivamente, hasta construir la ltima
   // ventana del ltimo nodo contenido.
   //

void GMenu::CreaSubMenus( GNodo * Nodo, int Col, int Lin )
 {
	// Para calcular las dimensiones de la ventana es necesario
	// contar las opciones que hay, y obtener la longitud de la mayor

	  // Obtener la primera de las opciones

	GOpcion * Opcion = Nodo -> Opciones . pPrimero() ;

	Nodo -> Actual = Opcion ; // Esa ser la opcin seleccionada en principio

	int NumOpc = 0 ; // Nmero de opciones que tiene el submen

	int Long = 0 ; // Longitud de la opcin ms larga

	while( Opcion != NULL ) // Mientras no hayamos llegado a la ltima
	 {
		if( Opcion -> Codigo == 0 ) // Si esa opcin tiene asociado un submen

	    // Hay que crear la ventana correspondiente

			CreaSubMenus( Opcion -> Nodo, Col + 2, Lin + NumOpc + 2 ) ;

		NumOpc++ ; // Contar la opcin

		if( istrlen( Opcion -> Titulo ) > Long ) // Si la longitud del ttulo es mayor

			Long = istrlen( Opcion -> Titulo ) ; // preservarla

		Opcion = Nodo -> Opciones . pSiguiente( * Opcion ) ; // Pasar a la siguiente opcin
	 }

	if( NumOpc != 0 ) // Si el nodo tiene opciones
	 {
		  // Se crea la ventana

		GVentana Ventana( Nodo -> Titulo, Long + 1, NumOpc, GVenBuf::Simple,
						  TintaNormal, FondoNormal ) ;

		Ventana << AspCursor( GVenBuf::SinCursor ) ; // El cursor no debe ser visible

		  // Mostrar en ella las distintas opciones

		  // Obtener la primera de las opciones

		Opcion = Nodo -> Opciones . pPrimero() ;

		int Linea = 1 ; // La primera opcin en la primera lnea

		while( Opcion != NULL ) // Mientras no hayamos llegado a la ltima
		 {
			Ventana << PosCursor( 1, Linea ) ; // Posicionar el cursor

			Opcion -> Linea = Linea ; // Se guarda la lnea

			Imprime( Ventana, Opcion -> Titulo, Normal ) ; // Imprimir el ttulo

				// Pasamos a la siguiente opcin

			Opcion = Nodo -> Opciones . pSiguiente( * Opcion ) ;

			Linea++ ;
		 }

			// Se calcula la columna y lnea donde aparecer la ventana
			// del submen, controlando que no se sobrepasen los lmites

		int Columna = Col + Long + 2 < 80 ? Col : Col - ( Col+Long+2 - 80 ) ;
		Linea = Lin + NumOpc + 2 < 25 ? Lin : Lin - ( Lin+NumOpc+2 - 25 ) ;

			// Se aade la ventana

		Pantalla -> Anade( Ventana, Columna, Linea ) ;
	 }
 }

   //
   // Mtodo : Longitud
   //
   // Calcula la longitud de la cadena pasada como parmetro,
   // no contando los indicadores de iniciales de opcin
   //

int GMenu::Longitud( char * C )
 {
	int N = 0 ; // En principio contador a 0

	while( *C ) // Mientras no estemos al final de la cadena

		if( *C++ != '*' ) // Si el carcter no es el indicador

			N++ ; // sumarlo a la longitud

	return N ; // Devolver la longitud total
 }

   //
   // Mtodo : Imprime
   //
   // Imprime el ttulo pasado como parmetro en la ventana indicada,
   // aplicando los atributos correspondientes segn el tercer parmetro
   //

void GMenu::Imprime( GVentana & Ventana, char * C, Modo M )
 {
	if( M == GMenu::Destacado ) // Si hay que destacar el ttulo

			// Utilizar el atributo de seleccionado

		Ventana << TintaA( TintaSeleccionado )
				<< FondoA( FondoSeleccionado ) ;

	else // Si no utilizar el atributo normal

		Ventana << TintaA( TintaNormal )
				<< FondoA( FondoNormal ) ;


	Ventana << " " ; // Espacio delimitador

	char Caracter[2] = " " ;

	while( *C ) // Mientras no lleguemos al final de la cadena
	 {
		Caracter[ 0 ] = *C++ ; // Tomar el carter

		if(  Caracter[0] == '*' ) // si es el delimitador
		 {
			Caracter[ 0 ] = *C++ ; // Tomar el siguiente carcter

				// Activar el atributo correspondiente y dar salida al carcter

			if( M == GMenu::Destacado ) // Si est seleccionado

				Ventana << TintaA( TintaInicialSeleccionada )
						<< FondoA( FondoInicialSeleccionada )
						<< Caracter
						<< TintaA( TintaSeleccionado )
						<< FondoA( FondoSeleccionado ) ;

			else // Si no

				Ventana << TintaA( TintaInicialNormal )
						<< FondoA( FondoInicialNormal )
						<< Caracter
						<< TintaA( TintaNormal )
						<< FondoA( FondoNormal ) ;
		 }
			else // Si no

				Ventana << Caracter ; // simplemente dar salida al carcter
		}

		Ventana << " " ; // Espacio delimitador final
 }

   //
   // Mtodo : LiberaSubMenus
   //
   // Elimina la ventana asociada con el nodo que se recibe como parmetro,
   // y en caso de que las opciones tengan asociados otros nodos se
   // llama a s mismo recursivamente, hasta eliminar todas las ventanas
   // existentes en esta rama.
   //

void GMenu::LiberaSubMenus( GNodo * Nodo )
 {
	Pantalla -> Elimina( Nodo -> Titulo ) ; // Elimina la ventana de este nodo

	GOpcion * Opcion = Nodo -> Opciones . pPrimero() ; // Obtiene la primera opcin

	while( Opcion != NULL ) // Mientras no hayamos llegado a la ltima
	 {
		if( Opcion -> Codigo == 0 ) // Si esa opcin tiene asociado un submen

			LiberaSubMenus( Opcion -> Nodo ) ; // Hay que liberar la ventana

		Opcion = Nodo -> Opciones . pSiguiente( * Opcion ) ; // Siguiente opcin
	 }
 }
