	//
	// GMenu.Hpp
	//
	// Definicin de las clases para creacin de mens
	//
	// 26 Agosto 1992
	//

#ifndef _GMENU_HPP_

#define _GMENU_HPP_

#ifndef __cplusplus
#error No se puede usar GMenu en C
#endif

#include "EstDat.Hpp" // Para utilizar colas de objetos
#include "GVentana.Hpp"	// Necesita conocer las clases GVentana
#include "GRaton.Hpp"

class GOpcion ;		// Referencias avanzadas
class GNodo ;

	//
	// Clase : RaizMenu
	//
	// Deriva de : -
	//
	// Base de : Cualquier clase que vaya a procesar
	//			 mensajes generados por el men
	//
	// Finalidad : Servir como base
	//
	// Fecha de creacin :
	//
	// Fecha de revisin :
	//

class RaizMenu
 {
	public :

		virtual int Procesa( unsigned ) = 0 ;
 } ;

	//
	// Clase : GOpcion
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Mantener la informacin de una opcin
	//
	// Fecha de creacin : 26 Agosto 1992
	//
	// Fecha de revisin : -
	//

class GOpcion
 {
	public :

  		friend class GMenu ; // La clase GMenu tiene acceso a esta

			GOpcion() ; // Constructor por defecto

			GOpcion( int ) ; // Constructor para crear objeto nulo

			GOpcion(  GOpcion & ) ; // Constructor de copia

			  // Crea una opcin asocindola a un nodo de opciones

			GOpcion( char *, GNodo & ) ;

			  // Crea una opcin asocindola a un mensaje

			GOpcion( char *, unsigned ) ;

			~GOpcion() ; // Destructor

			// Operador de asignacin

		void operator = (  GOpcion & ) ;

			// Operador relacional igualdad

		int operator == (  GOpcion & ) ;

			// Operador para insertar un objeto en un stream

		friend ostream & operator << ( ostream &, GOpcion & ) ;

			// Operador para extraer un objeto de un stream

		friend istream & operator >> ( istream &, GOpcion & ) ;

	private :

		char * Titulo ; // Para almacenar el ttulo de la opcin

		GNodo * Nodo ; // Nodo de opciones asociado

		int Linea ; // Lnea en la que est la opcin dentro de la ventana

		unsigned Codigo ; // Cdigo a enviar al mtodo Procesa

 } ; // GOpcion

	//
	// Mtodos inline de la clase GOpcion
	//

	//
	// Mtodo : Constructor por defecto
	//
	// Construye un objeto GOpcion vaco
	//

inline GOpcion::GOpcion() : Titulo( 0 ), Codigo( 0 ), Nodo( 0 )

	{}

	//
	// Mtodo : Constructor de conversin
	//
	// Permite crear objetos nulos, a efectos
	// de comparaciones
	//

inline GOpcion::GOpcion( int ) : Titulo( 0 ), Codigo( 0 ), Nodo( 0 )

	{}

	//
	// Clase : GNodo
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Mantener grupos de opciones, nodos,
	//			   que sern listas, mens
	//
	// Fecha de creacin : 26 Agosto 1992
	//
	// Fecha de revisin : -
	//

class GNodo
 {
	public :

			friend class GMenu ; // La clase GMenu tiene acceso a esta

			GNodo() ; // Constructor por defecto

			GNodo( int ) ; // Constructor para crear objetos nulos

			GNodo(  GNodo & ) ; // Constructor de copia

			 // Constructor que crea un objeto asocindole una
			 // lista de opciones

			GNodo( char *, Cola<GOpcion> & ) ;

			 // Constructor que crea un objeto asocindole un mensaje

			GNodo( char *, unsigned ) ;

			~GNodo() ; // Destructor

			// Operador de asignacin

		void operator = (  GNodo & ) ;

			// Operador relacional de igualdad

		int operator == (  GNodo & ) ;

			// Operador para insertar un objeto en un stream

		friend ostream & operator << ( ostream &, GNodo & ) ;

			// Operador para extraer un objeto de un stream

		friend istream & operator >> ( istream &, GNodo & ) ;

			// Operador para crear una cola de opciones a partir
			// de dos opciones individuales

		friend Cola<GOpcion> operator + ( GOpcion &, GOpcion & ) ;

			// Operador para aadir una opcin ms a una cola
			// de opciones

		friend Cola<GOpcion> operator + ( Cola<GOpcion> &, GOpcion & ) ;

	private :

		char * Titulo ; // Ttulo del nodo

		Cola<GOpcion> Opciones ; // Lista de opciones asociada

		int Columna ; // Columna en la que est el nodo dentro de la ventana

		GOpcion * Actual ; // Opcion actualmente seleccionada de este nodo

		unsigned Codigo ; // Mensaje a enviar a Procesa

 } ; // GNodo

	//
	// Implementacin mtodos inline clase GNodo
	//

	//
	// Mtodo : Constructor por defecto
	//
	// Construye un submen vaco
	//

inline GNodo::GNodo() : Titulo( 0 ), Codigo( 0 )

	{}

	//
	// Mtodo : Constructor de conversin
	//
	// Permite crear objetos nulos
	//

inline GNodo::GNodo( int ) : Titulo( 0 ), Codigo( 0 )

	 {}

	//
	// Clase : GMenu
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Gestionar la unin de opciones
	//		       y nodos que componen el men
	//
	// Fecha de creacin : 26 Agosto 1992
	//
	// Fecha de revisin : -
	//

class GMenu
 {
	public :

		enum Modo { Normal, Destacado } ; // Modo de vdeo

		  // Al crear un men hay que facilitar el GES sobre
		  // el que se utilizar y la cola de nodos que lo componen

		GMenu( GES *, Cola<GNodo> &, GVenBuf::Color, GVenBuf::Color,
									 GVenBuf::Color, GVenBuf::Color,
									 GVenBuf::Color, GVenBuf::Color,
									 GVenBuf::Color, GVenBuf::Color,
									 RaizMenu * = 0 ) ;

			// Destructor de la clase GMenu

		~GMenu();

		int Ejecuta() ; // Ejecuta el men

			// Operador para crear una cola de nodos a partir
			// de dos nodos individuales

		friend Cola<GNodo> operator + ( GNodo &, GNodo & ) ;

			// Operador para aadir un nodo a una cola

		friend Cola<GNodo> operator + ( Cola<GNodo> &, GNodo & ) ;

			// Indica si una tecla esta en la barra del menu

		int EstaEnBarraMenu( unsigned int tecla );

			// Activa la barra del menu.

		void ActivaBarraMenu( int Refrescar = 0 );

			// Desactiva la barra del menu.

		void DesactivaBarraMenu( int Refrescar = 0 );

	private :

		GES * Pantalla ; // Objeto GES sobre el que trabajar el men

		Cola<GNodo> Nodos ; // Cola de los nodos que componen el men

		GVenBuf::Color TintaNormal, FondoNormal,
					   TintaSeleccionado, FondoSeleccionado,
					   TintaInicialNormal, FondoInicialNormal,
					   TintaInicialSeleccionada, FondoInicialSeleccionada ;

		RaizMenu * Procesos ; // Objeto para procesar los mensajes

			// Mtodos privados

		void CreaBarraMenu() ; // Crea la barra del men

		void CreaSubMenus( GNodo *, int, int ) ; // Crea los submens

		void LiberaSubMenus( GNodo * ) ; // Libera las ventanas de los submens

		int Longitud( char * ) ; // Devuelve la longitud de un ttulo

		void Imprime( GVentana &, char *, Modo ) ; // Imprime un ttulo

	  static GRaton Raton ; // Para ocultar el cursor cuando sea necesario

 } ; // GMenu


#endif // _GMENU_HPP_
