	//ͻ
	// GRaton.Hpp                                     
	//                                                
	// Definicin de la clase para controlar el ratn 
	//                                                
	// 27 Mayo 1992                                   
	//ͼ

#ifndef _GRATON_HPP_

#define _GRATON_HPP_

#ifndef __cplusplus
#error No se puede utilizar GRaton en C
#endif

#include <dos.h>   // Para poder utilizar la estructura REGPACK

	//
	// Clase : GRaton
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Servir como interfaz con la
	//	       interrupcin de ratn
	//
	// Fecha de creacin : 27 Mayo 1992
	//
	// Fecha de revisin : 20 Julio 1992
	//

class GRaton
 {
	public :

		// Enumeracin con los servicios del ratn

		enum Servicio { Inicializar, MostrarCursor, OcultarCursor,
				LeerPosicionYBotones, FijarPosicion,
				LeerPulsacionBoton, LeerLiberacionBoton,
				FijarLimitesHorizontales, FijarLimitesVerticales,
				DefinirPunteroGrafico, DefinirPunteroTexto,
				LeerContadoresDesplazamiento, EstablecerLlamada,
				ActivarEmulacionLapiz, DesactivarEmulacionLapiz,
				FijarRelacionMovimiento, FijarLimitesVisualizacion,
				Desconocida1, Desconocida2, FijarVelocidad,
				IntercambiarLlamada, LeerEspacioBuffer,
				PreservarEstado, RestaurarEstado

		} ;

		// Enumeracin de los botones del ratn

		enum Boton { Izquierdo = 0, Derecho = 1, Central = 2 } ;

		GRaton() ; // Constructor que inicializar el ratn

		~GRaton() ; // Destructor, controla la desaparicin del ratn

 static  int	Instalado() ; // Informa sobre el estado del ratn

 static  void	MuestraCursor() ; // Visualizar el cursor del ratn

 static	 void	OcultaCursor() ;  // Oculta el cursor del ratn

 static	 unsigned 	PosicionHorizontal() ; // Devuelve la posicion actual en X

 static	 unsigned 	PosicionVertical() ; // Devuelve la posicion actual en Y

 static	 int	EstadoBoton( Boton ) ; // Devuelve el estado actual del botn indicado

 static	 void	FijaPosicion( unsigned, unsigned ) ; // Fija la posicin del cursor

 static	 void	FijaLimites( unsigned, unsigned, unsigned, unsigned ) ; // Fija los lmites

 static	 void FijaVelocidad( unsigned, unsigned ) ; // Fijar velocidad de desplazamiento

 static	 void	FijaExclusion( unsigned, unsigned, unsigned, unsigned ) ; // Fijar los lmites de exclusin

 static	 void	Preserva() ; // Preserva el estado actual del ratn

 static	 void Restaura() ; // Restaura el estado del ratn

 static  void 	Controlador( unsigned, void (far *) () ) ; // Establece un controlador

 static  void  PunteroSoftware( unsigned, unsigned ) ; // Definir el puntero como carcter software

 static  void	PunteroHardware( unsigned, unsigned ) ; // Definir el puntero como cursor hardware

 static  void	PunteroGrafico( unsigned, unsigned, unsigned far * ) ; // Definir aspecto cursor grfico

	 private :

		// Los datos son estticos, slo existir una instancia para
		// todos los objetos.

		static int Activado ;	// Indicador de existencia del ratn

		static char * Buffer ;	// Buffer para guardar estado del ratn

		// Mtodo para realizar las llamadas al controlador

  static REGPACK Raton( Servicio, unsigned BX = 0, unsigned CX = 0,
				 unsigned DX = 0, unsigned SI = 0,
				 unsigned DI = 0, unsigned ES = 0 ) ;

 } ;

 // Implementacin de las mtodos inline

inline int GRaton::Instalado()

	{ return Activado ; }

inline void GRaton::MuestraCursor()

	{ if( Activado ) Raton( MostrarCursor ) ; }

inline void GRaton::OcultaCursor()

	{ if( Activado ) Raton( OcultarCursor ) ; }

inline unsigned GRaton::PosicionHorizontal()

	{ return ( Activado ? ( Raton( LeerPosicionYBotones ) ) . r_cx  : 0 ) ; }

inline unsigned GRaton::PosicionVertical()

	{ return ( Activado ? ( Raton( LeerPosicionYBotones ) ) . r_dx : 0 ) ; }

inline int GRaton::EstadoBoton( Boton X )

	{ return ( Activado ? ( ( Raton( LeerPulsacionBoton, X ) ) . r_bx != 0 ) : 0 ) ; }

inline void GRaton::FijaPosicion( unsigned X, unsigned Y )

	{ if( Activado ) Raton( FijarPosicion, 0, X, Y ) ; }

inline void GRaton::FijaVelocidad( unsigned VH, unsigned VV )

	{ if( Activado ) Raton( FijarRelacionMovimiento, 0, VH, VV ) ; }

inline void GRaton::FijaExclusion( unsigned X1, unsigned Y1, unsigned X2, unsigned Y2 )

	{ if( Activado ) Raton( FijarLimitesVisualizacion, 0, X1, Y1, X2, Y2 ) ; }

inline void GRaton::Preserva()

	{ if( Activado ) Raton( PreservarEstado, 0, 0, FP_OFF( Buffer ), 0, 0, FP_SEG( Buffer ) ) ; }

inline void GRaton::Restaura()

	{ if( Activado ) Raton( RestaurarEstado, 0, 0, FP_OFF( Buffer ), 0, 0, FP_SEG( Buffer ) ) ; }

inline void GRaton::Controlador( unsigned Mascara, void (far * Funcion) () )

	{ if( Activado ) Raton( EstablecerLlamada, 0, Mascara, FP_OFF( Funcion ), 0, 0, FP_SEG( Funcion ) ) ; }

inline void GRaton::PunteroSoftware( unsigned MascaraPantalla, unsigned Puntero )

	{ if( Activado ) Raton( DefinirPunteroTexto, 0, MascaraPantalla, Puntero ) ; }

inline void GRaton::PunteroHardware( unsigned LineaInicio, unsigned LineaFin )

	{ if( Activado ) Raton( DefinirPunteroTexto, 1, LineaInicio, LineaFin ) ; }

inline void GRaton::PunteroGrafico( unsigned PX, unsigned PY, unsigned far * BitMaps )

	{ if( Activado ) Raton( DefinirPunteroGrafico, PX, PY, FP_OFF( BitMaps ), 0, 0, FP_SEG( BitMaps ) ) ; }

#endif // _GRATON_HPP_