	//
	// GRepreGr.Cpp
	//
	// Implementacin de los mtodos correspondientes
	// a la clase Representa
	//
	// Septiembre 1992
	//

#include "GxEstadi.Hpp" // Tratamiento estadsticos de los datos

#include "GRepreGr.Hpp"

	//
	// Mtodo : Constructor por defecto
	//
	// Construye un objeto vaco
	//

Representa::Representa() : Titulo( 0 ), Destino( 0 )

	{ }

	//
	// Mtodo : Constructor
	//
	// Construye el objeto a partir de la lista de datos
	// recibida
	//

Representa::Representa( char * N, Matriz<double> &  Dat, GVentana * Dest,
                        iTipo T, int Ef,
								double(*F)(double), char * TD[] )

		: Dato( Dat ), Destino( Dest ), TipoGrafico( T ),
		  Efectos( Ef ), nDivisiones( 10 ), Operacion( F ),
		  TitulosDatos( TD )

 {
	Titulo = new char[ istrlen( N ) + 1 ] ; // Espacio para el ttulo

	istrcpy( Titulo, N ) ; // Se copia

	if( Efectos & LlamadaFuncion ) // Si hay una funcin asociada a la representacin
	 {
		for( unsigned I = 0 ; I < Dato . Elementos() ; I++ )

						  // Ir obteniendo el resultado de aplicar la operacin

			  ARepresentar[ I ] = Operacion( Dato[ I ] ) ;
	 }
	else // en caso contrario

		  ARepresentar = Dato ; // Los datos a representar son estos mismos

	xEstadistica<double> Temporal( ARepresentar ) ;

	Min = Temporal . Minimo() ; // Se halla el valor mnimo
	Max = Temporal . Maximo() ; // y el mximo

       // Se calculan las dimensiones de la ventana

   Ancho = ( Ventana() -> rdbuf() -> Cols() - 2 ) * 8 ;
   Alto = ( Ventana() -> rdbuf() -> Lins() - 2 ) * 8 ;
 }

	//
	// Mtodo : Constructor de copia
	//
	// Construye un objeto a partir de los datos de otro
	//

Representa::Representa( Representa & O )

		: Dato( O . Dato ), Min( O . Min ), Max( O . Max ),
		  nDivisiones( O . nDivisiones ), Destino( O . Destino ),
		  TipoGrafico( O . TipoGrafico ), Efectos( O . Efectos ),
		  Operacion( O . Operacion ), TitulosDatos( O . TitulosDatos ),
		  ARepresentar( O . ARepresentar ), Ancho( O . Ancho ),
        Alto( O . Alto )
 {
	Titulo = new char[ istrlen( O . Titulo ) + 1 ] ;

	istrcpy( Titulo, O . Titulo ) ; // Se copia el ttulo
 }

	//
	// Mtodo : Operador de asignacin
	//
	// Facilita la asignacin entre objetos de la clase
	//

void Representa::operator = ( Representa & O )
 {
		// Se asignan los distintos miembros

	Dato = O . Dato ;
	Min = O . Min ;
	Max = O . Max ;
	nDivisiones = O . nDivisiones ;
	Destino = O . Destino ;
	TipoGrafico = O . TipoGrafico ;
	Efectos = O . Efectos ;
	Operacion = O . Operacion ;
	TitulosDatos = O . TitulosDatos ;
	ARepresentar = O . ARepresentar ;
   Ancho = O . Ancho ;
   Alto = O . Alto ;

		// Se libera el ttulo existente

	if( Titulo != 0 )

		delete [] Titulo ;

		// Y se copia el nuevo

	Titulo = new char[ istrlen( O . Titulo ) + 1 ] ;

	istrcpy( Titulo, O . Titulo ) ;
 }

	//
	// Mtodo : Operador relacional de igualdad
	//
	// Permite comprobar si dos elementos son iguales
	//

int Representa::operator == ( Representa & O )
 {
		// Si coinciden todos los miembros los objetos son iguales

	return Dato == O . Dato && Min == O . Min &&
		   Max == O . Max && nDivisiones == O . nDivisiones &&
			TipoGrafico == O . TipoGrafico && Efectos == O . Efectos &&
		   Operacion == O . Operacion &&
		   Destino == O . Destino && ! istrcmp( Titulo, O . Titulo ) ;
 }

	//
	// Mtodo : Minimo
	//
	// Establece el lmite inferior del rango a representar
	//

void Representa::Minimo( double N )

	{ Min = N ; }

	//
	// Mtodo : Minimo
	//
	// Devuelve el actual lmite inferior del rango
	//

double Representa::Minimo()

	{ return Min ; }

	//
	// Mtodo : Maximo
	//
	// Establece el lmite superior del rango a representar
	//

void Representa::Maximo( double N )

	{ Max = N ; }

	//
	// Mtodo : Maximo
	//
	// Devuelve el actual lmite superior del rango
	//

double Representa::Maximo()

	{ return Max ; }

	//
	// Mtodo : Divisiones
	//
	// Establece el nmero de divisiones a realizar en el rango
	//

void Representa::Divisiones( int N )

	{ nDivisiones = N ; }

	//
	// Mtodo : Divisiones
	//
	// Devuelve el nmero de divisiones actuales
	//

int Representa::Divisiones()

	{ return nDivisiones ; }

	//
	// Mtodo : Tipo
	//
	// Establece el tipo de grfico a utilizar para representar los datos
	//

void Representa::Tipo( iTipo T )

	{ TipoGrafico = T ; }

	//
	// Mtodo : Tipo
	//
	// Devuelve el tipo actual de representacin
	//

Representa::iTipo Representa::Tipo()

	{ return TipoGrafico ; }

	//
	// Mtodo : Efecto
	//
	// Permite establecer los efectos a aplicar al grfico
	//

void Representa::Efecto( int E )

	{ Efectos = E ; }

	//
	// Mtodo : Efecto
	//
	// Devuelve la palabra de efectos actualmente establecida
	//

int Representa::Efecto()

	{ return Efectos ; }

	//
	// Mtodo : Actualiza
	//
	// Dibuja el grfico en la ventana correspondiente
	//

void Representa::Actualiza()
 {
   struct viewporttype EstadoPantalla ;

          // Se preserva el rea establecida actualmente

	getviewsettings( & EstadoPantalla ) ;

        // Se delimita el rea de pantalla donde se llevar a cabo
        // la representacin

   setviewport( Ventana() -> rdbuf() -> Col() * 8,
                Ventana() -> rdbuf() -> Lin() * 8,
                ( Ventana() -> rdbuf() -> Col() + Ventana() -> rdbuf() -> Cols() - 2 ) * 8,
                ( Ventana() -> rdbuf() -> Lin() + Ventana() -> rdbuf() -> Lins() - 2 ) * 8,
                1 ) ;

   switch( TipoGrafico ) // Segn el tipo de grfico seleccionado
    {
      case Puntos :   // llamar a un mtodo u otro

			  DibujaPuntos() ;
           break ;

      case Lineal :

           DibujaLineal() ;
           break ;

      case Histograma :

           DibujaHistograma() ;
           break ;

      case Superficie :

           DibujaSuperficie() ;
           break ;

      case Sectorial :

			  DibujaSectorial() ;
           break ;

    } // switch

   if( Efectos & Rejilla )    // Segn los atributos fijados

       DibujaRejilla() ; // ir dibujando los distintos componentes de
                         // la representacin

   if( Efectos & Escala )

       DibujaEscala() ;

   if( Efectos & Texto )

       DibujaTitulo() ;

   if( Efectos & Datos )

		 DibujaDatos() ;

 } // Actualiza

	//
	// Mtodo : Ventana
	//
	// Fija la ventana destino donde se representarn los datos
	//

void Representa::Ventana( GVentana * V )

	{ Destino = V ; }

	//
	// Mtodo : Ventana
	//
	// Devuelve la ventana en la que se realiza la salida actualmente
	//

GVentana * Representa::Ventana()

	{ return Destino ; }

	//
	// Mtodo : Funcion
	//
	// Establece la funcin a la que hay que llamar para obtener
	// los valores a representar
	//

void Representa::Funcion( double(*F)(double) )

	{ Operacion = F ; }

	//
	// Mtodo : Funcion
	//
	// Devuelve la funcin que se est actualmente utilizando
	// para obtener los datos que se representan
	//

double (* Representa::Funcion() ) ( double )

	{ return Operacion ; }

	//
	// Mtodo : DibujaRejilla
	//
	// Dibuja la rejilla segn las divisiones establecidas y el
	// tipo de grfico a representar
	//

void Representa::DibujaRejilla()
 {
	if( TipoGrafico == Sectorial ) // Si la representacin es sectorial

		 return ; // no se dibuja rejilla alguna

       // Linea continua y ancha

	setlinestyle( SOLID_LINE, 0, THICK_WIDTH ) ;

     // El color ser el mismo actual en la ventana

	setcolor( Ventana() -> rdbuf() -> TintaActual() ) ;

		 // Se dibujan las lneas que contienen la rejilla

	line( Ancho * 0.2, Alto * 0.1, Ancho * 0.2, Alto * 0.9 ) ;
	line( Ancho * 0.2, Alto * 0.9, Ancho * 0.9, Alto * 0.9 ) ;

         // Se calcula la distancia vertical entre lnea y lnea

	double Intervalo = Alto * 0.8 / ( nDivisiones - 1 ) ;

          // Posicin vertical de la primera lnea

	double Actual = Alto * 0.9 - Intervalo ;

	setlinestyle( DOTTED_LINE, 0, 0 ) ; // Lneas de puntos

		 // Se dibujan tantas lneas como divisiones

	for( int I = 0 ; I < nDivisiones - 1 ; I++ )
	 {
		line( Ancho * 0.2, Actual, Ancho * 0.9, Actual ) ;

		Actual -= Intervalo ;  // Se va desplazando la posicin
    }

 }

	//
	// Mtodo : DibujaEscala
	//
	// Dibuja la escala, segn el mnimo y mximo que haya establecidos
	// y el nmero de divisiones a realizar
	//

void Representa::DibujaEscala()
 {
	if( TipoGrafico == Sectorial ) // Si la representacin es sectorial

		 return ; // no se dibuja la escala, ya que no tiene sentido

	double Actual = Min ; // Valor a mostrar

			 // Saltos para cumplir la escala establecida

	double Intervalo = ( Max - Min ) / ( nDivisiones - 1 ) ;

			 // Medidas para ir situando los ttulos

	double IntervaloPosicion = Alto * 0.8 / ( nDivisiones - 1 ) ;
	double ActualPosicion = Alto * 0.9 ;

			 // El texto estar ajustado a la derecha

	settextjustify( RIGHT_TEXT, TOP_TEXT ) ;

       // Utilizar el color actual de la ventana

	setcolor( Ventana() -> rdbuf() -> TintaActual() ) ;

       // Tantos letreros como divisiones

	for( int I = 0 ; I < nDivisiones ; I++ )
	 {
		char Buffer[ 128 ] ;

		gcvt( Actual, 16, Buffer ) ; // Convertir el valor a cadena

            // Si hay un punto decimal

		if( istrchr( Buffer, '.' ) )

                   // dejar slo dos decimales

			*( istrchr( Buffer, '.' ) + 3 ) = 0 ;

		else // Si no hay punto decimal

              // aadir .00, para que el ajuste sea correcto

			istrcat( Buffer, ".00" ) ;

				// Imprimirlo

		outtextxy( Ancho * 0.2 - 2, ActualPosicion, Buffer ) ;

		Actual += Intervalo ; // Siguiente valor

		ActualPosicion -= IntervaloPosicion ; // Siguiente posicin
	 }
 }

	//
	// Mtodo : DibujaTitulo
	//
	// Muestra el ttulo del grfico en la parte superior de la ventana
	//

void Representa::DibujaTitulo()
 {
		// El ttulo aparecer centrado en el rea superior de la ventana

	settextjustify( CENTER_TEXT, CENTER_TEXT ) ;

      // Con el color que tenga la ventana actualmente

	setcolor( Ventana() -> rdbuf() -> TintaActual() ) ;

	outtextxy( Ancho / 2, Alto * 0.1 / 2, Titulo ) ;
 }

	//
	// Mtodo : DibujaDatos
	//
	// Muestra los ttulos asociados a los datos en la parte inferior
	//

void Representa::DibujaDatos()
 {
	if( TipoGrafico == Sectorial ) // Si la representacin es sectorial

		 return ; // los datos no se muestran en este mtodo

		// Punto donde se mostrar el primer ttulo

	double Actual = Ancho * 0.2 + 2 ;

          // Distancia entre un ttulo y otro, dependiendo del
          // nmero de datos que se van a representar

	double Intervalo = Ancho * 0.7 / ARepresentar . Elementos() ;

          // Texto ajustado a la izquierda

	settextjustify( LEFT_TEXT, TOP_TEXT ) ;

        // Utilizar el color actual en la ventana

	setcolor( Ventana() -> rdbuf() -> TintaActual() ) ;

      // Bucle para recorrer todos los datos

	for( int I = 0 ; I < ARepresentar . Elementos() ; I++ )
	 {
			// Se muestra un ttulo

      outtextxy( Actual, Alto * 0.9 + 2, TitulosDatos[ I ] ) ;

		Actual += Intervalo ; // Se avanza a la posicin del siguiente
    }
 }

	//
	// Mtodo : DibujaPuntos
	//
	// Representa la lista de datos como puntos sueltos.
   // Los puntos en realidad sern representados como cruces
   // trazados por dos lneas rectas.
	//

void Representa::DibujaPuntos()
 {
		// Se calcula el factor multiplicativo/divisor

	double Factor = Alto * 0.8 / ( Max - Min ) ;

		// Separacin entre punto y punto, dependiendo del
		// nmero de puntos a representar

	double Intervalo = Ancho * 0.7 / ARepresentar . Elementos() ;

	double Actual = Ancho * 0.2 + 2 ; // Posicin del primer punto

	int Color = 0 ; // Color del punto

	setlinestyle( SOLID_LINE, 0, THICK_WIDTH ) ; // Lnea continua y gruesa

       // Bucle para recorrer todos los datos

	for( unsigned I = 0 ; I < ARepresentar . Elementos() ; I++ )
	 {
			// Se obtiene el punto a representar

		double Dato = ARepresentar[ I ] ;

				 // Si el dato est en el intervalo de la escala

		if( Dato >= Min && Dato <= Max )
		 {
			Dato -= Min ; // Ajustar

              // Si el color que corresponde usar coincide con
              // el color de fondo el punto no se ver

			if( Color % ( getmaxcolor() + 1 ) ==
				 Ventana() -> rdbuf() -> FondoActual() )

				Color++ ; // as que se avanza al siguiente color

			setcolor( Color % ( getmaxcolor() + 1 ) ) ; // Se fija el color

				// Se dibuja el punto como una cruz

			line( Actual - 2, Alto * 0.9 - Dato * Factor - 2,
					Actual + 2, Alto * 0.9 - Dato * Factor + 2 ) ;

			line( Actual - 2, Alto * 0.9 - Dato * Factor + 2,
					Actual + 2, Alto * 0.9 - Dato * Factor - 2 ) ;

			Color++ ; // Ir rotando los colores
		 }

		Actual += Intervalo ; // Avanzar horizontalmente
	 }
 }

	//
	// Mtodo : DibujaLineal
	//
	// Representa la lista de datos como una lnea quebrada de
	// tantos puntos como datos haya en la lista
	//

void Representa::DibujaLineal()
 {
      // Se calcula el factor multiplicativo/divisor

   double Factor = Alto * 0.8 / ( Max - Min ) ;

      // Separacin entre vrtices de lnea, dependiendo del
      // nmero de puntos a representar

	double Intervalo = Ancho * 0.7 / ARepresentar . Elementos() ;

	double Actual = Ancho * 0.2 + 2 ; // Posicin del primer punto

          // Utilizar el color actual en la ventana

	setcolor( Ventana() -> rdbuf() -> TintaActual() ) ;

             // Lnea slida de anchura normal

	setlinestyle( SOLID_LINE, 0, NORM_WIDTH ) ;

	for( unsigned I = 1 ; I < ARepresentar . Elementos() ; I++ )
	 {
			// Trazar la recta

		line( Actual, Alto * 0.9 - ( ARepresentar[I-1] - Min ) * Factor,
				Actual + Intervalo,
				Alto * 0.9 - ( ARepresentar[I] - Min ) * Factor ) ;

      Actual += Intervalo ; // Avanzar horizontalmente
	 }

 }

	//
	// Mtodo : DibujaHistograma
	//
	// Representa los datos en forma de barras
	//

void Representa::DibujaHistograma()
 {
      // Se calcula el factor multiplicativo/divisor

	double Factor = Alto * 0.8 / ( Max - Min ) ;

		// Ancho de cada barra y espacio de separacin

	double AnchoBarra = Ancho * 0.7 / ARepresentar . Elementos() ;
	double Intervalo = AnchoBarra / 4 ;

		// Punto del que se comenzar el trazado de la barra

	double Actual = Ancho * 0.2 + 2 ;

	int Color = 0 ; // Color que se utilizar
	int Relleno = 1 ; // Patrn de relleno que se usar

       // El color del borde ser blanco

	setcolor( Ventana() -> rdbuf() -> TintaActual() ) ;

         // El borde ser continuo y de anchura normal

	setlinestyle( SOLID_LINE, 0, NORM_WIDTH ) ;

	for( unsigned I = 0 ; I < ARepresentar . Elementos() ; I++ )
	 {
            // Si el color que se va a utilizar para el relleno
            // coincide con el color de fondo avanzar al siguiente

			if( Color % ( getmaxcolor() + 1 ) ==
				 Ventana() -> rdbuf() -> FondoActual() )

				Color++ ;

			// Se fija el tipo de relleno de la barra y su color

		setfillstyle( Relleno % CLOSE_DOT_FILL + 1,
                    Color % ( getmaxcolor() + 1 ) ) ;

         // Se dibuja la barra

		bar3d( Actual, Alto * 0.9,  Actual + AnchoBarra - Intervalo,
			  Alto * 0.9 - ( ARepresentar[ I ] - Min ) * Factor, 5, 1 ) ;

		Relleno++ ;	// Ir rotando tipos de relleno
		Color++ ; // y colores

		Actual += AnchoBarra ;
	 }

 }

	//
	// Mtodo : DibujaSuperficie
	//
	// Representa los datos como un rea cerrada
	//

void Representa::DibujaSuperficie()
 {
		// Se calcula el factor multiplicativo/divisor

	double Factor = Alto * 0.8 / ( Max - Min ) ;

		// Separacin entre punto y punto, dependiendo del
		// nmero de puntos a representar

	double Intervalo = Ancho * 0.7 / ARepresentar . Elementos() ;

	double Actual = Ancho * 0.2 + 2 ; // Posicin del primer punto

       // El rea ser bordeada por una lnea continua de ancho
       // normal y del mismo color establecido actualmente
       // en la ventana

	setcolor( Ventana() -> rdbuf() -> TintaActual() ) ;
	setlinestyle( SOLID_LINE, 0, NORM_WIDTH ) ;

       // Se crea una zona de memoria para contener los puntos
       // que sern los vrtices del reas

	int * Puntos = new int[ ( ARepresentar . Elementos() + 3 ) * 2 ] ;

       // El primer punto ser desde la parte inferior izquierda

	Puntos[ 0 ] = Actual ;
	Puntos[ 1 ] = Alto * 0.9 - ( 0 - Min ) * Factor ;

	int PuntoActual = 1 ; // Ya tenemos un punto

	for( unsigned I = 0 ; I < ARepresentar . Elementos() ; I++ )
	 {
			// Se obtiene el punto a representar

		double Dato = ARepresentar[ I ] ;

				 // Si el dato est en el intervalo de la escala

		if( Dato >= Min && Dato <= Max )
		 {
			Dato -= Min ; // Ajustar

              // Se almacena el punto en la matriz

			Puntos[ PuntoActual * 2 ] = Actual ;
			Puntos[ PuntoActual * 2 + 1 ] = Alto * 0.9 - Dato * Factor ;

			PuntoActual++ ; // Tenemos un punto ms
		 }

		Actual += Intervalo ; // Avanzar horizontalmente
	 }

      // Se cierra el rea, ya que de lo contrario no se puede rellenar

	Puntos[ PuntoActual * 2 ] = Puntos[ (PuntoActual-1) * 2 ] ;
	Puntos[ PuntoActual * 2 + 1 ] = Puntos[ 1 ] ;
	Puntos[ PuntoActual * 2 + 2 ] = Puntos[ 0 ] ;
	Puntos[ PuntoActual * 2 + 3 ] = Puntos[ 1 ] ;

      // Relleno slido con el color fijado actualmente

	setfillstyle( SOLID_FILL, Ventana() -> rdbuf() -> TintaActual() ) ;

       // Dibujar el rea

	fillpoly( PuntoActual + 1, Puntos ) ;

	delete [] Puntos ; // Liberar la memoria asignada para los puntos
 }

   // Macro para convertir grados a radianes

#define RADIANES( x )	2*3.1415972*(x)/360.0

	//
	// Mtodo : DibujaSectorial
	//
	// Representa los datos en forma de sectores
	//

void Representa::DibujaSectorial()
 {
      // Se crea un objeto estadstico

	xEstadistica<double> * Temporal = new xEstadistica<double>( ARepresentar ) ;

      // Para obtener la suma de los elementos y as poder
      // calcular el factor multiplicativo

	double Factor = 360.0 / Temporal -> SumaX() ;

	delete Temporal ; // Se libera el objeto estadstico

	int Color = 0 ; // Color y tipo de relleno del sector inicial
	int Relleno = 1 ;

       // El ngulo de inicio del primer sector ser 0

	int Inicio = 0, Fin ;

       // El radio de la circunferencia ser un tercio de
       // la anchura o la altura, lo que sea ms pequeo

	double Radio =  ( Ancho > Alto ? Ancho * 0.8 / 3 : Alto * 0.8 / 3 ) ;

          // El borde de los sectores ser del mismo color fijado
          // actualmente en la ventana

   setcolor( Ventana() -> rdbuf() -> TintaActual() ) ;

       // El tipo de lnea del borde ser continuo y de anchura normal

   setlinestyle( SOLID_LINE, 0, NORM_WIDTH ) ;

          // Tantos sectores como datos hay a representar

	for( int I = 0 ; I < ARepresentar . Elementos() ; I++ )
	 {
         // Se halla el ngulo de fin

		Fin = Inicio + Factor * ARepresentar[ I ] + 0.5 ;

		if( Fin > 360 ) // Si es el ltimo sector y por los redondeos
                     // se supera el lmite

			Fin = 360 ; // ajustar

             // Si el color que se va a utilizar es el mismo
             // que actualmente hay de fondo avanzar al siguiente

		if( Color % ( getmaxcolor() + 1 ) ==
			 Ventana() -> rdbuf() -> FondoActual() )

				Color++ ;

            // Se fija el tipo de relleno y el color

		setfillstyle( Relleno % CLOSE_DOT_FILL + 1,
                    Color % ( getmaxcolor() + 1 ) ) ;

          // Se dibuja el sector

		pieslice( Ancho / 2, Alto / 2, Inicio, Fin, Radio ) ;

                // Si el sector que hemos dibujado est a la izquierda

      if( cos( RADIANES( Inicio+(Fin-Inicio) / 2.0 )) < 0 )

               // el texto ser ajustado a la derecha

          settextjustify( RIGHT_TEXT, CENTER_TEXT ) ;

      else

            // En caso contrario a la izquierda

          settextjustify( LEFT_TEXT, CENTER_TEXT ) ;

      if( Efectos & Escala ) // Si hay que dibujar una escala
       {
       		char Buffer[128] ;

                 // Convertir el valor del sector a cadena

		      gcvt( ARepresentar[ I ], 16, Buffer ) ;

                // Mostrarlo dentro del sector
            outtextxy(

              cos( RADIANES( Inicio + (Fin-Inicio) / 2.0) ) *
                   (Radio/1.5) + Ancho / 2,
				  -sin( RADIANES( Inicio + (Fin-Inicio) / 2.0) ) *
                   (Radio/1.5) + Alto / 2,
				  Buffer ) ;
       }

      if( Efectos & Datos ) // Si hay que dibujar los titulos

                  // Mostrarlo fuera del sector

          outtextxy(

             cos( RADIANES( Inicio + (Fin-Inicio) / 2.0) ) *
                  (Radio+textwidth("O")*2) + Ancho / 2,
             -sin( RADIANES( Inicio + (Fin-Inicio) / 2.0 ) ) *
                   (Radio+textheight("O")*2) + Alto / 2,
             TitulosDatos[ I ] ) ;

		Inicio = Fin ; // El prximo sector comenzar donde ha terminado este

		Color++ ;    // Rotar los colores y rellenos
		Relleno ++ ;
	 }
 }