	//
	// GRepreGr.Hpp
	//
	// Define la clase Representa, para realizar
	// representaciones grficas de listas de datos
	//
	// Septiembre 1992
	//

#ifndef _GREPREGR_HPP_
#define _GREPREGR_HPP_

#ifndef __cplusplus
#error No se puede utilizar Representa en C
#endif

#include <graphics.h>
#include <string.h>

#include "EstDat.Hpp"

#include "GVentana.Hpp"

	// Si el modelo de memoria seleccionado permite varios segmentos de datos
	// habr que utilizar las versiones far de algunas funciones

#if defined(__LARGE__) || defined(__HUGE__) || defined(__COMPACT__)

	#define istrcpy( orig, dest )	_fstrcpy( (orig), (dest) )
	#define istrcmp( orig, dest )	_fstrcmp( (orig), (dest) )
	#define istrcat( orig, dest )	_fstrcat( (orig), (dest) )

#else

	#define istrcpy( orig, dest )	strcpy( (orig), (dest) )
	#define istrcmp( orig, dest )	strcmp( (orig), (dest) )
	#define istrcat( orig, dest )	strcat( (orig), (dest) )

#endif

	//
	// Clase : Representa
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Representar una lista de
	//			   datos, grficamente en
	//			   distintos formatos
	//
	// Fecha de creacin : 10 Septiembre 1992
	//
	// Fecha de revisin : -
	//

class Representa
 {
	public :

			// Tipos de representacin posibles

		enum iTipo { Puntos, Lineal, Histograma, Superficie, Sectorial } ;

			// Efectos aplicables

		enum iEfecto { Rejilla = 1, Escala = 2, Texto = 4,
					      Datos = 8, LlamadaFuncion = 16 } ;

		Representa() ; // Constructor por defecto

			// Constructor estndar

		Representa( char *, Matriz<double> &, GVentana *,
					   iTipo, int, double(*)(double) = 0, char * [] = 0 ) ;

		Representa( Representa & ) ; // Constructor de copia

			// Operador de asignacin

		void operator = ( Representa & ) ;

			// Operador de relacin de igualdad

		int operator == ( Representa & ) ;

		void Minimo( double ) ; // Establece el mnimo en la escala
		double Minimo() ; // Devuelve el mnimo

		void Maximo( double ) ; // Establece el mximo en la escala
		double Maximo() ; // Devuelve el mximo

		void Divisiones( int ) ; // Establece el nmero de divisiones
		int Divisiones() ; // Devuelve el nmero de divisiones

		void Tipo( iTipo ) ; // Establece el tipo de representacin
		iTipo Tipo() ; // Devuelve el tipo de representacin

		void Efecto( int ) ; // Establece efectos
		int Efecto() ; // Devuelve los efectos actuales

		void Actualiza() ; // Dibuja la representacin

		void Ventana( GVentana * ) ; // Establece la ventana destino
		GVentana * Ventana() ; // Devuelve la ventana destino

		void Funcion( double(*)( double ) ) ; // Establece la funcin a la que llamar
		double (* Funcion() )( double ) ; // Devuelve la funcin

	private :

		Matriz<double> Dato ; // Matriz con los datos

      Matriz<double> ARepresentar ; // Datos definitivos a representar

      char ** TitulosDatos ; // Matriz de punteros a los ttulos

		void DibujaRejilla() ; // Dibuja la rejilla

		void DibujaEscala() ; // Dibuja la escala

      void DibujaTitulo() ; // Dibuja el ttulo

      void DibujaDatos() ; // Muestra los datos

			// Representa los datos en forma de puntos

		void DibujaPuntos() ;

			// Representa los datos como una lnea

		void DibujaLineal() ;

			// Representa los datos en forma de barras

		void DibujaHistograma() ;

			// Representa los datos como un rea

		void DibujaSuperficie() ;

			// Representa los datos en forma de pastel

		void DibujaSectorial() ;

		double Min, Max ; // Lmites de la escala a representar

		int nDivisiones ; // Divisiones a realizar

		char * Titulo ; // Ttulo de la representacin

		GVentana * Destino ; // Ventana destino de la representacin

		iTipo TipoGrafico ; // Tipo de grfico a representar

		int Efectos ; // Efectos activos

		double( * Operacion )( double ) ; // Funcin a llamar para obtener datos

      int Alto, Ancho ; // Dimensiones del rea destino

 } ;

#endif // _GREPREGR_HPP_