	//
	//	GTeclado.Hpp
	//
	//	Definicin de la clase GTeclado
	//
	//	26 Mayo 1992
	//

#ifndef _GTECLADO_HPP_

#define _GTECLADO_HPP_

#include <bios.h>	// Para poder utilizar la funcin bioskey()

	//
	// Clase : GTeclado
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Facilitar mtodos de alto nivel
	//	       para acceso al teclado
	//
	// Fecha de creacin : Mayo 1992
	//
	// Fecha de revisin : Agosto 1992
	//

class GTeclado
 {
	public:

		// Enumeracin en la que se asigna un nombre a
		// cada cdigo de pulsacin.

		enum { NADA, CTRL_A, CTRL_B, CTRL_C, CTRL_D, CTRL_E, CTRL_F,
		       CTRL_G, CTRL_H, BS = CTRL_H, CTRL_I, TAB = CTRL_I,
		       CTRL_J, CTRL_INTRO = CTRL_J, CTRL_K, CTRL_L,
		       CTRL_M, INTRO = CTRL_M, CTRL_N, CTRL_O, CTRL_P,
		       CTRL_Q, CTRL_R, CTRL_S, CTRL_T, CTRL_U, CTRL_V,
		       CTRL_W, CTRL_X, CTRL_Y, CTRL_Z, ESC, CTRL_BS = 127,
		       ALT_ESC = 257, SHIFT_TAB = 271, ALT_Q, ALT_W,
		       ALT_E, ALT_R, ALT_T, ALT_Y, ALT_U, ALT_I, ALT_O,
		       ALT_P, ALT_INTRO = 284, ALT_A = 286, ALT_S, ALT_D,
		       ALT_F, ALT_G, ALT_H, ALT_J, ALT_K, ALT_L, ALT_Z = 300,
		       ALT_X, ALT_C, ALT_V, ALT_B, ALT_N, ALT_M, F1 = 315,
		       F2, F3, F4, F5, F6, F7, F8, F9, F10, INICIO = 327,
		       ARRIBA, REPAG, IZQUIERDA = 331, DERECHA = 333,
		       FIN = 335, ABAJO, AVPAG, INSERT, SUPR, SHIFT_F1,
		       SHIFT_F2, SHIFT_F3, SHIFT_F4, SHIFT_F5, SHIFT_F6,
		       SHIFT_F7, SHIFT_F8, SHIFT_F9, SHIFT_F10, CTRL_F1,
		       CTRL_F2, CTRL_F3, CTRL_F4, CTRL_F5, CTRL_F6, CTRL_F7,
		       CTRL_F8, CTRL_F9, CTRL_F10, ALT_F1, ALT_F2, ALT_F3,
		       ALT_F4, ALT_F5, ALT_F6, ALT_F7, ALT_F8, ALT_F9,
		       ALT_F10, CTRL_IZQUIERDA = 371, CTRL_DERECHA,
		       CTRL_FIN, CTRL_AVPAG, CTRL_INICIO, ALT_1, ALT_2,
		       ALT_3, ALT_4, ALT_5, ALT_6, ALT_7, ALT_8, ALT_9,
		       ALT_0, CTRL_REPAG = 388, F11, F12, SHIFT_F11,
		       SHIFT_F12, CTRL_F11, CTRL_F12, ALT_F11, ALT_F12,
		       CTRL_ARRIBA, CTRL_ABAJO = 401, CTRL_TAB = 404,
		       ALT_INICIO = 407, ALT_ARRIBA, ALT_REPAG,
		       ALT_IZQUIERDA = 411, ALT_DERECHA = 413, ALT_FIN = 415,
		       ALT_ABAJO, ALT_AVPAG, ALT_INSERT, ALT_SUPR,
		       ALT_TAB = 421
		     } ;

		unsigned Explora() ;  // Devuelve una pulsacin o NADA

		unsigned Recoge() ;   // Recoge una tecla, esperando si es necesario

		void     Limpia() ;   // Limpia el buffer de teclado

	     // Grupo de funciones inline para conocer el estado
	     // de las teclas bi-estado, como maysculas e insert,
	     // y de las teclas que no generan cdigo por s solas.

	      int InsertActivo() ;
	      int MayusculasActivo() ;
	      int MayusculasPulsado() ;
	      int NumerosActivo() ;
	      int NumerosPulsado() ;
	      int AltIPulsado() ;
	      int AltDPulsado() ;
	      int AltPulsado() ;
	      int CtrlIPulsado() ;
	      int CtrlDPulsado() ;
	      int CtrlPulsado() ;
	      int ShiftIPulsado() ;
	      int ShiftDPulsado() ;
	      int ShiftPulsado() ;
	      int PetSisPulsado() ;
 } ;

 //
 // Implementacin de las funciones , que hacen uso de la
 // funcin bioskey() para obtener la mscara  de estado de las
 // teclas que se quieren controlar.
 //

inline int GTeclado::InsertActivo()
 {
	return _bios_keybrd( _NKEYBRD_SHIFTSTATUS ) & 0x80 ;
 }

inline int GTeclado::MayusculasActivo()
 {
	return _bios_keybrd( _NKEYBRD_SHIFTSTATUS ) & 0x40 ;
 }

inline int GTeclado::MayusculasPulsado()
 {
	return _bios_keybrd( _NKEYBRD_SHIFTSTATUS ) & 0x4000 ;
 }

inline int GTeclado::NumerosActivo()
 {
	return _bios_keybrd( _NKEYBRD_SHIFTSTATUS ) & 0x20 ;
 }

inline int GTeclado::NumerosPulsado()
 {
	return _bios_keybrd( _NKEYBRD_SHIFTSTATUS ) & 0x2000 ;
 }

inline int GTeclado::AltIPulsado()
 {
	return _bios_keybrd( _NKEYBRD_SHIFTSTATUS ) & 0x200 ;
 }

inline int GTeclado::AltDPulsado()
 {
	return _bios_keybrd( _NKEYBRD_SHIFTSTATUS ) & 0x800 ;
 }

inline int GTeclado::AltPulsado()
 {
	return _bios_keybrd( _NKEYBRD_SHIFTSTATUS ) & 0x8 ;
 }

inline int GTeclado::CtrlIPulsado()
 {
	return _bios_keybrd( _NKEYBRD_SHIFTSTATUS ) & 0x100 ;
 }

inline int GTeclado::CtrlDPulsado()
 {
	return _bios_keybrd( _NKEYBRD_SHIFTSTATUS ) & 0x400 ;
 }

inline int GTeclado::CtrlPulsado()
 {
	return _bios_keybrd( _NKEYBRD_SHIFTSTATUS ) & 0x4 ;
 }

inline int GTeclado::ShiftIPulsado()
 {
	return _bios_keybrd( _NKEYBRD_SHIFTSTATUS ) & 0x2 ;
 }

inline int GTeclado::ShiftDPulsado()
 {
	return _bios_keybrd( _NKEYBRD_SHIFTSTATUS ) & 0x1 ;
 }

inline int GTeclado::ShiftPulsado()
 {
	return ShiftIPulsado() || ShiftDPulsado() ;
 }

inline int GTeclado::PetSisPulsado()
 {
	return _bios_keybrd( _NKEYBRD_SHIFTSTATUS ) & 0x8000 ;
 }

#endif