	//
	// GXEstadi.Hpp
	//
	// Define la plantilla de clase xEstadistica, para tratamiento
	// de listas de datos con obtencin de resultados estadsticos,
	// e implementa los mtodos correspondientes
	//
	// 7 Septiembre 1992
	//

#include <math.h>

#include "EstDat.Hpp"	// Necesita matrices

#ifndef _GXESTADI_HPP_
#define _GXESTADI_HPP_

#ifndef __cplusplus
#error No se puede usar xEstadistica en C
#endif

template <class Tipo>
	//
	// Clase : xEstadistica
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Aplicar clculos estadsticos
	//			   sobre listas de datos, con
	//			   una o dos variables
	//
	// Fecha de creacin : 7 Septiembre 1992
	//
	// Fecha de revisin : -
	//

class xEstadistica
 {
	public :

			xEstadistica() ; // Constructor por defecto

			xEstadistica(  xEstadistica & ) ; // Constructor de copia

         xEstadistica( Matriz<Tipo> & ) ; // Acepta una matriz como parmetro

			~xEstadistica() ; // Destructor

		void operator = (  xEstadistica & ) ; // Operador de asignacin

			// Operador de relacin de igualdad

		int operator == (  xEstadistica & ) ;

			// Operador de relacin de desigualdad

		int operator != (  xEstadistica & ) ;

			// Operador de insercin en un stream

		friend ostream & operator << ( ostream &, xEstadistica & ) ;

			// Operador de extraccin de un stream

		friend istream & operator >> ( istream &, xEstadistica & ) ;

		Tipo & operator [] ( int ) ; // Devuelve referencia a un elemento

		unsigned N () ; // Devuelve nmero de elementos en la lista

		Tipo SumaX() ; // Suma de los elementos

		Tipo SumaX2() ; // Suma de los elementos al cuadrado

		Tipo Maximo() ; // Valor mximo

		Tipo Minimo() ; // Valor mnimo

		Tipo Media() ; // Media de valores

		double Desviacion() ; // Desviacin tpica de los elementos

		double cDesviacion() ; // Cuasi desviacin tpica

			// Suma de elementos con dos variables

		friend Tipo SumaXY(  xEstadistica &,
							 xEstadistica & ) ;

			// Clculo de la covarianza

		friend Tipo Covarianza(  xEstadistica &,
								 xEstadistica & ) ;

			// Clculo del coeficiente de correlacin

		friend Tipo CoefCorr(  xEstadistica &,
								xEstadistica & ) ;

			// Clculo del valor terico de X sobre Y

		friend Tipo TeoricoX(  xEstadistica &,
								xEstadistica &,  Tipo ) ;

			// Clculo del valor terico de Y sobre X

		friend Tipo TeoricoY(  xEstadistica &,
								xEstadistica &,  Tipo ) ;

	private :

		Matriz<Tipo> Elemento ; // Lista con los elementos

 } ; // xEstadistica

	//
	// Implementacin de los mtodos
	//

	//
	// Mtodo : Constructor por defecto
	//
	// Crea una lista vaca
	//

template <class Tipo>
inline xEstadistica<Tipo>::xEstadistica()

	{ }

	//
	// Mtodo : Constructor de copia
	//
	// Crea una lista a partir de otra
	//

template <class Tipo>
inline xEstadistica<Tipo>::xEstadistica(  xEstadistica<Tipo> & O )

	: Elemento( O . Elemento ) // Copia los elementos

	{ }

   //
   // Mtodo : Constructor
   //
   // Permite construir un objeto a partir de una matriz
   //

template <class Tipo>
inline xEstadistica<Tipo>::xEstadistica( Matriz<Tipo> & O )

       : Elemento( O ) // Inicializa los datos

       { }

	//
	// Mtodo : Destructor
	//

template <class Tipo>
inline xEstadistica<Tipo>::~xEstadistica()

	{ }

	//
	// Mtodo : Operador de asignacin
	//
	// Asigna un objeto a otro
	//

template <class Tipo>
inline void xEstadistica<Tipo>::operator = (  xEstadistica<Tipo> & O )

	{ Elemento = O . Elemento ; }

	//
	// Mtodo : Operador de relacin de igualdad
	//
	// Permite comparar dos objetos y saber si son iguales
	//

template <class Tipo>
inline int xEstadistica<Tipo>::operator == (  xEstadistica<Tipo> & O )

	{ return Elemento == O . Elemento ; }

	//
	// Mtodo : Operador de relacin de desigualdad
	//
	// Negacin del operador de igualdad
	//

template <class Tipo>
inline int xEstadistica<Tipo>::operator != (  xEstadistica<Tipo> & O )

	{ return ! operator == ( O ) ; }

	//
	// Mtodo : Operador de insercin en stream
	//
	// Facilita la grabacin de un objeto en un stream
	//

template <class Tipo>
ostream & operator << ( ostream & Stream, xEstadistica<Tipo> & O )

	{ return Stream << O . Elemento ; }

	//
	// Mtodo : Operador de extraccin de stream
	//
	// Facilita la lectura de un objeto de un stream
	//

template <class Tipo>
istream & operator >> ( istream & Stream, xEstadistica<Tipo> & O )

	{ return Stream >> O . Elemento ; }

	//
	// Mtodo : Operador subndice
	//
	// Devuelve una referencia al elemento indicador, permitiendo
	// obtener su valor o manipularlo
	//

template <class Tipo>
inline Tipo & xEstadistica<Tipo>::operator [] ( int NE )

	{ return Elemento[ NE - 1 ] ; }

	//
	// Mtodo : N
	//
	// Devuelve el nmero de elementos que actualmente
	// contiene la lista de datos
	//

template <class Tipo>
inline unsigned xEstadistica<Tipo>::N ()

	{ return Elemento . Elementos() ; }

	//
	// Mtodo : SumaX
	//
	// Devuelve la suma de los elementos
	//

template <class Tipo>
Tipo xEstadistica<Tipo>::SumaX()
 {
	Tipo Resultado = 0 ; // Para almacenar la suma

	for( unsigned I = 0 ; I < Elemento . Elementos() ; I++ ) // Recorrer la matriz

		Resultado += Elemento[ I ] ; // Sumar cada elemento

	return Resultado ; // Devolver el resultado
 }

	//
	// Mtodo : SumaX2
	//
	// Devuelve la suma de los cuadrados de los elementos
	//

template <class Tipo>
Tipo xEstadistica<Tipo>::SumaX2()
 {
	Tipo Resultado = 0 ; // Para almacenar el resultado

	for( unsigned I = 0 ; I < Elemento . Elementos() ; I++ ) // Recorrer la matriz

		Resultado += Elemento[ I ] * Elemento[ I ] ; // Sumar el cuadrado

	return Resultado ; // Devolver el resultado
 }

	//
	// Mtodo : Maximo
	//
	// Devuelve el valor mximo de la lista de elementos
	//

template <class Tipo>
Tipo xEstadistica<Tipo>::Maximo()
 {
	Tipo Maximo = Elemento[ 0 ] ; // Asumir que el primero es el mximo

	for( unsigned I = 0 ; I < Elemento . Elementos() ; I++ ) // Recorrer la matriz

		if( Elemento[ I ] > Maximo ) // Si se encuentra un valor mayor

			Maximo = Elemento[ I ] ; // almacenarlo como mximo

	return Maximo ; // Devolver el mximo valor encontrado
 }

	//
	// Mtodo : Minimo
	//
	// Devuelve el valor mnimo de la lista de elementos

template <class Tipo>
Tipo xEstadistica<Tipo>::Minimo()
 {
	Tipo Minimo = Elemento[ 0 ] ; // Asumir que el primero es el minimo

	for( unsigned I = 0 ; I < Elemento . Elementos() ; I++ ) // Recorrer la matriz

		if( Elemento[ I ] < Minimo ) // Si se encuentra un valor menor

			Minimo = Elemento[ I ] ; // almacenarlo como mnimo

	return Minimo ; // Devolver el mnimo valor encontrado
 }

	//
	// Mtodo : Media
	//
	// Calcula la media de los valores y la devuelve
	//

template <class Tipo>
inline Tipo xEstadistica<Tipo>::Media()

	{ return SumaX() / N() ; }

	//
	// Mtodo : Desviacion
	//
	// Calcula la desviacin tpica y la devuelve
	//

template <class Tipo>
double xEstadistica<Tipo>::Desviacion()
 {
	Tipo Temporal = 0 ;

	for( unsigned I = 0 ; I < N() ; I++ )

			// Se suman los cuadrados de la operacin de restar
			// a cada elemento la media

		Temporal += ( Elemento[ I ] - Media() ) * ( Elemento[ I ] - Media() ) ;

	Temporal /= N() ; // Se divide entre el nmero de elementos

	return sqrt( Temporal ) ; // Se devuelve la raiz cuadrada
 }

	//
	// Mtodo : cDesviacion
	//
	// Calcula la cuasi desviacin tpica y la devuelve
	//

template <class Tipo>
double xEstadistica<Tipo>::cDesviacion()
 {
	Tipo Temporal = 0 ;

	for( unsigned I = 0 ; I < N() ; I++ )

			// Se suman los cuadrados de la operacin de restar
			// a cada elemento la media

		Temporal += ( Elemento[ I ] - Media() ) * ( Elemento[ I ] - Media() ) ;

	Temporal /= ( N() - 1 ) ; // Se divide entre el nmero de elementos menos 1

	return sqrt( Temporal ) ; // Se devuelve la raiz cuadrada
 }

	//
	// Mtodo : SumaXY
	//
	// Calcula la suma de valores teniendo en cuenta dos variables
	//

template <class Tipo>
Tipo SumaXY(  xEstadistica<Tipo> & O1,
			  xEstadistica<Tipo> & O2 )
 {
	Tipo Resultado = 0 ;

	for( unsigned I = 1 ; I <= O1 . N() ; I++ )

		Resultado += O1[ I ] * O2[ I ] ;  // Se va sumando el producto

	return Resultado ; // Se devuelve el resultado
 }

	//
	// Mtodo : Covarianza
	//
	// Realiza el clculo de la covarianza de dos variables y lo devuelve
	//

template <class Tipo>
Tipo Covarianza(  xEstadistica<Tipo> & O1,
				  xEstadistica<Tipo> & O2 )
 {
	Tipo Resultado = 0 ;

	for( unsigned I = 1 ; I <= O1 . N() ; I++ )

			// La covarianza es la suma de los cuadrados del producto
			// de cada par de elementos menos las medias

		Resultado += ( ( O1[ I ] - O1 . Media() ) * ( O2[ I ] - O2 . Media() ) ) ;

	return Resultado / O1 . N() ; // Se devuelve el resultado
 }

	//
	// Mtodo : CoefCorr
	//
	// Calcula el coeficiente de correlacin y lo devuelve
	//

template <class Tipo>
Tipo CoefCorr(  xEstadistica<Tipo> & O1,
				 xEstadistica<Tipo> & O2 )
 {
		// Se calcula y devuelve el resultado

	return Covarianza( O1, O2 ) / ( O1 . Desviacion() * O2 . Desviacion() ) ;
 }

	//
	// Mtodo : TeoricoX
	//
	// Calcula un valor terico o ideal para la variable X tomando
	// un valor dado en la variable Y
	//

template <class Tipo>
Tipo TeoricoX(  xEstadistica<Tipo> & O1,
				 xEstadistica<Tipo> & O2,  Tipo Y )
 {
		// Se valcula el valor y se devuelve

	return Covarianza( O1, O2 ) / ( O2 . Desviacion() * O2 . Desviacion() )
          * Y -
          Covarianza( O1, O2 ) / ( O2 . Desviacion() * O2 . Desviacion() )
			 * O2 . Media()
          + O1 . Media() ;
 }

	//
	// Mtodo : TeoricoY
	//
	// Calcula un valor terico o ideal para la variable Y tomando
	// un valor dado en la variable X
	//

template <class Tipo>
Tipo TeoricoY(  xEstadistica<Tipo> & O1,
				 xEstadistica<Tipo> & O2,  Tipo X )
 {
		// Se valcula el valor y se devuelve

	return Covarianza( O1, O2 ) / ( O1 . Desviacion() * O1 . Desviacion() )
			 *	X -
			 Covarianza( O1, O2 ) / ( O1 . Desviacion() * O1 . Desviacion() )
			 * O1 . Media()
          + O2 . Media() ;
 }


#endif // _GXESTADI_HPP_
