	//
	// GXMatriz.Hpp
	//
	// Define la plantilla de clase xMatriz, para tratamiento de
	// matrices matemticas, y sus mtodos correspondientes
	//
	// 7 Septiembre 1992
	//

#include "EstDat.Hpp" // Necesita matrices

#ifndef _GXMATRIZ_HPP_
#define _GXMATRIZ_HPP_

#ifndef __cplusplus
#error No se puede utilizar GXMATRIZ en C
#endif

				// Enumeracin de los errores posibles

			enum ErrxMatriz { Correcto, NoEsCuadrada, MatrizNula,
					  NoHayInversa, DistintasDimensiones,
					  OperacionInvalida } ;


template <class Tipo>
	//
	// Clase : xMatriz
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Facilitar el tratamiento de
	//			   matrices matemticas
	//
	// Fecha de creacin : 7 Septiembre
	//
	// Fecha de revisin : -
	//
class xMatriz
 {
	public :

			xMatriz() ; // Construye una matriz vaca

			xMatriz( int, int ) ; // Matriz de n filas por m columnas

			xMatriz( int ) ; // Matriz cuadrada de orden n

			xMatriz(  xMatriz & ) ; // Constructor de copia

			~xMatriz() ; // Destructor

			// Operador de asignacin

		void operator = (  xMatriz & ) ;

			// Operador de relacin de igualdad

		int operator == ( xMatriz & ) ;

			// Operador de relacin de desigualdad

		int operator != ( xMatriz & ) ;

			// Operador de insercin en stream

		friend ostream & operator << ( ostream &, xMatriz & ) ;

			// Operador de extraccin de stream

		friend istream & operator >> ( istream &, xMatriz & ) ;

			// Devuelve una referencia a un elemento de la matriz

		Tipo & operator () ( int, int ) ;

		int Filas() ; // Devuelve nmero de filas de la matriz

		int Columnas() ; // Devuelve nmero de columnas de la matriz

		int EsCuadrada() ; // Devuelve 1 si la matriz es cuadrada

		int Orden() ; // Devuelve el orden de la matriz

			// Devuelve los elementos que componen la diagonal

		Matriz<Tipo> Diagonal() ;

			// Devuelve los elementos que componen la diagonal secundaria

		Matriz<Tipo> DiagonalS() ;

			// Suma dos matrices

		friend xMatriz operator + ( xMatriz &, xMatriz & ) ;

			// Multiplica una matriz por un objeto

		friend xMatriz operator * ( xMatriz &, Tipo ) ;

			// Multiplica dos matrices

		friend xMatriz operator * ( xMatriz &, xMatriz & ) ;

			// Divide una matriz entre un objeto

		friend xMatriz operator / ( xMatriz &, Tipo ) ;

			// Devuelve la matriz traspuesta

		xMatriz Traspuesta() ;

			// Devuelve la matriz inversa

		xMatriz Inversa() ;

			// Devuelve 1 si la matriz es simtrica

		int EsSimetrica() ;

			// Calcula y devuelve el determinante de la matriz

		Tipo Determinante() ;

			// Calcula el menor complementario correspondiente a
			// una fila y una columna dadas y lo devuelve

		Tipo MenorComplementario( int, int ) ;

			// Calcula el adjunto a una fila y columna dadas y
			// lo devuelve

		Tipo Adjunto( int, int ) ;

			// Construye la matriz adjunta y la devuelve

		xMatriz Adjunta() ;

			// Devuelve la condicin de error de la ltima operacin

		ErrxMatriz Estado( ErrxMatriz = Correcto ) ;

	private :

		ErrxMatriz UltimoEstado ; // Estado de la ltima operacin

		int nFilas, nColumnas ; // Dimensiones de la matriz

		Matriz<Matriz<Tipo> > Elemento ; // Matriz con los elementos

 } ; // xMatriz

	// Implementacin de los mtodos

	//
	// Mtodo : Constructor por defecto
	//
	// Crea una matriz vaca
	//

template <class Tipo>
inline xMatriz<Tipo>::xMatriz() :

		// Se inicializan los miembros

	nFilas( 0 ), nColumnas( 0 ), UltimoEstado( Correcto )

	{ }

	//
	// Mtodo : Constructor
	//
	// Crea una matriz de F filas por C columnas
	//

template <class Tipo>
xMatriz<Tipo>::xMatriz( int F, int C ) :

			// Se inicializa miembros

		nFilas( F ), nColumnas( C ), UltimoEstado( Correcto )
 {
	for( int I = 0 ; I < F ; I++ ) // Bucle para recorrer las filas

		for( int J = 0 ; J < C ; J++ ) // Bucle para recorrer las columnas

			Elemento[ I ][ J ] = 0 ; // Se inicializan los elementos
 }

	//
	// Mtodo : Constructor
	//
	// Crea una matriz de orden O, que no es ms que una matriz
	// cuadrado, con O filas y O columnas
	//

template <class Tipo>
xMatriz<Tipo>::xMatriz( int O ) :

		nFilas( O ), nColumnas( O ), UltimoEstado( Correcto )
 {
	for( int I = 0 ; I < nFilas ; I++ ) // Bucle para recorrer las filas

		for( int J = 0 ; J < nColumnas ; J++ ) // Bucle para recorrer las columnas

			Elemento[ I ][ J ] = 0 ; // Se inicializan los elementos
 }

	//
	// Mtodo : Constructor de copia
	//
	// Crea una matriz copiando los miembros y elementos de otra
	//

template <class Tipo>
inline xMatriz<Tipo>::xMatriz(  xMatriz<Tipo> & O ) :

	nFilas( O . nFilas ), nColumnas( O . nColumnas ), UltimoEstado( Correcto ),
	Elemento( O . Elemento ) // Copiar todos los miembros

	{ }

	//
	// Mtodo : Destructor
	//

template <class Tipo>
inline xMatriz<Tipo>::~xMatriz()

	{ }

	//
	// Mtodo : Operador de asignacin
	//
	// Asigna un objeto a otro
	//

template <class Tipo>
void xMatriz<Tipo>::operator = (  xMatriz<Tipo> & O )
 {
	nFilas = O . nFilas ; // Se asignan los miembros
	nColumnas = O . nColumnas ;

	Estado( Correcto ) ;

	Elemento = Matriz<Matriz<Tipo> >(0) ; // La matriz est vaca

	for( int I = 0 ; I < nFilas ; I++ ) // Recorrer todas las filas

		for( int J = 0 ; J < nColumnas ; J++ ) // Recorrer todas las columnas

			Elemento[ I ][ J ] = O . Elemento[ I ][ J ] ; // Copiar
 }

	//
	// Mtodo : Operador relacional de igualdad
	//
	// Dos matrices son iguales cuando tienen las mismas dimensiones
	// y todos sus elementos son iguales
	//

template <class Tipo>
int xMatriz<Tipo>::operator == ( xMatriz<Tipo> & O )
 {
	return nFilas == O . nFilas && nColumnas == O . nColumnas &&
			Elemento == O . Elemento ;
 }

	//
	// Mtodo : Operador relacional de desigualdad
	//
	// Es la negacin del operador relacional de igualdad
	//

template <class Tipo>
inline int xMatriz<Tipo>::operator != ( xMatriz<Tipo> & O )
 {
	return ! operator== ( O ) ;
 }

	//
	// Mtodo : Operador de insercin en un stream
	//
	// Permite escribir una matriz en un stream
	 //

template <class Tipo>
ostream & operator << ( ostream & Stream, xMatriz<Tipo> & O )
 {
		// Escribir las dimensiones de la matriz

	Stream . write( (char * ) & O . nFilas, sizeof( int ) ) ;
	Stream . write( (char * ) & O . nColumnas, sizeof( int ) ) ;

		// Escribir todos los elementos

	for( int I = 0 ; I < O . nFilas ; I++ )

		for( int J = 0 ; J < O . nColumnas ; J++ )

			Stream . write( (char *) & O . Elemento[ I ][ J ], sizeof( Tipo ) ) ;

	return Stream ;
 }

	//
	// Mtodo : Operador de extraccin de un stream
	//
	// Permite leer una matriz desde un stream
	//

template <class Tipo>
istream & operator >> ( istream & Stream, xMatriz<Tipo> & O )
 {
		// Leer las dimensiones de la matriz

	Stream . read( (char * ) & O . nFilas, sizeof( int ) ) ;
	Stream . read( (char * ) & O . nColumnas, sizeof( int ) ) ;

		// Leer todos los elementos

	for( int I = 0 ; I < O . nFilas ; I++ )

		for( int J = 0 ; J < O . nColumnas ; J++ )

			Stream . read( (char *) & O . Elemento[ I ][ J ], sizeof( Tipo ) ) ;

	O . Estado( Correcto ) ; // El estado es inicializado

	return Stream ;
 }

	//
	// Mtodo : Operador () sobrecargado
	//
	// Devuelve una referencia al elemento indicado, permitindose
	// as la asignacin de valores
	//

template <class Tipo>
Tipo & xMatriz<Tipo>::operator () ( int F, int C )
 {
	Tipo NULO = 0 ;

	return ( F >= 1 && F <= nFilas && C >= 1 && C <= nColumnas ) ?

			Elemento[ F-1 ][ C-1 ] : // Devuelve la referencia
			NULO ; // o el objeto nulo
 }

	//
	// Mtodo : Filas
	//
	// Devuelve el nmero de filas de la matriz
	//

template <class Tipo>
inline int xMatriz<Tipo>::Filas()

	{ return nFilas ; }

	//
	// Mtodo : Columnas
	//
	// Devuelve el nmero de columnas de la matriz
	//

template <class Tipo>
inline int xMatriz<Tipo>::Columnas()

	{ return nColumnas ; }

	//
	// Mtodo : EsCuadrada
	//
	// Devuelve 1 si la matriz es cuadrada o 0 en caso contrario
	//

template <class Tipo>
inline int xMatriz<Tipo>::EsCuadrada()

		// Una matriz es cuadrada si tiene el mismo nmero de filas
		// que de columnas

	{ return nFilas == nColumnas ; }

	//
	// Mtodo : Orden
	//
	// Devuelve el orden de la matriz, siempre que sta sea cuadrada
	//

template <class Tipo>
int xMatriz<Tipo>::Orden()
 {
	if( EsCuadrada() ) // Si la matriz es cuadrada

		return nFilas ; // Devuelve el orden, que son las filas o las columnas

	// si no es cuadrada

	Estado( NoEsCuadrada ) ; // Se activa el indicador de error

	return -1 ; // Devolver valor no vlido
 }

	//
	// Mtodo : Diagonal
	//
	// Devuelve una matriz con los elementos que componen
	// la diagonal de la matriz, siempre que sta sea cuadrada
	//

template <class Tipo>
Matriz<Tipo> xMatriz<Tipo>::Diagonal()
 {
	if( ! EsCuadrada() ) // Si la matriz no es cuadrada
	 {
		Estado( NoEsCuadrada ) ; // Se activa indicador de error

		return 0 ; // valor nulo
	 }

	Matriz<Tipo> ElementoDiagonal ; // Matriz para los elementos

	for( int I = 0 ; I < nFilas ; I++ ) // Recorrer la matriz

		ElementoDiagonal[ I ] = Elemento[ I ][ I ] ; // Recoger los elementos

	return ElementoDiagonal ; // Devolver la matriz
 }

	//
	// Mtodo : DiagonalS
	//
	// Devuelve la matriz con los elementos que forman la
	// diagonal secundaria de la matriz, siempre que sea cuadrada
	//

template <class Tipo>
Matriz<Tipo> xMatriz<Tipo>::DiagonalS()
 {
	if( ! EsCuadrada() ) // Si la matriz no es cuadrada
	 {
		Estado( NoEsCuadrada ) ; // Se activa indicador de error

		return 0 ; // valor nulo
	 }

	Matriz<Tipo> ElementoDiagonal ; // Matriz para los elementos

	int I, J ;

	for( I = 0, J = nColumnas - 1 ; I < nFilas ; I++, J-- ) // Recorrer la matriz

		ElementoDiagonal[ I ] = Elemento[ I ][ J ] ; // Recoger los elementos

	return ElementoDiagonal ; // Devolver la matriz
 }

	//
	// Mtodo : Operador +
	//
	// Permite sumar dos matrices, siempre que tengan las misma dimensin
	//

template <class Tipo>
xMatriz<Tipo> operator + ( xMatriz<Tipo> & O1, xMatriz<Tipo> & O2 )
 {
		// Si las matrices no tienen la misma dimensin

	if( O1 . nFilas != O2 . nFilas || O1 . nColumnas != O2 . nColumnas )
	 {
		O1 . Estado( DistintasDimensiones ) ; // Activar indicador de error

		return 0 ; // Devolver nulo
	 }

		// Matriz donde se almacenar el resultado

	xMatriz<Tipo> Resultado( O1 . nFilas, O1 . nColumnas ) ;

	for( int I = 1 ; I <= O1 . nFilas ; I++ ) // Recorrer todas las filas

		for( int J = 1 ; J <= O1 . nColumnas ; J++ ) // y todas las columnas

				// Sumando los pares de elementos

			Resultado( I, J ) = O1( I, J ) + O2( I, J ) ;

	return Resultado ; // Devolver la matriz resultado
 }

	//
	// Mtodo : Operador *
	//
	// Multiplica un factor por la matriz
	//

template <class Tipo>
xMatriz<Tipo> operator * ( xMatriz<Tipo> & O, Tipo & N )
 {
	xMatriz Resultado( O . nFilas, O . nColumnas ) ; // Matriz resultado

	for( int I = 1 ; I <= O . nFilas ; I++ ) // Recorrer las filas

		for( int J = 1 ; J <= O . nColumnas ; J++ ) // y las columnas

				// Operar para obtener el resultado

			Resultado( I, J ) = O( I, J ) * N ;

	return Resultado ; // Devolver la matriz resultado
 }

	//
	// Mtodo : Operador *
	//
	// Multiplica dos matrices
	//

template <class Tipo>
xMatriz<Tipo> operator * ( xMatriz<Tipo> & O1, xMatriz<Tipo> & O2 )
 {
		// Si el nmero de columnas de la primera matriz no es
		// igual al nmero de filas de la segunda la operacin no es posible

	if( O1 . Columnas() != O2 . Filas() )
	 {
		O1 . Estado( OperacionInvalida ) ;
		return 0 ;
	 }

	  // Se crea la matriz donde se almacenar el resultado

	xMatriz<Tipo> Resultado( O1 . Filas(), O2 . Columnas() ) ;

		// Recorrer las filas de la primera matriz

	for( int I = 1 ; I <= O1 . Filas() ; I++ )

			// Recorrer las columnas de la segunda matriz

		for( int J = 1 ; J <= O2 . Columnas() ; J++ )
		 {
			Resultado( I, J ) = 0 ; // Inicializar elemento resultado

				// Recorrer las filas de la segunda matriz

			for( int K = 1 ; K <= O2 . Filas() ; K++ )

					// Ir multiplicando los elementos

				Resultado( I, J ) += O1( I, K ) * O2( K, J ) ;
		 }

	return Resultado ; // Devolver la matriz resultado
 }

	//
	// Mtodo : Operador /
	//
	// Divide una matriz entre un factor
	//

template <class Tipo>
xMatriz<Tipo> operator / ( xMatriz<Tipo> & O, Tipo N )
 {
	xMatriz<Tipo> Resultado( O . nFilas, O . nColumnas ) ; // Matriz resultado

	for( int I = 1 ; I <= O . nFilas ; I++ ) // Recorrer las filas

		for( int J = 1 ; J <= O . nColumnas ; J++ ) // y las columnas

				// Operar para obtener el resultado

			Resultado( I, J ) = O( I, J ) / N ;

	return Resultado ; // Devolver la matriz resultado
 }

	//
	// Mtodo : Traspuesta
	//
	// Compone la matriz traspuesta y la devuelve
	//

template <class Tipo>
xMatriz<Tipo> xMatriz<Tipo>::Traspuesta()
 {
		// La matriz traspuesta tendr tantas filas como columnas
		// tiene la original y tantas columnas como filas tiene
		// la original

	xMatriz Resultado( nColumnas, nFilas ) ;

	for( int I = 1 ; I <= nFilas ; I++ ) // Recorrer las filas

		for( int J = 1 ; J <= nColumnas ; J++ ) // y las columnas

				// Ir trasponiendo

			Resultado( J, I ) = Elemento[ I-1 ][ J-1 ] ;

	return Resultado ; // Devolver la matriz
 }

	//
	// Mtodo : Inversa
	//
	// Construye la matriz inversa y la devuelve
	//

template <class Tipo>
xMatriz<Tipo> xMatriz<Tipo>::Inversa()
 {
	if( Determinante() == 0 ) // Si el determinante es 0
	 {
		Estado( NoHayInversa ) ;  // no hay matriz inversa
		return 0 ;
	 }

	xMatriz Resultado = Adjunta() ; // Se obtiene la matriz adjunta

	Resultado = Resultado . Traspuesta() ; // Se traspone

		// Se divide la adjunta traspuesta entre el determinante
		// de la matriz original

	Resultado = Resultado / Determinante() ;

	return Resultado ;
 }

	//
	// Mtodo : EsSimetrica
	//
	// Devuelve 1 si la matriz es simtrica o 0 en caso contrario
	//

template <class Tipo>
int xMatriz<Tipo>::EsSimetrica()
 {
	xMatriz Temporal = Traspuesta() ; // Se obtiene la traspuesta

	return *this == Temporal ; // Si son iguales es porque es simtrica
 }

	//
	// Mtodo : Determinante
	//
	// Calcula el determinante de la matriz, si es cuadrada, y
	// lo devuelve
	//

template <class Tipo>
Tipo xMatriz<Tipo>::Determinante()
 {
	Tipo Resultado ;

	if( ! EsCuadrada() )   // Si la matriz no es cuadrada no hay determinante
	 {
		Estado( NoEsCuadrada ) ;
		return 0 ;
	 }

	switch( Orden() ) // Segn el orden de la matriz se utiliza un mtodo u otro
	 {
		case 2 : // Si la matriz es de orden 2

				// El determinante es la diagonal principal menos
				// la diagonal secundaria

			Resultado = Elemento[0][0] * Elemento[1][1] -
							Elemento[0][1] * Elemento[1][0] ;

			break ;

		case 3 : // Si la matriz es de orden 3

			Resultado = Elemento[0][0] * Elemento[1][1] * Elemento[2][2] +
							Elemento[0][2] * Elemento[1][0] * Elemento[2][1] +
							Elemento[0][1] * Elemento[2][0] * Elemento[1][2] -

							Elemento[0][2] * Elemento[1][1] * Elemento[2][0] -
							Elemento[0][0] * Elemento[2][1] * Elemento[1][2] -
							Elemento[0][1] * Elemento[2][2] * Elemento[1][0] ;

			break ;

		default : // Cualquier otro orden mayor

			Resultado = 0 ;

			for( int I = 1 ; I <= nFilas ; I ++ )

					// Sumar los adjuntos de la primera fila

				Resultado += Adjunto( I, 1 ) ;
	 }

	return Resultado ; // Devuelve el resultado
 }

	//
	// Mtodo : MenorComplementario
	//
	// Calcula el menor complementario de la matriz en la fila F
	// columna C, y lo devuelve
	//

template <class Tipo>
Tipo xMatriz<Tipo>::MenorComplementario( int F, int C )
 {
		// Crear una matriz en la que se elimine la fila F y columna C

	xMatriz Temporal( nFilas - 1, nColumnas - 1 ) ;

	int Fi = 1, Co = 1 ; // Fila y columna en la matriz Temporal

	for( int I = 0 ; I < nFilas ; I++ )
	 {
		if( I == (F-1) ) // Si esta es la fila a eliminar

			continue ; // no la proceses

		for( int J = 0 ; J < nColumnas ; J++ )
		 {
			if( J == (C-1) ) // Si esta es la columna a eliminar

				continue ; // no la proceses

			Temporal( Fi, Co++ ) = Elemento[ I ][ J ] ; // Copiar
		 }

		Co = 1 ;
		Fi++ ;
	 }

		// Se devuelve el menor complementario, que es el determinante
		// de la matriz creada

	return Temporal . Determinante() ;
 }

	//
	// Mtodo : Adjunto
	//
	// Calcula el adjunto de la matriz en la fila F columna C
	// y lo devuelve
	//

template <class Tipo>
Tipo xMatriz<Tipo>::Adjunto( int F, int C )
 {
		// Si al sumar la fila y la columna obtenemos un nmero impar,
		// que al dividir por dos quede resto, el adjunto es el
		// negativo del menor complementario, en caso contrario el
		// adjunto es el mismo menor complementario

	return (F+C) % 2 ? -( MenorComplementario( F, C ) ) :
								 MenorComplementario( F, C ) ;
 }

	//
	// Mtodo : Adjunta
	//
	// Construye la matriz adjunta y la devuelve
	//

template <class Tipo>
xMatriz<Tipo> xMatriz<Tipo>::Adjunta()
 {
	xMatriz Resultado( nFilas, nColumnas ) ; // Para almacenar la matriz resultado

	for( int I = 1 ; I <= nFilas ; I++ ) // Recorrer las filas

		for( int J = 1 ; J <= nColumnas ; J++ ) // y columnas de la matriz

				// Hallando el adjunto de cada elemento

			Resultado( I, J ) = Adjunto( I, J ) ;

	return Resultado ; // Se devuelve el resultado
 }

	//
	// Mtodo : Estado
	//
	// Devuelve el estado de error actual, y asigna un nuevo
	// valor, que por defecto es Correcto

template <class Tipo>
ErrxMatriz xMatriz<Tipo>::Estado( ErrxMatriz Error )
 {
	ErrxMatriz Temporal = UltimoEstado ; // Se preserva el estado actual

	UltimoEstado = Error ; // Se asigna el nuevo estado

	return Temporal ; // Se devuelve el estado
 }

#endif // _GXMATRIZ_HPP_