	//
	// Pmanip.Hpp
	//
	// Plantillas para la definicin de manipuladores con
	// uno o dos parmetros de cualquier tipo.
	//
	// Francisco Charte - Jan Agosto 1992
	//

#ifndef _PMANIP_HPP_	// Incluir una sola vez
#define _PMANIP_HPP_

#ifndef __cplusplus
#error No se puede usar Pmanip en C   // Es necesario el compilador de C++
#endif

#include <iostream.h>

	//
	// Primero se definen las plantillas para creacin de manipuladores
	// que toman un slo parmetro.
	//

	//
	// Clase : SManip
        //
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Permitir la creacin de manipuladores
	//	       que acten sobre objetos ios
	//
	// Fecha de creacin : Junio 1992
	//
	// Fecha de revisin : Agosto 1992
	//

template <class Tipo> class SManip
 {
	public :

		// El constructor recibe un puntero a una funcin, que es
		// nuestro manipulador, y un parmetro, que tendr que enviarle.
		// Almacena ambos datos en los miembros declarados
		// al efecto.

		SManip( ios & ( * F ) ( ios &, Tipo ), Tipo Par ) :

			Funcion( F ), Parametro( Par ) {}

		// Se sobrecargan los operadores de extraccin e insercin
		// para que trabajen recibiendo una referencia a un objeto
		// de esta clase, SManip, llamando a la funcin cuyo puntero est
		// almacenado en el objeto, pasndole el parmetro que
		// tambin est en el objeto.

		// Operador para utilizar el manipulador en una cadena
		// de extraccin.

		friend istream & operator>> ( istream & Stream, SManip & Objeto )

			{ return Objeto . Funcion( Stream, Objeto . Parametro ) }

		// Operador para utilizar el manipulador en una cadena
		// de insercin.

		friend ostream & operator<< ( ostream & Stream, SManip & Objeto )

			{ return Objeto . Funcion( Stream, Objeto . Parametro ) }

	private :

		// Puntero a la funcin que es el manipulador, que recibe
		// un parmetro del tipo Tipo y devuelve una referencia
		// al mismo objeto ios que recibe.

		ios & ( * Funcion ) ( ios &, Tipo ) ;

		// Parmetro a pasar

		Tipo Parametro ;
 } ;

	//
	// Clase : OManip
        //
	// Deriva de : -
	//
	// Base de : IOManip
	//
	// Finalidad : Permitir la creacin de manipuladores
	//	       a usar con objetos ostream
	//
	// Fecha de creacin : Junio 1992
	//
	// Fecha de revisin : Agosto 1992
	//

template <class Tipo> class OManip
 {
	public :

		// El constructor almacena el puntero a funcin y el
		// parmetro adicional en los miembros definidos para
		// este fin.

		OManip( ostream & ( * F ) ( ostream &, Tipo ), Tipo Par ) :

			Funcion( F ), Parametro( Par ) {}

		// Se sobrecarga el operador de insercin definiendo
		// un insertador para objetos de la clase OManip. Lo
		// que hace el operador es llamar a la funcin cuyo
		// puntero est almacenado en el objeto, pasndo como
		// parmetro el dato que tambin est en el objeto.

		friend ostream & operator<< ( ostream & Stream, OManip & Objeto )

			{ return Objeto . Funcion( Stream, Objeto . Parametro ) ; }

	private :

		// Puntero a la funcin que es el manipulador

		ostream & ( * Funcion ) ( ostream &, Tipo ) ;

		// Parmetro adicional

		Tipo Parametro ;

 } ;

	//
	// Clase : IManip
        //
	// Deriva de : -
	//
	// Base de : IOManip
	//
	// Finalidad : Permitir la creacin de manipuladores
	//	       a usar con objetos istream
	//
	// Fecha de creacin : Junio 1992
	//
	// Fecha de revisin : Agosto 1992
	//

template <class Tipo> class IManip
 {
	public :

		// El constructor es similar al de la clase OManip, recibe
		// el puntero a la funcin y el parmetro que tiene que
		// enviarle, y guarda ambos datos en los miembros privados.

		IManip( istream & ( * F ) ( istream &, Tipo ), Tipo Par ) :

			Funcion( F ), Parametro( Par ) {}

		// El operador de extraccin se sobrecarga para que cuando
		// sea llamado con un objeto de la clase IManip en realidad
		// no extraiga nada, sino que llame al manipulador pasndole
		// el parmetro.

		friend istream & operator>> ( istream & Stream, IManip & Objeto )

			{ return Objeto . Funcion( Stream, Objeto . Parametro ) ; }

	private :

		// Funcin es un puntero a una funcin que recibe una
		// referencia a un istream y un parmetro adicional.

		istream & ( * Funcion ) ( istream &, Tipo ) ;

		// Variable para guardar el parmetro

		Tipo Parametro ;
 } ;


	//
	// Clase : IOManip
        //
	// Deriva de : OManip e IManip
	//
	// Base de : -
	//
	// Finalidad : Permitir la creacin de manipuladores
	//	       para objetos de la clase iostream
	//
	// Fecha de creacin : Junio 1992
	//
	// Fecha de revisin : Agosto 1992
	//

		// Se hereda de forma pblica y mltiple de las clases
		// OManip e IManip, heredando as todos los miembros necesarios,

template <class Tipo> class IOManip : public OManip<Tipo>, public IManip<Tipo>
 {
	public :

		// Tan slo implementa el constructor, que llama a los
		// constructores de las clases base moldeando el tipo
		// del puntero a funcin.

		IOManip( iostream & ( * F ) ( iostream &, Tipo ), Tipo Par ) :

			OManip<Tipo>( ( ostream & (*)( ostream &, Tipo ) ) F, Par ),
			IManip<Tipo>( ( istream & (*)( istream &, Tipo ) ) F, Par ) {}
 } ;

	//
	// Clase : SManip2
        //
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Permitir la creacin de manipuladores
	//	       que tomen dos parmetros, a utilizar
	//	       con objetos ios.
	//
	// Fecha de creacin : Junio 1992
	//
	// Fecha de revisin : Agosto 1992
	//

template <class Tipo1, class Tipo2> class SManip2
 {
	public :

		// El constructor ahora recibe un puntero a una funcin,
		// que recibe una referencia a un objeto ios y dos parmetros,
		// y dos parmetros adicionales, que preservar en los miembros
		// privados previstos para ello.

		SManip2( ios & ( * F ) ( ios &, Tipo1, TIpo2 ), Tipo1 Par1, Tipo2 Par2 ) :

			Funcion( F ), Parametro1( Par1 ), Parametro2( Par2 ) {}

		// Cuando se encuentre un objeto SManip2 en una cadena de
		// extraccin se llamar a esta implementacin del operador,
		// que en realidad lo que har ser llamar a la funcin,
		// pasndole los dos parmetros necesarios.

		friend istream & operator>> ( istream & Stream, SManip & Objeto )

			{ return Objeto . Funcion( Stream, Objeto . Parametro1, Objeto . Parametro2 ) }

		// El operador de insercin acta de forma similar

		friend ostream & operator<< ( ostream & Stream, SManip & Objeto )

			{ return Objeto . Funcion( Stream, Objeto . Parametro1, Objeto . Parametro2 ) }

	private :

		// Puntero a una funcin que toma una referencia a un
		// objeto ios y dos objetos adicionales, devolviendo
		// la misma referencia al objeto ios.

		ios & ( * Funcion ) ( ios &, Tipo1, Tipo2 ) ;

		// Dos parmetros a tomar por el manipulador

		Tipo1 Parametro1 ;
		Tipo2 Parametro2 ;
 } ;

	//
	// Clase : OManip2
        //
	// Deriva de : -
	//
	// Base de : IOManip2
	//
	// Finalidad : Permitir la creacin de manipuladores
	//	       que tomen dos parmetros, a utilizar
	//	       con objetos de la clase ostream
	//
	// Fecha de creacin : Junio 1992
	//
	// Fecha de revisin : Agosto 1992
	//

template <class Tipo1, class Tipo2> class OManip2
 {
	public :

		// El constructor toma la direccin de la funcin y los
		// parmetros, y los guarda en los miembros privados.

		OManip2( ostream & ( * F ) ( ostream &, Tipo1, Tipo2 ), Tipo1 Par1, Tipo2 Par2 ) :

			Funcion( F ), Parametro1( Par1 ), Parametro2( Par2 ) {}

		// El operador de insercin realiza la llamada al manipulador,
		// pasndole los parmetros que el constructor guard.

		friend ostream & operator<< ( ostream & Stream, OManip2 & Objeto )

			{ return Objeto . Funcion( Stream, Objeto . Parametro1, Objeto . Parametro2 ) ; }

	private :

		// Puntero a la funcin

		ostream & ( * Funcion ) ( ostream &, Tipo1, Tipo2 ) ;

		// Parmetros a tomar por el manipulador

		Tipo1 Parametro1 ;
		Tipo2 Parametro2 ;
 } ;

	//
	// Clase : IManip2
        //
	// Deriva de : -
	//
	// Base de : IOManip2
	//
	// Finalidad : Permitir la creacin de manipuladores
	//	       que tomen dos parmetros, a utilizar
	//	       con objetos de la clase istream
	//
	// Fecha de creacin : Junio 1992
	//
	// Fecha de revisin : Agosto 1992
	//

template <class Tipo1, class Tipo2> class IManip2
 {
	public :

		// El constructor preserva los parmetros recibidos

		IManip2( istream & ( * F ) ( istream &, Tipo1, Tipo2 ), Tipo1 Par1, Tipo2 Par2 ) :

			Funcion( F ), Parametro1( Par1 ), Parametro2( Par2 ) {}

		// Y el operador de extraccin los utiliza para llamar
		// al manipulador.

		friend istream & operator>> ( istream & Stream, IManip2 & Objeto )

			{ return Objeto . Funcion( Stream, Objeto . Parametro1, Objeto . Parametro2 ) ; }

	private :

		// Miembros para guardar la direccin de la funcin

		istream & ( * Funcion ) ( istream &, Tipo1, Tipo2 ) ;

		// y los parmetros tomar por el manipulador

		Tipo1 Parametro1 ;
		Tipo2 Parametro2 ;
 } ;

	//
	// Clase : IOManip2
        //
	// Deriva de : OManip2 e IManip2
	//
	// Base de : -
	//
	// Finalidad : Permitir la creacin de manipuladores
	//	       que tomen dos parmetros, a utilizar
	//	       con objetos de la clase iostream
	//
	// Fecha de creacin : Junio 1992
	//
	// Fecha de revisin : Agosto 1992
	//

		// Deriva pblicamente y de forma mltiple, heredando
		// as los miembros necesarios para las operaciones
		// de insercin y extraccin.

template <class Tipo1, class Tipo2> class IOManip2 : public OManip2<Tipo1, Tipo2>, public IManip2<Tipo1, Tipo2>
 {
	public :

		// Al construir un objeto de esta clase se pasan los parmetros
		// a los constructores de las clases base, que son los que realmente
		// formarn el objeto

		IOManip2( iostream & ( * F ) ( iostream &, Tipo1, Tipo2 ), Tipo1 Par1, Tipo2 Par2 ) :

			OManip2<Tipo1, Tipo2>( ( ostream & (*)( ostream &, Tipo1, Tipo2 ) ) F, Par1, Par2 ),
			IManip2<Tipo1, Tipo2>( ( istream & (*)( istream &, Tipo1, Tipo2 ) ) F, Par1, Par2 ) {}
 } ;

	//
	// Clase : SApp
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Servir como "aplicador" de la
	//	       clase SManip, implementando el
	//	       operador (), que devolver la
	//	       creacin de un objeto SManip
	//	       con el parmetro adecuado.
	//
	// Fecha de creacin : Junio 1992
	//
	// Fecha de revisin : Agosto 1992
	//

template <class Tipo> class SApp
 {
	public :

		// El constructor recibe un puntero a una funcin,
		// y lo guarda en el miembro privado Funcion.

		SApp( ios & ( * F ) ( ios &, Tipo ) ) :

			Funcion( F ) {}

		// El operador () recibe un parmetro del tipo Tipo,
		// y con l y la direccin de la funcin construye
		// un objeto SManip, que devuelve para ser insertado.

		SManip<Tipo> operator() ( Tipo Par )

			{ return SManip<Tipo>( Funcion, Par ) ; }

	private :

		// Puntero a la funcin

		ios & ( * Funcion ) ( ios &, Tipo ) ;
 } ;

	//
	// Clase : IApp
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Servir como "aplicador" de la
	//	       clase IManip, implementando el
	//	       operador (), que devolver la
	//	       creacin de un objeto IManip
	//	       con el parmetro adecuado.
	//
	// Fecha de creacin : Junio 1992
	//
	// Fecha de revisin : Agosto 1992
	//

template <class Tipo> class IApp
 {
	public :

		// Guarda la direccin de la funcin al crear
		// el objeto.

		IApp( istream & ( * F ) ( istream &, Tipo ) ) :

			Funcion( F ) {}

		// Implementa el operador (), que al ser llamado
		// con un parmetro construye un objeto IManip
		// y lo devuelve.

		IManip<Tipo> operator() ( Tipo Par )

			{ return IManip<Tipo>( Funcion, Par ) ; }

	private :

		// Miembro para guardar la direccin de la funcin

		istream & ( * Funcion ) ( istream &, Tipo ) ;
 } ;

	//
	// Clase : OApp
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Servir como "aplicador" para los
	//	       manipuladores creados con OManip.
	//
	// Fecha de creacin : Junio 1992
	//
	// Fecha de revisin : Agosto 1992
	//

template <class Tipo> class OApp
 {
	public :

		// Direccin del manipulador

		OApp( ostream & ( * F ) ( ostream &, Tipo ) ) :

			Funcion( F ) {}

		// El operador () recibe el parmetro a pasar

		OManip<Tipo> operator() ( Tipo Par )

			{ return OManip<Tipo>( Funcion, Par ) ; }

	private :

		// Miembro para guardar la direccin del manipulador

		ostream & ( * Funcion ) ( ostream &, Tipo ) ;
 } ;

	//
	// Clase : IOApp
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : La de las dos clases anteriores combinada
	//
	// Fecha de creacin : Junio 1992
	//
	// Fecha de revisin : Agosto 1992
	//

template <class Tipo> class IOApp
 {
	public :

		IOApp( iostream & ( * F ) ( iostream &, Tipo ) ) :

			Funcion( F ) {}

		IOManip<Tipo> operator() ( Tipo Par )

			{ return IOManip<Tipo>( Funcion, Par ) ; }

	private :

		iostream & ( * Funcion ) ( iostream &, Tipo ) ;
 } ;

	//
	// Clase : SApp2
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Igual que SApp pero para los
	//	       manipuladores que toman dos parmetros
	//
	// Fecha de creacin : Junio 1992
	//
	// Fecha de revisin : Agosto 1992
	//

template <class Tipo1, Tipo2> class SApp2
 {
	public :

		// Puntero a funcin que toma una referencia ios y
		// otros dos parmetros.

		SApp2( ios & ( * F ) ( ios &, Tipo1, Tipo2 ) ) :

			Funcion( F ) {} // Se guarda la direccin

		// El operador () recibe dos parmetros, y con ellos
		// y la direccin almacenada en Funcion construye
		// un objeto SManip2 y lo devuelve.

		SManip2<Tipo1, Tipo2> operator() ( Tipo1 Par1, Tipo2 Par2 )

			{ return SManip2<Tipo1, Tipo2>( Funcion, Par1, Par2 ) ; }

	private :

		// Para guardar la direccin de la funcin

		ios & ( * Funcion ) ( ios &, Tipo1, Tipo2 ) ;
 } ;

	//
	// Clase : IApp2
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Como IApp pero para los manipuladores
	//	       que toman dos parmetros
	//
	// Fecha de creacin : Junio 1992
	//
	// Fecha de revisin : Agosto 1992
	//

template <class Tipo1, class Tipo2> class IApp2
 {
	public :

		// Al crear el objeto se recibe un puntero a una funcin

		IApp2( istream & ( * F ) ( istream &, Tipo1, Tipo2 ) ) :

			Funcion( F ) {} // que se guarda en el miembro privado

		// El operador () es el encargado de construir un objeto
		// IManip2 con los datos recibidos y la direccin de
		// la funcin almacenada anteriormente.

		IManip2<Tipo1, Tipo2> operator() ( Tipo1 Par1, Tipo2 Par2 )

			{ return IManip2<Tipo1, Tipo2>( Funcion, Par1, Par2 ) ; }

	private :

		// Miembro que almacena la direccin de la funcin

		istream & ( * Funcion ) ( istream &, Tipo1, Tipo2 ) ;
 } ;

	//
	// Clase : OApp2
	//
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Como OApp2 aplicndose a los
	//	       manipuladores que toman dos parmetros
	//
	// Fecha de creacin : Junio 1992
	//
	// Fecha de revisin : Agosto 1992
	//

template <class Tipo1, class Tipo2> class OApp2
 {
	public :

		// Se recibe un puntero al manipulador y se guarda

		OApp2( ostream & ( * F ) ( ostream &, Tipo1, Tipo2 ) ) :

			Funcion( F ) {}

		// Cuando se realice la llamada con el operador ()
		// crear el objeto OManip2 y devolverlo.

		OManip2<Tipo1, Tipo2> operator() ( Tipo1 Par1, Tipo2 Par2 )

			{ return OManip2<Tipo1, Tipo2>( Funcion, Par1, Par2 ) ; }

	private :

		ostream & ( * Funcion ) ( ostream &, Tipo1, Tipo2 ) ;
 } ;

	//
	// Clase : IOApp2
        //
	// Deriva de : -
	//
	// Base de : -
	//
	// Finalidad : Como IOApp2 aplicndose a los
	//	       manipuladores que toman dos parmetros
	//
	// Fecha de creacin : Junio 1992
	//
	// Fecha de revisin : Agosto 1992
	//

template <class Tipo1, class Tipo2> class IOApp2
 {
	public :

		// Recibe el la direccin del manipulador

		IOApp2( iostream & ( * F ) ( iostream &, Tipo1, Tipo2 ) ) :

			Funcion( F ) {} // La guarda

		// El operador () recibe los dos parmetros, y con ellos
		// y la direccin de la funcin construye el objeto
		// IOManip2 y lo devuelve.

		IOManip2<Tipo1, Tipo2> operator() ( Tipo1 Par1, Tipo2 Par2 )

			{ return IOManip2<Tipo1, Tipo2>( Funcion, Par1, Par2 ) ; }

	private :

		iostream & ( * Funcion ) ( iostream &, Tipo1, Tipo2 ) ;
 } ;

#endif // _PMANIP_HPP_

