/*******************************************************************************
 *------------------------------------------------------------------------------
 *
 *      FICHERO:
 *              GLISTA.CPP
 *
 *------------------------------------------------------------------------------
 ******************************************************************************/
/*******************************************************************************
 *------------------------------------------------------------------------------
 *
 *      PROFILE INFORMATION:
 *
 *      OBJECT MODIFICATION DATE:
 *              Tue May 14 10:31:15 1996
 *
 *------------------------------------------------------------------------------
 ******************************************************************************/

/*******************************************************************************
 *
 *      INCLUDES:
 *
 ******************************************************************************/
#include <stdio.h>

#include "glista.hpp"
#include "tracelog.hpp"


/*******************************************************************************
 *
 *      VARIABLES EXPORTADAS:
 *
 ******************************************************************************/
#define GLISTA_LOG             ".\\GLISTA"


/*******************************************************************************
 *
 *      FUNCTION:
 *              int DatoNameCmp( const void* p1, const void* p2 )
 *
 ******************************************************************************/
int DatoNameCmp( const void* p1, const void* p2 )
{
	return strcmp( ((DatoName*)p1)->name, ((DatoName*)p2)->name );
}


/*******************************************************************************
 *
 *      CONSTRUCTOR:
 *              GLista::GLista()
 *
 ******************************************************************************/
GLista::GLista():p( 0 )
{ 
	numLines = 0;
	numColus = 0;
	numDatos = 0;
	currLine = 1;
	currDato = 0;
	name[0] = '\0';
	selectedHandler = NULL;

	for( unsigned int i = 0; i < MAX_DATOS_NUM; i++ )
	{
		datos[i] = NULL;
		lista[i] = NULL;
	}
}


/*******************************************************************************
 *
 *      CONSTRUCTOR:
 *              GLista::GLista( unsigned int numC,
 *				unsigned int numL,
 *                              GVentana* v,
 *                              GES* p,
 *				SelectedHandler *handler )
 *
 ******************************************************************************/
GLista::GLista( unsigned int numC,
		unsigned int numL,
		char* n,
		GES* p,
		SelectedHandler *handler )
:p( p )
{
	unsigned int            i = 0;


	numLines = numL;
	numColus = numC;
	numDatos = 0;
	currLine = 1;
	currDato = 0;
	istrcpy( &name[0], n );
	selectedHandler = handler;

	for( i = 0; i < MAX_DATOS_NUM; i++ )
	{
		datos[i] = NULL;
		lista[i] = NULL;
	}
}


/*******************************************************************************
 *
 *      CONSTRUCTOR:
 *              GLista::GLista( const GLista& O )
 *
 ******************************************************************************/
GLista::GLista( const GLista& O )
{ 
	unsigned int            i = 0;



	p = O.p;
	numLines = O.numLines;
	numColus = O.numColus;
	currLine = O.currLine;
	currDato = O.currDato;
	istrcpy( &name[0], &O.name[0] );
	selectedHandler = O.selectedHandler;

	for( i = 0; i < numDatos; i++ )
	{
		delete[] datos[i];
		datos[i] = NULL;
		delete[] lista[i];
		lista[i] = NULL;
	}
	numDatos = O.numDatos;

	for( i = 0; i < numDatos; i++ )
	{
		if( ( datos[i] = new char[MAX_DATOS_LEN] ) == NULL )
		{
			TraceLog( GLISTA_LOG, "GLista::GLista( const GLista& ): newErr" );
			break;
		}
		istrcpy( datos[i], O.datos[i] );
		if( ( lista[i] = new char[MAX_DATOS_LEN] ) == NULL )
		{
			TraceLog( GLISTA_LOG, "GLista::GLista( const GLista& ): newErr" );
			break;
		}
		istrcpy( lista[i], O.lista[i] );
	}
	for( ; i < MAX_DATOS_NUM; i++ )
	{
		datos[i] = O.datos[i];
		lista[i] = O.lista[i];
	}
}


/*******************************************************************************
 *
 *      DESTRUCTOR:
 *              GLista::~GLista()
 *
 ******************************************************************************/
GLista::~GLista()
{
	unsigned int    i = 0;


	for( i = 0; i < numDatos; i++ )
	{
		delete[] datos[i];
		datos[i] = NULL;
		delete[] lista[i];
		lista[i] = NULL;
	}
}


/*******************************************************************************
 *
 *      OPERATOR:
 *              void GLista::operator = ( const GLista& O )
 *
 ******************************************************************************/
void GLista::operator = ( const GLista& O )
{
	unsigned int            i = 0;



	p = O.p;
	numLines = O.numLines;
	numColus = O.numColus;
	currLine = O.currLine;
	currDato = O.currDato;
	istrcpy( &name[0], &O.name[0] );
	selectedHandler = O.selectedHandler;

	for( i = 0; i < numDatos; i++ )
	{
		delete[] datos[i];
		datos[i] = NULL;
		delete[] lista[i];
		lista[i] = NULL;
	}
	numDatos = O.numDatos;

	for( i = 0; i < numDatos; i++ )
	{
		if( ( datos[i] = new char[MAX_DATOS_LEN] ) == NULL )
		{
			TraceLog( GLISTA_LOG, "GLista::operator = ( const GLista& ): newErr" );
			break;
		}
		istrcpy( datos[i], O.datos[i] );

		if( ( lista[i] = new char[MAX_DATOS_LEN] ) == NULL )
		{
			TraceLog( GLISTA_LOG, "GLista::operator = ( const GLista& ): newErr" );
			break;
		}
		istrcpy( lista[i], O.lista[i] );
	}
	for( ; i < MAX_DATOS_NUM; i++ )
	{
		datos[i] = O.datos[i];
		lista[i] = O.lista[i];
	}
}


/*******************************************************************************
 *
 *      OPERATOR:
 *              void GLista::operator == ( const GLista& O )
 *
 ******************************************************************************/
int GLista::operator == ( const GLista& O )
{
	unsigned int            i = 0;



	if( !( p == O.p ) ) {
		return 0;
	}
	if( numLines != O.numLines ) {
		return 0;
	}
	if( numColus != O.numColus ) {
		return 0;
	}
	if( numDatos != O.numDatos ) {
		return 0;
	}
	if( currLine != O.currLine ) {
		return 0;
	}
	if( currDato != O.currDato ) {
		return 0;
	}
	if( istrcmp( &name[i], &O.name[i] ) )
	{
		return 0;
	}
	if( selectedHandler != O.selectedHandler )
	{
		return 0;
	}
	for( i = 0; i < numDatos; i++ )
	{
		if( istrcmp( datos[i], O.datos[i] ) )
		{
			return 0;
		}
		if( istrcmp( lista[i], O.lista[i] ) )
		{
			return 0;
		}
	}
	for( ; i < MAX_DATOS_NUM; i++ )
	{
		if( datos[i] != O.datos[i] )
		{
			return 0;
		}
		if( lista[i] != O.lista[i] )
		{
			return 0;
		}
	}
	return 1;
}

  
/*******************************************************************************
 *
 *      METODO:
 *              int GLista::Actualiza( XMSarray<char> *pClassNames, long numLista )
 *
 ******************************************************************************/
int GLista::Actualiza( XMSarray<char> *pClassNames, long numLista )
{
	long            i = 0;
	long            j = 0;
	long            l = 0;



	for( i = 0; i < numDatos; i++ )
	{
		delete[] datos[i];
		datos[i] = NULL;
		delete[] lista[i];
		lista[i] = NULL;
	}
	(*p)( &name[0] )->rdbuf()->Limpia();

	if( (unsigned int)numLista <= MAX_DATOS_NUM )
	{
		numDatos = (unsigned int)numLista;
	}
	else
	{
		numDatos = MAX_DATOS_NUM; 
	}
	currDato = 0;
	currLine = 1;

	for( i = 0; i < numDatos; i++ )
	{
		if( ( datos[i] = new char[MAX_DATOS_LEN] ) == NULL )
		{
			TraceLog( GLISTA_LOG, "GLista::Actualiza(): newErr" );
			return 0;
		}
		if( ( lista[i] = new char[MAX_DATOS_LEN] ) == NULL )
		{
			TraceLog( GLISTA_LOG, "GLista::Actualiza(): newErr" );
			return 0;
		}
		for( j = 0; j < MAX_DATOS_LEN; j++, l++ )
		{
			datos[i][j] = pClassNames->at( l );
			lista[i][j] = pClassNames->at( l );
		}

		j = strlen( lista[i] );

		if( j < numColus )
		{
			for( ; j < numColus; j++ )
			{
				lista[i][j] = ' ';
			}
		}
		lista[i][numColus] = '\0';
	}
	for( i = 0; i < numDatos; i++, currLine++ )
	{
		*(*p)( &name[0] ) << PosCursor( 1, currLine ) << lista[i];

		if( currLine >= numLines )
		{
			break;
		}
	}
	currLine = 1;

	return 1;
}


/*******************************************************************************
 *
 *      METODO:
 *              int GLista::Actualiza( char *pLista[MAX_DATOS_LEN], long numLista )
 *
 ******************************************************************************/
int GLista::Actualiza( char *pLista[MAX_DATOS_LEN], long numLista )
{
	unsigned int           	i = 0;
	unsigned int 		j = 0;
	DatoName		*pDN;


	for( i = 0; i < numDatos; i++ )
	{
		delete[] datos[i];
		datos[i] = NULL;
		delete[] lista[i];
		lista[i] = NULL;
	}
	(*p)( &name[0] )->rdbuf()->Limpia();

	if( (unsigned int)numLista <= MAX_DATOS_NUM )
	{
		numDatos = (unsigned int)numLista;
	}
	else
	{
		numDatos = MAX_DATOS_NUM; 
	}
	currDato = 0;
	currLine = 1;

	if( ( pDN = new DatoName[numDatos] ) == NULL )
	{
		TraceLog( GLISTA_LOG, "GLista::Actualiza(): newErr" );
		return 0;
	}
	for( i = 0; i < numDatos; i++ )
	{
		strcpy( pDN[i].name, pLista[i] );
	}
	qsort( pDN, numDatos, sizeof(DatoName), DatoNameCmp );

	for( i = 0; i < numDatos; i++ )
	{
		if( ( datos[i] = new char[MAX_DATOS_LEN] ) == NULL )
		{
			TraceLog( GLISTA_LOG, "GLista::Actualiza(): newErr" );
			return 0;
		}
		if( ( lista[i] = new char[MAX_DATOS_LEN] ) == NULL )
		{
			TraceLog( GLISTA_LOG, "GLista::Actualiza(): newErr" );
			return 0;
		}
		strcpy( datos[i], pDN[i].name );
		strcpy( lista[i], pDN[i].name );

		j = strlen( lista[i] );

		if( j < numColus )
		{
			for( ; j < numColus; j++ )
			{
				lista[i][j] = ' ';
			}
		}
		lista[i][numColus] = '\0';
	}
	for( i = 0; i < numDatos; i++, currLine++ )
	{
		*(*p)( &name[0] ) << PosCursor( 1, currLine ) << lista[i];

		if( currLine >= numLines )
		{
			break;
		}
	}
	currLine = 1;

	delete [] pDN;

	return 1;
}


/*******************************************************************************
 *
 *      METODO:
 *              int GLista::Actualiza( XMSarray<long> *pLista, long numLista )
 *
 ******************************************************************************/
int GLista::Actualiza( XMSarray<long> *pLista, long numLista )
{
	unsigned int           	i = 0;
	unsigned int 		j = 0;
	long			index = 0;
	char			buf[MAX_DATOS_LEN];



	for( i = 0; i < numDatos; i++ )
	{
		delete[] datos[i];
		datos[i] = NULL;
		delete[] lista[i];
		lista[i] = NULL;
	}
	(*p)( &name[0] )->rdbuf()->Limpia();

	if( (unsigned int)numLista <= MAX_DATOS_NUM )
	{
		numDatos = (unsigned int)numLista;
	}
	else
	{
		numDatos = MAX_DATOS_NUM; 
	}
	currDato = 0;
	currLine = 1;

	for( i = 0; i < numDatos; i++ )
	{
		if( ( datos[i] = new char[MAX_DATOS_LEN] ) == NULL )
		{
			TraceLog( GLISTA_LOG, "GLista::Actualiza(): newErr" );
			return 0;
		}
		if( ( lista[i] = new char[MAX_DATOS_LEN] ) == NULL )
		{
			TraceLog( GLISTA_LOG, "GLista::Actualiza(): newErr" );
			return 0;
		}
		index = (*pLista)[i];
		strcpy( datos[i], ltoa( index, buf, 10 ) );
		strcpy( lista[i], ltoa( index, buf, 10 ) );

		j = strlen( lista[i] );

		if( j < numColus )
		{
			for( ; j < numColus; j++ )
			{
				lista[i][j] = ' ';
			}
		}
		lista[i][numColus] = '\0';
	}
	for( i = 0; i < numDatos; i++, currLine++ )
	{
		*(*p)( &name[0] ) << PosCursor( 1, currLine ) << lista[i];

		if( currLine >= numLines )
		{
			break;
		}
	}
	currLine = 1;

	return 1;
}


/*******************************************************************************
 *
 *      METODO:
 *              int GLista::ProcesaMensaje( Evento& suceso )
 *
 ******************************************************************************/
int GLista::ProcesaMensaje( Evento& suceso )
{
	// Si el evento fue producido por el teclado
	if( suceso.Quien == Evento::Teclado )
	{
		// veamos que tecla se puls
		switch( suceso.Tecla ) 
		{
			case GTeclado::ARRIBA: // Cursor arriba
 
				// No estamos en el primero
				if( currDato > 0 )
				{
					// Ir al anterior
					currDato--;
 
					// No es la primera lnea
					if( currLine > 1 )
					{
						// Ir a la anterior
						currLine--;
					}
					// Estamos al principio de la ventana
					else 
					{
						// Desplazar el contenido actual
						(*p)( &name[0] )->rdbuf()->DespAbajo();
					}
				}
			break;

			case GTeclado::ABAJO : // Cursor abajo

				// Si no estamos en el ltimo.
				if( currDato < numDatos - 1 )
				{ 
					// Ir al siguiente
					currDato++ ;
 
					// No es la ltima lnea
					if( currLine < numLines )
					{
						// ir a la siguiente
						currLine++;
					}
					// Estamos en el fondo de la ventana
					else
					{
						// Desplazar el contenido
						(*p)( &name[0] )->rdbuf()->DespArriba();
					}
				}
			break;

			case GTeclado::INTRO:
 
				if( selectedHandler != NULL )
				{
					selectedHandler->Selected( currDato );
				}
				else	
				{
					return 1;
				}

			case GTeclado::TAB:
 
				// Activar indicador de salida del programa
				return 1;

		}
	}
	// El evento ha sido producido por el ratn
	else
	{
		// Si se puls el botn derecho
		if( suceso.Boton == Evento::ClickD )
		{
		}
		// Si el botn pulsado fue el izquierdo
		else
		{
			// Veamos en qu ventana estaba el cursor
			GVentana* cual = (*p)( p->Ventana( 	suceso.Posicion.X / 8 + 1,
							 	suceso.Posicion.Y / 8 + 1 ) );

			// Si estaba en la ventana de la lista
			if( cual == (*p)( &name[0] ) ) 
			{
				// Convertir la lnea a ordenada de ventana
				int linea = suceso.Posicion.Y / 8 + 1 - cual->rdbuf()->Lin();

				// Si la lnea queda en los bordes de la ventana
				if( linea < 1 || linea > cual->rdbuf()->Lins() )
				{
					// no procesar el evento
					return 0;
				}
				// Desplazarse hasta el elemnto marcado si es que hay
				if( currDato + ( linea - currLine  ) < numDatos )
				{
					currDato += ( linea - currLine );
					currLine = linea;
					if( selectedHandler != NULL )
					{
						selectedHandler->Selected( currDato );
					}
				}
			}
			// Estaba en la ventana de archivos
			else
			{
				return 1;
			}

		}
	}
	return 0;
}


/*******************************************************************************
 *
 *      METODO:
 *              int GLista::Ejecuta( void )
 *
 ******************************************************************************/
int GLista::Ejecuta( void )
{
	GEvento         control;        // Para leer eventos
	Evento          suceso;
	int             salida = 0;     // Indicador de salida



	// Mientras no se active el indicador de salida
	while( !salida )
	{
		// Si hay directorios en la lista
		if( currDato < numDatos )
		{
			// Destaca el dato actual del resto, mostrndolo en vdeo inverso
			*(*p)( &name[0] )       << TintaA( GVenBuf::Blanco ) << FondoA( GVenBuf::Negro )
						<< PosCursor( 1, currLine )
						<< (char *)lista[currDato];

		}
		// Actualizar el aspecto de la pantalla
		p->Refresca();

		// Captura un evento
		do { control >> suceso ; } while( suceso . Quien == Evento::Vacio ) ;

		// Si hay datos en la lista se destac el actual
		if( currDato < numDatos )
		{       
			// Devolver el aspecto normal
			*(*p)( &name[0] )       << TintaA( GVenBuf::Negro ) << FondoA( GVenBuf::GrisClaro )
						<< PosCursor( 1, currLine )
						<< (char *)lista[currDato];

		}
		// Procesar el evento capturado
		salida = ProcesaMensaje( suceso );
	}
	return 0;
}


/*******************************************************************************
 *
 *      METODO:
 *              char* GLista::GetDato( char* buf )
 *
 ******************************************************************************/
char* GLista::GetDato( char* buf )
{
	strcpy( buf, datos[currDato] );

	return buf;
}


/*******************************************************************************
 *------------------------------------------------------------------------------
 *
 *      FIN DE FICHERO
 *
 *------------------------------------------------------------------------------
 ******************************************************************************/

