/*******************************************************************************
 *------------------------------------------------------------------------------
 *
 *	FICHERO:
 *		GRAFI3D.HPP
 *
 *------------------------------------------------------------------------------
 ******************************************************************************/
/*******************************************************************************
 *------------------------------------------------------------------------------
 *
 * 	PROFILE INFORMATION:
 *
 * 	OBJECT MODIFICATION DATE:
 *		Mon May 06 10:33:20 1996
 *
 *------------------------------------------------------------------------------
 ******************************************************************************/
#ifndef _GRAFI3D_HPP_
#define _GRAFI3D_HPP_


#ifndef __cplusplus
#error No se puede utilizar Grafi3D en C
#endif


/*******************************************************************************
 *
 *	INCLUDES:
 *
 ******************************************************************************/
#include <graphics.h>
#include <string.h>
#include <iostream.h>

#include "doserror.h"
#include "xmsarray.h"
#include "grafico.hpp"

#include "gventana.hpp"
#include "qstruct.h"
#include "vstruct.h"

	// Si el modelo de memoria seleccionado permite varios segmentos de datos
	// habr que utilizar las versiones far de algunas funciones

#if defined(__LARGE__) || defined(__HUGE__) || defined(__COMPACT__)

	#define istrcpy( orig, dest )	_fstrcpy( (orig), (dest) )
	#define istrcmp( orig, dest )	_fstrcmp( (orig), (dest) )
	#define istrcat( orig, dest )	_fstrcat( (orig), (dest) )

#else

	#define istrcpy( orig, dest )	strcpy( (orig), (dest) )
	#define istrcmp( orig, dest )	strcmp( (orig), (dest) )
	#define istrcat( orig, dest )	strcat( (orig), (dest) )

#endif


/*******************************************************************************
 *
 *	DEFINES:
 *
 ******************************************************************************/
#define PI	3.14159


/*******************************************************************************
 *
 *	PROTOTIPOS:
 *
 ******************************************************************************/
float* BBoxToA( float p[8][3], boundbox_t b );
float* BBoxToA( float p[8][3], shortbox_t b );
GLine* BBoxToA( GLine g[24], float p[8][3] );


/*******************************************************************************
 *
 *	CLASS:
 *		Grafi3D:
 *
 ******************************************************************************/
class Grafi3D: public Grafico
{
    public:

				Grafi3D();

				Grafi3D( GVentana* v );

				Grafi3D( const Grafi3D& R );

	void			operator = ( const Grafi3D& R );

	int			operator == ( const Grafi3D& R );

	Grafi3D&		operator * ( float r );

				// Mueve la representacion.
	void			Mover( Direccion direccion, int flag = 0 );

	void			Actualiza(	GVenBuf::Color color,
						XMSarray<dleaf_t> *XL,
						long maxXLs,
						XMSarray<long> *XLI = NULL,
						long maxXLIs = 0,
						dleaf_t *pCenter = NULL );
 
	void			Actualiza(	GVenBuf::Color color,
						XMSarray<dhull_t>* XH,
						long maxXHs );
 
	void			Actualiza(	GVenBuf::Color color,
						XMSarray<node_t>* XN );
 
	void			Actualiza(	GVenBuf::Color color,
						XMSarray<GLine>* GP,
						long numGPS,
						dleaf_t *pCenter = NULL );

	void			Vertices(	GVenBuf::Color color,
						XMSarray<GLine>* GP,
						long numGPS );

    private:

				// factor de perspectiva angular.
	float			d;

				// angulo de deriva en radianes.
	double			r1;

				// angulo de balanza en radianes.
	double			r2;

				// angulo de caida en radianes.
	double			r3;

				// factores de rotacion de seno.
	double			sr1, sr2, sr3;

				// factores de rotacion de coseno.
	double			cr1, cr2, cr3;

				// posicion del punto de vista.
	float			mx, my, mz;

	void 			Rotacion( void );

	void 			Calcula3D(	float *sx, float *sy,
						float x, float y, float z );
};

#endif // _GRAFI3D_HPP_

/*******************************************************************************
 *------------------------------------------------------------------------------
 *
 *	FIN DE FICHERO
 *
 *------------------------------------------------------------------------------
 ******************************************************************************/

