/*******************************************************************************
 *------------------------------------------------------------------------------
 *
 *      FICHERO:
 *              GRAFICO.CPP
 *
 *------------------------------------------------------------------------------
 ******************************************************************************/
/*******************************************************************************
 *------------------------------------------------------------------------------
 *
 *      PROFILE INFORMATION:
 *
 *      OBJECT MODIFICATION DATE:
 *              Mon May 06 10:43:15 1996
 *
 *------------------------------------------------------------------------------
 ******************************************************************************/

/*******************************************************************************
 *
 *      INCLUDES:
 *
 ******************************************************************************/
#include <stdio.h>
#include <math.h>

#include <graton.hpp>

#include "grafico.hpp"
#include "tracelog.hpp"


/*******************************************************************************
 *
 *      VARIABLES GLOBALES:
 *
 ******************************************************************************/
float anchura = 0.0;
float altura = 0.0;


/*******************************************************************************
 *
 *      FUNCTION:
 *              void GetAspectRatio( void )
 *
 ******************************************************************************/
void GetAspectRatio( void )
{
	int             an = 0;
	int             al = 0;

	// Obtenemos la relacion ancho/alto.
	getaspectratio( &an, &al );

	anchura = (float)an;
	altura = (float)al;

}


/*******************************************************************************
 *
 *      FUNCTION:
 *              float* BBoxToA( float p[4][2], float bbox[2][3], Plano plano )
 *
 ******************************************************************************/
float* BBoxToA( float p[4][2], float bbox[2][3], Plano plano )
{
	switch( plano )
	{
		case X0:
			p[0][0] = bbox[1][1];
			p[0][1] = bbox[1][2];
			p[1][0] = bbox[0][1];
			p[1][1] = bbox[1][2];
			p[2][0] = bbox[0][1];
			p[2][1] = bbox[0][2];
			p[3][0] = bbox[1][1];
			p[3][1] = bbox[0][2];
		return &p[0][0];

		case Y0:
			p[0][0] = bbox[0][0];
			p[0][1] = bbox[1][2];
			p[1][0] = bbox[1][0];
			p[1][1] = bbox[1][2];
			p[2][0] = bbox[1][0];
			p[2][1] = bbox[0][2];
			p[3][0] = bbox[0][0];
			p[3][1] = bbox[0][2];
		return &p[0][0];

		case Z0:
		default:
			p[0][0] = bbox[0][0];
			p[0][1] = bbox[0][1];
			p[1][0] = bbox[1][0];
			p[1][1] = bbox[0][1];
			p[2][0] = bbox[1][0];
			p[2][1] = bbox[1][1];
			p[3][0] = bbox[0][0];
			p[3][1] = bbox[1][1];
		return &p[0][0];

	}
}


/*******************************************************************************
 *
 *      FUNCTION:
 *              float* BBoxToA( float p[4][2], boundbox_t b, Plano plano )
 *
 ******************************************************************************/
float* BBoxToA( float p[4][2], boundbox_t b, Plano plano )
{
	float                   bbox[2][3];



	bbox[0][0] = b.min.x;
	bbox[0][1] = b.min.y;
	bbox[0][2] = b.min.z;

	bbox[1][0] = b.max.x;
	bbox[1][1] = b.max.y;
	bbox[1][2] = b.max.z;

	return BBoxToA( p, bbox, plano );

}



/*******************************************************************************
 *
 *      FUNCTION:
 *              float* BBoxToA( float p[4][2], shortbox_t b, Plano plano )
 *
 ******************************************************************************/
float* BBoxToA( float p[4][2], shortbox_t b, Plano plano )
{
	float                   bbox[2][3];



	bbox[0][0] = (float)b.min.x;
	bbox[0][1] = (float)b.min.y;
	bbox[0][2] = (float)b.min.z;

	bbox[1][0] = (float)b.max.x;
	bbox[1][1] = (float)b.max.y;
	bbox[1][2] = (float)b.max.z;

	return BBoxToA( p, bbox, plano );

}



/*******************************************************************************
 *
 *      CONSTRUCTOR:
 *              Grafico::Grafico()
 *
 ******************************************************************************/
Grafico::Grafico():v( 0 )
{ 
	razon = 0.05;

	GetAspectRatio();
}


/*******************************************************************************
 *
 *      CONSTRUCTOR:
 *              Grafico::Grafico( GVentana* v )
 *
 ******************************************************************************/
Grafico::Grafico( GVentana* v ):v( v )
{
	razon = 0.05;

	GetAspectRatio();

	origenX = Ventana()->rdbuf()->Cols()*8/2; 
	origenY = Ventana()->rdbuf()->Lins()*8/2;
}


/*******************************************************************************
 *
 *      CONSTRUCTOR:
 *              Grafico::Grafico( const Grafico& O )
 *
 ******************************************************************************/
Grafico::Grafico( const Grafico& O )
{ 
	razon = O.razon;

	GetAspectRatio();

	v = O.v;
	origenX = O.origenX;
	origenY = O.origenY;
}


/*******************************************************************************
 *
 *      OPERATOR:
 *              void Grafico::operator = ( const Grafico& O )
 *
 ******************************************************************************/
void Grafico::operator = ( const Grafico& O )
{
	razon = O.razon;
	v = O.v;
	origenX = O.origenX;
	origenY = O.origenY;
}


/*******************************************************************************
 *
 *      OPERATOR:
 *              void Grafico::operator == ( const Grafico& O )
 *
 ******************************************************************************/
int Grafico::operator == ( const Grafico& O )
{
	if( razon != O.razon )
	{
		return 0;
	}
	if( !( v == O.v ) )
	{
		return 0;
	}
	if( origenX != O.origenX || origenY != O.origenY )
	{
		return 0;
	}
	return 1;
}

  
/*******************************************************************************
 *
 *      OPERATOR:
 *              Grafico& Grafico::operator * ( float r )
 *
 ******************************************************************************/
Grafico& Grafico::operator * ( float r )
{
	razon = razon*r;

	return *this;
}

  
/*******************************************************************************
 *
 *      METODO:
 *              void Grafico::Actualiza(        GVenBuf::Color color,
 *                                              Plano plano,
 *                                              XMSarray<dleaf_t>* XL,
 *                                              long maxXLs )
 *
 ******************************************************************************/
void Grafico::Actualiza(        GVenBuf::Color color,
				Plano plano,
				XMSarray<dleaf_t>* XL,
				long maxXLs )
{
	struct viewporttype             estadoPantalla;
	long                            i = 0;
	dleaf_t                         dleaf;



	// Se preserva el rea establecida actualmente
	getviewsettings( &estadoPantalla );

	// Se delimita el rea de pantalla donde se llevar a cabo la representacin
	setviewport(    ( Ventana() -> rdbuf() -> Col() ) * 8,
			( Ventana() -> rdbuf() -> Lin() ) * 8,
			( Ventana() -> rdbuf() -> Col() + Ventana() -> rdbuf() -> Cols() ) * 8,
			( Ventana() -> rdbuf() -> Lin() + Ventana() -> rdbuf() -> Lins() ) * 8,
			1) ;

	// Para todos los elementos de la matriz...
	for( i = 0; i < maxXLs; i++ )
	{
		dleaf = (*XL)[i];
		DibujaRect( color, plano, dleaf.bound );
	}
	setviewport( estadoPantalla.left, estadoPantalla.top, estadoPantalla.right, estadoPantalla.bottom, estadoPantalla.clip );

} // Actualiza


/*******************************************************************************
 *
 *      METODO:
 *              void Grafico::Actualiza(        GVenBuf::Color color,
 *                                              Plano plano,
 *                                              XMSarray<dhull_t>* FM )
 *
 ******************************************************************************/
void Grafico::Actualiza(        GVenBuf::Color color,
				Plano plano,
				XMSarray<dhull_t>* FM )
{
	struct viewporttype             estadoPantalla;
	long                            n = 0;
	long                            i = 0;
	dhull_t                         dhull;


	// Se preserva el rea establecida actualmente
	getviewsettings( &estadoPantalla );

	// Se delimita el rea de pantalla donde se llevar a cabo la representacin
	setviewport(    ( Ventana() -> rdbuf() -> Col() ) * 8,
			( Ventana() -> rdbuf() -> Lin() ) * 8,
			( Ventana() -> rdbuf() -> Col() + Ventana() -> rdbuf() -> Cols() ) * 8,
			( Ventana() -> rdbuf() -> Lin() + Ventana() -> rdbuf() -> Lins() ) * 8,
			1) ;

	// Obtenemos los elementos de la matriz...
	n = FM->size();

	// Para todos los elementos de la matriz...
	for( i = 0; i < n; i++ )
	{
		dhull = (*FM)[i];
		DibujaRect( color, plano, dhull.bound );
	}
	setviewport( estadoPantalla.left, estadoPantalla.top, estadoPantalla.right, estadoPantalla.bottom, estadoPantalla.clip );

} // Actualiza


/*******************************************************************************
 *
 *      METODO:
 *              void Grafico::Actualiza(        GVenBuf::Color color,
 *                                              Plano plano,
 *                                              XMSarray<node_t>* FM )
 *
 ******************************************************************************/
void Grafico::Actualiza(        GVenBuf::Color color,
				Plano plano,
				XMSarray<node_t>* FM )
{
	struct viewporttype             estadoPantalla;
	long                            n = 0;
	long                            i = 0;
	node_t                          node;



	// Se preserva el rea establecida actualmente
	getviewsettings( &estadoPantalla );

	// Se delimita el rea de pantalla donde se llevar a cabo la representacin
	setviewport(    ( Ventana() -> rdbuf() -> Col() ) * 8,
			( Ventana() -> rdbuf() -> Lin() ) * 8,
			( Ventana() -> rdbuf() -> Col() + Ventana() -> rdbuf() -> Cols() ) * 8,
			( Ventana() -> rdbuf() -> Lin() + Ventana() -> rdbuf() -> Lins() ) * 8,
			1) ;

	// Obtenemos los elementos de la matriz...
	n = FM->size();

	// Para todos los elementos de la matriz...
	for( i = 0; i < n; i++ )
	{
		node = (*FM)[i];
		DibujaRect( color, plano, node.box );
	}
	setviewport( estadoPantalla.left, estadoPantalla.top, estadoPantalla.right, estadoPantalla.bottom, estadoPantalla.clip );

} // Actualiza


/*******************************************************************************
 *
 *      METODO:
 *              void Grafico::Actualiza(        GVenBuf::Color color,
 *                                              XMSarray<GLine>* FG,
 *                                              long numFGs,
 *                                              Plano plano )
 *
 ******************************************************************************/
void Grafico::Actualiza(        GVenBuf::Color color,
				XMSarray<GLine>* FG,
				long numFGs,
				Plano plano )
{
	struct viewporttype             estadoPantalla;
	float                           p1[2];
	float                           p2[2];
	GLine                           gline;



	// Se preserva el rea establecida actualmente
	getviewsettings( &estadoPantalla );

	// Se delimita el rea de pantalla donde se llevar a cabo la representacin
	setviewport(    ( Ventana() -> rdbuf() -> Col() ) * 8,
			( Ventana() -> rdbuf() -> Lin() ) * 8,
			( Ventana() -> rdbuf() -> Col() + Ventana() -> rdbuf() -> Cols() ) * 8,
			( Ventana() -> rdbuf() -> Lin() + Ventana() -> rdbuf() -> Lins() ) * 8,
			1 );


	// Para todos los elementos de la matriz...
	for( long i = 0; i < numFGs; i++ )
	{
		gline = (*FG)[i];

		switch( plano )
		{
			case X0:
				p1[0] = gline.sv.y;
				p1[1] = gline.sv.z;
				p2[0] = gline.ev.y;
				p2[1] = gline.ev.z;
			break;
			case Y0:
				p1[0] = gline.sv.x;
				p1[1] = gline.sv.z;
				p2[0] = gline.ev.x;
				p2[1] = gline.ev.z;
			break;
			case Z0:
			default:
				p1[0] = gline.sv.x;
				p1[1] = gline.sv.y;
				p2[0] = gline.ev.x;
				p2[1] = gline.ev.y;
			break;
		}
		DibujaLinea( color, p1, p2 );
	}
	setviewport( estadoPantalla.left, estadoPantalla.top, estadoPantalla.right, estadoPantalla.bottom, estadoPantalla.clip );

} // Actualiza


/*******************************************************************************
 *
 *      METODO:
 *              void Grafico::Vertices( GVenBuf::Color color,
 *                                      XMSarray<GLine>* FG,
 *                                      long numFGs,
 *                                      Plano plano )
 *
 ******************************************************************************/
void Grafico::Vertices( GVenBuf::Color color,
			XMSarray<GLine>* FG,
			long numFGs,
			Plano plano )
{
	struct viewporttype             estadoPantalla;
	float                           p1[2];
	float                           p2[2];
	GLine                           gline;



	// Se preserva el rea establecida actualmente
	getviewsettings( &estadoPantalla );

	// Se delimita el rea de pantalla donde se llevar a cabo la representacin
	setviewport(    ( Ventana() -> rdbuf() -> Col() ) * 8,
			( Ventana() -> rdbuf() -> Lin() ) * 8,
			( Ventana() -> rdbuf() -> Col() + Ventana() -> rdbuf() -> Cols() ) * 8,
			( Ventana() -> rdbuf() -> Lin() + Ventana() -> rdbuf() -> Lins() ) * 8,
			1) ;

	// Para todos los elementos de la matriz...
	for( long i = 0; i < numFGs; i++ )
	{
		gline = (*FG)[i];

		switch( plano )
		{
			case X0:
				p1[0] = gline.sv.y;
				p1[1] = gline.sv.z;
				p2[0] = gline.ev.y;
				p2[1] = gline.ev.z;
			break;
			case Y0:
				p1[0] = gline.sv.x;
				p1[1] = gline.sv.z;
				p2[0] = gline.ev.x;
				p2[1] = gline.ev.z;
			break;
			case Z0:
			default:
				p1[0] = gline.sv.x;
				p1[1] = gline.sv.y;
				p2[0] = gline.ev.x;
				p2[1] = gline.ev.y;
			break;
		}
		DibujaPunto( color, p1 );
		DibujaPunto( color, p2 );
	}
	setviewport( estadoPantalla.left, estadoPantalla.top, estadoPantalla.right, estadoPantalla.bottom, estadoPantalla.clip );

} // Vertices


/*******************************************************************************
 *
 *      METODO:
 *              void Grafico::Entities( GVenBuf::Color color,
 *                                      Plano plano,
 *                                      unsigned long mask,
 *                                      XMSarray<GEntiti>* XGE,
 *                                      long numXGE,
 *                                      XMSArray<dhull_t>* XDH = NULL,
 *                                      XMSarray<long>* XGI = NULL,
 *                                      long numXGIs = 0 )
 *
 ******************************************************************************/
void Grafico::Entities( GVenBuf::Color color,
			Plano plano,
			unsigned long mask,
			XMSarray<GEntiti>* XGE,
			long numXGE,
			XMSarray<dhull_t>* XDH,
			XMSarray<long>* XGI,
			long numXGIs )
{
	struct viewporttype             estadoPantalla;
	long                            ng = 0;
	long                            i = 0;
	GEntiti                         gentiti;



	// Se preserva el rea establecida actualmente
	getviewsettings( &estadoPantalla );

	// Se delimita el rea de pantalla donde se llevar a cabo la representacin
	setviewport(    ( Ventana() -> rdbuf() -> Col() ) * 8,
			( Ventana() -> rdbuf() -> Lin() ) * 8,
			( Ventana() -> rdbuf() -> Col() + Ventana() -> rdbuf() -> Cols() ) * 8,
			( Ventana() -> rdbuf() -> Lin() + Ventana() -> rdbuf() -> Lins() ) * 8,
			1);

	// Si no hay indices para los gentities a dibujar
	if( XGI == NULL )
	{ 
		// Obtenemos los elementos de la matriz de gentities.
		ng = numXGE;

		// Para todos los elementos de la matriz.
		for( i = 0; i < ng; i++ )
		{
			// Obtenemos el gentiti correspondiente.
			gentiti = (*XGE)[i];

			Dibuja( color, plano, mask, gentiti, XDH );
		}
	}
	// Si se nos pasan los indices de las gentities a dibujar
	else
	{
	
		// Para todos las gentities a dibujar.
		for( i = 0; i < numXGIs; i++ )
		{
			// Obtenemos la gentiti correspondiente al indice.
			gentiti = (*XGE)[(*XGI)[i]];

			Dibuja( color, plano, mask, gentiti, XDH );
		}
	}
	setviewport( estadoPantalla.left, estadoPantalla.top, estadoPantalla.right, estadoPantalla.bottom, estadoPantalla.clip );

}


/*******************************************************************************
 *
 *      METODO:
 *              void Grafico::Leafs(    GVenBuf::Color color,
 *                                      Plano plano,
 *                                      XMSarray<dleaf_t>* XL,
 *                                      long numXL,
 *                                      XMSarray<long>* XLI,
 *                                      long numXLI )
 *
 ******************************************************************************/
void Grafico::Leafs(    GVenBuf::Color color,
			Plano plano,
			XMSarray<dleaf_t> *XL,
			long numXL,
			XMSarray<long>* XLI,
			long numXLI )
{
	struct viewporttype             estadoPantalla;
	long                            i = 0;
	dleaf_t                         leaf;



	// Se preserva el rea establecida actualmente
	getviewsettings( &estadoPantalla );

	// Se delimita el rea de pantalla donde se llevar a cabo la representacin
	setviewport(    ( Ventana() -> rdbuf() -> Col() ) * 8,
			( Ventana() -> rdbuf() -> Lin() ) * 8,
			( Ventana() -> rdbuf() -> Col() + Ventana() -> rdbuf() -> Cols() ) * 8,
			( Ventana() -> rdbuf() -> Lin() + Ventana() -> rdbuf() -> Lins() ) * 8,
			1);

	
	// Para todos las gentities a dibujar.
	for( i = 0; i < numXLI; i++ )
	{
		// Obtenemos la gentiti correspondiente al indice.
		leaf = (*XL)[(*XLI)[i]];

		DibujaBar( color, plano, leaf.bound );
	}
	setviewport( estadoPantalla.left, estadoPantalla.top, estadoPantalla.right, estadoPantalla.bottom, estadoPantalla.clip );

}


/*******************************************************************************
 *
 *      METODO:
 *              void Grafico::Dibuja(   GVenBuf::Color color,
 *                                      Plano plano,
 *                                      unsigned long mask,
 *                                      const GEntiti& gentiti,
 *                                      XMSarray<dhull_t>* XDH = NULL,
 *
 ******************************************************************************/
void Grafico::Dibuja(   GVenBuf::Color color,
			Plano plano,
			unsigned long mask,
			const GEntiti& gentiti,
			XMSarray<dhull_t>* XDH )
{
	boundbox_t                      bbox;
	dhull_t                         dhull;
	unsigned long                   c = 0x00000000;



	// Obtenemos el tipo 
	c = gentiti.c & 0x00FFFF00;

	// Si tipo coincide.
	if( mask & c )
	{
		// Si esta seleccionado o marcado como un target.
		if( mask & 0x80000000 )
		{
			if( gentiti.c & 0x01000000 )
			{
				dhull = (*XDH)[gentiti.model];
				bbox.min.x = dhull.bound.min.x-10;
				bbox.min.y = dhull.bound.min.y-10;
				bbox.min.z = dhull.bound.min.z-10;
				bbox.max.x = dhull.bound.max.x+10;
				bbox.max.y = dhull.bound.max.y+10;
				bbox.max.z = dhull.bound.max.z+10;
			}
			// Si no, es un punto.
			else 
			{
				bbox.min.x = gentiti.x-20;
				bbox.min.y = gentiti.y-20;
				bbox.min.z = gentiti.z-20;
				bbox.max.x = gentiti.x+20;
				bbox.max.y = gentiti.y+20;
				bbox.max.z = gentiti.z+20;
			}
			DibujaRect( color, plano, bbox );
		}
		else if( mask & 0x40000000 ) 
		{
			if( gentiti.c & 0x01000000 )
			{
				dhull = (*XDH)[gentiti.model];
				bbox.min.x = dhull.bound.min.x-10;
				bbox.min.y = dhull.bound.min.y-10;
				bbox.min.z = dhull.bound.min.z-10;
				bbox.max.x = dhull.bound.max.x+10;
				bbox.max.y = dhull.bound.max.y+10;
				bbox.max.z = dhull.bound.max.z+10;
			}
			// Si no, es un punto.
			else 
			{
				bbox.min.x = gentiti.x-20;
				bbox.min.y = gentiti.y-20;
				bbox.min.z = gentiti.z-20;
				bbox.max.x = gentiti.x+20;
				bbox.max.y = gentiti.y+20;
				bbox.max.z = gentiti.z+20;
			}
			DibujaRect( color, plano, bbox );
		}
		// Si no esta seleccionado o no es un target.
		else
		{
			// Si es un dhull.
			if( gentiti.c & 0x01000000 )
			{
				dhull = (*XDH)[gentiti.model];
				bbox.min.x = dhull.bound.min.x;
				bbox.min.y = dhull.bound.min.y;
				bbox.min.z = dhull.bound.min.z;
				bbox.max.x = dhull.bound.max.x;
				bbox.max.y = dhull.bound.max.y;
				bbox.max.z = dhull.bound.max.z;
			}
			// Si no, es un punto.
			else 
			{
				bbox.min.x = gentiti.x-10;
				bbox.min.y = gentiti.y-10;
				bbox.min.z = gentiti.z-10;
				bbox.max.x = gentiti.x+10;
				bbox.max.y = gentiti.y+10;
				bbox.max.z = gentiti.z+10;
			}
			DibujaBar( color, plano, bbox );
		}
	}
}


/*******************************************************************************
 *
 *      METODO:
 *              int Grafico::GetLeaf(   Plano plano,
 *                                      int *x,
 *                                      int *y,
 *                                      int *z,
 *                                      XMSarray<long>* XLI,
 *                                      long* numXLI,
 *                                      long maxXLI,
 *                                      XMSarray<dleaf_t>* XL,
 *                                      long maxXL )
 *
 ******************************************************************************/
int Grafico::GetLeaf(   Plano plano,
			int *x,
			int *y,
			int *z,
			XMSarray<long> *XLI,
			long* numXLI,
			long maxXLI,
			XMSarray<dleaf_t> *XL,
			long maxXL )
{
	long                    i = 0;
	long                    j = 0;
	dleaf_t                 leaf;
	int                     esta = 0;
	float                   xf = 0.0, yf = 0.0, zf = 0.0;
	int                     xi = 0, yi = 0, zi = 0;
	boundbox_t              bound;

	xi = *x;
	yi = *y;
	zi = *z;


	// Para todos los elementos de la matriz...
	for( i = 0; i < maxXL; i++ )
	{
		esta = 0;

		leaf = (*XL)[i];

		bound.min.x = leaf.bound.min.x;
		bound.min.y = leaf.bound.min.y;
		bound.min.z = leaf.bound.min.z;
		bound.max.x = leaf.bound.max.x;
		bound.max.y = leaf.bound.max.y;
		bound.max.z = leaf.bound.max.z;

		switch( plano )
		{
			case X0:
				if( BBoxEsta( yi, zi, bound, plano ) )
				{
					if( ( xi >= bound.min.x ) && ( xi <= bound.max.x ) )
					{
						esta = 1;
						ScreenToMap( &yf, &zf, yi, zi );
						yi = (int)yf;
						zi = (int)zf;
					}
					// esta = 1;
				}
			break;

			case Y0:
				if( BBoxEsta( xi, zi, bound, plano ) )
				{
					if( ( yi >= bound.min.y ) && ( yi <= bound.max.y ) )
					{
						esta = 1;
						ScreenToMap( &xf, &zf, xi, zi );
						xi = (int)xf;   
						zi = (int)zf;
					}
					// esta = 1;
				}
			break;

			default:
			case Z0:
				if( BBoxEsta( xi, yi, bound, plano ) )
				{
					if( ( zi >= bound.min.z ) && ( zi <= bound.max.z ) )
					{
						esta = 1;
						ScreenToMap( &xf, &yf, xi, yi );
						xi = (int)xf;   
						yi = (int)yf;
					}
					// esta = 1;
				}
			break;
		}
		if( j < maxXLI )
		{
			if( esta )
			{
				(*XLI)[j++] = i;
			}
		}
		else
		{
			break;
		}
	}
	*x = xi;        
	*y = yi;
	*z = zi;

	*numXLI = j;

	return j;
}


/*******************************************************************************
 *
 *      METODO:
 *              int Grafico::GetGEntiti(        Plano plano,
 *                                              unsigned long mask,
 *                                              int x,
 *                                              int y,
 *                                              long* pGI,
 *                                              XMSarray<GEntiti>* XDH,
 *                                              long numXGE,
 *                                              XMSarray<dhull_t>* XDH )
 *
 ******************************************************************************/
int Grafico::GetGEntiti(        Plano plano,
				int x,
				int y,
				unsigned long mask,
				long* pGI,
				XMSarray<GEntiti>* XGE,
				long numXGE,
				XMSarray<dhull_t>* XDH )
{
	long                            ng = 0;
	long                            i = 0;
	boundbox_t                      bbox;
	dhull_t                         dhull;
	GEntiti                         gentiti;
	unsigned long                   c = 0x00000000;


	// Obtenemos los elementos de la matriz...
	ng = numXGE;

	// Para todos los elementos de la matriz...
	for( i = 0; i < ng; i++ )
	{
		gentiti = (*XGE)[i];

		c = gentiti.c & 0x00FFFF00;

		// Si classname coincide.
		if( mask & c )
		{
			// Si es un dhull.
			if( gentiti.c & 0x01000000 )
			{
				dhull = (*XDH)[gentiti.model];
				bbox = dhull.bound;
			}
			// Si no, es un punto.
			else 
			{
				bbox.min.x = gentiti.x-10;
				bbox.min.y = gentiti.y-10;
				bbox.min.z = gentiti.z-10;
				bbox.max.x = gentiti.x+10;
				bbox.max.y = gentiti.y+10;
				bbox.max.z = gentiti.z+10;
			}
			if( BBoxEsta( x, y, bbox, plano ) )
			{
				break;
			}
		}
	}
	if( i == ng )
	{
		*pGI = -1L;
		return 0;
	}
	else
	{
		*pGI = i;
		return 1;
	}
}


/*******************************************************************************
 *
 *      METODO:
 *              int Grafico::SetGEntiti(        Plano plano,
 *                                              int x,
 *                                              int y,
 *                                              Gentiti* pGE ) 
 *
 ******************************************************************************/
int Grafico::SetGEntiti(        Plano plano,
				int x,
				int y,
				GEntiti* pGE )
{
	float           xv = 0.0;
	float           yv = 0.0;



	ScreenToMap( &xv, &yv, x, y );

	switch( plano )
	{
		case X0:
			pGE->y = xv;
			pGE->z = yv;
		break;

		case Y0:
			pGE->x = xv;
			pGE->z = yv;
		break;

		case Z0:
		default:
			pGE->x = xv;
			pGE->y = yv;
		break;
	}
	return 1;
}


/*******************************************************************************
 *
 *      METODO:
 *              int Grafico::SetArea(   Plano plano,
 *                                      shortbox_t *pSBox ) 
 *
 ******************************************************************************/
int Grafico::SetArea(   Plano plano,
			shortbox_t *pSBox ) 
{
	boundbox_t                      fbox;
	shortbox_t                      sbox;
	int                             xi = 0, yi = 0, zi = 0, xf = 0, yf = 0, zf = 0;
	GRaton                          raton;
	struct viewporttype             estadoPantalla;


	sbox = *pSBox;

	fbox.min.x = (float)sbox.min.x;
	fbox.min.y = (float)sbox.min.y;
	fbox.min.z = (float)sbox.min.z;
	fbox.max.x = (float)sbox.max.x;
	fbox.max.y = (float)sbox.max.y;
	fbox.max.z = (float)sbox.max.z;

	switch( plano )
	{
		case X0:
			yi = raton.PosicionHorizontal();
			zi = raton.PosicionVertical();
			sbox.min.y = yi;
			sbox.min.z = zi;
		break;

		case Y0:
			xi = raton.PosicionHorizontal();
			zi = raton.PosicionVertical();
			sbox.min.x = xi;
			sbox.min.z = zi;
		break;

		case Z0:
		default:
			xi = raton.PosicionHorizontal();
			yi = raton.PosicionVertical();
			sbox.min.x = xi;
			sbox.min.y = yi;
		break;
	}

	do {

	} while( !raton.EstadoBoton( GRaton::Izquierdo ) );

	switch( plano )
	{
		case X0:
			yf = raton.PosicionHorizontal();
			zf = raton.PosicionVertical();
			if( yf < yi )
			{
				sbox.min.y = yf;
				sbox.max.y = yi;
			}
			else
			{
				sbox.min.y = yi;
				sbox.max.y = yf;
			}
			if( zf < zi )
			{
				sbox.min.z = zf;
				sbox.max.z = zi;
			}
			else
			{
				sbox.min.z = zi;
				sbox.max.z = zf;
			}
			ScreenToMap( &fbox.min.y, &fbox.min.z, sbox.min.y, sbox.min.z );
			ScreenToMap( &fbox.max.y, &fbox.max.z, sbox.max.y, sbox.max.z );
		break;

		case Y0:
			xf = raton.PosicionHorizontal();
			zf = raton.PosicionVertical();
			if( xf < xi )
			{
				sbox.min.x = xf;
				sbox.max.x = xi;
			}
			else
			{
				sbox.min.x = xi;
				sbox.max.x = xf;
			}
			if( zf < zi )
			{
				sbox.min.z = zf;
				sbox.max.z = zi;
			}
			else
			{
				sbox.min.z = zi;
				sbox.max.z = zf;
			}
			ScreenToMap( &fbox.min.x, &fbox.min.z, sbox.min.x, sbox.min.z );
			ScreenToMap( &fbox.max.x, &fbox.max.z, sbox.max.x, sbox.max.z );
		break;

		case Z0:
		default:
			xf = raton.PosicionHorizontal();
			yf = raton.PosicionVertical();
			if( xf < xi )
			{
				sbox.min.x = xf;
				sbox.max.x = xi;
			}
			else
			{
				sbox.min.x = xi;
				sbox.max.x = xf;
			}
			if( yf < yi )
			{
				sbox.min.y = yf;
				sbox.max.y = yi;
			}
			else
			{
				sbox.min.y = yi;
				sbox.max.y = yf;
			}
			ScreenToMap( &fbox.min.x, &fbox.min.y, sbox.min.x, sbox.min.y );
			ScreenToMap( &fbox.max.x, &fbox.max.y, sbox.max.x, sbox.max.y );
		break;
	}

	// Se preserva el rea establecida actualmente
	getviewsettings( &estadoPantalla );

	// Se delimita el rea de pantalla donde se llevar a cabo la representacin
	setviewport(    ( Ventana() -> rdbuf() -> Col() ) * 8,
			( Ventana() -> rdbuf() -> Lin() ) * 8,
			( Ventana() -> rdbuf() -> Col() + Ventana() -> rdbuf() -> Cols() ) * 8,
			( Ventana() -> rdbuf() -> Lin() + Ventana() -> rdbuf() -> Lins() ) * 8,
			1) ;

	DibujaRect( GVenBuf::Blanco, plano, fbox );

	setviewport( estadoPantalla.left, estadoPantalla.top, estadoPantalla.right, estadoPantalla.bottom, estadoPantalla.clip );

	sbox.min.x = (short)fbox.min.x;
	sbox.min.y = (short)fbox.min.y;
	sbox.min.z = (short)fbox.min.z;
	sbox.max.x = (short)fbox.max.x;
	sbox.max.y = (short)fbox.max.y;
	sbox.max.z = (short)fbox.max.z;

	*pSBox = sbox;

	return 1;
}

/*******************************************************************************
 *
 *      METODO:
 *              void Grafico::LimpiarGrafico( void )
 *
 ******************************************************************************/
void Grafico::LimpiarGrafico( void )
{
	struct viewporttype             estadoPantalla;



	// Se preserva el rea establecida actualmente
	getviewsettings( &estadoPantalla ) ;

	// Se delimita el rea del cliente 
	setviewport(    ( v -> rdbuf() -> Col() ) * 8,
			( v -> rdbuf() -> Lin() ) * 8,
			( v -> rdbuf() -> Col() + v -> rdbuf() -> Cols() ) * 8,
			( v -> rdbuf() -> Lin() + v -> rdbuf() -> Lins() ) * 8,
			1 ) ;
	clearviewport();

	setviewport( estadoPantalla.left, estadoPantalla.top, estadoPantalla.right, estadoPantalla.bottom, estadoPantalla.clip );
}


/*******************************************************************************
 *
 *      METODO:
 *              void Grafico::Ventana( GVentana * V )
 *
 ******************************************************************************/
void Grafico::Ventana( GVentana * V )
{
	v = V;
}


/*******************************************************************************
 *
 *      METODO:
 *              GVentana* Grafico::Ventana( void )
 *
 ******************************************************************************/
GVentana* Grafico::Ventana( void )
{
	return v;
}


/*******************************************************************************
 *
 *      METODO:
 *              void Grafico::Mover( Direccion direccion )
 *
 ******************************************************************************/
void Grafico::Mover( Direccion direccion )
{
	// Mueve el 10% del View en la direccion especificada.

	switch( direccion )
	{
		case Left:
			origenX -= ( Ventana()->rdbuf()->Cols()*8 ) * 0.1;
		break;
	
		case Up:
			origenY -= ( Ventana()->rdbuf()->Lins()*8 ) * 0.1;
		break;

		case Down:
			origenY += ( Ventana()->rdbuf()->Lins()*8 ) * 0.1;
		break;

		case Right:
		default:
			origenX += ( Ventana()->rdbuf()->Cols()*8 ) * 0.1;
		break;

	}
}


/*******************************************************************************
 *
 *      METODO:
 *              void Grafico::DibujaRect(       GVenBuf::Color color,
 *                                              Plano plano,
 *                                              boundbox_t b )
 *
 ******************************************************************************/
void Grafico::DibujaRect(       GVenBuf::Color color,
				Plano plano,
				boundbox_t b )
{
	float           fr[4][2];



	// Obtenemos los puntos en el plano correspondiente.
	BBoxToA( fr, b, plano );

	// Dibujamos el rectangulo.
	DibujaRect( color, fr );

}


/*******************************************************************************
 *
 *      METODO:
 *              void Grafico::DibujaRect(       GVenBuf::Color color,
 *                                              Plano plano,
 *                                              shortbox_t b )
 *
 ******************************************************************************/
void Grafico::DibujaRect(       GVenBuf::Color color,
				Plano plano,
				shortbox_t b )
{
	float           fr[4][2];


	// Obtenemos los puntos en el plano correspondiente.
	BBoxToA( fr, b, plano );

	// Dibujamos el rectangulo.
	DibujaRect( color, fr );

}


/*******************************************************************************
 *
 *      METODO:
 *              void Grafico::DibujaRect(       GVenBuf::Color color,
 *                                              float fr[4][2] )
 *
 ******************************************************************************/
void Grafico::DibujaRect(       GVenBuf::Color color,
				float fr[4][2] )
{
	int             i = 0;
	int             ir[4][2];



	// Calculamos los puntos a representar en las coordenadas de la View.
	for( i = 0; i < 4; i++ ) {
		MapToView( &ir[i][0], &ir[i][1], fr[i][0], fr[i][1] );
	}
	// Utilizar el color actual en la ventana
	setcolor( color );

	// Lnea slida de anchura normal
	setlinestyle( SOLID_LINE, 0, NORM_WIDTH );

	// dibujamos los datos.
	for( i = 0; i < 4; i++ ) {
		moveto( ir[i][0], ir[i][1] );
		if( i != 3 ) {
			lineto( ir[i+1][0], ir[i+1][1] );
		} else {
			lineto( ir[0][0], ir[0][1] );
		}
	}
}


/*******************************************************************************
 *
 *      METODO:
 *              void Grafico::DibujaBar(        GVenBuf::Color color,
 *                                              Plano plano,
 *                                              boundbox_t bbox )
 *
 ******************************************************************************/
void Grafico::DibujaBar(        GVenBuf::Color color,
				Plano plano,
				boundbox_t bbox )
{
	float           fr[4][2];


	// Obtenemos los puntos en el plano correspondiente.
	BBoxToA( fr, bbox, plano );

	// Dibuja el Area.
	DibujaBar( color, fr );

}


/*******************************************************************************
 *
 *      METODO:
 *              void Grafico::DibujaBar(        GVenBuf::Color color,
 *                                              Plano plano,
 *                                              shortbox_t bbox )
 *
 ******************************************************************************/
void Grafico::DibujaBar(        GVenBuf::Color color,
				Plano plano,
				shortbox_t bbox )
{
	float           fr[4][2];


	// Obtenemos los puntos en el plano correspondiente.
	BBoxToA( fr, bbox, plano );

	// Dibuja el Area.
	DibujaBar( color, fr );

}


/*******************************************************************************
 *
 *      METODO:
 *              void Grafico::DibujaBar(        GVenBuf::Color color,
 *                                              float fr[4][2] )
 *
 ******************************************************************************/
void Grafico::DibujaBar(        GVenBuf::Color color,
				float fr[4][2] )
{
	int             i = 0;
	int             ir[4][2];



	// Calculamos los puntos a representar en las coordenadas de la View.
	for( i = 0; i < 4; i++ ) {
		MapToView( &ir[i][0], &ir[i][1], fr[i][0], fr[i][1] );
	}
	// Estilo de relleno.
	setfillstyle( SOLID_FILL, color );

	// Dibujamos un rectangulo.
	bar( ir[0][0], ir[1][1], ir[2][0], ir[3][1] );
	
}


/*******************************************************************************
 *
 *      METODO:
 *              void Grafico::DibujaLinea(      GVenBuf::Color color,
 *                                              float fr1[2],
 *                                              float fr2[2],
 *
 ******************************************************************************/
void Grafico::DibujaLinea(      GVenBuf::Color color,
				float fr1[2],
				float fr2[2] )
{
	int             p1[2];
	int             p2[2];



	// Calculamos los puntos a representar en las coordenadas de la View.
	MapToView( &p1[0], &p1[1], fr1[0], fr1[1] );
	MapToView( &p2[0], &p2[1], fr2[0], fr2[1] );

	// Utilizar el color actual en la ventana
	setcolor( color );

	// Lnea slida de anchura normal
	setlinestyle( SOLID_LINE, 0, NORM_WIDTH );

	// Situamos en el punto inicial.
	moveto( p1[0], p1[1] );

	// Trazamos una linea al punto final.
	lineto( p2[0], p2[1] );

}



/*******************************************************************************
 *
 *      METODO:
 *              void Grafico::DibujaPunto(      GVenBuf::Color color,
 *                                              float fr1[2] )
 *
 ******************************************************************************/
void Grafico::DibujaPunto(      GVenBuf::Color color,
				float fr1[2] )
{
	int             p1[2];



	// Calculamos los puntos a representar en las coordenadas de la View.
	MapToView( &p1[0], &p1[1], fr1[0], fr1[1] );

	// Utilizar el color actual en la ventana
	setcolor( color );

	// Lnea slida de anchura normal
	setlinestyle( SOLID_LINE, 0, THICK_WIDTH );

	// Nos situamos en el punto.
	moveto( p1[0], p1[1] );

	// Dibujamos el punto como una cruz.
	line( p1[0]-2, p1[1]-2, p1[0]+2, p1[1]+2 );
	line( p1[0]-2, p1[1]+2, p1[0]+2, p1[1]-2 );

}


/*******************************************************************************
 *
 *      METODO:
 *              int Grafico::BBoxEsta( int x, int y, boundbox_t b, Plano plano )
 *
 ******************************************************************************/
int Grafico::BBoxEsta( int x, int y, boundbox_t b, Plano plano )
{
	float           xf = 0.0;
	float           yf = 0.0;



	ScreenToMap( &xf, &yf, x, y );

	switch( plano )
	{
		case X0:
			if( xf <  b.min.y )
			{
				return 0;
			}
			else if( xf > b.max.y )
			{
				return 0;
			}
			else if( yf <  b.min.z )
			{
				return 0;
			}
			else if( yf > b.max.z )
			{
				return 0;
			}
		return 1;

		case Y0:
			if( xf < b.min.x )
			{
				return 0;
			}
			else if( xf > b.max.x )
			{
				return 0;
			}
			else if( yf <  b.min.z )
			{
				return 0;
			}
			else if( yf > b.max.z )
			{
				return 0;
			}
		return 1;

		case Z0:
		default:
			if( xf < b.min.x )
			{
				return 0;
			}
			else if( xf > b.max.x )
			{
				return 0;
			}
			else if( yf <  b.min.y )
			{
				return 0;
			}
			else if( yf >  b.max.y )
			{
				return 0;
			}
		return 1;
	}
}


/*******************************************************************************
 *
 *      METODO:
 *              int Grafico::BBoxEsta( int x, int y, shortbox_t b, Plano plano )
 *
 ******************************************************************************/
int Grafico::BBoxEsta( int x, int y, shortbox_t b, Plano plano )
{
	float           mxf = 0.0;
	float           myf = 0.0;
	short           xf = 0.0;
	short           yf = 0.0;


	ScreenToMap( &mxf, &myf, x, y );

	xf = (short)mxf;
	yf = (short)myf;

	switch( plano )
	{
		case X0:
			if( xf <  b.min.y )
			{
				return 0;
			}
			else if( xf > b.max.y )
			{
				return 0;
			}
			else if( yf <  b.min.z )
			{
				return 0;
			}
			else if( yf > b.max.z )
			{
				return 0;
			}
		return 1;

		case Y0:
			if( xf < b.min.x )
			{
				return 0;
			}
			else if( xf > b.max.x )
			{
				return 0;
			}
			else if( yf <  b.min.z )
			{
				return 0;
			}
			else if( yf > b.max.z )
			{
				return 0;
			}
		return 1;

		case Z0:
		default:
			if( xf < b.min.x )
			{
				return 0;
			}
			else if( xf > b.max.x )
			{
				return 0;
			}
			else if( yf <  b.min.y )
			{
				return 0;
			}
			else if( yf >  b.max.y )
			{
				return 0;
			}
		return 1;
	}
}


/*******************************************************************************
 *
 *      METODO:
 *              void Grafico::ScreenToMap( float *px, float *py, int x, int y )
 *
 ******************************************************************************/
void Grafico::ScreenToMap( float *px, float *py, int x, int y )
{
	float           xf = 0.0;
	float           yf = 0.0;



	xf = x - ( v->rdbuf()->Col()*8 );
	yf = y - ( v->rdbuf()->Lin()*8 );

	xf = ( ( xf - origenX ) * anchura ) / altura;
	yf = ( yf - origenY );

	xf = xf / razon;
	yf = yf / razon;

	*px = xf;
	*py = yf;
}


/*******************************************************************************
 *
 *      METODO:
 *              void Grafico::ViewToMap( float *px, float *py, int x, int y )
 *
 ******************************************************************************/
void Grafico::ViewToMap( float *px, float *py, int x, int y )
{
	float           xf = 0.0;
	float           yf = 0.0;



	xf = ( ( x - origenX ) * anchura ) / altura;
	yf = ( y - origenY );

	xf = xf / razon;
	yf = yf / razon;

	*px = xf;
	*py = yf;
}


/*******************************************************************************
 *
 *      METODO:
 *              void Grafico::ScreenToView( int *px, int *py, int x, int y )
 *
 ******************************************************************************/
void Grafico::ScreenToView( int *px, int *py, int x, int y )
{
	*px = x - ( v->rdbuf()->Col()*8 );
	*py = y - ( v->rdbuf()->Lin()*8 );
}


/*******************************************************************************
 *
 *      METODO:
 *              void Grafico::MapToScreen( int *px, int *py, float x, float y )
 *
 ******************************************************************************/
void Grafico::MapToScreen( int *px, int *py, float x, float y )
{


	// Calculamos el tamano dependiendo del factor de escala.
	x = x * razon;
	y = y * razon;


	// Calculamos la posicion en las cordenadas del View.
	x = ( ( x * altura ) / anchura ) + origenX; 
	y = y + origenY;

	// Pasamos a enteros
	*px = (int)x;   
	*py = (int)y;   

	// Calculamos la posicion en funcion del Screen.
	*px = *px + ( v->rdbuf()->Col()*8 );
	*py = *py + ( v->rdbuf()->Lin()*8 );

}


/*******************************************************************************
 *
 *      METODO:
 *              void Grafico::MapToView( int *px, int *py, float x, float y )
 *
 ******************************************************************************/
void Grafico::MapToView( int *px, int *py, float x, float y )
{

	// Calculamos el tamano dependiendo del factor de escala.
	x = x * razon;
	y = y * razon;

	// Calculamos la posicion en las cordenadas del View.
	x = ( ( x * altura ) / anchura ) + origenX; 
	y = y + origenY;

	// Pasamos los floats a integers.
	*px = (int)x;   
	*py = (int)y;   
}


/*******************************************************************************
 *
 *      METODO:
 *              void Grafico::ViewToScreen( int *px, int *py, int x, int y )
 *
 ******************************************************************************/
void Grafico::ViewToScreen( int *px, int *py, int x, int y )
{
	*px = x + ( v->rdbuf()->Col()*8 );
	*py = y + ( v->rdbuf()->Lin()*8 );
}


/*******************************************************************************
 *------------------------------------------------------------------------------
 *
 *      FIN DE FICHERO
 *
 *------------------------------------------------------------------------------
 ******************************************************************************/

