/*******************************************************************************
 *------------------------------------------------------------------------------
 *
 *	FICHERO:
 *		GRAFICO.HPP
 *
 *------------------------------------------------------------------------------
 ******************************************************************************/
/*******************************************************************************
 *------------------------------------------------------------------------------
 *
 * 	PROFILE INFORMATION:
 *
 * 	OBJECT MODIFICATION DATE:
 *		Mon May 06 10:33:20 1996
 *
 *------------------------------------------------------------------------------
 ******************************************************************************/
#ifndef _GRAFICO_HPP_
#define _GRAFICO_HPP_


#ifndef __cplusplus
#error No se puede utilizar Grafico en C
#endif



/*******************************************************************************
 *
 *	INCLUDES:
 *
 ******************************************************************************/
#include <graphics.h>
#include <string.h>
#include <iostream.h>


#include "doserror.h"
#include "xms.h"
#include "xmsarray.h"



#include "GVentana.Hpp"
#include "qstruct.h"
#include "vstruct.h"

	// Si el modelo de memoria seleccionado permite varios segmentos de datos
	// habr que utilizar las versiones far de algunas funciones

#if defined(__LARGE__) || defined(__HUGE__) || defined(__COMPACT__)

	#define istrcpy( orig, dest )	_fstrcpy( (orig), (dest) )
	#define istrcmp( orig, dest )	_fstrcmp( (orig), (dest) )
	#define istrcat( orig, dest )	_fstrcat( (orig), (dest) )

#else

	#define istrcpy( orig, dest )	strcpy( (orig), (dest) )
	#define istrcmp( orig, dest )	strcmp( (orig), (dest) )
	#define istrcat( orig, dest )	strcat( (orig), (dest) )

#endif


/*******************************************************************************
 *
 *	PROTOTIPOS:
 *
 ******************************************************************************/
void GetAspectRatio( void );
float* BBoxToA( float p[4][2], float bbox[2][3], Plano plano );
float* BBoxToA( float p[4][2], boundbox_t b, Plano plano );
float* BBoxToA( float p[4][2], shortbox_t b, Plano plano );


/*******************************************************************************
 *
 *	CLASS:
 *		Grafico:
 *
 ******************************************************************************/
class Grafico
{
    public:

				Grafico();

				Grafico( GVentana* v );

				Grafico( const Grafico& R );

	void			operator = ( const Grafico& R );

	int			operator == ( const Grafico& R );

	Grafico&		operator * ( float r );

 				// Dibuja la representacin
	void			Actualiza(	GVenBuf::Color color,
						Plano plano,
						XMSarray<dleaf_t>* XL,
						long maxXLs );
 
	void			Actualiza(	GVenBuf::Color color,
						Plano plano,
						XMSarray<dhull_t>* FM );
 
	void			Actualiza(	GVenBuf::Color color,
						Plano plano,
						XMSarray<node_t>* FM );
 
	void			Actualiza(	GVenBuf::Color color,
						XMSarray<GLine>* GP,
						long numGPS,
						Plano plano );

	void			Vertices(	GVenBuf::Color color,
						XMSarray<GLine>* GP,
						long numGPs,
						Plano plano );

	void			Entities( 	GVenBuf::Color color,
						Plano plano,
						unsigned long mask,
						XMSarray<GEntiti>* XGE,
						long numXGEs,
						XMSarray<dhull_t>* XDH = NULL,
						XMSarray<long>* XGI = NULL,
						long numXGIs = 0 );

	void			Leafs( 		GVenBuf::Color color,
						Plano plano,
						XMSarray<dleaf_t>* XL,
						long numXLs,
						XMSarray<long>* XLI,
						long numXLIs );

	int			GetLeaf( 	Plano plano,
						int *x,
						int *y,
						int *z,
						XMSarray<long> *XLI,
						long *numXLI,
						long maxXLI,
						XMSarray<dleaf_t> *XL,
						long maxXL );

	int			GetGEntiti( 	Plano plano,
						int x,
						int y,
						unsigned long mask,
						long* pGI,
						XMSarray<GEntiti>* XGE,
						long numXGE,
						XMSarray<dhull_t>* XDH );

	int			SetGEntiti( 	Plano plano,
						int x,
						int y,
						GEntiti *pGE );

	int			SetArea( 	Plano plano,
						shortbox_t *sbox );

				// Limpia la representacion. 
	void			LimpiarGrafico( void );

				// Mueve la representacion.
	void			Mover( Direccion direccion );

				// Establece la ventana destino
	void			Ventana( GVentana* v );

				 // Devuelve la ventana destino
	GVentana*		Ventana( void );

	float			razon; // Razon para representar los graficos


    protected:

	void			DibujaLinea(	GVenBuf::Color color,
						float p1[2],
						float p2[2] );

	void			DibujaPunto(	GVenBuf::Color color,
						float p1[2] );

    private:

	float			origenX,
				origenY;

	GVentana*		v; // Ventana destino de la representacin


	void			Dibuja( 	GVenBuf::Color color,
						Plano plano,
						unsigned long mask,
						const GEntiti& gentiti,
						XMSarray<dhull_t>* XDH = NULL );

	void			DibujaRect(	GVenBuf::Color color,
						float fr[4][2] );

	void			DibujaRect(	GVenBuf::Color color,
						Plano plano,
						shortbox_t b );

	void			DibujaRect(	GVenBuf::Color color,
						Plano plano,
						boundbox_t b );

	void			DibujaBar(	GVenBuf::Color color,
						Plano plano,
						boundbox_t b );

	void			DibujaBar(	GVenBuf::Color color,
						Plano plano,
						shortbox_t b );

	void			DibujaBar(	GVenBuf::Color color,
						float fr[4][2] );

	int 			BBoxEsta( 	int x,
						int y,
						boundbox_t b,
						Plano plano );
	int 			BBoxEsta( 	int x,
						int y,
						shortbox_t b,
						Plano plano );

	void 			ScreenToMap( 	float *px,
						float *py,
						int x,
						int y );

	void 			ViewToMap( 	float *px,
						float *py,
						int x,
						int y );

	void 			ScreenToView( 	int *px,
						int *py,
						int x,
						int y );

	void 			MapToScreen( 	int *px,
						int *py,
						float x,
						float y );

	void 			MapToView( 	int *px,
						int *py,
						float x,
						float y );

	void 			ViewToScreen( 	int *px,
						int *py,
						int x,
						int y );

};


#endif // _GRAFICO_HPP_


/*******************************************************************************
 *------------------------------------------------------------------------------
 *
 *	FIN DE FICHERO
 *
 *------------------------------------------------------------------------------
 ******************************************************************************/

