/*******************************************************************************
 *------------------------------------------------------------------------------
 *
 *	FICHERO:
 *		QSTRUCT.H
 *
 *------------------------------------------------------------------------------
 ******************************************************************************/
/*******************************************************************************
 *------------------------------------------------------------------------------
 *
 * 	PROFILE INFORMATION:
 *
 * 	OBJECT MODIFICATION DATE:
 *		Wen Apr 24 10:11:34 1996
 *
 *------------------------------------------------------------------------------
 ******************************************************************************/
#ifndef _QSTRUCT_H_
#define _QSTRUCT_H_


/*******************************************************************************
 *
 *	INCLUDES:
 *
 ******************************************************************************/
/*******************************************************************************
 *
 *	VERSION 23:
 *
 ******************************************************************************/
#ifdef V23

typedef float scalar_t;

typedef struct
{
	scalar_t x;
	scalar_t y;
	scalar_t z;
} vec3_t;

typedef struct
{
	vec3_t   min;
	vec3_t   max;
} boundbox_t;

typedef struct 
{
	vec3_t normal;
	scalar_t dist;
	long type;
	long firstsurf;
	long numsurf;
} plane_t;

typedef struct 
{
	float x;
	float y;
	float z;
} vertex_t;

typedef struct 
{
	long planenum;
	unsigned short front;
	unsigned short back;
	boundbox_t box;
} node_t;

typedef struct 
{
	unsigned short planenum;
	unsigned short side;
	unsigned char texnum;
	unsigned char sofs;
	unsigned char tofs;
	unsigned char flips;
	long firstedge;
	long numedge;
	unsigned char light;
	unsigned char unknown0;
	unsigned short unknown1;
	unsigned long lightmap;
} surface_t;

typedef struct 
{
	unsigned long planenum;
	short front;
	short back;
} dhullbound_t;

typedef struct 
{
	unsigned long code;
	boundbox_t bound;
	unsigned long vislist;
	long firstsurf;
	long numsurf;
	unsigned long zeroes[3];
	unsigned short zero;
	unsigned short flag;
} dleaf_t;

typedef struct
{
	unsigned short startvertex;
	unsigned short endvertex;
} edge_t;

typedef struct 
{
	boundbox_t bound;
	long zero[3];
	long node;
	long boundnode;
	long numleafs;
	long firstsurface;
	long numsurfaces;
} dhull_t;

typedef struct			// A Directory entry
{
	long offset;		// Offset to entry, in bytes, from start of file
	long size;		// Size of entry in file, in bytes
} dentry_t;

typedef struct			// The BSP file header
{
	long version;		// Model version, must be 0x17 (23).
	dentry_t entities;	// List of Entities.
	dentry_t planes;	// Map Planes.
				// numplanes = size/sizeof(plane_t).
	dentry_t miptex;	// Wall Textures.	
	dentry_t vertices;	// Map Vertices.
				// numvertices = size/sizeof(vertex_t).
	dentry_t visilist;	// Leaves Visibility lists.
	dentry_t nodes;		// BSP Nodes.
				// numnodes = size/sizeof(node_t).
	dentry_t surfaces;	// Map Surfaces.
				// numsurfaces = size/sizeof(surface_t).
	dentry_t lightmaps;	// Wall Light Maps.
	dentry_t boundnodes;	// BSP bound nodes, for Hulls.
				// numbounds = size/sizeof(hullbound_t).
	dentry_t leaves;	// BSP Leaves.
				// numleaves = size/sizeof(leaf_t).
	dentry_t lstsurf;	// List of Surfaces.	
	dentry_t edges;		// Original surface Edges.
				// numedges = size/sizeof(edge_t).
	dentry_t lstedges;	// list of surfaces Edges.
	dentry_t hulls;		// List of Hulls.
				// numhulls = size/sizeof(hull_t).
} dheader_t;

#endif // V23


/*******************************************************************************
 *
 *	VERSION 28:
 *
 ******************************************************************************/
#ifdef V28

#define MAXLIGTHMAPS	4
#define NUM_AMBIENTS	4
#define MAX_MAP_HULLS	4
#define MIPLEVELS 	4
#define MAX_MIPTEXS_NUM	512


typedef float scalar_t;

typedef struct
{
	scalar_t x;
	scalar_t y;
	scalar_t z;
} vec3_t;

typedef struct
{
	vec3_t   min;
	vec3_t   max;
} boundbox_t;

typedef struct
{
	short x;
	short y;
	short z;
} short3_t;

typedef struct
{
	short3_t min;
	short3_t max;
} shortbox_t;

typedef struct 
{
	vec3_t normal;
	scalar_t dist;
	long type;
} plane_t;

typedef struct 
{
	float x;
	float y;
	float z;
} vertex_t;

typedef struct
{
	long nummiptex;
	long dataofs[MAX_MIPTEXS_NUM];
} dmiptexlump_t;

typedef struct
{
	char name[16];
	unsigned long width;
	unsigned long height;
	unsigned long offsets[MIPLEVELS];
} miptex_t;

typedef struct 
{
	long planenum;
	short front;
	short back;
	shortbox_t box;
	unsigned short firstface;
	unsigned short numfaces;
} node_t;

typedef struct 
{
	short planenum;
	short side;
	long firstedge;
	short numedge;
	short texnum;
	unsigned char styles[MAXLIGTHMAPS];
	long lightmap;
} surface_t;

typedef struct 
{
	long planenum;
	short front;
	short back;
} dhullbound_t;

typedef struct 
{
	long code;
	long vislist;
	shortbox_t bound;
	unsigned short firstsurf;
	unsigned short numsurf;
	unsigned char ambient_level[NUM_AMBIENTS];
} dleaf_t;

typedef struct
{
	unsigned short startvertex;
	unsigned short endvertex;
} edge_t;

typedef struct 
{
	boundbox_t bound;
	float origin[3];
	long node[MAX_MAP_HULLS];
	long visleafs;
	long firstsurface;
	long numsurfaces;
} dhull_t;

typedef struct			// A Directory entry
{
	long offset;		// Offset to entry, in bytes, from start of file
	long size;		// Size of entry in file, in bytes
} dentry_t;

typedef struct			// The BSP file header
{
	long version;		// Model version, must be 0x1C (28).
	dentry_t entities;	// List of Entities.
	dentry_t planes;	// Map Planes.
				// numplanes = size/sizeof(plane_t).
	dentry_t miptex;	// Wall Textures.	
	dentry_t vertices;	// Map Vertices.
				// numvertices = size/sizeof(vertex_t).
	dentry_t visilist;	// Leaves Visibility lists.
	dentry_t nodes;		// BSP Nodes.
				// numnodes = size/sizeof(node_t).
	dentry_t texinfo;	// Textures information.
				// numtexinfos = size/sizeof(texinfo_s).
	dentry_t surfaces;	// Map Surfaces.
				// numsurfaces = size/sizeof(surface_t).
	dentry_t lightmaps;	// Wall Light Maps.
	dentry_t boundnodes;	// BSP bound nodes, for Hulls.
				// numbounds = size/sizeof(hullbound_t).
	dentry_t leaves;	// BSP Leaves.
				// numleaves = size/sizeof(leaf_t).
	dentry_t lstsurf;	// List of Surfaces.	
	dentry_t edges;		// Original surface Edges.
				// numedges = size/sizeof(edge_t).
	dentry_t lstedges;	// list of surfaces Edges.
	dentry_t hulls;		// List of Hulls.
				// numhulls = size/sizeof(hull_t).
} dheader_t;

#endif // V28


#endif


/*******************************************************************************
 *------------------------------------------------------------------------------
 *
 *	FIN DE FICHERO
 *
 *------------------------------------------------------------------------------
 ******************************************************************************/

