/*******************************************************************************
 *------------------------------------------------------------------------------
 *
 *      FICHERO:
 *              UNBSP.CPP
 *
 *------------------------------------------------------------------------------
 ******************************************************************************/
/*******************************************************************************
 *------------------------------------------------------------------------------
 *
 *      PROFILE INFORMATION:
 *
 *      OBJECT MODIFICATION DATE:
 *              Wen Apr 24 15:34:55 1996
 *
 *------------------------------------------------------------------------------
 ******************************************************************************/

/*******************************************************************************
 *
 *      INCLUDES
 *
 ******************************************************************************/
#include <stdio.h>
#include <math.h>
#include <fstream.h>


#include "string.hpp"
#include "tracelog.hpp"
#include "working.hpp"
#include "test055.hpp"
#include "unbsp.hpp"



/*******************************************************************************
 *
 *      DEFINES:
 *
 ******************************************************************************/
#define ENTRY00         "ENTRY00"
#define ENTRY01         "ENTRY01"
#define ENTRY02         "ENTRY02"
#define ENTRY03         "ENTRY03"
#define ENTRY04         "ENTRY04"
#define ENTRY05         "ENTRY05"
#define ENTRY06         "ENTRY06"
#define ENTRY07         "ENTRY07"
#define ENTRY08         "ENTRY08"
#define ENTRY09         "ENTRY09"
#define ENTRY10         "ENTRY10"
#define ENTRY11         "ENTRY11"
#define ENTRY12         "ENTRY12"
#define ENTRY13         "ENTRY13"
#define ENTRY14         "ENTRY14"


/*******************************************************************************
 *
 *      VARIABLES IMPORTADAS:
 *
 ******************************************************************************/
extern Working                  *working;


/*******************************************************************************
 *
 *      GLOBALS VARIABLES
 *
 ******************************************************************************/

 
/*******************************************************************************
 *
 *      FUNCTION:
 *              int ReadBytes( FILE *file, void *addr, unsigned long size )
 *
 ******************************************************************************/
int ReadBytes( FILE *file, void *addr, unsigned long size )
{
	while( size > 0x8000 )
	{
		if( fread( addr, 1, 0x8000, file) != 0x8000 )
		{
			return 0;
		}
		addr = (char*)addr + 0x8000;
		size -= 0x8000;
	}
	if( fread( addr, 1, size, file ) != size )
	{
		return 0;
	}
	return 1;
}

 
/*******************************************************************************
 *
 *      FUNCTION:
 *              int WriteBytes( FILE *file, void *addr, unsigned long size )
 *
 ******************************************************************************/
int WriteBytes( FILE *file, void *addr, unsigned long size )
{
	while( size > 0x8000 )
	{
		if( fwrite( addr, 1, 0x8000, file ) != 0x8000 )
		{
			return 0;
		}
		addr = (char*)addr + 0x8000;
		size -= 0x8000;
	}
	if( fwrite( addr, 1, size, file ) != size )
	{
		return 0;
	}
	return 1;
}


/*******************************************************************************
 *
 *      FUNCTION:
 *              long SaveBMP( FILE *file, BitMap *bmptr, RGB *palette256 );
 *
 ******************************************************************************/
long SaveBMP( FILE *file, BitMap *bmptr, RGB *palette256 )
{
  struct BMPHeader      bmphdr;
  char                  *x;
  char*                 buffer;
  short                 r, c;
  unsigned short        linesize;
  unsigned long         imgsize;
  unsigned long         imgoffset;
  long                  i;


 
	if( file == NULL || bmptr->width == 0 || bmptr->height == 0 || palette256 == NULL )
	{
		return 0L;
	}
	linesize = (bmptr->width + 3) & (~3);
	imgsize = (unsigned long)linesize * (unsigned long)bmptr->height;
	imgoffset = 2L + (unsigned long)sizeof(struct BMPHeader) + 1024L;

	/* write the BMP header */
	if( !WriteBytes( file, "BM", 2L ) )
	{
		return 0L;
	}
	bmphdr.bfSize = imgoffset + imgsize;
	bmphdr.bfReserved1 = 0;
	bmphdr.bfReserved2 = 0;
	bmphdr.bfOffBits = imgoffset;
	bmphdr.biSize = 40L;
	bmphdr.biWidth = (unsigned long)bmptr->width;
	bmphdr.biHeight =(unsigned long)bmptr->height;
	bmphdr.biPlanes = 1;
	bmphdr.biBitCount = 8;
	bmphdr.biCompression = 0L;
	bmphdr.biSizeImage = imgsize;
	bmphdr.biXPelsPerMeter = 1181L;  /* 30 dpi - low res */
	bmphdr.biYPelsPerMeter = 1181L;  /* 30 dpi - low res */
	bmphdr.biClrUsed = 256L;
	bmphdr.biClrImportant = 0L;
  
	if( !WriteBytes( file, &bmphdr, sizeof(struct BMPHeader) ) )
	{
		return 0L;
	}
	if( linesize < 1024 )
	{
		buffer = (char*)malloc( 1024L );
	}
	else
	{
		buffer = (char*)malloc((unsigned long)linesize);
	}
	/* write the color palette */
	x = buffer;

	for( i = 0; i < 256; i++ )
	{
		*x++ = palette256[i].B;
		*x++ = palette256[i].G;
		*x++ = palette256[i].R;
		*x++ = 0;
	}
	if( !WriteBytes( file, buffer, 1024L ) )
	{
		free( buffer );
		return 0L;
	}
	unsigned long size = (unsigned long)bmptr->width * (unsigned long)bmptr->height;

	/* write the bitmap data (from bottom to top) */
	for( r = bmptr->height - 1; r >= 0; r-- )
	{
		for ( c = 0; c < bmptr->width; c++ )
		{
			buffer[c] = bmptr->data[c + bmptr->width * r];
		}
		for( ; c < linesize; c++ )
		{
			buffer[c] = 0;
		}
		if( !WriteBytes( file, buffer, (unsigned long)linesize ) )
		{
			free( buffer );
			return 0L;
		}
	}
	free( buffer );

	return imgoffset + imgsize;
}


/*******************************************************************************
 *
 *      FUNCTION:
 *              long SavePCX( FILE *file, BitMap *bmptr, RGB *palette256 );
 *
 ******************************************************************************/
long SavePCX( FILE *file, BitMap *bmptr, RGB *palette256 )
{
	struct PCXHeader        pcxhdr;
	char                    *x;
	char                    *buf;
	unsigned short          linesize;
	unsigned long           imgsize;
	unsigned long           imgoffset;
	short                   r, c;
	long                    i, j;


 
	if( file == NULL || bmptr->width == 0 || bmptr->height == 0 || palette256 == NULL )
	{
		return 0L;
	}
	linesize = (bmptr->width + 3) & (~3);
	imgsize = (unsigned long)linesize * (unsigned long)bmptr->height;
	imgoffset = (unsigned long)sizeof(struct PCXHeader) + 768L;

	pcxhdr.originator = 0xA;
	pcxhdr.version = 5;
	pcxhdr.encoding = 1;
	pcxhdr.bitsperpixel = 8;
	pcxhdr.x1 = 0;
	pcxhdr.y1 = 0;
	pcxhdr.x2 = bmptr->width-1;
	pcxhdr.y2 = bmptr->height-1;
	pcxhdr.modehres = 0x96;
	pcxhdr.modevres = 0x96;

	for( i = 0; i < 48; i++ )
	{
		pcxhdr.pal[i] = '\0';
	}
	pcxhdr.charsize = '\0'; 
	pcxhdr.nplanes = 1;
	pcxhdr.bytesperline = linesize; 
	pcxhdr.displaytype = 1;

	for( i = 0; i < 58; i++ )
	{
		pcxhdr.filter[i] = '\0';
	}
	fseek( file, 0, SEEK_SET ); 

	if( !WriteBytes( file, &pcxhdr, sizeof(struct PCXHeader) ) )
	{
		return 0L;
	}
	linesize = (unsigned short)pcxhdr.nplanes * (unsigned short)pcxhdr.bytesperline;

	if( linesize < 1024 )
	{
		buf = (char*)malloc( 1024L );
	}
	else
	{
		buf = (char*)malloc((unsigned long)linesize);
	}
	fseek( file, 128, SEEK_SET ); 

	for( r = 0; r < bmptr->height; r++ )
	{
		for ( c = 0; c < bmptr->width; c++ )
		{
			buf[c] = bmptr->data[c + bmptr->width * r];

			if( ( buf[c] & 192 ) == 192 )
			{
				buf[c++] =  0x01;
			}
		}
		for( ; c < linesize; c++ )
		{
			buf[c] = 0;
		}
		if( !WriteBytes( file, buf, (unsigned long)linesize ) )
		{
			free( buf );
			return 0L;
		}
	}
	/* write the color palette */
	for( i = 0, j = 0; i < 256; i++ )
	{
		buf[j++] = palette256[i].R;
		buf[j++] = palette256[i].G;
		buf[j++] = palette256[i].B;
	}
	if( !WriteBytes( file, buf, 768 ) )
	{
		free( buf );
		return 0L;
	}
	free( buf );

	return imgoffset + imgsize;
}


/*******************************************************************************
 *
 *      FUNCTION:
 *              long fsize( FILE * file )
 *
 ******************************************************************************/
long fsize( FILE * file )
{
	long pos, end;
	pos = ftell(file);
	fseek(file, 0, SEEK_END);
	end = ftell(file);
	fseek(file, pos, SEEK_SET);
	return end;
}


/*******************************************************************************
 *
 *      FUNCTION:
 *              XMSarray<char>* AllocFile( XMSarray<char> *pX, long* size, char* file )
 *
 ******************************************************************************/
XMSarray<char>* AllocFile( XMSarray<char>* pX, long* size, char* file )
{
	FILE            *fthing;        // descriptor para el fichero THI.
	char            buf[BUFFSIZE];  // array para ir copiando bloques del fichero a memoria.
	char            *ptr;           // puntero auxiliar para desplazarse por el buffer.
	unsigned long   n;              // # de bytes escritos.
	int             nb;             // # de veces que se debe escribir.
	int             cb = BUFFSIZE;  // # de bytes que se deben escribir.
	int             rb;             // # resto de bytes que quedan por escibir.
	long            i = 0;          // contador de # de ciclos de copia.
	long            j = 0;          // contador de caracteres del buffer.
	long            l = 0;          // contador de caracteres del array de XMS.



	if( ( fthing = fopen( file, "rb" ) ) == NULL ) {
		TraceLog( TEST_LOG, "AllocFile(): fopenErr %s", file );
		return pX = NULL;
	}
	*size = fsize( fthing );

	nb = *size / cb;
	rb = *size % cb;

	fseek( fthing, 0, 0 );

	for( i = 0; i < nb; i++ )
	{
		if( ( n = fread( &buf[0], sizeof(char), cb, fthing ) ) != cb )
		{
			TraceLog( TEST_LOG, "AllocFile(): freadErr n=%ld", n );
			return pX = NULL;
		}
		for( j = 0; j < cb; j++, l++ )
		{
			(*pX)[l] = buf[j];
		}
	}
	if( ( n = fread( &buf[0], sizeof(char), rb, fthing ) ) != rb )
	{
		TraceLog( TEST_LOG, "AllocFile(): freadErr n=%ld", n );
		return pX = NULL;
	}
	fclose( fthing );

	for( j = 0; j < rb; j++, l++ )
	{
		(*pX)[l] = buf[j];
	}
	*size = l;

	return pX;
}


/*******************************************************************************
 *
 *      FUNCTION:
 *              int FreeFile( XMSarray<char>* pX, long size, char* file )
 *
 ******************************************************************************/
int FreeFile( XMSarray<char>* pX, long size, char* file )
{
	FILE            *fthing;        // descriptor para el fichero THI.
	char            buf[BUFFSIZE];  // buffer auxiliar para ir copiando caracteres.
	char            *ptr;           // puntero para ir copiando bloques del fichero a memoria.
	unsigned long   n;              // # de bytes escritos.
	int             nb;             // contador para el # de veces que se debe escribir.
	int             cb = BUFFSIZE;  // # de bytes que se deben escribir.
	int             rb;             // # resto de bytes que quedan por escibir.
	long            i = 0;          // contador de # de ciclos de copia.
	long            j = 0;          // contador de caracteres del buffer.
	long            l = 0;          // contador de caracteres del array de XMS.



	if( ( fthing = fopen( file, "wb" ) ) == NULL ) {
		TraceLog( TEST_LOG, "FreeFile(): fopenErr %s", file );
		return 0;
	}
	nb = size / cb;
	rb = size % cb;

	fseek( fthing, 0, 0 );

	for( i = 0; i < nb; i++ )
	{
		for( j = 0; j < cb; j++, l++ )
		{
			buf[j] = (*pX)[l];
		}
		if( ( n = fwrite( &buf[0], sizeof(char), cb, fthing ) ) != cb )
		{
			TraceLog( TEST_LOG, "FreeFile(): fwriteErr n=%ld", n );
			return 0;
		}
	}
	for( j = 0; j < rb; j++, l++ )
	{
		buf[j] = (*pX)[l];
	}
	if( ( n = fwrite( &buf[0], sizeof(char), rb, fthing ) ) != rb )
	{
		TraceLog( TEST_LOG, "FreeFile(): fwriteErr n=%ld", n );
		return 0;
	}
	fclose( fthing );

	return 1;
}



/*******************************************************************************
 *
 *      FUNCTION:
 *              long InitMipTexs( char* dirname, XMSarray<miptex_t>* gMipTex )
 *
 ******************************************************************************/
long InitMipTexs( char* dirname, XMSarray<miptex_t>* gMipTex )
{
	FILE*                   lmp;
	char                    filename[257];
	miptex_t                mipTex;
	dmiptexlump_t           dMipTexLump;
	long                    entcount = 0;
	long                    i = 0;
	long                    nh = 0;


	strcpy( filename, dirname );
	strcat( filename, "\\" );
	strcat( filename, ENTRY02 );

	lmp = fopen( filename, "rb" );

	if( lmp == NULL ) {
		TraceLog( TEST_LOG, "File not found (%s)", filename );
		return 0;
	}
	fseek( lmp, 0, SEEK_SET );

	fread( &dMipTexLump, sizeof( dmiptexlump_t ), 1, lmp );

	entcount = dMipTexLump.nummiptex;

	if( gMipTex != NULL )
	{
		nh = (long)gMipTex->size();
	
		for( i = 0; i < nh; i++ )
		{
			fseek( lmp, dMipTexLump.dataofs[i], SEEK_SET );

			fread( &mipTex, sizeof( miptex_t ), 1, lmp );

			(*gMipTex)[i] = mipTex;
		}
	}
	else
	{
		if( entcount < MAX_MIPTEXS_NUM )
		{
			nh = entcount;
		}
		else
		{
			nh = MAX_MIPTEXS_NUM;
		}
	}
	fclose( lmp );

	return nh;
}


/*******************************************************************************
 *
 *      FUNCTION:
 *              long InitDHulls( char* dirname, XMSarray<dhull_t>* gDHull )
 *
 ******************************************************************************/
long InitDHulls( char* dirname, XMSarray<dhull_t>* gDHull )
{
	FILE*                   lmp;
	char                    filename[257];
	dhull_t                 dhull;
	long                    entcount = 0;
	long                    i = 0;
	long                    nh = 0;


	strcpy( filename, dirname );
	strcat( filename, "\\" );
	strcat( filename, ENTRY14 );

	lmp = fopen( filename, "rb" );

	if( lmp == NULL ) {
		TraceLog( TEST_LOG, "File not found (%s)", filename );
		return 0;
	}
	entcount = fsize( lmp ) / sizeof( dhull_t );

	if( gDHull != NULL )
	{
		nh = (long)gDHull->size();

		for( i = 0; i < nh; i++ )
		{
			fread( &dhull, sizeof( dhull_t ), 1, lmp );

			(*gDHull)[i] = dhull;
		}
	}
	else
	{
		if( entcount < MAX_DHULLS_NUM )
		{
			nh = entcount;
		}
		else
		{
			nh = MAX_DHULLS_NUM;
		}
	}
	fclose( lmp );

	return nh;
}


/*******************************************************************************
 *
 *      FUNCTION:
 *              long InitNodes( char* dirname, XMSarray<node_t>* gNode )
 *
 ******************************************************************************/
long InitNodes( char* dirname, XMSarray<node_t>* gNode )
{
	FILE*                   lmp;
	char                    filename[257];
	node_t                  node;
	long                    entcount = 0;
	long                    i = 0;
	long                    nh = 0;


	strcpy( filename, dirname );
	strcat( filename, "\\" );
	strcat( filename, ENTRY05 );

	lmp = fopen( filename, "rb" );

	if( lmp == NULL ) {
		TraceLog( TEST_LOG, "File not found (%s)", filename );
		return 0;
	}
	entcount = fsize( lmp ) / sizeof( node_t );

	if(  gNode != NULL )
	{
		nh = (long)gNode->size();
	
		for( i = 0; i < nh; i++ )
		{
			fread( &node, sizeof( node_t ), 1, lmp );

			(*gNode)[i] = node;
		}
	}
	else
	{
		if( entcount < MAX_NODES_NUM )
		{
			nh = entcount;
		}
		else
		{
			nh = MAX_NODES_NUM;
		}
	}
	fclose( lmp );

	return nh;
}


/*******************************************************************************
 *
 *      FUNCTION:
 *              long InitDLeafs( char* dirname, XMSarray<dleaf_t>* gDLeaf )
 *
 ******************************************************************************/
long InitDLeafs( char* dirname, XMSarray<dleaf_t>* gDLeaf )
{
	FILE*                   lmp;
	char                    filename[257];
	dleaf_t                 dleaf;
	long                    entcount = 0;
	long                    i = 0;
	long                    nh = 0;



	strcpy( filename, dirname );
	strcat( filename, "\\" );
	strcat( filename, ENTRY10 );

	lmp = fopen( filename, "rb" );

	if( lmp == NULL ) {
		TraceLog( TEST_LOG, "File not found (%s)", filename );
		return 0;
	}
	entcount = fsize( lmp ) / sizeof( dleaf_t );

	if( gDLeaf != NULL )
	{
		nh = (long)gDLeaf->size();

	
		for( i = 0; i < nh; i++ )
		{
			fread( &dleaf, sizeof( dleaf_t ), 1, lmp );

			(*gDLeaf)[i] =  dleaf;
		}
	}
	else
	{
		if( entcount < MAX_LEAFS_NUM )
		{
			nh = entcount;
		}
		else
		{
			nh = MAX_LEAFS_NUM;
		}
	}
	fclose( lmp );

	return nh;
}


/*******************************************************************************
 *
 *      FUNCTION:
 *              long InitSurfaces( char* dirname, XMSarray<surface_t>* gSurface )
 *
 ******************************************************************************/
long InitSurfaces( char* dirname, XMSarray<surface_t>* gSurface )
{
	FILE*                   lmp;
	char                    filename[257];
	surface_t               surface;
	long                    entcount = 0;
	long                    i = 0;
	long                    nh = 0;


	strcpy( filename, dirname );
	strcat( filename, "\\" );
	strcat( filename, ENTRY07 );

	lmp = fopen( filename, "rb" );

	if( lmp == NULL ) {
		TraceLog( TEST_LOG, "File not found (%s)", filename );
		return 0;
	}
	entcount = fsize( lmp ) / sizeof( surface_t );

	if( gSurface != NULL )
	{       
		nh = (long)gSurface->size();
	
		for( i = 0; i < nh; i++ )
		{
			fread( &surface, sizeof( surface_t ), 1, lmp );

			(*gSurface)[i] = surface;

		}
	}
	else
	{
		if( entcount < MAX_SURFACES_NUM )
		{
			nh = entcount;
		}
		else
		{
			nh = MAX_SURFACES_NUM;
		}
	}
	fclose( lmp );

	return nh;
}


/*******************************************************************************
 *
 *      FUNCTION:
 *              long InitEdges( char* dirname, XMSarray<edge_t>* gEdge )
 *
 ******************************************************************************/
long InitEdges( char* dirname, XMSarray<edge_t>* gEdge )
{
	FILE*                   lmp;
	char                    filename[257];
	edge_t                  edge, edge2;
	long                    entcount = 0;
	long                    i = 0;
	long                    nh = 0;


	strcpy( filename, dirname );
	strcat( filename, "\\" );
	strcat( filename, ENTRY12 );

	lmp = fopen( filename, "rb" );

	if( lmp == NULL ) {
		TraceLog( TEST_LOG, "File not found (%s)", filename );
		return 0;
	}
	entcount = fsize( lmp ) / sizeof( edge_t );

	if( gEdge != NULL )
	{
		nh = gEdge->size();
	
		for( i = 0; i < nh; i++ )
		{
			fread( &edge, sizeof( edge_t ), 1, lmp );

			(*gEdge)[i] = edge;
		}
		fseek( lmp, 0, SEEK_SET );

		for( i = 0; i < nh; i++ )
		{
			fread( &edge, sizeof( edge_t ), 1, lmp );

			edge2 = (*gEdge)[i];

			if( ( edge2.startvertex != edge.startvertex ) && 
				( edge2.endvertex != edge.endvertex ) )
			{
				return nh = 0;
			}
		}
	}
	else
	{
		if( entcount < MAX_EDGES_NUM )
		{
			nh = entcount;
		}
		else
		{
			nh = MAX_EDGES_NUM;
		}
	}
	fclose( lmp );

	return nh;
}


/*******************************************************************************
 *
 *      FUNCTION:
 *              long InitVertexs( char* dirname, XMSarray<vertex_t>* gVertex )
 *
 ******************************************************************************/
long InitVertexs( char* dirname, XMSarray<vertex_t>* gVertex )
{
	FILE*                   lmp;
	char                    filename[257];
	vertex_t                vertex;
	long                    entcount = 0;
	long                    i = 0;
	long                    nh = 0;


	strcpy( filename, dirname );
	strcat( filename, "\\" );
	strcat( filename, ENTRY03 );

	lmp = fopen( filename, "rb" );

	if( lmp == NULL ) {
		TraceLog( TEST_LOG, "File not found (%s)", filename );
		return 0;
	}
	entcount = fsize( lmp ) / sizeof( vertex_t );

	if( gVertex != NULL )
	{       
		nh = (long)gVertex->size();
	
		for( i = 0; i < nh; i++ )
		{
			fread( &vertex, sizeof( vertex_t ), 1, lmp );

			(*gVertex)[i] = vertex;
		}
	}
	else
	{
		if( entcount < MAX_VERTEXS_NUM )
		{
			nh = entcount;
		}
		else
		{
			nh = MAX_VERTEXS_NUM;
		}
	}
	fclose( lmp );

	return nh;
}


/******************************************************************************
 *
 *      FUNCTION:
 *              char* GetClass( char *buf,
 *                              char* classname )
 *
 ******************************************************************************/
char* GetClass( char *buf,
		char* classname )
{
	char            *ptr;



	strcpy( buf, classname );

	for( ptr = buf; *ptr != '_'; ptr++ );

	*ptr = '\0';

	return buf;
}


/******************************************************************************
 *
 *      FUNCTION:
 *              long InitClassId(       char *filename,
 *                                      XMSarray<ClassId> *pClassIds,
 *                                      long *numClassIds )
 *
 ******************************************************************************/
long InitClassIds(      char *filename,
			XMSarray<ClassId> *pClassIds,
			long *numClassIds )
{
	ifstream        ifs;            // descriptor del fichero de classnames.
	char            line[257];      // linea para los datos.
	long            i = 0;          // contador para los classnames de cada class
	long            j = 0;          // contador para los classnames
	long            k = 0;          // contador para los classMembers
	ClassId         classid, oldclassid;
	char            *ptr;
	char            buf[MAX_CLASSNAME_LEN];
	char            oldbuf[MAX_CLASSNAME_LEN];
	int             valid = 0;



	strcpy( buf, "" );
	strcpy( oldbuf, "p" );
	strcpy( classid.className, "" );
	classid.classId = 0x00000000;
	classid.classMembers = 0;
	strcpy( oldclassid.className, "" );
	oldclassid.classId = 0x00000000;
	oldclassid.classMembers = 0;

	ifs.open( filename, ios::in | ios::binary );

	if( !ifs ) {
		TraceLog( TEST_LOG, "InitClassIds(): File not found (%s)", filename );
		*numClassIds = 0;
		ifs.close();
		return *numClassIds;
	}
	do {
		valid = 0;

		ifs.getline( line, 257 );

		if( ifs.eof() )
		{
			break;
		}
		if( ( line[0] != '#' ) && ( line[0] != ' ' ) && ( line[0] != '\n' ) )
		{
			line[strlen(line)-1] = '\0';

			strcpy( classid.className, line );
	
			strcpy( oldbuf, buf );
	
			GetClass( buf, classid.className );

			if( !strcmp( buf, oldbuf ) )
			{
				i++;
			}
			else
			{
				for( k = 0; k < i+1; k++ )
				{
					oldclassid = (*pClassIds)[j-k-1];
					oldclassid.classMembers = i+1;
					(*pClassIds)[j-k-1] = oldclassid;
				}
				i = 0;
			} 
			if( !strcmp( buf, "light" ) )
			{
				classid.classId = 0x00000100 | i;
				valid = 1;
			}
			else if( !strcmp( buf, "weapon" ) )
			{
				classid.classId = 0x00000200 | i;
				valid = 1;
			}
			else if( !strcmp( buf, "monster" )  )
			{
				classid.classId = 0x00000400 | i;
				valid = 1;
			}
			else if( !strcmp( buf, "item" ) )
			{
				classid.classId = 0x00000800 | i;
				valid = 1;
			}
			else if( !strcmp( buf, "info" ) )
			{
				classid.classId = 0x00001000 | i;
				valid = 1;
			}
			else if( !strcmp( buf, "path" ) )
			{
				classid.classId = 0x00002000 | i;
				valid = 1;
			}
			else if( !strcmp( buf, "func" ) )
			{
				classid.classId = 0x00004000 | i;
				valid = 1;
			}
			else if( !strcmp( buf, "trigger" ) )
			{
				classid.classId = 0x00008000 | i;
				valid = 1;
			}
			else if( !strcmp( buf, "worldspawn" ) )
			{
				classid.classId = 0x00010000 | i;
				valid = 1;
			}
			else if( !strcmp( buf, "ambient" ) )
			{
				classid.classId = 0x00020000 | i;
				valid = 1;
			}
			if ( valid )
			{
				(*pClassIds)[j++] = classid;
			}
		}
	} while( ifs );

	ifs.close();

	*numClassIds = j;

	for( j = 0; j < *numClassIds; j++ )
	{
		classid = (*pClassIds)[j];
		TraceLog( TEST_LOG, "className=%s classId=%lx classMembers=%ld", classid.className, classid.classId, classid.classMembers );
	}
	return *numClassIds;
}


/******************************************************************************
 *
 *      FUNCTION:
 *              long GetClassId(        XMSarray<ClassId> *pClassIds,
 *                                      long maxClassIds,
 *                                      char *buf,
 *                                      unsigned long* mask )
 *
 ******************************************************************************/
long GetClassId(        XMSarray<ClassId> *pClassIds,
			long maxClassIds,
			char *buf,
			unsigned long *mask )
{
	long                    i = 0;
	ClassId                 classid;


	strlwr( buf );

	for( i = 0; i < maxClassIds; i++ )
	{
		classid = (*pClassIds)[i];

		if( !strcmp( buf, classid.className ) )
		{
			*mask = *mask | classid.classId;
			break;
		}
	}
	return *mask;
}


/******************************************************************************
 *
 *      FUNCTION:
 *              char* GetClassName(     XMSarray<ClassId> *pClassIds,
 *                                      long maxClassIds,
 *                                      char *buf,
 *                                      unsigned long mask )
 *
 ******************************************************************************/
char* GetClassName(     XMSarray<ClassId> *pClassIds,
			long maxClassIds,
			char *buf,
			unsigned long mask )
{
	unsigned long           id = 0x00000000;
	long                    i = 0;
	ClassId                 classid;



	id = mask & 0x00FFFFFF;

	strcpy( buf, "" );

	for( i = 0; i < maxClassIds; i++ )
	{
		classid = (*pClassIds)[i];

		if( id == classid.classId )
		{
			strcpy( buf, classid.className );
			break;
		}
	}
	return buf;
}


/******************************************************************************
 *
 *      FUNCTION:
 *              long GetMipTex( char *nameMipTex,
 *                              XMSarray<miptex_t> *XM,
 *                              long numXMs,
 *                              miptex_t *pMipTex )
 *
 ******************************************************************************/
long GetMipTex( char *nameMipTex,
		XMSarray<miptex_t> *XM,
		long numXMs,
		miptex_t *pMipTex )
{
	long                    i = 0;
	miptex_t                mipTex;


	for( i = 0; i < numXMs; i++ )
	{
		mipTex = (*XM)[i];

		if( !strcmp( nameMipTex, mipTex.name ) )
		{
			break;
		}
	}
	if( i == numXMs )
	{
		return -1;
	}
	*pMipTex = mipTex;

	return i;
}


/******************************************************************************
 *
 *      FUNCTION:
 *              long GetClassNum(     XMSarray<ClassId> *pClassIds,
 *                                    long maxClassIds,
 *                                    long *num,
 *                                    unsigned long mask )
 *
 ******************************************************************************/
long GetClassNum(       XMSarray<ClassId> *pClassIds,
			long maxClassIds,
			long *num,
			unsigned long mask )
{
	unsigned long           id = 0x00000000;
	long                    i = 0;
	ClassId                 classid;



	id = mask & 0x00FFFFFF;

	for( i = 0; i < maxClassIds; i++ )
	{
		classid = (*pClassIds)[i];

		if( id == classid.classId )
		{
			*num = classid.classMembers;
			break;
		}
	}
	return *num;
}


/******************************************************************************
 *
 *      FUNCTION:
 *              long GetListClassName(  XMSarray<char> *pClassNames,
 *                                      long *numClassNames,
 *                                      XMSarray<ClassId> *pClassId,
 *                                      long maxClassId,
 *                                      unsigned long mask )
 *
 ******************************************************************************/
long GetListClassName(  XMSarray<char> *pClassNames,
			long *numClassNames,
			XMSarray<ClassId> *pClassId,
			long maxClassId,
			unsigned long mask )
{
	unsigned long           id = 0x00000000;
	long                    index = 0;
	long                    num = 0;
	long                    l = 0;
	long                    classMembers = 0;
	unsigned int            i = 0;
	char                    buf[MAX_CLASSNAME_LEN];
	

	num = 0;

	if(  mask & 0x00000200 )
	{       
		GetClassNum( pClassId, maxClassId, &classMembers, 0x00000200 );

		for( index = 0; index < classMembers; index++, num++ )
		{
			id = 0x00000200 | index;

			GetClassName( pClassId, maxClassId, buf, id );
		
			for( i = 0; i < MAX_CLASSNAME_LEN; i++ )
			{
				(*pClassNames)[l++] = buf[i];
			}
		}
	}
	if(  mask & 0x00000400 )
	{       
		GetClassNum( pClassId, maxClassId, &classMembers, 0x00000400 );

		for( index = 0; index < classMembers; index++, num++ )
		{
			id = 0x00000400 | index;

			GetClassName( pClassId, maxClassId, buf, id );

			for( i = 0; i < MAX_CLASSNAME_LEN; i++ )
			{
				(*pClassNames)[l++] = buf[i];
			}
		}
	}
	if(  mask & 0x00000800 )
	{       
		GetClassNum( pClassId, maxClassId, &classMembers, 0x00000800 );

		for( index = 0; index < classMembers; index++, num++ )
		{
			id = 0x00000800 | index;

			GetClassName( pClassId, maxClassId, buf, id );

			for( i = 0; i < MAX_CLASSNAME_LEN; i++ )
			{
				(*pClassNames)[l++] = buf[i];
			}
		}
	}
	*numClassNames = num;

	return num;
}


/******************************************************************************
 *
 *      FUNCTION:
 *              unsigned long GetClassModel( char* buf, unsigned long* mask )
 *
 ******************************************************************************/
unsigned long GetClassModel( char* buf, unsigned long* mask )
{
	long            c = 0;


	
	*mask = *mask | 0x01000000;

	c = atol( &buf[1] );

	return c;
}


/******************************************************************************
 *
 *      FUNCTION:
 *              unsigned long GetClassTarget( char* buf, unsigned long* mask )
 *
 ******************************************************************************/
unsigned long GetClassTarget( char* buf, unsigned long* mask )
{
	long            c = 0;


	
	*mask = *mask | 0x02000000;

	c = atol( &buf[1] );

	return c;
}


/******************************************************************************
 *
 *      FUNCTION:
 *              unsigned long GetClassTargetName( char* buf, unsigned long* mask )
 *
 ******************************************************************************/
unsigned long GetClassTargetName( char* buf, unsigned long* mask )
{
	long            c = 0;

	
	*mask = *mask | 0x04000000;

	c = atol( &buf[1] );

	return c;
}


/*******************************************************************************
 *
 *      FUNCTION:
 *              long SurfaceToGLine(    surface_t s,
 *                                      XMSarray<edge_t> *XME,
 *                                      XMSarray<vertex_t> *XMV,
 *                                      XMSarray<GLine> *pGLine,
 *                                      long *numGLines )
 *
 ******************************************************************************/
long SurfaceToGLine(    surface_t s,
			XMSarray<edge_t> *XME,
			XMSarray<vertex_t> *XMV,
			XMSarray<GLine> *pGLine,
			long *numGLines )
{
	long            i = 0;          // contador para el # de la surface...
	long            fe = 0;         // 1er edge de la surface...
	long            ne = 0;         // # de edges de la surface...
	unsigned short  sv = 0;         // primer vertice del edge de la surface...
	unsigned short  ev = 0;         // ultimo vertice del edge de la surface...
	edge_t          edge;           // edge auxiliar...
	vertex_t        svertex;        // primer vertex auxiliar...
	vertex_t        evertex;        // ultimo vertex auxiliar...
	GLine           gline;          // GLine auxiliar ...



	// Calculamos el primer edge de la surface.
	fe = s.firstedge;

	// Calculamos el numero de edges de la surface.
	ne = s.numedge;

	// Para todos los edges de la surface.
	for( i = 0; i < ne; i++ )
	{
		if( (fe+i) > MAX_EDGES_NUM ) { 
			// TraceLog( TEST_LOG, "SurfaceToGLine(): edges overflowErr %ld", fe+i );
			*numGLines = 0;
			return *numGLines;
		}
		// Obtenemos el edge de la superficie.
		edge = (*XME)[fe+i];

		// Calculamos los nuevos vertices.
		sv = edge.startvertex;
		ev = edge.endvertex;
		svertex = (*XMV)[sv];
		evertex = (*XMV)[ev];
		gline.sv.x = svertex.x;
		gline.sv.y = svertex.y;
		gline.sv.z = svertex.z;
		gline.ev.x = evertex.x;
		gline.ev.y = evertex.y;
		gline.ev.z = evertex.z;
		(*pGLine)[*numGLines+i] = gline;
	}
	// Actualizamos el # de vertexs que habia mas los nuevos;
	*numGLines += i;

	// Devolvemos la direccion del primer vertex;
	return *numGLines;
}


/*******************************************************************************
 *
 *      METODO:
 *              long InitGLines(        XMSarray<dleaf_t> *FML,
 *                                      long numXLs,
 *                                      XMSarray<surface_t> *FMS,
 *                                      XMSarray<edge_t> *FME,
 *                                      XMSarray<vertex_t> *FMV,
 *                                      XMSarray<GLine> *FMG,
 *                                      long *numXGs )
 *
 ******************************************************************************/
long InitGLines(        XMSarray<dleaf_t> *FML,
			long numXLs,
			XMSarray<surface_t> *FMS,
			XMSarray<edge_t> *FME,
			XMSarray<vertex_t> *FMV,
			XMSarray<GLine> *FMG,
			long *numXGs )
{
	long            i = 0;                  // contador para el # de dleafs...
	long            k = 0;                  // contador para el # de vertexs...
	long            l = 0;                  // contador para el # de GPoints...
	unsigned short  j = 0;                  // contador para el # de surfaces...
	unsigned short  fs = 0;                 // indice de la 1era surface...
	unsigned short  ns = 0;                 //  # de superficies del leaf...
	dleaf_t         dleaf;                  // dleaf auxiliar...
	surface_t       surface;                // surface auxiliar...
	XMSarray<GLine> *pGLine;                // array para los vertices de la superficie...
	long            numGLines = 0;          // # de vertexs de la superficie...


	if( ( pGLine = new XMSarray<GLine>( MAX_SURFGLINES_NUM ) ) == NULL )
	{
		TraceLog( TEST_LOG, "InitGLines(): newErr" );
		return l = 0;
	}
	if( !pGLine->valid()  )
	{
		TraceLog( TEST_LOG, "InitGLines(): validErr GLines" );
		return l = 0;
	}
	// Para todos los leafs de la matriz...
	for( i = 0; i < numXLs; i++ )
	{
		// Obtenemos la primera surface del leaf.
		dleaf = (*FML)[i];
		ns = dleaf.numsurf;
		fs = dleaf.firstsurf;

		// Para todas las surface del dleaf 
		for( j = 0; j < ns; j++ )
		{
			// Obtenemos la superficie del leaf.
			surface = (*FMS)[j+fs];

			// Inicializamos el # de vertexs.
			numGLines = 0;

			// Aumentamos el array con los vertices de la suferficie. 
			SurfaceToGLine( surface, FME, FMV, pGLine, &numGLines );

			// Si se han obtenido los GLine.
			if( numGLines != 0 )
			{
				for( k = 0; k < numGLines; k++ )
				{
					if( l < MAX_GLINES_NUM )
					{
						(*FMG)[l++] = (*pGLine)[k];
					}
					else
					{
						delete pGLine;
						*numXGs = l;
						return l;
					}
				}
			}
		working->AddChar( '.' );
		}
	}
	delete pGLine;
	*numXGs = l;
	return l;

} // InitGLines


/*******************************************************************************
 *
 *      METODO:
 *              long InitGLines(        XMSarray<dleaf_t> *FML,
 *                                      XMSarray<long> *XLI,
 *                                      long numXLIs,
 *                                      XMSarray<surface_t> *FMS,
 *                                      long numXSs,
 *                                      XMSarray<edge_t> *FME,
 *                                      XMSarray<vertex_t> *FMV,
 *                                      XMSarray<GLine> *FMG,
 *                                      long *numXGs )
 *
 ******************************************************************************/
long InitGLines(        XMSarray<dleaf_t> *FML,
			XMSarray<long> *XLI,
			long numXLIs,
			XMSarray<surface_t> *FMS,
			XMSarray<edge_t> *FME,
			XMSarray<vertex_t> *FMV,
			XMSarray<GLine> *FMG,
			long *numXGs )
{
	long            i = 0;                  // contador para el # de dleafs...
	long            k = 0;                  // contador para el # de vertexs...
	long            l = 0;                  // contador para el # de points...
	unsigned short  j = 0;                  // contador para el # de surfaces...
	unsigned short  fs = 0;                 // indice de la 1era surface...
	unsigned short  ns = 0;                 // # de superficies del leaf...
	dleaf_t         dleaf;                  // dleaf auxiliar...
	surface_t       surface;                // surface auxiliar...
	XMSarray<GLine> *pGLine;                // array para los vertices de la superficie...
	long            numGLines = 0;          // # de vertexs de la superficie...


	if( ( pGLine = new XMSarray<GLine>( MAX_SURFGLINES_NUM ) ) == NULL )
	{
		TraceLog( TEST_LOG, "InitGLines(): newErr" );
		return l = 0;
	}
	if( !pGLine->valid()  )
	{
		TraceLog( TEST_LOG, "InitGLines(): validErr GLines" );
		return l = 0;
	}
	// Para todos los leafs de la matriz...
	for( i = 0; i < numXLIs; i++ )
	{
		// Obtenemos la primera surface del leaf.
		dleaf = (*FML)[(*XLI)[i]];
		ns = dleaf.numsurf;
		fs = dleaf.firstsurf;

		// Para todas las surface del dleaf 
		for( j = 0; j < ns; j++ )
		{
			// Obtenemos la superficie del leaf.
			surface = (*FMS)[j+fs];

			// Inicializamos el # de vertexs.
			numGLines = 0;

			// Aumentamos el array con los vertices de la suferficie. 
			SurfaceToGLine( surface, FME, FMV, pGLine, &numGLines );

			// Si se han obtenido los GLine.
			if( numGLines != 0 )
			{
				for( k = 0; k < numGLines; k++ )
				{
					if( l < MAX_GLINESELECS_NUM )
					{
						// La introducimos y aumentamos el contador.
						(*FMG)[l++] = (*pGLine)[k];
					}
					else
					{
						delete pGLine;
						*numXGs = l;
						return l;
					}
				}
			}
		}
	}
	delete pGLine;
	*numXGs = l;
	return l;

} // InitGLines


/*******************************************************************************
 *
 *      METODO:
 *              long InitGLines(        shortbox_t sbox,
 *                                      XMSarray<GLine> *XG,
 *                                      long numXGs,
 *                                      XMSarray<GLine> *XGI,
 *                                      long *numXGIs )
 *
 ******************************************************************************/
long InitGLines(        shortbox_t sbox,
			XMSarray<GLine> *XG,
			long numXGs,
			XMSarray<GLine> *XGI,
			long *numXGIs )
{
	long            i = 0;                  // contador para el # de GLines...
	long            l = 0;                  // contador para el # de GLines...
	GLine           gline;                  // GLine auxiliar...



	// Para todos los leafs de la matriz...
	for( i = 0; i < numXGs; i++ )
	{
		// Obtenemos la primera surface del leaf.
		gline = (*XG)[i];

		if(     ( ( (short)gline.sv.x >= sbox.min.x ) && 
			( (short)gline.sv.x <= sbox.max.x ) &&
			( (short)gline.sv.y >= sbox.min.y ) && 
			( (short)gline.sv.y <= sbox.max.y ) &&
			( (short)gline.sv.z >= sbox.min.z ) && 
			( (short)gline.sv.z <= sbox.max.z ) ) &&
			( ( (short)gline.ev.x >= sbox.min.x ) && 
			( (short)gline.ev.x <= sbox.max.x ) &&
			( (short)gline.ev.y >= sbox.min.y ) && 
			( (short)gline.ev.y <= sbox.max.y ) &&
			( (short)gline.ev.z >= sbox.min.z ) && 
			( (short)gline.ev.z <= sbox.max.z ) ) )
		{ 
			if( l < MAX_GLINESELECS_NUM )
			{
				(*XGI)[l++] = gline;
			}
			else
			{
				*numXGIs = l;
				return l;
			}
		}
	}
	*numXGIs = l;
	return l;

} // InitGLines


/*******************************************************************************
 *
 *      METODO:
 *              long InitGLines(        XMSarray<dleaf_t> *XL,
 *                                      XMSarray<long> *XLI,
 *                                      long numXLIs,
 *                                      XMSarray<GLine> *XG,
 *                                      long numXGs,
 *                                      XMSarray<GLine> *XGI,
 *                                      long *numXGIs )
 *
 ******************************************************************************/
long InitGLines(        XMSarray<dleaf_t> *XL,
			XMSarray<long> *XLI,
			long numXLIs,
			XMSarray<GLine> *XG,
			long numXGs,
			XMSarray<GLine> *XGI,
			long *numXGIs )
{
	long            i = 0;                  // contador para el # de dleafs...
	long            k = 0;                  // contador para el # de vertexs...
	long            l = 0;                  // contador para el # de points...
	dleaf_t         dleaf;                  // dleaf auxiliar...
	XMSarray<GLine> *pGLine;                // array para los vertices de la superficie...
	long            numGLines = 0;          // # de vertexs de la superficie...


	if( ( pGLine = new XMSarray<GLine>( MAX_SURFGLINES_NUM ) ) == NULL )
	{
		TraceLog( TEST_LOG, "InitGLines(): newErr" );
		return l = 0;
	}
	if( !pGLine->valid()  )
	{
		TraceLog( TEST_LOG, "InitGLines(): validErr GLines" );
		return l = 0;
	}
	// Para todos los leafs de la matriz...
	for( i = 0; i < numXLIs; i++ )
	{
		// Obtenemos la primera surface del leaf.
		dleaf = (*XL)[(*XLI)[i]];

		numGLines = 0;

		if( InitGLines( dleaf.bound, XG, numXGs, pGLine, &numGLines ) )
		{
			// Si se han obtenido los GLine.
			if( numGLines != 0 )
			{
				for( k = 0; k < numGLines; k++ )
				{
					if( l < MAX_GLINESELECS_NUM )
					{
						// La introducimos y aumentamos el contador.
						(*XGI)[l++] = (*pGLine)[k];
					}
					else
					{
						delete pGLine;
						*numXGIs = l;
						return l;
					}
				}
			}
		}
	}
	delete pGLine;
	*numXGIs = l;
	return l;

} // InitGLines


/*******************************************************************************
 *
 *      METODO:
 *              long InitLeafs( shortbox_t sbox,
 *                              XMSarray<dleaf_t> *XL,
 *                              long numXLs,
 *                              XMSarray<long> *XLI,
 *                              long *numXLIs )
 *
 ******************************************************************************/
long InitLeafs( shortbox_t sbox,
		XMSarray<dleaf_t> *XL,
		long numXLs,
		XMSarray<long> *XLI,
		long *numXLIs )
{
	long            i = 0;                  // contador para el # de dleafs...
	long            l = 0;                  // contador para el # de GPoints...
	dleaf_t         dleaf;                  // dleaf auxiliar...



	// Para todos los leafs de la matriz...
	for( i = 0; i < numXLs; i++ )
	{
		// Obtenemos la primera surface del leaf.
		dleaf = (*XL)[i];

		if(     ( dleaf.bound.min.x >= sbox.min.x ) && 
			( dleaf.bound.max.x <= sbox.max.x ) &&
			( dleaf.bound.min.y >= sbox.min.y ) && 
			( dleaf.bound.max.y <= sbox.max.y ) &&
			( dleaf.bound.min.z >= sbox.min.z ) && 
			( dleaf.bound.max.z <= sbox.max.z ) )
		{ 
			if( l < MAX_LEAFSELECS_NUM )
			{
				(*XLI)[l++] = i;
			}
			else
			{
				*numXLIs = l;
				return l;
			}
		}
	}
	*numXLIs = l;
	return l;

} // InitLeafs


/*******************************************************************************
 *
 *      METODO:
 *              long LeafSurfaces(      XMSarray<dleaf_t> *XL,
 *                                      XMSarray<long> *XLI
 *                                      long numXLIs,
 *                                      XMSarray<long> *XSI,
 *                                      long *numXIs )
 *
 ******************************************************************************/
long LeafSurfaces(      XMSarray<dleaf_t> *XL,
			XMSarray<long> *XLI,
			long numXLIs,
			XMSarray<long> *XSI,
			long *numXSIs )
{
	long                    i = 0;          // contador para el # de indices de los leaf_t.
	unsigned short          j = 0;          // contador para el # de surfaces.
	unsigned short          ns = 0;         // # total de surfaces.
	unsigned short          fs = 0;         // indice de la 1era surface.
	long                    l = 0;          // contador para el # de indices de superficies seleccionadas.
	dleaf_t                 dleaf;          //  dleaf_t auxiliar.



	// Para todos los leafs de la matriz...
	for( i = 0; i < numXLIs; i++ )
	{
		// Obtenemos la primera surface del leaf.
		dleaf = (*XL)[(*XLI)[i]];
		ns = dleaf.numsurf;
		fs = dleaf.firstsurf;

		// Para todas las surface del dleaf 
		for( j = 0; j < ns; j++ )
		{
			if( l < MAX_SURFSELECS_NUM )
			{
				(*XSI)[l++] = j+fs;
			}
			else
			{
				*numXSIs = l;
				return l;
			}
		}
	}
	*numXSIs = l;
	return l;
} 


/******************************************************************************
 *
 *      FUNCTION:
 *              long GetGEntitiTargetName(      long targetname,
 *                                              XMSarray<GEntiti>* XGE,
 *                                              long numGEs,
 *                                              XMSarray<long>* XGI,
 *                                              long* numXGIs )
 *
 ******************************************************************************/
long GetGEntitiTargetName(      long targetname,
				XMSarray<GEntiti>* XGE,
				long numXGEs,
				XMSarray<long>* XGI,
				long* numXGIs )
{
	long            i = 0;          // contador de gentities.
	long            j = 0;          // contador de gentities con el targetname especigicado.
	GEntiti         gentiti;        // gentiti auxiliar.



	for( i = 0; i < numXGEs; i++ )
	{
		gentiti = (*XGE)[i];

		if( gentiti.c & 0x04000000 )
		{
			if( gentiti.targetname == targetname )
			{
				if( j < MAX_TAGS_NUM )
				{
					(*XGI)[j++] = i;
				}
				else
				{
					break;
				}
			}
		}
	}
	*numXGIs = j;

	return j;
}


/******************************************************************************
 *
 *      FUNCTION:
 *              long GetGEntiti(        long id,
 *                                      XMSarray<GEntiti>* XGE,
 *                                      long numGEs,
 *                                      XMSarray<long>* XGI,
 *                                      long* numXGIs )
 *
 ******************************************************************************/
long GetGEntiti(        long id,
			XMSarray<GEntiti>* XGE,
			long numXGEs,
			XMSarray<long>* XGI,
			long* numXGIs )
{
	long            i = 0;  // contador de gentities.
	long            j = 0;  // contador de gentities con el targetname especigicado.
	GEntiti         gentiti;// gentiti auxiliar.



	for( i = 0; i < numXGEs; i++ )
	{
		gentiti = (*XGE)[i];

		if( gentiti.c  == id )
		{
			if( j < *numXGIs )
			{
				(*XGI)[j++] = i;
			}
			else
			{
				break;
			}
		}
	}
	*numXGIs = j;

	return j;
}



/*******************************************************************************
 *
 *      CLASS:
 *              Bsp
 *
 ******************************************************************************/
/*******************************************************************************
 *
 *      CONSTRUCTOR:
 *              Bsp::Bsp( char *filename )
 *
 ******************************************************************************/
Bsp::Bsp( char *filename )
{
	char            *ptr;



	// Copiamos el nombre del fichero BSP.
	strcpy( filebsp, filename );

	// Eliminamos la extension .BSP y anadimos la extension DIR.
	strcpy( pathbsp, filename );
	for( ptr = pathbsp; *ptr != '\0'; ptr++ );
	for( ; *ptr != '.'; ptr-- );
	strcpy( ptr, ".DIR\\" );
}


/*******************************************************************************
 *
 *      DESTRUCTOR:
 *              Bsp::~Bsp()
 *
 ******************************************************************************/
Bsp::~Bsp()
{
}


/*******************************************************************************
 *
 *      METODO:
 *              int Bsp::UnPackBsp( void )
 *
 ******************************************************************************/
int Bsp::UnPackBsp( void )
{
	FILE            *fbsp;      // descriptor para el fichero BSP.
	unsigned int    i;          // indice de la dentry_t. 



	if( ( fbsp = fopen( filebsp, "rb" ) ) == NULL ) {
		TraceLog( TEST_LOG, "UnPackBsp(): fopenErr Unable to load %s", filebsp );
		return 0;
	}
	sizebsp = fsize( fbsp );

	fseek( fbsp, 0, SEEK_SET );

	fread( &header, sizeof( dheader_t ), 1, fbsp );

	fclose( fbsp );

	strcpy( file[0], pathbsp );
	strcat( file[0], ENTRY00 );
	strcpy( file[1], pathbsp );
	strcat( file[1], ENTRY01 );
	strcpy( file[2], pathbsp );
	strcat( file[2], ENTRY02 );
	strcpy( file[3], pathbsp );
	strcat( file[3], ENTRY03 );
	strcpy( file[4], pathbsp );
	strcat( file[4], ENTRY04 );
	strcpy( file[5], pathbsp );
	strcat( file[5], ENTRY05 );
	strcpy( file[6], pathbsp );
	strcat( file[6], ENTRY06 );
	strcpy( file[7], pathbsp );
	strcat( file[7], ENTRY07 );
	strcpy( file[8], pathbsp );
	strcat( file[8], ENTRY08 );
	strcpy( file[9], pathbsp );
	strcat( file[9], ENTRY09 );
	strcpy( file[10], pathbsp );
	strcat( file[10], ENTRY10 );
	strcpy( file[11], pathbsp );
	strcat( file[11], ENTRY11 );
	strcpy( file[12], pathbsp );
	strcat( file[12], ENTRY12 );
	strcpy( file[13], pathbsp );
	strcat( file[13], ENTRY13 );
	strcpy( file[14], pathbsp );
	strcat( file[14], ENTRY14 );

	entry[0] = header.entities;
	entry[1] = header.planes;
	entry[2] = header.miptex;
	entry[3] = header.vertices;
	entry[4] = header.visilist;
	entry[5] = header.nodes;
	entry[6] = header.texinfo;
	entry[7] = header.surfaces;
	entry[8] = header.lightmaps;
	entry[9] = header.boundnodes;
	entry[10] = header.leaves;
	entry[11] = header.lstsurf;
	entry[12] = header.edges;
	entry[13] = header.lstedges;
	entry[14] = header.hulls;

	for( i = 0; i < MAX_ENTRY_NUM; i++ )
	{
		if( !UnPackBsp( entry[i], &file[i][0] ) ) {
			TraceLog( TEST_LOG, "UnPackBsp(): UnPackBspErr %s", file[i] );
			return 0;
		}
	}
	return 1;
}


/*******************************************************************************
 *
 *      METODO:
 *              int Bsp::PackBsp(  XMSarray<char>* pBSP, XMSarray<char>* pXMS, char* filename = NULL )
 *
 ******************************************************************************/
int Bsp::PackBsp( XMSarray<char>* pBSP, XMSarray<char>* pXMS, char* filename )
{
	FILE            *fbsp;          // descriptor para el fichero BSP.
	unsigned int    i = 0;          // indice para las dentry_t



	for( i = 0; i < MAX_ENTRY_NUM; i++ )
	{
		if( !PackBsp( &entry[i], file[i], pBSP, pXMS ) ) {
			TraceLog( TEST_LOG, "PackBsp(): PackBspErr %s", file[i] );
			return 0;
		}
	}
	header.entities = entry[0];
	header.planes = entry[1];
	header.miptex = entry[2];
	header.vertices = entry[3];
	header.visilist = entry[4];
	header.nodes = entry[5];
	header.texinfo = entry[6];
	header.surfaces = entry[7];
	header.lightmaps = entry[8];
	header.boundnodes = entry[9];
	header.leaves = entry[10];
	header.lstsurf = entry[11];
	header.edges = entry[12];
	header.lstedges = entry[13];
	header.hulls = entry[14];

	for( long l = 0; l < sizeof( dheader_t ); l++ ) {
		(*pBSP)[l] = ((char*)&header)[l];
	}
	if( filename != NULL )
	{
		if( !FreeFile( pBSP, sizebsp, filename ) ) {
			TraceLog( TEST_LOG, "PackBsp(): FreeFileErr" );
			return 0;
		}
	}
	else {
		if( !FreeFile( pBSP, sizebsp, filebsp ) ) {
			TraceLog( TEST_LOG, "PackBsp(): FreeFileErr" );
			return 0;
		}
	}
	return 1;
}


/*******************************************************************************
 *
 *      METODO:
 *              int Bsp::UnPackBsp( dentry_t entry, char *filename )
 *
 ******************************************************************************/
int Bsp::UnPackBsp( dentry_t entry, char* filename )
{
	char            *ptr;           // puntero auxiliar.
	FILE            *fbsp;          // descriptor para el fichero BSP.
	FILE            *fpack;         // descriptor para el fichero THI.
	unsigned long   n;              // # de bytes escritos.
	int             nb;             // contador para el # de veces que se debe escribir.
	int             cb = BUFFSIZE;  // # de bytes que se deben escribir.
	int             rb;             // # resto de bytes que quedan por escibir.
	char            buf[BUFFSIZE];  // buffer para los datos.



	if( ( fbsp = fopen( filebsp, "rb" ) ) == NULL ) {
		TraceLog( TEST_LOG, "UnPackBsp(): fopenErr Unable to load %s", filebsp );
		return 0;
	}
	if( ( fpack = fopen( filename, "wb" ) ) == NULL ) {
		TraceLog( TEST_LOG, "UnPackBsp(): fopenErr Unable to load %s", filename );
		return 0;
	}
	nb = entry.size / cb;
	rb = entry.size % cb;

	fseek( fbsp, entry.offset, SEEK_SET );
	fseek( fpack, 0, SEEK_SET );

	for( int i = 0; i < nb; i++ )
	{
		if( ( n = fread( buf, sizeof(char), cb, fbsp ) ) != cb )
		{
			TraceLog( TEST_LOG, "UnPackBsp(): freadErr n=%ld", n );
			return 0;
		}
		if( ( n = fwrite( buf, sizeof(char), cb, fpack ) ) != cb )
		{
			TraceLog( TEST_LOG, "UnPackBsp(): fwriteErr n=%ld", n );
			return 0;
		}
	}
	if( ( n = fread( buf, sizeof(char), rb, fbsp ) ) != rb )
	{
		TraceLog( TEST_LOG, "UnPackBsp(): freadErr n=%ld", n );
		return 0;
	}
	if( ( n = fwrite( buf, sizeof(char), rb, fpack ) ) != rb )
	{
		TraceLog( TEST_LOG, "UnPackBsp(): fwriteErr n=%ld", n );
		return 0;
	}
	fclose( fbsp );
	fclose( fpack );

	return 1;
}



/*******************************************************************************
 *
 *      FUNCTION:
 *              int Bsp::PackBsp( dentry_t *entry, char *filename, XMSarray<char>* pBSP, XMSarray<char>* pXMS )
 *
 ******************************************************************************/
int Bsp::PackBsp( dentry_t *entry, char *filename, XMSarray<char>* pBSP, XMSarray<char>* pXMS )
{
	FILE            *fpack;        // descriptor para el fichero XMS.
	long            sizeXMS;        // tamano del fichero XMS.
	int             nb;             // # de veces que se debe escribir.
	int             cb = BUFFSIZE;  // # de bytes que se deben escribir.
	int             rb;             // # resto de bytes que quedan por escibir.
	long            i = 0;          // contador de # de ciclos de copia.
	long            j = 0;          // contador de # de caracteres por ciclos.
	long            l = 0;          // contador de # de caracteres escritos.


	if( ( AllocFile( pXMS, &sizeXMS, filename ) ) == NULL ) {
		TraceLog( TEST_LOG, "PackBsp(): AllocFileErr" );
		return 0;
	}
	nb = sizeXMS / cb;
	rb = sizeXMS % cb;

	for( i = 0; i < nb; i++ ) {
		for( j = 0; j < cb; l++, j++ ) {
			(*pBSP)[entry->offset+l] = (*pXMS)[l];
		}
		working->AddChar( '.' );
	}
	for( j = 0; j < rb; l++, j++ ) {
		(*pBSP)[entry->offset+l] = (*pXMS)[l];
	}
	working->AddChar( '.' );

	entry->size = sizeXMS;

	return 1;
}


/*******************************************************************************
 *
 *      METODO:
 *              int Bsp::UpdateEntry( unsigned int index, long newsize )
 *
 ******************************************************************************/
int Bsp::UpdateEntry( unsigned int index, long newsize )
{
	unsigned int            i = 0;
	long                    oldsize;


	oldsize = entry[index].size;
	entry[index].size = newsize;

	sizebsp = sizebsp + newsize - oldsize;

	for( i = 0; i < MAX_ENTRY_NUM; i++ )
	{
		if( entry[i].offset > entry[index].offset )
		{
			entry[i].offset = entry[i].offset + newsize - oldsize;
		}
	}
	return 1;
}


/*******************************************************************************
 *
 *      METODO:
 *              long Bsp::UnTexBsp( XMSarray<miptex_t> *XM, long numXMs, char *palettename, int type )
 *
 ******************************************************************************/
long Bsp::UnTexBsp( XMSarray<miptex_t> *XM, long numXMs, char *palettename, int type )
{
	long            i = 0;
	char            mipname[257];
	char            lumpname[257];
	char            buf[257];
	char            pal[769];
	miptex_t        miptex;
	dmiptexlump_t   miptexlump;
	BitMap          bmptr;
	FILE            *fbmp, *flump, *fpal;
	char            *p;
	unsigned long   size;
	RGB             palette256[256];


	if( ( fpal = fopen( palettename, "rb" ) ) == NULL )
	{
		return 0L;
	}
	if( !ReadBytes( fpal, pal, 768 ) )
	{ 
		fclose( fpal );
		return 0L;
	}
	p = pal;
	for( i = 0; i < 256; i++ )
	{
		palette256[i].R = *p++;
		palette256[i].G = *p++;
		palette256[i].B = *p++;
	}
	fclose( fpal );

	strcpy( lumpname, pathbsp );
	strcat( lumpname, "\\" );
	strcat( lumpname, ENTRY02 );

	if( ( flump = fopen( lumpname, "rb" ) ) == NULL )
	{
		return 0L;
	}
	fseek( flump, 0, SEEK_SET );

	fread( &miptexlump, sizeof( dmiptexlump_t ), 1, flump );

	for( i = 0; i < miptexlump.nummiptex; i++ )
	{
		miptex = (*XM)[i];

		strcpy( mipname, pathbsp );
		if( miptex.name[0] != '*' && miptex.name[0] != '+' )
		{
			strcat( mipname, miptex.name );
		}
		else
		{
			strcpy( buf, miptex.name );
			buf[0] = '_';
			strcat( mipname, buf );
		}
		switch( type )
		{
			case 1:
				strcat( mipname, ".BMP" );
			break;

			case 0:
			default:
				strcat( mipname, ".PCX" );
			break;
		}
		bmptr.width = (unsigned short)miptex.width;
		bmptr.height = (unsigned short)miptex.height;

		size = (unsigned long)bmptr.width * (unsigned long)bmptr.height;

		if( ( bmptr.data = (char*)malloc( size ) ) == NULL )
		{
			return 0;
		}
		if( ( fbmp = fopen( mipname, "wb" ) ) == NULL )
		{
			fclose( flump );
			free( bmptr.data );
			return 0L;
		}
		if( fseek( flump, miptexlump.dataofs[i]+miptex.offsets[0], SEEK_SET ) < 0 )
		{
			fclose( fbmp );
			fclose( flump );
			free( bmptr.data );
			return 0L;
		}
		if( !ReadBytes( flump, bmptr.data, size ) )
		{ 
			fclose( fbmp );
			fclose( flump );
			free( bmptr.data );
			return 0L;
		}
		switch( type )
		{
			case 1:
				if( !SaveBMP( fbmp, &bmptr, palette256 ) )
				{
					fclose( fbmp );
					fclose( flump );
					free( bmptr.data );
					return 0L;
				}
			break;

			case 0:
			default:
				if( !SavePCX( fbmp, &bmptr, palette256 ) )
				{
					fclose( fbmp );
					fclose( flump );
					free( bmptr.data );
					return 0L;
				}
			break;
		}
		fclose( fbmp );
		free( bmptr.data );
	}
	fclose( flump );

	return numXMs;
}


/*******************************************************************************
 *
 *      METODO:
 *              int Bsp::SaveMipTex( miptex_t miptex, long index, char *filename, char* palettename, int type )
 *
 ******************************************************************************/
int Bsp::SaveMipTex( miptex_t miptex, long index, char *filename, char *palettename, int type )
{
	long            i = 0;
	char            lumpname[257];
	char            buf[257];
	char            *ptr;
	char            pal[769];
	dmiptexlump_t   miptexlump;
	BitMap          bmptr;
	FILE            *fbmp, *flump, *fpal;
	char            *p;
	unsigned long   size;
	RGB             palette256[256];


	if( ( fpal = fopen( palettename, "rb" ) ) == NULL )
	{
		TraceLog( TEST_LOG, "Bsp::SaveMipTex(): fopenErr %s", palettename );
		return 0;
	}
	if( !ReadBytes( fpal, pal, 768 ) )
	{ 
		TraceLog( TEST_LOG, "Bsp::SaveMipTex(): ReadBytesErr" );
		fclose( fpal );
		return 0;
	}
	p = pal;
	for( i = 0; i < 256; i++ )
	{
		palette256[i].R = *p++;
		palette256[i].G = *p++;
		palette256[i].B = *p++;
	}
	fclose( fpal );

	strcpy( lumpname, pathbsp );
	strcat( lumpname, "\\" );
	strcat( lumpname, ENTRY02 );

	if( ( flump = fopen( lumpname, "rb" ) ) == NULL )
	{
		TraceLog( TEST_LOG, "Bsp::SaveMipTex(): fopenErr %s", lumpname );
		return 0;
	}
	fseek( flump, 0, SEEK_SET );

	fread( &miptexlump, sizeof( dmiptexlump_t ), 1, flump );

	bmptr.width = (unsigned short)miptex.width;
	bmptr.height = (unsigned short)miptex.height;

	size = (unsigned long)bmptr.width * (unsigned long)bmptr.height;

	if( ( bmptr.data = (char*)malloc( size ) ) == NULL )
	{
		TraceLog( TEST_LOG, "Bsp::SaveMipTex(): newErr" );
		return 0;
	}
	if( ( fbmp = fopen( filename, "wb" ) ) == NULL )
	{
		TraceLog( TEST_LOG, "Bsp::SaveMipTex(): fopenErr %s", filename );
		fclose( flump );
		free( bmptr.data );
		return 0;
	}
	if( fseek( flump, miptexlump.dataofs[index]+miptex.offsets[0], SEEK_SET ) < 0 )
	{
		TraceLog( TEST_LOG, "Bsp::SaveMipTex(): fseekErr" );
		fclose( fbmp );
		fclose( flump );
		free( bmptr.data );
		return 0;
	}
	if( !ReadBytes( flump, bmptr.data, size ) )
	{ 
		TraceLog( TEST_LOG, "Bsp::SaveMipTex(): ReadBytesErr" );
		fclose( fbmp );
		fclose( flump );
		free( bmptr.data );
		return 0;
	}
	switch( type )
	{
		case 1:
			if( !SaveBMP( fbmp, &bmptr, palette256 ) )
			{
				TraceLog( TEST_LOG, "Bsp::SaveMipTex(): SaveBMPErr" );
				fclose( fbmp );
				fclose( flump );
				free( bmptr.data );
				return 0;
			}
		break;

		case 0:
		default:
			if( !SavePCX( fbmp, &bmptr, palette256 ) )
			{
				TraceLog( TEST_LOG, "Bsp::SaveMipTex(): SavePCXErr" );
				fclose( fbmp );
				fclose( flump );
				free( bmptr.data );
				return 0;
			}
		break;
	}
	fclose( fbmp );
	free( bmptr.data );

	return 1;
}


/*******************************************************************************
 *
 *      METODO:
 *              XMSarray<char>* Bsp::GetBspEntiti(      long index,
 *                                                              XMSarray<char> *pX,
 *                                                              long *size,
 *                                                              long *size12,
 *                                                              long *size2 )
 *
 *******************************************************************************/
XMSarray<char>* Bsp::GetBspEntiti(      long index,
					XMSarray<char> *pX,
					long *size,
					long *size12,
					long *size2 )
{
	long            i = 0,
			j = 0,
			k = 0,
			l = 0;

	l = *size-1;

	if( (*pX)[l] != '\n' )
	{
		for( ; (*pX)[l] != '}'; l-- );
		l++;
		(*pX)[l] = '\n';
		*size = l+1;
	}       
	/* empezamos a recorrer la cadena hasta la palabra. */
	for( i = 0, j = 0; i < index; i++ ) {
		/* si no hay llave al principio de la cadena ... */
		if( (*pX)[j] != '{' ) {
			for( ; (*pX)[j] != '{'; j++ );
		}
		/* si hay llave al principio de la cadena ... */
		if( (*pX)[j] == '{' ) {
			for( ; (*pX)[j] != '}'; j++ );
		}
	}
	/* si no hay llave al principio de la cadena ... */
	if( (*pX)[j] != '{' ) {
		for( ; (*pX)[j] != '{'; j++ );
	}
	/* ahora que p1 apunta al principio de la cadena hacemos que p2 apunte al final de la palabra. */
	for( k = 0; (*pX)[j] != '}'; k++, j++ );

	/* si no hay llave al principio de la cadena ... */
	if( (*pX)[j] != '{' ) {
		for( ; (*pX)[j] != '{' && j < *size; j++, k++ );
	}
	*size12 = k;
	*size2 = *size - j;

	return pX;

}


/*******************************************************************************
 *
 *      METODO:
 *              char* Bsp::GetEntitiSpecific(   char *specific,
 *                                              char *ptr,
 *                                              long size,
 *                                              char **ptr1,
 *                                              long *size12,
 *                                              char **ptr2,
 *                                              long *size2 )
 *
 *******************************************************************************/
char* Bsp::GetEntitiSpecific(   char *specific,
				char *ptr,
				long size,
				char **ptr1,
				long *size12,
				char **ptr2,
				long *size2 )
{
	char            *p1;            // puntero al principio de la specific.
	char            *p2;            // puntero al principio de la siguiente specific.
	char            *p3;            // puntero auxiliar para contar caracteres hasta el final de la entiti.
	long            i = 0;          // contador para el # de caracteres de la specific.
	long            j = 0;          // contador para el # de caracteres de la siguiente specific al final.



	if( ( p1 = strstr( ptr, specific ) ) == NULL ) {
		*size12 = *size2 = 0;
		return *ptr1 = *ptr2 = NULL;
	}
	for( p2 = p1, i = 0; *p2 != '\n' && *p2 != '\r';  p2++, i++ ); 

	for( ; *p2 != '"' && *p2 != '}'; p2++, i++ ); 

	if( *p2 != '}' )
	{
		for( p3 = p2, j = 0; *p3 != '}'; p3++, j++ ); 
		
	}
	else
	{
		j = 0;
	}
	*size12 = i;
	*size2 = j;
	*ptr1 = p1;
	*ptr2 = p2;

	return p1;

}



/******************************************************************************
 *
 *      METODO:
 *              long Bsp::InitGEntities(        XMSarray<GEntiti> *pGE,
 *                                              long *numGE,
 *                                              XMSarray<ClassId> *pClassIds,
 *                                              long maxClassIds )
 *
 ******************************************************************************/
long Bsp::InitGEntities(        XMSarray<GEntiti> *pGE,
				long *numGE,
				XMSarray<ClassId> *pClassIds,
				long maxClassIds )
{
	ifstream        thi;            // descriptor del fichero de entities.
	char            line[257];      // linea para los datos.
	char            specific[257];  // specifies.
	char            args[257];      // argumentos.
	char            buf[257];       // buffer auxiliar.
	GEntiti         gEntiti;        // GEntiti auxiliar.
	long            entcount1 = 0;  // contador 1 de gentities
	long            entcount2 = 0;  // contador 2 de gentities
	long            i = 0;          // contador para los elementos.
	long            nh = 0;         // # de elementos del array.


	thi.open( file[0], ios::in | ios::binary );

	if( !thi ) {
		TraceLog( TEST_LOG, "InitGEntities(): File not found (%s)", file[0] );
		*numGE = 0;
		return 0;
	}
	if( pGE != NULL )
	{
		nh = pGE->size();
	}
	do {
		thi.getline( line, 257 );
		if( thi.eof() ) {
			break;
		}
		strcpy( buf, line );
		strcpy( specific, Word2( 0, buf ) );

		// Si el array de gentities no es NULL, lo inicializamos.
		if( pGE != NULL )
		{
			if( !strcmp( specific, "{" ) )
			{
				entcount1++;

				gEntiti.x = 0.0;
				gEntiti.y = 0.0;
				gEntiti.z = 0.0;
				gEntiti.c = 0x00000000;
				gEntiti.model = 0;
				gEntiti.target = 0;
				gEntiti.targetname = 0;
				gEntiti.index = -1;

				do {

					thi.getline( line, 257 );
					strcpy( buf, line );
					strcpy( specific, Word2( 0, buf ) );

					if( !strcmp( specific, "origin" ) )
					{
						strcpy( buf, line );
						gEntiti.x = atof( Word2( 1, buf ) );
						strcpy( buf, line );
						gEntiti.y = atof( Word2( 2, buf ) );
						strcpy( buf, line );
						gEntiti.z = atof( Word2( 3, buf ) );
					}
					else if( !strcmp( specific, "classname" ) )
					{
						strcpy( buf, line );
						strcpy( args, Word2( 1, buf ) );
						gEntiti.c = GetClassId( pClassIds, maxClassIds, args, &gEntiti.c );
					}
					else if( !strcmp( specific, "model" ) )
					{
						strcpy( buf, line );
						strcpy( args, Word2( 1, buf ) );
						gEntiti.model = GetClassModel( args, &gEntiti.c );
					}
					else if( !strcmp( specific, "target" ) )
					{
						strcpy( buf, line );
						strcpy( args, Word2( 1, buf ) );
						gEntiti.target = GetClassTarget( args, &gEntiti.c );
					}
					else if( !strcmp( specific, "targetname" ) )
					{
						strcpy( buf, line );
						strcpy( args, Word2( 1, buf ) );
						gEntiti.targetname = GetClassTargetName( args, &gEntiti.c );
					}
				} while( strcmp( specific, "}" ) );

				entcount2++;

				if( i < nh )
				{
					gEntiti.index = i;
					(*pGE)[i++] = gEntiti;
					// TraceLog( TEST_LOG, "i = %ld GEntiti.c=%lx",i, gEntiti.c );
				}
				else
				{
					break;
				}
			}

		}
		// Si en NULL contamos el # de gentities.
		else 
		{
			if( !strcmp( specific, "{" ) )
			{
				// TraceLog( TEST_LOG, "entcount1 = %ld",entcount1 );
				entcount1++;
			}
			else if( !strcmp( specific, "}" ) )
			{
				// TraceLog( TEST_LOG, "entcount2 = %ld",entcount2 );
				entcount2++;
			}
		}
	} while( !thi.eof() );

	if( entcount1 != entcount2 )
	{
		nh = 0;
	}
	else
	{
		if( entcount1 < MAX_GENTITIES_NUM )
		{
			nh = entcount1;
		}
		else
		{
			nh = MAX_GENTITIES_NUM;
		}
	}
	thi.close();

	*numGE = nh;

	return nh;
}


/******************************************************************************
 *
 *      METODO:
 *              long Bsp::Move( XMSarray<char> *pX, long *size, GEntiti *pGE )
 *
 ******************************************************************************/
long Bsp::Move( XMSarray<char> *pX, long *size, GEntiti *pGE )
{
	char            *ptr1;          // puntero al principio de la entiti.
	char            *ptr2;          // puntero a la siguiente entiti.
	char            *ptr3;          // puntero al principio de la linea de la entiti.
	char            *ptr4;          // puntero a la siguiente linea.
	long            size1;          // # de caracteres hasta la entiti.
	long            size12;         // # de caracteres de la entiti.
	long            size2;          // # de caracteres desde la siguiente entiti al final.
	long            size3;          // # de caracteres hasta la specific.
	long            size34;         // # de caracteres de la specific.
	long            size4;          // # de caracteres de la siguiente specific al final.
	long            size5;          // # de caracteres de la nueva specific.
	long            size6;          // # de caracteres de la nueva entiti.
	long            size7;          // # de caracteres del nuevo fichero.
	long            i = 0;          // contador para el # de caracters de la entiti.
	long            j = 0;          // contador para el # de caracters de la entiti.
	long            k = 0;          // contador para el # de caracters de la linea.
	char            entiti[MAX_ENTITI_LEN];
					// buffer auxiliar para la entiti.
	char            specific[MAX_SPECIFIC_LEN];
					// buffer auxiliar para la linea.
	char            fbuf[33];       // buffer auxiliar para los floats.


	// Obtenemos los punteros y tamanos de la entrada de la entiti.
	GetBspEntiti( pGE->index, pX, size, &size12, &size2 );

	// Obtenemos el # de caracters hasta la entiti.
	size1 = *size - size12 - size2;

	// Copiamos en el buffer auxiliar la entiti.
	for( i = 0; i < size12; i++ )
	{
		entiti[i] = (*pX)[size1+i];
	}

	// Obtenemos los punteros a la specific de la entiti.
	GetEntitiSpecific( "\"origin\"", &entiti[0], size12, &ptr3, &size34, &ptr4, &size4 );

	// Obtenemos el # de caracteres hasta la specific.
	size3 = size12 - size34 - size4;

	// Cambiamos los campos de la specific de la entiti.
	strcpy( specific, "\"origin\"" );
	strcat( specific, " " );
	strcat( specific, "\"" ); 
	sprintf( fbuf, "%ld ", (long)pGE->x );
	strcat( specific, fbuf );
	sprintf( fbuf, "%ld ", (long)pGE->y );
	strcat( specific, fbuf );
	sprintf( fbuf, "%ld", (long)pGE->z );
	strcat( specific, fbuf );
	strcat( specific, "\"\n}\n" ); 

	// Obtenemos el tamano de la specific de la entiti.
	size5 = strlen( specific );

	// copiamos el final de la specific en la entiti. 
	for( j = 0; j < size4; j++ )
	{
		ptr3[j] = ptr4[j];
	}
	// Copiamos la nueva specific de la entiti al final.
	for( k = 0; k < size5; j++, k++ )
	{
		ptr3[j] = specific[k];
	}
	// Obtenemos el nuevo tamano de la entiti.
	size6 = size3 + j-2;

	// Copiamos el final del fichero.
	for( i = 0; i < size2; i++ )
	{
		(*pX)[size1+i] = (*pX)[size1+size12+i];
	}
	// Copiamos la nueva entiti al final del fichero.
	for( j = 0; j < size6; i++, j++ )
	{
		(*pX)[size1+i] = entiti[j];
	}
	// Pongo nulo al final del buffer;
	(*pX)[size1+i] = '\0';
	i++;
	// Calculamos el nuevo tamano del fichero.
	size7 = size1 + i;

	// Devolvemos el nuevo tamano del fichero.
	*size = size7;

	return size7;

}


/******************************************************************************
 *
 *      METODO:
 *              long Bsp::Supr( XMSarray<char> *pX, long *size, GEntiti *pGE )
 *
 ******************************************************************************/
long Bsp::Supr( XMSarray<char> *pX, long *size, GEntiti *pGE )
{
	long            size1;          // # de caracteres hasta la entiti.
	long            size12;         // # de caracteres de la entiti.
	long            size2;          // # de caracteres desde la siguiente entiti al final.
	long            size7;          // # de caracteres del nuevo fichero.
	long            i = 0;          // contador para el # de caracters de la entiti.


	// Obtenemos los punteros y tamanos de la entrada de la entiti.
	GetBspEntiti( pGE->index, pX, size, &size12, &size2 );

	// Obtenemos el # de caracters hasta la entiti.
	size1 = *size - size12 - size2;

	// Copiamos el final del fichero.
	for( i = 0; i < size2; i++ )
	{
		(*pX)[size1+i] = (*pX)[size1+size12+i];
	}
	// Pongo nulo al final del buffer;
	(*pX)[size1+i] = '\0';
	i++;
	// Calculamos el nuevo tamano del fichero.
	size7 = size1 + i;

	// Devolvemos el nuevo tamano del fichero.
	*size = size7;

	return size7;

}


/******************************************************************************
 *
 *      METODO:
 *              long Bsp::Insert(       XMSarray<char> *pX,
 *                                      long *sizeX,
 *                                      GEntiti *pGE,
 *                                      XMSarray<ClassId> *pClassIds,
 *                                      long maxClassIds )
 *
 ******************************************************************************/
long Bsp::Insert(       XMSarray<char> *pX,
			long *size,
			GEntiti *pGE,
			XMSarray<ClassId> *pClassIds,
			long maxClassIds )
{
	long            size1;          // # de caracteres hasta la entiti.
	long            size12;         // # de caracteres de la entiti.
	long            size2;          // # de caracteres desde la siguiente entiti al final.
	long            i = 0;          // contador para el # de caracters de la entiti.
	char            entiti[MAX_ENTITI_LEN];
					// buffer auxiliar para la entiti.
	char            cbuf[MAX_CLASSNAME_LEN];        
					// buffer auxiliar para el classname.
	char            fbuf[33];       // buffer auxiliar para los floats.


	// Obtenemos los campos de la specific de la entiti.
	strcpy( entiti, "{\n" );
	strcat( entiti, "\"classname\"" );
	strcat( entiti, " " );
	strcat( entiti, "\"" ); 
	strcpy( cbuf, GetClassName( pClassIds, maxClassIds, cbuf, pGE->c ) );
	strcat( entiti, cbuf );
	strcat( entiti, "\"\n" ); 
	strcat( entiti, "\"origin\"" );
	strcat( entiti, " " );
	strcat( entiti, "\"" ); 
	sprintf( fbuf, "%ld ", (long)pGE->x );
	strcat( entiti, fbuf );
	sprintf( fbuf, "%ld ", (long)pGE->y );
	strcat( entiti, fbuf );
	sprintf( fbuf, "%ld", (long)pGE->z );
	strcat( entiti, fbuf );
	strcat( entiti, "\"\n}\n" ); 
	
	//  Ponemos el final del fichero a fin de linea y calculamos el tamano.
	i = *size-1;

	if( (*pX)[i] != '\n' )
	{
		for( ; (*pX)[i] != '}'; i-- );
		i++;
		(*pX)[i] = '\n';
	}       
	size1 = i+1;

	// Obtenemos el tamano de la entiti.
	size12 = strlen( entiti );

	// copiamos el final de la specific en la entiti. 
	for( i = 0; i < size12; i++ )
	{
		(*pX)[size1+i] = entiti[i];
	}
	// Pongo nulo al final del buffer;
	(*pX)[size1+i] = '\0';
	i++;
	// Calculamos el nuevo tamano del fichero.
	size2 = size1 + i;

	// Devolvemos el nuevo tamano del fichero.
	*size = size2;

	return size2;

}


/*******************************************************************************
 *------------------------------------------------------------------------------
 *
 *      FIN DE FICHERO
 *
 *------------------------------------------------------------------------------
 ******************************************************************************/

